'use strict';

/**
 * Component that draws the boundingbox of an entity.
 * @moduleName Debug
 */
bento.define('components/debug', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Utils, Tween) {
    'use strict';

    var globalHold = null;
    return function (settings) {
        var viewport = Bento.getViewport(),
            drawBoundingBox = settings.drawBoundingBox || true,
            move = settings.move || false,
            entity,
            color = settings.color || [0, 0, 1, 0.5],
            onUpdate = settings.update || function () {},
            isHolding = false,
            pointerPos = new Vector2(0, 0),
            holdPos = new Vector2(0, 0),
            startPos = new Vector2(0, 0),
            clickable = new Clickable({
            onClick: function onClick(data) {
                if (!globalHold || globalHold.z <= entity.z) {
                    globalHold = entity;
                } else {
                    return;
                }
                holdPos = entity.position.substract(data.worldPosition);
                pointerPos = data.worldPosition;
                startPos = entity.position.clone();
            },
            pointerMove: function pointerMove(data) {
                if (globalHold === entity) {
                    pointerPos = data.worldPosition;
                }
            },
            pointerUp: function pointerUp() {
                if (globalHold === entity) {
                    globalHold = null;
                    console.log(entity.name, 'Vector2(' + Math.floor(entity.position.x) + ', ' + Math.floor(entity.position.y) + ')');
                    console.log('moved:', 'Vector2(' + Math.floor(entity.position.x - startPos.x) + ', ' + Math.floor(entity.position.y - startPos.y) + ')');
                    console.log('============');
                }
            },
            onHoldEnd: function onHoldEnd() {}
        }),
            postDraw = function postDraw(data) {
            var box;
            if (entity) {
                box = entity.getBoundingBox();
                data.renderer.fillRect(color, box.x - viewport.x, box.y - viewport.y, box.width, box.height);
            }
        },
            component = {
            name: 'debug',
            setColor: function setColor(array) {
                color = array;
            },
            start: function start() {
                startPos = entity.position.clone();
            },
            update: function update(data) {
                if (globalHold === entity) {
                    entity.position = pointerPos.add(holdPos);
                }

                onUpdate();
            },
            attached: function attached(data) {
                var originalDraw;
                entity = data.entity;
                originalDraw = entity.draw;
                if (entity) {
                    // take over draw function
                    entity.extend({
                        draw: function draw(data) {
                            // after drawing, call the postDraw to draw the boundingbox
                            originalDraw.apply(entity, [data]);
                            postDraw(data);
                        }
                    });
                }
                if (move) {
                    // allow dragging
                    entity.attach(clickable);
                }

                // move self to top
                entity.moveComponentTo(component, 0);
            }
        };
        return component;
    };
});