'use strict';

/**
 * This component swap's an existing sprite's draw function so it only draws within
 * a certain bounding box.
 * NOTE: bug with sprite animations
 * @moduleName MaskSprite
 */
bento.define('components/masksprite', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'bento/components/sprite'], function (Bento, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, Sprite) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        // a specific sprite can be passed, otherwise the component will search for one
        var sprite = settings.sprite;
        var component = {
            name: 'maskSprite',
            boundingBox: settings.boundingBox,
            start: function start(data) {
                if (!sprite) {
                    sprite = entity.getComponent('sprite');
                    if (!sprite) {
                        Utils.log('WARNING: entity has no sprite');
                        return;
                    }
                }
                // swap
                sprite.draw = maskDraw;
            },
            destroy: function destroy(data) {
                if (!sprite) {
                    return;
                }
                // restore
                if (sprite.draw === maskDraw) {
                    sprite.draw = Sprite.prototype.draw;
                }
            },
            attached: function attached(data) {
                entity = data.entity;
            }
        };
        // overwritten draw function, replicates most of sprite's draw
        var maskDraw = function maskDraw(data) {
            if (!this.currentAnimation || !this.visible) {
                return;
            }

            // do the sprite update
            this.updateFrame();

            // get source and target rectangles
            var source = new Rectangle(this.sourceX, this.sourceY, this.frameWidth, this.frameHeight);
            var target = new Rectangle(-this.origin.x | 0, -this.origin.y | 0, this.frameWidth, this.frameHeight);
            // calculate source and target bounding boxes
            var sourceBounding = component.boundingBox.clone();
            var targetBounding = component.boundingBox.clone();
            // move sourceBounding with the source offset
            sourceBounding.x += source.x;
            sourceBounding.y += source.y;
            // bounding box is relative to the origin point
            sourceBounding.x += this.origin.x;
            sourceBounding.y += this.origin.y;

            // what we need to draw in the end is the intersection between bounding and source/target
            source = source.intersection(sourceBounding);
            target = target.intersection(targetBounding);

            // nothing to draw?
            if (!source.width || !source.height || !target.width || !target.height) {
                return;
            }
            // draw
            data.renderer.drawImage(this.spriteImage, source.x, source.y, source.width, source.height, target.x, target.y, target.width, target.height);
        };

        if (!component.boundingBox) {
            Utils.log('ERROR: pass Rectangle as boundingbox');
            return {};
        }

        return component;
    };
});