'use strict';

bento.define('components/pathfollower', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/eventlistener', 'modules/box2d'], function (Bento, Vector2, Rectangle, Sprite, Entity, EventSystem, Text, Utils, Screen, Tween, Globals, EventListener, Box2D) {
    return function (settings) {
        var viewport = Bento.getViewport();
        var pathName = settings.pathName;
        var entity;

        /* properties that we get from the path */

        var points; // array of points e.g. [ {x:10,y:20} , {x:5,y:3} ]
        var timePerPoint;
        var ease;

        /* variables for movement logic */
        var startPos;
        var currentPointIndex = 0;
        var currentPoint;
        var nextPoint;
        var closedLoop = false; // we move back and forth unless the final point is the same as the first point
        var reversing = false; // are we moving backwards along the path?

        var tween;
        var body;

        // var debug = {
        //     name: 'pathFollowerDebug',
        //     z: Globals.Layers.BOX2D_DEBUG,
        //     pos: new Vector2(0, 0),
        //     draw ({renderer}) {
        //         renderer.fillCircle(Utils.color('000000'), this.pos.x, this.pos.y, 3);
        //     }
        // };

        var baseDt = Globals.simulationSpeed / 60;

        var onStep = function onStep(dt) {
            if (tween) {
                tween.update({
                    speed: baseDt / dt
                });
            }
        };

        var component = {
            name: 'pathFollower',
            start: function start(data) {
                startPos = entity.position.clone();
                body = entity.getComponent('body');
                // Bento.objects.attach(debug);
                Box2D.addStepFunction(onStep);
            },
            destroy: function destroy(data) {
                Box2D.removeStepFunction(onStep);
                // Bento.objects.remove(debug);
            },
            update: function update(data) {
                if (!points) {
                    var path = Globals.paths[pathName];
                    if (!path) {
                        return;
                    }
                    points = path.points;
                    timePerPoint = path.timePerPoint || 120;
                    ease = path.ease || 'easeInOutQuad';

                    currentPointIndex = 0;
                    currentPoint = points[0];
                    nextPoint = points[1];

                    if (points[0].x === points[points.length - 1].x && points[0].y === points[points.length - 1].y) {
                        closedLoop = true;
                    }

                    moveToNextPoint();
                }
            },
            attached: function attached(data) {
                entity = data.entity;
            }
        };

        var tweenUpdate = function tweenUpdate(v, t) {
            var pathX = currentPoint.x + (nextPoint.x - currentPoint.x) * v;
            var pathY = currentPoint.y + (nextPoint.y - currentPoint.y) * v;

            var x1 = entity.position.x;
            var y1 = entity.position.y;
            var x2 = startPos.x + pathX;
            var y2 = startPos.y + pathY;

            // sometimes tiled polylines are not normalised... so we should subtract the first point
            x2 -= points[0].x;
            y2 -= points[0].y;

            // debug.pos.x = x2;
            // debug.pos.y = y2;
            if (!body) {
                body = entity.getComponent('body');
                if (!body) return;
            }
            body.setLinearVelocity(new Vector2(x2 - x1, y2 - y1).scale(16.0)); // higher scale = tigher following
        };

        // figure out the next point
        // start the tween to go there
        // when we're done, the current point becomes the next point
        // then we repeat the process.
        var moveToNextPoint = function moveToNextPoint() {
            var nextPointIndex;
            if (!reversing) {
                nextPointIndex = currentPointIndex + 1;
                if (closedLoop) {
                    if (nextPointIndex >= points.length - 1) {
                        nextPointIndex = 0;
                    }
                } else {
                    if (nextPointIndex >= points.length) {
                        reversing = true;
                        nextPointIndex = currentPointIndex - 1;
                    }
                }
            } else {
                nextPointIndex = currentPointIndex - 1;
                if (nextPointIndex < 0) {
                    nextPointIndex = 1;
                    reversing = false;
                }
            }
            nextPoint = points[nextPointIndex];

            tween = new Tween.TweenBehavior({
                from: 0, to: 1,
                in: timePerPoint / Globals.simulationSpeed,
                ease: ease,
                onUpdate: tweenUpdate,
                onComplete: function onComplete() {
                    currentPointIndex = nextPointIndex;
                    currentPoint = nextPoint;
                    moveToNextPoint();
                }
            });
        };

        return component;
    };
});