'use strict';

/**
 * Lightweight graphical object with just origin, position and scale
 * @moduleName SimpleSprite
 * @snippet SimpleSprite|snippet
SimpleSprite({
    imageName: '${1}',
    position: new Vector2(${2:0}, ${3:0}),
    scale: new Vector2(${4:1}, ${5:1})
})
 */
bento.define('components/simplesprite', ['bento', 'bento/math/vector2', 'bento/utils'], function (Bento, Vector2, Utils) {
    'use strict';

    return function (settings) {
        var image = settings.image ? settings.image : Bento.assets.getImage(settings.imageName);
        var p = settings.position || new Vector2(0, 0);
        var o = settings.origin || new Vector2(image.width / 2, image.height / 2);
        var scale = settings.scale || new Vector2(1, 1);
        var component = {
            name: settings.name || 'simpleSprite',
            image: image,
            position: p,
            origin: o,
            scale: scale,
            draw: function draw(data) {
                data.renderer.drawImage(image, 0, 0, // source pos
                image.width, image.height, // source dimensions
                p.x - o.x * scale.x, p.y - o.y * scale.y, // dest pos
                image.width * scale.x, image.height * scale.y // dest size
                );
            },
            setImage: function setImage(img) {
                image = Utils.isString(img) ? Bento.assets.getImage(img) : img;
                component.image = image;
            }
        };
        return component;
    };
});