'use strict';

/**
 * Sparkle emitter from kaiju rush
 * @moduleName Sparkles
 * @snippet Sparkles|snippet
Sparkles({
    offset: new Vector2(0, 0),
    scale: new Vector2(0.12, 0.12),
    radius: new Vector2(16, 16),
    spawnRate: 1.0,
    speed: 1
})
 */
bento.define('components/sparkles', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Utils, Tween) {
    'use strict';

    return function (settings) {
        var offset = settings.offset || new Vector2(0, 0);
        var radius = settings.radius || new Vector2(16, 16);
        var spawnRate = settings.spawnRate || 0.025;
        var speed = settings.speed || 1;
        var scale = settings.scale || new Vector2(1, 1);
        var imageName = settings.imageName || 'gui/sparkle';
        var nextSparkleTime = Utils.getRandomFloat(20);

        // allow sparkles to spawn at intervals, for a more 'natural' effect
        // once remainingSparkles is 0, we need to wait for the next burst to regenerate them
        var nextBurstTime = 0;
        var remainingSparkles = 0;

        var lastPos = new Vector2(0, 0);

        var spawnSparkle = function spawnSparkle() {
            var verticalSpeed = Utils.getRandomRangeFloat(0.04, 0.25) * speed;
            var shrinkSpeed = scale.x / Utils.getRandomRangeFloat(14, 16) * speed;
            var growSpeed = scale.x / Utils.getRandomRangeFloat(3, 3.5) * speed;
            var sparkleBehavior = {
                name: 'sparkleBehavior',
                update: function update() {
                    // fly up
                    sparkle.position.y -= verticalSpeed;
                    // grow/shrink
                    if (sparkle.timer < 6) {
                        sparkle.scale.x += growSpeed;
                        sparkle.scale.y += growSpeed;
                    } else if (sparkle.timer < 16) {
                        sparkle.scale.x -= shrinkSpeed;
                        sparkle.scale.y -= shrinkSpeed;
                    } else {
                        sparkle.scale.x -= shrinkSpeed * 0.8;
                        sparkle.scale.y -= shrinkSpeed * 0.8;
                    }

                    if (sparkle.scale.x < 0) {
                        sparkle.removeSelf();
                    }
                }
            };
            var sparkle = new Entity({
                name: 'sparkle',
                scale: new Vector2(0, 0),
                position: new Vector2(Utils.getRandomRangeFloat(-radius.x, radius.x) + offset.x, Utils.getRandomRangeFloat(-radius.y, radius.y) + offset.y),
                components: [sparkleBehavior, new Sprite({
                    imageName: imageName,
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });

            // try to avoid spawning sparkles too close to each other
            for (var i = 0; i < 2; i++) {
                if (sparkle.position.isCloserThan(lastPos, 5)) {
                    sparkle.position = new Vector2(Utils.getRandomRangeFloat(-radius.x, radius.x) + offset.x, Utils.getRandomRangeFloat(-radius.y, radius.y) + offset.y);
                } else {
                    break;
                }
            }
            lastPos = sparkle.position.clone();

            entity.attach(sparkle);
            return sparkle;
        };
        var behavior = {
            name: 'behavior',
            update: function update(data) {
                if (entity.timer >= nextSparkleTime) {
                    if (Math.random() < 0.8 * spawnRate) {
                        nextSparkleTime += Utils.getRandomRangeFloat(20, 100);
                    } else {
                        nextSparkleTime += Utils.getRandomRangeFloat(80, 170) / spawnRate;
                    }
                    spawnSparkle();
                }
            }
        };
        var entity = new Entity({
            z: settings.z,
            name: 'sparkles',
            position: settings.position,
            alpha: 0.95,
            components: [behavior]
        }).extend({
            setSpawnRate: function setSpawnRate(value) {
                spawnRate = value;
            }
        });
        return entity;
    };
});