'use strict';

/**
 * Placement for text
 * @moduleName TextPlacement
 * @snippet TextPlacement|snippet
TextPlacement({
    textPos: new Vector2(0, 0),
    textSize: new Vector2(0, 0),
    scaleFactor: 0.44
});
 */
bento.define('components/textplacement', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/gui/text', 'bento/utils', 'bento/tween', 'bento/components/fill'], function (Bento, Vector2, Rectangle, Entity, Text, Utils, Tween, Fill) {
    'use strict';

    return function (settings) {
        var gamepixelDimension = new Rectangle((settings.textPos ? settings.textPos.x || 0 : 0) / (settings.scaleFactor || 1), (settings.textPos ? settings.textPos.y || 0 : 0) / (settings.scaleFactor || 1), (settings.gamepixelDimension ? settings.gamepixelDimension.width || 0 : 0) / (settings.scaleFactor || 1), (settings.gamepixelDimension ? settings.gamepixelDimension.height || 0 : 0) / (settings.scaleFactor || 1));

        var viewportDimension = settings.viewportDimension || new Rectangle(0, 0, 0, 0);
        var inheritDimension = settings.inheritDimension || new Rectangle(0, 0, 0, 0);

        var finalSize = new Rectangle(0, 0, 0, 0);

        var text;
        var entity = new Entity({
            name: settings.name || 'textplacement',
            scale: new Vector2(settings.scaleFactor || 1, settings.scaleFactor || 1)
        });
        if (Bento.isUITool) {
            text = new Fill({
                name: settings.name + '_text',
                color: [1, 0, 1, 0.5],
                dimension: finalSize
            });
        } else {
            // gameuiloader should take care of this
        }
        if (text) {
            entity.attach(text);
        }

        entity.attach({
            name: 'textplacementBehaviour',
            start: function start() {

                finalSize.width = Math.max(0, gamepixelDimension.width + viewportDimension.width * Bento.getViewport().width + inheritDimension.width * (entity.parent && entity.parent.parent ? entity.parent.parent.dimension.width : 0));
                finalSize.height = Math.max(0, gamepixelDimension.height + viewportDimension.height * Bento.getViewport().height + inheritDimension.height * (entity.parent && entity.parent.parent ? entity.parent.parent.dimension.height : 0));
                finalSize.x -= finalSize.width / 2;
                finalSize.y -= finalSize.height / 2;

                entity.dimension = finalSize;

                if (entity.parent && entity.parent.dimension.width === 0 && entity.parent.dimension.height === 0) {
                    entity.parent.dimension.width = entity.dimension.width * entity.scale.x;
                    entity.parent.dimension.height = entity.dimension.height * entity.scale.y;
                    entity.parent.dimension.x = -entity.dimension.width * 0.5 * entity.scale.x;
                    entity.parent.dimension.y = -entity.dimension.height * 0.5 * entity.scale.y;
                    var positioner = entity.parent.getComponent('positioner');
                    if (positioner) {
                        positioner.reposition();
                    }
                }
            }
        });
        return entity;
    };
});