'use strict';

bento.define('entities/cage', ['bento', 'bento/components/sprite', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'globals', 'modules/box2d', 'bento/components/eventlistener', 'modules/tapticengine', 'entities/lemming'], function (Bento, Sprite, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, Globals, Box2D, EventListener, TapticEngine, Lemming) {

    var Cage = function Cage(settings) {
        var position = settings.position,
            tiled = settings.tiled,
            _settings$b = settings.b,
            b = _settings$b === undefined ? '' : _settings$b;


        var characterId = Globals.cageCharacter;
        if (characterId == null) {
            console.log('No char id, skipping cage');
            return;
        }
        var characterDummy = new Entity({
            name: 'cageCharacterDummy',
            position: new Vector2(-2, 116),
            scale: new Vector2(0.8, 0.8),
            components: [new Sprite(Lemming.getAnimations(characterId)['idle']), new Object({
                name: 'behavior',
                update: function update() {
                    var lemming = Bento.objects.getByFamily('lemming')[0];
                    if (lemming) {
                        var pos = characterDummy.toComparablePosition(lemming.position);
                        if (characterDummy.position.x < pos.x && characterDummy.scale.x < 0 || characterDummy.position.x > pos.x && characterDummy.scale.x > 0) {
                            characterDummy.scale.x *= -1;
                            characterDummy.position.x *= -1;
                        }
                    }
                }
            })]
        });

        var anchorA = void 0,
            anchorB = void 0;

        if (tiled.polyline) {
            // stuff required for chain and joint
            anchorA = new Box2D.b2Vec2(Box2D.toMeters(tiled.x + tiled.polyline[0].x), Box2D.toMeters(tiled.y + tiled.polyline[0].y));
            anchorB = new Box2D.b2Vec2(Box2D.toMeters(tiled.x + tiled.polyline[1].x), Box2D.toMeters(tiled.y + tiled.polyline[1].y));
        }

        var sprite = new Sprite({
            imageName: 'boss/cage',
            originRelative: new Vector2(0.5, 0.1)
        });

        var falling = false;
        var destroyed = false;

        var explode = function explode() {
            if (destroyed) {
                return;
            }
            destroyed = true;
            Bento.objects.attach(new GibCage({
                position: entity.position.add(new Vector2(0, 16)),
                rotation: entity.rotation,
                linearVelocity: body.getLinearVelocity(),
                angularVelocity: body.getAngularVelocity()
            }));
            Bento.objects.attach(new Lemming({
                position: characterDummy.getWorldPosition(),
                facing: characterDummy.scale.x > 0 ? 'right' : 'left',
                characterId: characterId,
                isRescued: true
            }));

            entity.removeSelf();
        };

        var solidFixture = {
            filter: new Box2D.Filter(Globals.Categories.WALL),
            shape: new Box2D.BoxShape(10, 11, new Vector2(0, 13)),
            friction: 0.1,
            restitution: 0.1,
            density: 5.0,
            onBeginContact: function onBeginContact() {
                if (falling && !destroyed) {
                    Box2D.defer(explode);
                }
            }
        };

        var body = new Box2D.Body({
            type: 'dynamic',
            fixtures: [solidFixture]
        });

        var pixiChain = void 0,
            pixiChainEnd = void 0;

        var behavior = {
            name: 'behavior',
            start: function start() {},
            update: function update(_ref) {
                var speed = _ref.speed;

                var force = Math.sin(entity.timer * 0.03);
                if (Math.abs(force) > 0.1) {
                    body.applyForce(new Vector2(force * 4, 0), entity.position.add(new Vector2(0, 10)));
                    // body.applyForce(new Vector2(force*200, 0), entity.position.add(new Vector2(0, 10)));
                }
            },
            draw: function draw(_ref2) {
                var renderer = _ref2.renderer;

                // renderer.scale(1/entity.scale.x, 1/entity.scale.y);
                // Box2D.drawShape(renderer, solidFixture.shape, [255,255,0,1.0]);
                // renderer.scale(entity.scale.x, entity.scale.y);

                // draw chain
                if (b2Joint) {
                    var p1 = new Vector2(0, 0);
                    // let p2 = entity.toLocalPosition(Box2D.toBento(anchorB));
                    var p2 = entity.toLocalPosition(Box2D.toBento(b2Joint.GetAnchorB()));
                    // let p2 = entity.toComparablePosition(entityB.toWorldPosition(localAnchorB));
                    // renderer.drawLine([0,0,0,1.0], p1.x, p1.y, p2.x, p2.y, 4.0);
                    pixiChain.rotation = p2.angleBetween(p1) + Math.PI * 0.5;
                    pixiChain.height = p2.distance(p1);

                    var v = new Vector2(0, pixiChain.height);
                    v.rotateRadian(pixiChain.rotation);
                    pixiChain.x = v.x;
                    pixiChain.y = v.y;

                    // reduce length by end of chain length, then move up to make room for it
                    pixiChain.height -= pixiChainEnd.height;
                    pixiChain.pivot.y = 1.0 * pixiChain.height;

                    pixiChainEnd.rotation = pixiChain.rotation;
                    v = new Vector2(0, pixiChainEnd.height);
                    v.rotateRadian(pixiChain.rotation);
                    pixiChainEnd.x = v.x;
                    pixiChainEnd.y = v.y;
                    renderer.drawPixi(pixiChain);
                    renderer.drawPixi(pixiChainEnd);
                }
            }
        };

        var b2Joint = void 0;
        var localAnchorB = void 0;
        var entityB = void 0;

        var setupJoints = function setupJoints() {

            var bodyB = void 0;

            if (b === '') {
                // static background body
                bodyB = new Box2D.Body({
                    type: 'static'
                });
                entityB = new Entity({
                    name: 'fixedBody',
                    position: Box2D.toBento(anchorB),
                    visible: false,
                    active: false,
                    components: [bodyB]
                });
                Bento.objects.attach(entityB);
            } else {
                entityB = Bento.objects.get(b);
                if (!entityB) {
                    Utils.log('No such entity \'' + b + '\' for cage joint');
                }
                bodyB = entityB.getComponent('body');
                if (!bodyB) {
                    Utils.log('No body for entity \'' + b + '\' which should be joined to cage');
                }
            }

            localAnchorB = entityB.toComparablePosition(Box2D.toBento(anchorB));

            // setup joint
            var bA = body.getB2Body();
            var bB = bodyB.getB2Body();
            var jointDef = new Box2D.b2DistanceJointDef();
            var dX = anchorB.x - anchorA.x;
            var dY = anchorB.y - anchorA.y;
            jointDef.length = Math.sqrt(dX * dX + dY * dY);
            jointDef.dampingRatio = 0.99;
            jointDef.frequencyHz = 5.0;
            jointDef.bodyA = bA;
            jointDef.bodyB = bB;
            jointDef.localAnchorA.SetV(bA.GetLocalPoint(anchorA));
            jointDef.localAnchorB.SetV(bB.GetLocalPoint(anchorB));
            jointDef.collideConnected = false;
            b2Joint = Box2D.box2DWorld.CreateJoint(jointDef);

            pixiChain = new window.PIXI.extras.TilingSprite(Utils.newTexture('boss/chain'), 44, 0);
            pixiChain.pivot.x = 0.5 * pixiChain.width;
            pixiChain.pivot.y = 1.0 * pixiChain.height;
            pixiChainEnd = new window.PIXI.Sprite(Utils.newTexture('boss/chain-end'));
            pixiChainEnd.pivot.x = 0.5 * pixiChain.width;
            pixiChainEnd.pivot.y = 0 * pixiChain.height;
            pixiChainEnd.scale.y = -1;
        };
        var setupJointsListener = new EventListener({
            eventName: 'setupJoints',
            onEvent: setupJoints
        });

        var leverListener = new EventListener({
            eventName: 'pullLever',
            onEvent: function onEvent() {
                Utils.timeout(14, function () {
                    if (b2Joint && !falling) {
                        Box2D.box2DWorld.DestroyJoint(b2Joint);
                        b2Joint = null;
                        falling = true;

                        // spawn chain gibs
                        var p1 = entity.getWorldPosition();
                        var p2 = Box2D.toBento(anchorB);
                        var distance = p2.distance(p1);
                        var angle = p2.angleBetween(p1);
                        // console.log(distance, p1, p2);
                        for (var y = 0; y < distance; y += 70 * entity.scale.y) {
                            var v = new Vector2(0, y).rotateRadian(angle + Math.PI * 0.5).addTo(p1);
                            Bento.objects.attach(new ChainGib({
                                position: v
                            }));
                        }
                    } else {
                        explode();
                    }
                });
            }
        });

        var entity = new Entity({
            z: Globals.Layers.CAGE,
            name: 'cage',
            family: ['cage'],
            position: anchorA ? Box2D.toBento(anchorA) : position.addTo(new Vector2(tiled.width / 2, 0)),
            scale: new Vector2(0.14, 0.14),
            components: [behavior, characterDummy, sprite, body, leverListener]
        });

        if (tiled.polyline) {
            entity.attach(setupJointsListener);
        }

        characterDummy.scale.scalarMultiplyWith(1 / entity.scale.x);

        return entity;
    };

    var ChainGib = function ChainGib(settings) {
        var position = settings.position;


        var vel = new Vector2(Utils.getRandomRangeFloat(-1.6, 1.6), Utils.getRandomRangeFloat(-2, -4));
        var rotVel = Utils.getRandomRangeFloat(-0.4, 0.4);

        var sprite = new Sprite({
            imageName: Utils.pickRandom('boss/chain', 'boss/chain-end'),
            originRelative: new Vector2(0.5, 0.5)
        });

        var behavior = {
            name: 'behavior',
            update: function update() {
                vel.y += 0.24;
                entity.position.addTo(vel);
                entity.rotation += rotVel;
                if (entity.timer > 200) {
                    entity.removeSelf();
                }
            }
        };

        var entity = new Entity({
            z: Utils.pickRandom(Globals.Layers.CHAIN, Globals.Layers.TERRAIN_SHADOW),
            name: 'chainGib',
            position: position,
            rotation: Math.random() * Math.PI * 2,
            scale: new Vector2(0.14, 0.14),
            components: [sprite, behavior]
        });

        return entity;
    };

    var GibCage = function GibCage(settings) {
        var position = settings.position,
            rotation = settings.rotation,
            linearVelocity = settings.linearVelocity,
            angularVelocity = settings.angularVelocity;


        var debugDraw = false;
        var GibDebug = function GibDebug(entity, body) {
            return {
                name: 'gibDebug',
                draw: function draw(_ref3) {
                    var renderer = _ref3.renderer;

                    renderer.scale(1 / entity.scale.x, 1 / entity.scale.y);
                    Box2D.drawShape(renderer, body.fixtures[0].shape, [255, 255, 0, 1.0]);
                    renderer.scale(entity.scale.x, entity.scale.y);
                }
            };
        };
        var filter = new Box2D.Filter(Globals.Categories.CAGE, Globals.Categories.WALL | Globals.Categories.SLICEABLE | Globals.Categories.CAGE);
        var friction = 0.1;
        var restitution = 0.5;
        var density = 2.0;

        var Bar = function Bar(pos) {
            var sprite = new Sprite({
                imageName: 'boss/cage-gib-2',
                originRelative: new Vector2(0.5, 0.5)
            });
            var body = new Box2D.Body({
                type: 'dynamic',
                fixture: {
                    filter: filter,
                    shape: new Box2D.BoxShape(1, 8),
                    friction: friction,
                    restitution: restitution,
                    density: density
                }
            });
            var entity = new Entity({
                name: 'bar',
                family: ['gib'],
                position: pos,
                scale: new Vector2(0.14, 0.14),
                components: [body, sprite]
            });
            if (debugDraw) entity.attach(new GibDebug(entity, body));
            return entity;
        };
        var Plate = function Plate(pos, flip) {
            var sprite = new Sprite({
                imageName: 'boss/cage-gib',
                originRelative: new Vector2(0.5, 0.5)
            });
            var body = new Box2D.Body({
                type: 'dynamic',
                fixture: {
                    filter: filter,
                    shape: new Box2D.BoxShape(10, 2),
                    friction: friction,
                    restitution: restitution,
                    density: density
                }
            });
            var entity = new Entity({
                name: 'bar',
                family: ['gib'],
                position: pos,
                scale: new Vector2(0.14, 0.14 * (flip ? -1 : 1)),
                components: [body, sprite]
            });
            if (debugDraw) entity.attach(new GibDebug(entity, body));
            return entity;
        };
        var Ring = function Ring(pos) {
            var sprite = new Sprite({
                imageName: 'boss/cage-gib-3',
                originRelative: new Vector2(0.5, 0.5)
            });
            var body = new Box2D.Body({
                type: 'dynamic',
                fixture: {
                    filter: filter,
                    shape: new Box2D.BoxShape(3, 2),
                    friction: friction,
                    restitution: 0,
                    density: density * 4
                }
            });
            var entity = new Entity({
                name: 'bar',
                family: ['gib'],
                position: pos,
                scale: new Vector2(0.14, 0.14),
                components: [body, sprite]
            });
            if (debugDraw) entity.attach(new GibDebug(entity, body));
            return entity;
        };

        var parts = [Bar(position.add(Vector2(-8, 0))), Bar(position.add(Vector2(-4, 0))), Bar(position.add(Vector2(0, 0))), Bar(position.add(Vector2(4, 0))), Bar(position.add(Vector2(8, 0))), Ring(position.add(Vector2(0, -13))), Plate(position.add(Vector2(0, -9.6)), true), Plate(position.add(Vector2(0, 9.6)))];

        var gibCage = {
            name: 'gibCage',
            start: function start() {
                parts.forEach(function (e, i) {
                    e.rotation = rotation;
                    e.position.subtractFrom(position).rotateRadian(rotation).addTo(position);
                    Bento.objects.attach(e);
                    e.getComponent('body', function (body) {
                        body.setLinearVelocity(linearVelocity);
                        body.setAngularVelocity(angularVelocity + Utils.getRandomRangeFloat(-0.4, 0.4));
                        var centerDiff = e.position.subtract(position);
                        body.applyImpulse(centerDiff.scale(0.28).add(new Vector2(0, -6)), position.add(Vector2(Utils.getRandomRangeFloat(-10, 10), Utils.getRandomRangeFloat(-10, 10))));
                    });
                });
            },
            destroy: function destroy() {
                parts.forEach(function (e) {
                    Bento.objects.remove(e);
                });
            }
        };

        return gibCage;
    };

    return Cage;
});