'use strict';

bento.define('entities/camera', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Entity, EventSystem, Utils, Tween) {
	return function (settings) {
		var viewport = Bento.getViewport();
		var renderer = Bento.getRenderer();

		var shakePos = Vector2(0, 0);
		var shakeTimer = 0;
		var shakeDuration = 0;
		var shakeStrength = 0;
		var updateShake = function updateShake() {
			if (shakeTimer > 0) {
				shakeTimer -= 1;
				if (shakeTimer % 2) {
					shakePos.x = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
					shakePos.y = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
				}
			} else {
				shakePos.x = 0;
				shakePos.y = 0;
			}
		};

		var camera = {
			name: 'camera',
			z: settings.startLayer || 0,

			// mutable fields
			position: settings.position || Vector2(0, 0),
			zoom: settings.zoom || 1,
			rotation: settings.rotation || 0,

			start: function start(data) {
				Bento.objects.attach(cameraEnd);
			},
			destroy: function destroy(data) {
				Bento.objects.remove(cameraEnd);
			},

			update: function update() {
				updateShake();
			},

			// transform the renderer
			draw: function draw() {
				var pos = camera.position;
				var cx = viewport.width / 2;
				var cy = viewport.height / 2;
				renderer.save();
				renderer.translate(cx, cy);
				renderer.scale(camera.zoom, camera.zoom);
				renderer.rotate(camera.rotation);
				renderer.translate(-pos.x + shakePos.x, -pos.y + shakePos.y);
			},

			shake: function shake(duration, strength) {
				shakeTimer = duration;
				shakeDuration = duration;
				shakeStrength = strength;
			},

			cameraCoords: function cameraCoords(pos, offset) {
				var ox = offset ? offset.x : 0;
				var oy = offset ? offset.y : 0;
				var w = viewport.width;
				var h = viewport.height;
				var c = Math.cos(camera.rotation);
				var s = Math.sin(camera.rotation);
				var x = pos.x - camera.position.x;
				var y = pos.y - camera.position.y;
				x = c * x - s * y;
				y = s * x + c * y;
				return Vector2(x * camera.zoom + w / 2 + ox, y * camera.zoom + h / 2 + oy);
			},
			worldCoords: function worldCoords(pos, offset) {
				var ox = offset ? offset.x : 0;
				var oy = offset ? offset.y : 0;
				var w = viewport.width;
				var h = viewport.height;
				var c = Math.cos(-camera.rotation);
				var s = Math.sin(-camera.rotation);
				var x = (pos.x - w / 2 - ox) / camera.zoom;
				var y = (pos.y - h / 2 - oy) / camera.zoom;
				x = c * x - s * y;
				y = s * x + c * y;
				return Vector2(x + camera.position.x, y + camera.position.y);
			}
		};

		var cameraEnd = {
			name: 'cameraEnd',
			z: settings.endLayer || 99999,

			// un-transform the renderer
			draw: function draw() {
				renderer.restore();
			}
		};

		return camera;
	};
});