'use strict';

bento.define('entities/collectible', ['bento', 'bento/components/sprite', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/components/eventlistener', 'bento/utils', 'bento/tween', 'globals', 'modules/box2d', 'modules/savemanager'], function (Bento, Sprite, Vector2, Rectangle, Entity, EventSystem, EventListener, Utils, Tween, Globals, Box2D, SaveManager) {

    var playerOnly = new Box2D.Filter(Globals.Categories.COLLECTIBLE, Globals.Categories.PLAYER);

    return function (settings) {
        var position = settings.position;


        // Note:
        // In order to keep track of which ones have been collected, each collectible
        // should automatically give itself a unique name from tiled data, but the designer could
        // choose to override it if necessary?

        var sprite = new Sprite({
            imageName: 'main/gem-idle',
            originRelative: new Vector2(0.5, 0.5),
            frameCountX: 6,
            animations: {
                default: {
                    speed: 0.15,
                    frames: [0, 1, 2, 3, 4, 5]
                }
            }
        });

        var collected = false;

        var sensor = {
            shape: new Box2D.CircleShape(6, new Vector2(0, 0)),
            filter: playerOnly,
            isSensor: true,
            onBeginContact: function onBeginContact(other, contact) {
                if (!collected) {
                    collected = true;
                    console.log('HIT');
                    entity.remove(body);
                    Tween({
                        from: entity.position.y,
                        to: entity.position.y - 20,
                        in: 60,
                        delay: 0,
                        applyOnDelay: 0,
                        ease: 'linear',
                        decay: 1,
                        oscillations: 1,
                        onUpdate: function onUpdate(v, t) {
                            entity.position.y = v;
                        }
                    });
                    Tween({
                        delay: 30,
                        from: 1, to: 0, in: 20,
                        applyOnDelay: 0,
                        ease: 'easeInOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            entity.alpha = v;
                        }
                    });
                }
            }
        };

        var body = new Box2D.Body({
            type: 'static',
            fixedRotation: true,
            fixtures: [sensor]
        });

        var behavior = {
            name: 'behavior',
            start: function start() {
                console.log('a');
            },
            update: function update(_ref) {
                var speed = _ref.speed;

                entity.transform.y = Math.sin(entity.timer * 0.07) * 3;
                if (collected) {
                    entity.rotation += speed * 0.4;
                }
            },
            draw: function draw(_ref2) {
                // Box2D.drawShape(renderer, sensor.shape);

                var renderer = _ref2.renderer;
            }
        };

        var entity = new Entity({
            z: Globals.Layers.COLLECTIBLE,
            name: 'collectible',
            position: position,
            scale: new Vector2(0.75, 0.75),
            components: [sprite, body, behavior]
        });

        return entity;
    };
});