'use strict';

bento.define('entities/failregion', ['bento', 'bento/components/sprite', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/components/eventlistener', 'bento/utils', 'bento/tween', 'globals', 'modules/box2d', 'modules/savemanager'], function (Bento, Sprite, Vector2, Rectangle, Entity, EventSystem, EventListener, Utils, Tween, Globals, Box2D, SaveManager) {

    var playerOnly = new Box2D.Filter(Globals.Categories.REGION, Globals.Categories.PLAYER);

    var decomp = window.decomp;

    return function (settings) {
        var position = settings.position,
            tiled = settings.tiled;


        var shape = void 0;
        if (tiled.polygon) {
            var concavePoly = tiled.polygon.map(function (v) {
                return [v.x, v.y];
            });
            decomp.makeCCW(concavePoly);
            shape = new Box2D.PolygonShape(concavePoly.map(function (v) {
                return new Vector2(v[0], v[1]);
            })); // only convex polys for now
        } else if (tiled.ellipse) {
            shape = new Box2D.CircleShape(tiled.width / 2, new Vector2(tiled.width / 2, tiled.height / 2));
        } else {
            shape = new Box2D.PolygonShape([new Vector2(tiled.width, 0), new Vector2(tiled.width, tiled.height), new Vector2(0, tiled.height), new Vector2(0, 0)]);
        }

        var sensor = {
            shape: shape,
            filter: playerOnly,
            isSensor: true,
            onBeginContact: function onBeginContact(other, contact) {
                EventSystem.fire('fail');
                entity.removeSelf();
            }
        };

        var body = new Box2D.Body({
            type: 'static',
            fixtures: [sensor]
        });

        var entity = new Entity({
            z: Globals.Layers.REGION,
            name: 'failRegion',
            position: position,
            visible: false,
            components: [body]
        });

        return entity;
    };
});