'use strict';

bento.define('entities/goal', ['bento', 'bento/components/sprite', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'globals', 'modules/box2d', 'modules/tapticengine'], function (Bento, Sprite, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, Globals, Box2D, TapticEngine) {
    return function (settings) {
        var position = settings.position,
            _settings$type = settings.type,
            type = _settings$type === undefined ? 'static' : _settings$type,
            _settings$solid = settings.solid,
            solid = _settings$solid === undefined ? false : _settings$solid,
            _settings$rotationDeg = settings.rotationDegrees,
            rotationDegrees = _settings$rotationDeg === undefined ? 0 : _settings$rotationDeg;


        if (settings.bodyType) {
            type = settings.bodyType; // legacy parameter name
        }

        // let currentStage = Utils.currentLevel % 5;
        // let isLever = (currentStage === 4);
        var isLever = Globals.cageCharacter !== null;
        var hasWon = false;

        var red = {
            imageName: 'flagpole-red',
            originRelative: new Vector2(0.92, 1)
        };
        var green = {
            imageName: 'flagpole-green',
            originRelative: new Vector2(0.08, 1)
        };
        var leverOff = {
            imageName: 'boss/lever-off',
            originRelative: new Vector2(0.5, 1)
        };
        var leverOn = {
            imageName: 'boss/lever-on',
            originRelative: new Vector2(0.5, 1)
        };

        var sprite = new Sprite(isLever ? leverOff : red);
        var scale = isLever ? 0.10 : 0.05;

        var extraRotation = 0;

        var fixture = {
            shape: new Box2D.BoxShape(2, 12, new Vector2(0, -12)),
            isSensor: true,
            filter: new Box2D.Filter(Globals.Categories.GOAL, Globals.Categories.PLAYER),
            onBeginContact: function onBeginContact(other, contact) {
                if (!hasWon && !Globals.hasWon) {
                    hasWon = true;

                    if (isLever) {
                        new Tween({
                            delay: 4,
                            from: leverHandle.rotation,
                            to: Utils.toRadian(45),
                            in: 25,
                            ease: 'easeInOutQuad',
                            onStart: function onStart() {
                                EventSystem.fire('pullLever');
                                Utils.timeout(14, function () {
                                    // change image
                                    sprite.setup(leverOn);
                                    // Bento.audio.playSound('sfx_button');
                                    Bento.audio.playSound('sfx_switch');
                                });
                            },
                            onUpdate: function onUpdate(v, t) {
                                leverHandle.rotation = v;
                            },
                            onComplete: function onComplete() {
                                TapticEngine.impact({ type: 'medium' });
                                Bento.audio.playSound('sfx_victory_jingle_cheer');

                                Utils.timeout(100, function () {
                                    EventSystem.fire('win');
                                });
                            }
                        });
                    } else {
                        EventSystem.fire('win');
                        TapticEngine.impact({ type: 'medium' });
                        Bento.audio.playSound('sfx_victory_jingle_cheer');

                        // flip flag horizontally
                        new Tween({
                            from: scale, to: scale * 0.1, in: 8,
                            ease: 'easeInQuad',
                            onUpdate: function onUpdate(v, t) {
                                entity.scale.x = v;
                            },
                            onComplete: function onComplete() {
                                // change image
                                sprite.setup(green);

                                // resume flipping
                                new Tween({
                                    from: entity.scale.x, to: scale, in: 8,
                                    ease: 'easeOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        entity.scale.x = v;
                                    }
                                });
                            }
                        });
                        new Tween({
                            from: 0,
                            to: 0.2,
                            in: 7,
                            ease: 'linear',
                            onUpdate: function onUpdate(v) {
                                extraRotation = v;
                            },
                            onComplete: function onComplete() {
                                new Tween({
                                    from: 0.2, to: 0, in: 30,
                                    ease: 'elastic',
                                    decay: 4,
                                    oscillations: 2,
                                    onUpdate: function onUpdate(v) {
                                        extraRotation = v;
                                    }
                                });
                            }
                        });
                    }
                }
            }
        };

        var collisionMask = solid ? Globals.Categories.WALL | Globals.Categories.SLICEABLE : 0;

        var solidFixture = {
            filter: new Box2D.Filter(Globals.Categories.GOAL, collisionMask),
            shape: new Box2D.BoxShape(2, 10, new Vector2(0, -10)),
            friction: 0.1,
            restitution: 0.1,
            density: 0.2
        };

        var body = new Box2D.Body({
            type: type,
            fixtures: [fixture, solidFixture],
            ignoreRotation: true
        });

        var behavior = {
            name: 'behavior',
            update: function update() {
                entity.rotation = body.getAngle() + extraRotation;
            }
        };

        var entity = new Entity({
            z: Globals.Layers.GOAL,
            name: 'goal',
            family: ['goal'],
            position: position,
            rotation: Utils.toRadian(rotationDegrees),
            scale: new Vector2(scale, scale),
            components: [sprite, body, behavior]
        });

        var leverHandle = void 0;
        if (isLever) {
            leverHandle = new Entity({
                name: 'leverHandle',
                position: new Vector2(0, -36),
                scale: new Vector2(1.2, 1.2),
                rotation: Utils.toRadian(-45),
                components: [new Sprite({
                    imageName: 'boss/lever',
                    originRelative: new Vector2(0.5, 1.0)
                })]
            });
            entity.attach(leverHandle);
            entity.moveComponentTo(leverHandle, 0);
        }

        return entity;
    };
});