'use strict';

bento.define('entities/gui/devmenu', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'modules/levelmanager', 'modules/savemanager', 'modules/timemanager', 'modules/textstyles', 'gui/gui', 'entities/gui/fpscounter', 'gui/dialog', 'modules/vipmanager', 'gui/askrating', 'modules/admanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Screen, Tween, Globals, Fill, EventListener, LevelManager, SaveManager, TimeManager, TextStyles, Gui, FPSCounter, Dialog, VipManager, AskRating, AdManager) {

    var CheatButton = function CheatButton(settings) {
        var position = settings.position,
            text = settings.text,
            onClick = settings.onClick,
            _settings$size = settings.size,
            size = _settings$size === undefined ? 20 : _settings$size;


        var circle = new window.PIXI.Graphics();
        circle.lineStyle(1, 0xffffff);
        circle.drawCircle(0, 0, size / 2);

        var behavior = {
            name: 'behavior',
            draw: function draw(_ref) {
                var renderer = _ref.renderer;

                renderer.drawPixi(circle);
            }
        };
        var textObj = new Text({
            text: text,
            scale: new Vector2(0.5, 0.5),
            position: new Vector2(0, -1),
            fontSettings: TextStyles.get('buttonLabel'),
            align: 'center',
            maxWidth: size * 1.6,
            maxHeight: 28
            // drawDebug: true
        });
        var clickable = new Clickable({ onClick: onClick });

        var button = new Entity({
            name: 'cheatButton',
            position: position,
            dimension: new Rectangle(-size / 2, -size / 2, size, size),
            components: [behavior, textObj, clickable]
        });
        return button;
    };

    return function (settings) {
        var viewport = Bento.getViewport();

        var nameText = new Text({
            z: 9999,
            position: new Vector2(0, 30),
            text: LevelManager.getName(Utils.currentLevel, Utils.currentLevelGroup),
            font: '',
            fontSize: 8,
            fontColor: '#fff',
            align: 'center',
            textBaseline: 'top',
            lineWidth: 0.5, // set to add an outline
            strokeStyle: '#000000'
        });

        var updateText = function updateText() {
            nameText.setText(LevelManager.getName(Utils.currentLevel, Utils.currentLevelGroup));
        };

        var fpsCounter = new FPSCounter({});
        fpsCounter.position.x = -22;
        fpsCounter.position.y += 8;

        var devMenu = new Entity({
            global: true,
            float: true,
            z: Globals.Layers.GUI,
            name: 'devMenu',
            position: new Vector2(viewport.width / 2, 0),
            components: [fpsCounter, nameText]
        });

        var btnX = -46;
        var btnY = viewport.height - 18;
        var addButton = function addButton(text, onClick) {
            devMenu.attach(new CheatButton({
                position: new Vector2(btnX, btnY),
                text: text,
                onClick: onClick
            }));
            btnX += 24;
            if (btnX > 40) {
                btnX = -46;
                btnY -= 24;
            }
        };

        addButton('hide gui', function () {
            Gui.get('root', function (root) {
                root.visible = !root.visible;
                devMenu.visible = root.visible;
                Bento.objects.get('cageProgressBar', function (cageProgressBar) {
                    cageProgressBar.visible = root.visible;
                });
            });
        });
        addButton('show hand', function () {
            if (handCursor.parent) {
                Bento.objects.remove(handCursor);
            } else {
                Bento.objects.attach(handCursor);
            }
        });
        addButton('<', function () {
            var next = LevelManager.getPrevious(Utils.currentLevel, Utils.currentLevelGroup);
            if (next !== -1) {
                Utils.currentLevel = next;
                Bento.screens.show('screens/main');
            }
        });
        addButton('>', function () {
            var next = LevelManager.getNext(Utils.currentLevel, Utils.currentLevelGroup);
            if (next !== -1) {
                Utils.currentLevel = next;
                Bento.screens.show('screens/main');
            }
        });
        addButton('clear save', function () {
            Bento.saveState.clear();
            Utils.currentLevel = 0;
            Bento.screens.show('screens/titlescreen');
            handCursor.removeSelf();
            devMenu.removeSelf();
        });
        addButton('notification', function () {
            EventSystem.fire('scheduleNotification', {
                id: 'testNotification',
                title: 'Go Slice',
                body: 'Let\'s slice and help me reach the flag!',
                time: 1 * 60 * 1000 // 1 minute
            });
        });
        addButton('+500 coins', function () {
            SaveManager.add('coins', 500);
            EventSystem.fire('updateCoins', SaveManager.load('coins'));
        });
        addButton('give vip', function () {
            Bento.saveState.save('VipLastValidated', Math.floor(TimeManager.getTime() / VipManager.getTimeSpan()));
            Bento.saveState.save('VipFailedValidations', 0);
            Bento.saveState.save('VIP', true);
            Bento.saveState.save('VipDowngraded', false);
            SaveManager.save('lastTimeVipClaimed', TimeManager.getTime() - 1000 * 60 * 60 * 24);
            console.log('lastTimeVipClaimed', TimeManager.getTime() - 1000 * 60 * 60 * 24);
        });
        addButton('askrating', function () {
            Bento.saveState.save('hasRatedV' + window.VERSION, false);
            Bento.saveState.save('askedRatingV' + window.VERSION, false);
            Bento.objects.attach(new AskRating({}));
        });
        addButton('interstitial', function () {
            AdManager.tryShowInterstitial(function (success, err) {
                if (success) {
                    Bento.objects.attach(new Dialog({
                        name: 'testInterstitialResult',
                        text: 'interstitial success',
                        type: 'ok'
                    }));
                } else {
                    Bento.objects.attach(new Dialog({
                        name: 'testInterstitialResult',
                        title: 'Interstitial Failed',
                        text: 'Reason: ' + err,
                        type: 'ok'
                    }));
                }
            }, '', true); // force interstitial
        });
        addButton('send crash', function () {
            window.fabric.Crashlytics.addLog('about to send a crash for testing');
            window.fabric.Crashlytics.sendCrash();
        });
        addButton('js error', function () {
            null();
        });
        addButton('text oom test', function () {
            var f = function f() {
                var txt = new Text({
                    z: Globals.Layers.GUI,
                    position: new Vector2(20, 100),
                    text: Utils.getRandomRange(100000000, 1000000000) + '',
                    scale: new Vector2(0.38, 0.38),
                    font: 'main',
                    fontSize: 70,
                    fontColor: '#ffffff',
                    align: 'left',
                    innerStroke: false,
                    maxWidth: 400,
                    maxHeight: 100,
                    drawDebug: false
                });
                txt.float = true;
                Bento.objects.attach(txt);
                Utils.timeout(2, function () {
                    Bento.objects.remove(txt);
                });
            };
            window.setInterval(function () {
                f();
            }, 20);
        });
        addButton('show banner', function () {
            AdManager.showBanner();
        });

        devMenu.attach(new EventListener({
            eventName: 'levelLoaded',
            onEvent: updateText
        }));

        devMenu.attach(new EventListener({
            eventName: 'screenShown',
            onEvent: function onEvent(screen) {
                if (screen.name !== 'screens/main') {
                    devMenu.removeSelf();
                }
            }
        }));

        var handSprite = new Sprite({
            imageName: 'gui/hand-frames',
            originRelative: new Vector2(0.38, 0.06),
            frameCountX: 2,
            animations: {
                default: { speed: 0, frames: [0] },
                press: { speed: 0, frames: [1] }
            }
        });

        var handCursor = new Entity({
            z: Globals.Layers.GUI,
            name: 'devHand',
            position: new Vector2(0, 0),
            scale: new Vector2(0.14, 0.14),
            components: [handSprite, EventListener({
                eventName: 'pointerDown',
                onEvent: function onEvent(data) {
                    handCursor.position.x = data.position.x;
                    handCursor.position.y = data.position.y;
                    handSprite.setAnimation('press');
                    Tween({
                        from: 0, to: 1, in: 30,
                        ease: 'easeInOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            handCursor.alpha = v;
                        }
                    });
                }
            }), EventListener({
                eventName: 'pointerUp',
                onEvent: function onEvent(data) {
                    handSprite.setAnimation('default');
                    Tween({
                        from: 1, to: 0, in: 30,
                        ease: 'easeInOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            handCursor.alpha = v;
                        }
                    });
                }
            }), EventListener({
                eventName: 'pointerMove',
                onEvent: function onEvent(data) {
                    handCursor.position.x += 0.5 * (data.position.x - handCursor.position.x);
                    handCursor.position.y += 0.5 * (data.position.y - handCursor.position.y);
                }
            })]
        });

        return devMenu;
    };
});