'use strict';

bento.define('entities/gui/hintline', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, Globals, Fill, EventListener) {
    var _slicedToArray = function () {
        function sliceIterator(arr, i) {
            var _arr = [];
            var _n = true;
            var _d = false;
            var _e = undefined;

            try {
                for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
                    _arr.push(_s.value);

                    if (i && _arr.length === i) break;
                }
            } catch (err) {
                _d = true;
                _e = err;
            } finally {
                try {
                    if (!_n && _i["return"]) _i["return"]();
                } finally {
                    if (_d) throw _e;
                }
            }

            return _arr;
        }

        return function (arr, i) {
            if (Array.isArray(arr)) {
                return arr;
            } else if (Symbol.iterator in Object(arr)) {
                return sliceIterator(arr, i);
            } else {
                throw new TypeError("Invalid attempt to destructure non-iterable instance");
            }
        };
    }();

    var PIXI = window.PIXI;

    return function (settings) {
        var position = settings.position,
            points = settings.points,
            _settings$hintLevel = settings.hintLevel,
            hintLevel = _settings$hintLevel === undefined ? 1 : _settings$hintLevel;


        var freeHint = hintLevel <= 0;
        if (freeHint) {
            hintLevel = Math.abs(hintLevel) + 1;
        }

        var _points = _slicedToArray(points, 2),
            a = _points[0],
            b = _points[1];

        a = new Vector2(a.x, a.y);
        b = new Vector2(b.x, b.y);
        var diff = b.subtract(a);

        var lineGraphics = void 0;

        var shown = false;
        var isSlicing = false;

        var behavior = {
            name: 'behavior',
            start: function start() {
                lineGraphics = new PIXI.Graphics();
                lineGraphics.moveTo(a.x, a.y);

                // draw black/white dashed line
                var segments = diff.magnitude() / 3;
                for (var i = 1; i < segments; i++) {
                    var x = a.x + diff.x * (i / segments);
                    var y = a.y + diff.y * (i / segments);
                    lineGraphics.lineStyle(0.7, i % 2 ? 0x000000 : 0xffffff);
                    lineGraphics.lineTo(x, y);
                }

                Bento.objects.attach(hand);
            },
            destroy: function destroy() {
                Bento.objects.remove(hand);
            },
            update: function update() {
                if (!shown && (Globals.hintsRevealed >= hintLevel || freeHint)) {
                    shown = true;
                    // fade in
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 30,
                        ease: 'easeOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            entity.alpha = v;
                            hand.alpha = v;
                        }
                    });

                    // animateHand();
                }
                var handWasVisible = hand.visible;
                hand.visible = shown && !isSlicing && (hintLevel === 0 || Globals.sliceCount === hintLevel - 1);
                if (!handWasVisible && hand.visible) {
                    animateHand();
                    // fade in hand
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        ease: 'easeOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            hand.alpha = v;
                        }
                    });
                }
                entity.visible = shown;
            },
            draw: function draw(_ref) {
                var renderer = _ref.renderer;

                renderer.drawPixi(lineGraphics);
                // renderer.drawLine(Utils.color('000000'), a.x, a.y, b.x, b.y, 1);
            }
        };

        var sprite = new Sprite({
            imageName: 'gui/hand-frames',
            originRelative: new Vector2(0.4, 0.06),
            frameCountX: 2,
            animations: {
                default: { speed: 0, frames: [0] },
                press: { speed: 0, frames: [1] } // new hand doesn't have a down frame?
            }
        });

        var setHandPressed = function setHandPressed(pressed) {
            if (pressed) {
                sprite.setAnimation('press');
            } else {
                sprite.setAnimation('default');
            }
        };

        var hand = new Entity({
            z: Globals.Layers.HINT_LINE,
            name: 'tutorialHand',
            scale: new Vector2(0.1, 0.1),
            position: position.add(a),
            components: [sprite, new EventListener({
                eventName: 'beginSlicing',
                onEvent: function onEvent() {
                    isSlicing = true;
                }
            }), new EventListener({
                eventName: 'endSlicing',
                onEvent: function onEvent() {
                    isSlicing = false;
                }
            })]
        });

        var curTween = null;

        var animateHand = function animateHand() {

            if (curTween) {
                curTween.removeSelf();
            }

            Utils.timeout(10, function () {
                sprite.setAnimation('press');
            });

            // move along line
            curTween = new Tween({
                from: position.add(a),
                to: position.add(b),
                in: 60,
                delay: 20,
                ease: 'easeInOutQuad',
                onUpdate: function onUpdate(v) {
                    hand.position.x = v.x;
                    hand.position.y = v.y;
                },
                onComplete: function onComplete() {

                    // fade out
                    new Tween({
                        from: 1, to: 0.4, in: 26,
                        ease: 'easeInOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            hand.alpha = v;
                        }
                    });

                    // move away from line
                    curTween = new Tween({
                        from: 0, to: 8, in: 32,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            sprite.setAnimation('default');
                        },
                        onUpdate: function onUpdate(v, t) {
                            hand.transform.y = v;
                        },
                        onComplete: function onComplete() {
                            // move back towards line
                            new Tween({
                                from: 8, to: 0, in: 32,
                                delay: 20,
                                ease: 'easeInQuad',
                                onUpdate: function onUpdate(v, t) {
                                    hand.transform.y = v;
                                }
                            });
                        }
                    });

                    // move back along line
                    curTween = new Tween({
                        from: hand.position.clone(),
                        // to: position.add(a).add(new Vector2(0, 10)),
                        to: position.add(a),
                        in: 40,
                        delay: 18,
                        ease: 'easeInOutQuad',
                        onUpdate: function onUpdate(v) {
                            hand.position.x = v.x;
                            hand.position.y = v.y;
                        },
                        onComplete: function onComplete() {

                            // fade in
                            new Tween({
                                from: 0.4, to: 1.0, in: 26,
                                ease: 'easeInOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    hand.alpha = v;
                                },
                                onComplete: function onComplete() {
                                    animateHand();
                                }
                            });

                            // // move towards line
                            // new Tween({
                            //     from: hand.position.clone(),
                            //     to: position.add(a),
                            //     in: 20,
                            //     ease: 'easeOutQuad',
                            //     delay: 12,
                            //     onUpdate (v) {
                            //         hand.position.x = v.x;
                            //         hand.position.y = v.y;
                            //     },
                            //     onComplete () {
                            //         animateHand();
                            //     }
                            // });
                        }
                    });

                    // // move away from line
                    // new Tween({
                    //     from: hand.position.clone(),
                    //     to: hand.position.add(new Vector2(0, 10)),
                    //     in: 20,
                    //     ease: 'easeOutQuad',
                    //     onStart () {
                    //         sprite.setAnimation('default');
                    //     },
                    //     onUpdate (v) {
                    //         hand.position.x = v.x;
                    //         hand.position.y = v.y;
                    //     },
                    //     onComplete () {

                    //     }
                    // });
                }
            });
        };

        var entity = new Entity({
            z: Globals.Layers.HINT_LINE,
            name: 'hintLine',
            position: position,
            components: [behavior, new EventListener({
                eventName: 'slice',
                onEvent: function onEvent() {
                    if (Globals.sliceCount >= hintLevel) {
                        entity.removeSelf();
                    }
                }
            })]
        });

        entity.visible = false;
        hand.visible = false;
        entity.alpha = 0;
        hand.alpha = 0;

        return entity;
    };
});