'use strict';

bento.define('entities/gui/slicecounter', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/simplesprite', 'entities/gui/devmenu', 'modules/savemanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, Globals, Fill, EventListener, SimpleSprite, DevMenu, SaveManager) {

    var PIXI = window.PIXI;

    var FollowBehavior = function FollowBehavior(settings) {
        var debug = new Entity({
            z: 9999,
            name: 'debug',
            position: new Vector2(0, 0),
            components: [{
                name: 'behavior',
                gfx: new PIXI.Graphics().beginFill(0xff0000).drawCircle(0, 0, 4),
                update: function update() {
                    debug.position.x = followPos.x;
                    debug.position.y = followPos.y;
                },
                draw: function draw(_ref) {
                    var renderer = _ref.renderer;

                    renderer.drawPixi(this.gfx);
                }
            }]
        });
        var entity = void 0;
        var initialPos = new Vector2(0, 0); // where does it belong in the slice list
        var followPos = new Vector2(0, 0); // 
        var followVel = new Vector2(0, 0);
        var slicer = void 0;
        var component = {
            name: 'followBehavior',
            start: function start() {
                // Bento.objects.attach(debug);
                initialPos = entity.getWorldPosition();
                followPos = initialPos.clone();
                slicer = Bento.objects.get('slicer');
            },
            destroy: function destroy() {
                // Bento.objects.remove(debug);
            },
            update: function update() {
                if (slicer.isPressed) {
                    var startPos = Globals.camera.cameraCoords(slicer.startPos);
                    var endPos = Globals.camera.cameraCoords(slicer.endPos);
                    var a = startPos.angleBetween(endPos);
                    var diff = startPos.add(Vector2(-10, 0).rotateRadian(a)).subtract(followPos);
                    followVel.addTo(diff.scalarMultiply(0.04)).scalarMultiplyWith(0.82);
                    followPos.addTo(followVel);
                    var relPos = entity.toComparablePosition(followPos);
                    entity.position.x = relPos.x;
                    entity.position.y = relPos.y;
                    entity.rotation = followPos.angleBetween(endPos) + Math.PI * 0.5;

                    // let pos = Globals.camera.cameraCoords(slicer.endPos);
                    // let diff = pos.add(Vector2(-18, -20)).subtract(followPos);
                    // followVel.addTo(diff.scalarMultiply(0.04)).scalarMultiplyWith(0.82);
                    // followPos.addTo(followVel);
                    // let relPos = entity.toComparablePosition(followPos);
                    // entity.position.x = relPos.x;
                    // entity.position.y = relPos.y;
                    // entity.rotation = followPos.angleBetween(pos) + Math.PI*0.5;

                    // let startPos = Globals.camera.cameraCoords(slicer.startPos);
                    // let endPos = Globals.camera.cameraCoords(slicer.endPos);
                    // let a = startPos.angleBetween(endPos);
                    // let halfLen = startPos.distance(endPos) / 2;
                    // let diff = startPos.add(Vector2(halfLen, 10).rotateRadian(a)).subtract(followPos);
                    // followVel.addTo(diff.scalarMultiply(0.04)).scalarMultiplyWith(0.82);
                    // followPos.addTo(followVel);
                    // let relPos = entity.toComparablePosition(followPos);
                    // entity.position.x = relPos.x;
                    // entity.position.y = relPos.y;
                    // entity.rotation = followPos.angleBetween(endPos.add(Vector2(0, 10).rotateRadian(a)));
                } else {
                    var _diff = initialPos.subtract(followPos);
                    followVel.addTo(_diff.scalarMultiply(0.20)).scalarMultiplyWith(0.6);
                    followPos.addTo(followVel);
                    var _relPos = entity.toComparablePosition(followPos);
                    entity.position.x = _relPos.x;
                    entity.position.y = _relPos.y;
                    entity.rotation = followPos.angleBetween(initialPos.subtract(Vector2(0, 20))) + Math.PI * 0.5;
                }
            },
            attached: function attached(data) {
                entity = data.entity;
            }
        };
        return component;
    };

    var AnimScissor = function AnimScissor(settings) {
        var from = settings.from,
            to = settings.to,
            scale = settings.scale,
            imageName = settings.imageName;


        var sprite = new Sprite({
            imageName: imageName,
            frameCountX: 2,
            originRelative: new Vector2(0.5, 0.5),
            animations: {
                default: { speed: 0.14, frames: [1, 0, 1], loop: false
                    // default: { speed: 0.40, frames: [1,1,0,0,0,0,1,1,0,0,0,1,1], loop: false }
                    // default: { speed: 0, frames: [1] },
                } }
        });

        var entity = new Entity({
            z: Globals.Layers.ANIM_SCISSOR,
            name: 'animScissor',
            position: from.clone(),
            scale: new Vector2(scale, scale),
            rotation: from.angleBetween(to) + Math.PI * 0.5,
            components: [sprite]
        });

        // grow in
        Tween({
            from: 0,
            to: scale,
            in: 12,
            ease: 'easeOutQuad',
            onUpdate: function onUpdate(v, t) {
                entity.scale.x = entity.scale.y = v;
            }
        });
        // move from a to b
        Tween({
            from: from, to: to,
            delay: 0,
            in: 18,
            ease: 'easeInQuad',
            onUpdate: function onUpdate(v, t) {
                v.copyInto(entity.position);
            }
        });
        // shrink out
        Tween({
            from: scale,
            to: 0,
            delay: 22,
            // delay: 14,
            in: 12,
            ease: 'easeInQuad',
            onUpdate: function onUpdate(v, t) {
                entity.scale.x = entity.scale.y = v;
            }
        });

        return entity;
    };

    return function (settings) {
        var position = settings.position;


        var icons = [];

        var scissorName = SaveManager.load('scissor');
        var getScissorTexture = function getScissorTexture() {
            var texture = Utils.newTexture('scissors/scissors-' + scissorName);
            texture.frame.width /= 2;
            texture.frame.x += texture.frame.width;
            texture._updateUvs();
            return texture;
        };
        var getShadowTexture = function getShadowTexture() {
            return Utils.newTexture('scissors/scissors-' + scissorName.split('-')[0] + '-shadow');
        };

        var addIcon = function addIcon() {
            var container = new PIXI.Container();
            var sprite = new PIXI.Sprite(getScissorTexture());
            sprite.anchor.x = 0.5;
            sprite.anchor.y = 0.5;
            var shadow = new PIXI.Sprite(getShadowTexture());
            shadow.anchor.x = 0.5;
            shadow.anchor.y = 0.5;
            shadow.x = 14;
            shadow.y = 14;
            container.addChild(shadow);
            container.addChild(sprite);

            var icon = new Entity({
                name: 'slice',
                position: new Vector2(0, icons.length * -25),
                components: {
                    name: 'behavior',
                    draw: function draw(_ref2) {
                        var renderer = _ref2.renderer;

                        renderer.drawPixi(container);
                    }
                }
            });

            icon.attach(new EventListener({
                eventName: 'redrawScissors',
                onEvent: function onEvent(data) {
                    scissorName = SaveManager.load('scissor');
                    sprite.texture = getScissorTexture();
                    shadow.texture = getShadowTexture();
                }
            }));

            icons.push(icon);

            // grow icon from nothing
            new Tween({
                delay: icons.length * 10,
                from: 0, to: 0.07, in: 60,
                ease: 'elastic',
                decay: 8,
                oscillations: 2,
                applyOnDelay: true,
                onStart: function onStart() {
                    entity.attach(icon);
                },
                onUpdate: function onUpdate(v, t) {
                    icon.scale.x = v;
                    icon.scale.y = v;
                }
            });
        };
        var removeIcon = function removeIcon() {
            var icon = icons.pop();
            var followBehavior = icon.getComponent('followBehavior');
            if (followBehavior) {
                icon.remove(followBehavior);
                // animate out differently to below?
            }
            new Tween({
                from: icon.scale.x, to: 0, in: 20,
                ease: 'easeInBack',
                applyOnDelay: true,
                onUpdate: function onUpdate(v, t) {
                    icon.scale.x = v;
                    icon.scale.y = v;
                },
                onComplete: function onComplete() {
                    entity.remove(icon);
                }
            });
        };

        var repaint = function repaint() {
            var remaining = Globals.maxSlices - Globals.sliceCount;
            while (icons.length < remaining) {
                addIcon();
            }
            while (icons.length > remaining) {
                removeIcon();
            }

            // more icons = smaller scale
            new Tween({
                from: entity.scale.x,
                to: 1.0 - icons.length * 0.075,
                in: 20,
                ease: 'easeInOutQuad',
                onUpdate: function onUpdate(v, t) {
                    entity.scale.x = v;
                    entity.scale.y = v;
                }
            });

            // icons.forEach((icon, i) => {
            //     if (i < icons.length-1) {
            //         icon.removeByName('followBehavior');
            //     } else {
            //         if (!icon.getComponent('followBehavior')) {
            //             icon.attach(new FollowBehavior({}));
            //         }
            //     }
            // });
        };

        var sliceListener = new EventListener({
            eventName: 'slice',
            onEvent: function onEvent() {
                repaint();
                Bento.objects.get('slicer', function (slicer) {
                    Bento.objects.attach(new AnimScissor({
                        imageName: 'scissors/scissors-' + scissorName,
                        scale: entity.scale.x * 0.07,
                        from: slicer.startPos.clone(),
                        to: slicer.endPos.clone()
                    }));
                });
            }
        });
        var loadedListener = new EventListener({
            eventName: 'levelLoaded',
            onEvent: repaint
        });

        var entity = new Entity({
            z: Globals.Layers.GUI,
            float: true,
            name: 'sliceCounter',
            position: position,
            components: [sliceListener, loadedListener]
        });

        repaint();

        var showDev = function showDev() {
            var devMenu = Bento.objects.get('devMenu') || new DevMenu({});
            if (devMenu.isAdded) {
                devMenu.removeSelf();
            } else {
                Bento.objects.attach(devMenu);
            }
        };

        entity.dimension = new Rectangle(-15, -15, 30, 30);
        if (Utils.isDev() || Bento.saveState.load('isDev')) {
            Bento.saveState.save('isDev', true);
            entity.attach(new Clickable({
                onClick: showDev
            }));
        } else {
            var t = 0;
            var isDown = false;
            entity.attach({
                name: 'devChecker',
                update: function update() {
                    var _this = this;

                    if (isDown) {
                        if (t < 300) {
                            t++;
                            if (t >= 300) {
                                Utils.prompt('', '?', function (input) {
                                    if (input === window.atob('aW1hbG92ZXI=')) {
                                        Bento.saveState.save('isDev', true);
                                        Utils.setDev(true);
                                        showDev();
                                        entity.remove(_this);
                                    }
                                });
                            }
                        }
                    }
                }
            });
            entity.attach(new Clickable({
                onClick: function onClick(data) {
                    isDown = true;
                },
                onClickUp: function onClickUp(data) {
                    t = 0;
                    isDown = false;
                }
            }));
        }

        return entity;
    };
});