'use strict';

bento.define('entities/gui/starsearned', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, Globals, Fill, EventListener) {

    var Star = function Star(settings) {
        var position = settings.position,
            earned = settings.earned,
            delay = settings.delay;


        var sprite = new Sprite({
            imageName: 'gui/star-icon-empty',
            originRelative: new Vector2(0.5, 0.5)
        });
        var shadowSprite = new Sprite({
            imageName: 'gui/star-icon-shadow',
            originRelative: new Vector2(0.47, 0.47)
        });

        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'starsEarned',
            position: position,
            scale: new Vector2(0.8 * 0.2, 0.8 * 0.2),
            components: [shadowSprite, sprite]
        });

        if (earned) {
            Utils.timeout(delay, function () {
                // change graphic
                sprite.setup({
                    imageName: 'gui/star-icon',
                    originRelative: new Vector2(0.5, 0.5)
                });

                Bento.audio.playSound('sfx_star_single');

                // grow and shrink star
                Utils.sequenceTweens({
                    from: 0.8, to: 1.1, in: 4,
                    onUpdate: function onUpdate(v) {
                        entity.scale.x = entity.scale.y = v * 0.2;
                    }
                }, {
                    to: 0.8, in: 20,
                    ease: 'elastic',
                    decay: 3,
                    oscillations: 1
                });

                // rotate star back and forth
                Utils.sequenceTweens({
                    from: 0, to: 0.2, in: 10,
                    onUpdate: function onUpdate(v) {
                        entity.rotation = v;
                    }
                }, {
                    to: 0, in: 40,
                    ease: 'elastic',
                    decay: 3,
                    oscillations: 2
                });
            });
        }

        return entity;
    };

    return function (settings) {
        var position = settings.position;


        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'starsEarned',
            position: position,
            float: true
        });

        for (var i = 0; i < 3; i++) {
            var earned = i < Globals.starsEarned;
            var delay = 22 + (earned ? i * 20 : (Globals.starsEarned - 1) * 20);
            entity.attach(new Star({
                earned: earned,
                delay: delay,
                position: new Vector2((i - 1) * 38, i === 1 ? -6 : 5)
            }));
        }
        return entity;
    };
});