'use strict';

bento.define('entities/gui/tutorialhand', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, Globals, Fill, EventListener) {
    return function (settings) {
        var position = settings.position,
            text = settings.text,
            _settings$textOffset = settings.textOffset,
            textOffset = _settings$textOffset === undefined ? new Vector2(0, 0) : _settings$textOffset,
            _settings$removeOnWin = settings.removeOnWin,
            removeOnWin = _settings$removeOnWin === undefined ? false : _settings$removeOnWin,
            _settings$squishAmoun = settings.squishAmount,
            squishAmount = _settings$squishAmoun === undefined ? 1.2 : _settings$squishAmoun,
            _settings$moveAmount = settings.moveAmount,
            moveAmount = _settings$moveAmount === undefined ? 8 : _settings$moveAmount,
            _settings$innerScale = settings.innerScale,
            innerScale = _settings$innerScale === undefined ? 0.1 : _settings$innerScale;


        var sprite = new Sprite({
            imageName: 'gui/hand',
            originRelative: new Vector2(0.4, 0.1),
            frameCountX: 1
        });

        var container = new Entity({
            name: 'container',
            position: new Vector2(0, 0),
            scale: new Vector2(innerScale, innerScale),
            components: [sprite]
        });

        var animate = function animate() {
            // move towards point
            new Tween({
                from: moveAmount, to: 0, in: 20,
                ease: 'easeInQuad',
                onUpdate: function onUpdate(v, t) {
                    container.position.y = v;
                },
                onComplete: function onComplete() {
                    // squish
                    new Tween({
                        from: 1.0, to: squishAmount,
                        in: 10,
                        ease: 'easeOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            container.scale.y = innerScale * 1 / v;
                            container.scale.x = innerScale * v;
                        },
                        onComplete: function onComplete() {
                            // unsquish
                            new Tween({
                                from: squishAmount, to: 1.0,
                                in: 30,
                                ease: 'elastic',
                                decay: 7,
                                oscillations: 3,
                                onUpdate: function onUpdate(v, t) {
                                    container.scale.y = innerScale * 1 / v;
                                    container.scale.x = innerScale * v;
                                }
                            });
                            // move back to start
                            new Tween({
                                from: 0, to: moveAmount,
                                in: 40,
                                ease: 'easeInOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    container.position.y = v;
                                },

                                onComplete: animate
                            });
                        }
                    });
                }
            });
        };

        animate();

        var textObj = void 0;
        if (text) {
            textObj = new GameText({
                position: position && position.add(textOffset || new Vector2(0, 0)),
                scale: new Vector2(0.5, 0.5),
                fontColor: '#ffffff',
                strokeStyle: '#000000',
                // strokeStyle: '#054494',
                lineWidth: 1,
                text: text
            });
        }

        var behavior = {
            name: 'behavior',
            start: function start() {
                if (textObj) {
                    entity.parent.attach(textObj);
                }
            },
            destroy: function destroy() {
                if (textObj) {
                    entity.parent.remove(textObj);
                }
            },
            update: function update() {
                if (removeOnWin && Globals.hasWon) {
                    entity.removeSelf();
                }
            }
        };

        var entity = new Entity({
            z: Globals.Layers.GUI,
            float: true,
            name: 'tutorialHand',
            position: position,
            // rotation: Math.PI * -0.5,
            alpha: 0,
            components: [container, behavior]
        }).extend({
            hide: function hide() {
                // fade out
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                        if (textObj) {
                            textObj.alpha = v;
                        }
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                    }
                });
            }
        });

        // fade in
        new Tween({
            from: 0,
            to: 1,
            in: 30,
            ease: 'easeInOutQuad',
            onUpdate: function onUpdate(v, t) {
                entity.alpha = v;
                if (textObj) {
                    textObj.alpha = v;
                }
            }
        });

        return entity;
    };
});