'use strict';

bento.define('entities/joint', ['bento', 'bento/components/sprite', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'globals', 'modules/box2d'], function (Bento, Sprite, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, Globals, Box2D) {
    var _slicedToArray = function () {
        function sliceIterator(arr, i) {
            var _arr = [];
            var _n = true;
            var _d = false;
            var _e = undefined;

            try {
                for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
                    _arr.push(_s.value);

                    if (i && _arr.length === i) break;
                }
            } catch (err) {
                _d = true;
                _e = err;
            } finally {
                try {
                    if (!_n && _i["return"]) _i["return"]();
                } finally {
                    if (_d) throw _e;
                }
            }

            return _arr;
        }

        return function (arr, i) {
            if (Array.isArray(arr)) {
                return arr;
            } else if (Symbol.iterator in Object(arr)) {
                return sliceIterator(arr, i);
            } else {
                throw new TypeError("Invalid attempt to destructure non-iterable instance");
            }
        };
    }();

    /**
     * When attached to the world, finds two objects and creates a joint between them
     */
    var Joint = function Joint(settings) {
        var _settings$a = settings.a,
            a = _settings$a === undefined ? '' : _settings$a,
            _settings$b = settings.b,
            b = _settings$b === undefined ? '' : _settings$b,
            position = settings.position,
            tiled = settings.tiled,
            showJoint = settings.showJoint,
            motorSpeed = settings.motorSpeed,
            maxMotorTorque = settings.maxMotorTorque,
            _settings$enableLimit = settings.enableLimit,
            enableLimit = _settings$enableLimit === undefined ? false : _settings$enableLimit,
            _settings$lowerTransl = settings.lowerTranslation,
            lowerTranslation = _settings$lowerTransl === undefined ? -0.01 : _settings$lowerTransl,
            _settings$upperTransl = settings.upperTranslation,
            upperTranslation = _settings$upperTransl === undefined ? 0.01 : _settings$upperTransl;


        var jointDef = void 0;
        var entityA = void 0,
            entityB = void 0;
        var bodyA = void 0,
            bodyB = void 0; // entity body components

        // initial anchor settings (Box2D world coordinates, not useful after the bodies have moved)
        var anchorA = new Box2D.b2Vec2(Box2D.toMeters(tiled.x + tiled.polyline[0].x), Box2D.toMeters(tiled.y + tiled.polyline[0].y));
        var anchorB = new Box2D.b2Vec2(Box2D.toMeters(tiled.x + tiled.polyline[1].x), Box2D.toMeters(tiled.y + tiled.polyline[1].y));
        var axis = void 0; // axis unit vector, for line joints and prismatic joints

        if (a === '') {
            bodyA = new Box2D.Body({
                type: 'static'
            });
            entityA = new Entity({
                name: 'fixedBody',
                position: Box2D.toBento(anchorA),
                visible: false,
                active: false,
                components: [bodyA]
            });
            Bento.objects.attach(entityA);
        }
        if (b === '') {
            bodyB = new Box2D.Body({
                type: 'static'
            });
            entityB = new Entity({
                name: 'fixedBody',
                position: Box2D.toBento(anchorB),
                visible: false,
                active: false,
                components: [bodyB]
            });
            Bento.objects.attach(entityB);
        }

        var setup = function setup() {
            if (!entityA) {
                entityA = Bento.objects.get(a);
                if (entityA) {
                    // console.log('Found ', a);
                    bodyA = entityA.getComponent('body');
                    if (!bodyA) {
                        Utils.log('Can\'t create joint because ' + a + ' doesn\'t have a body');
                        Bento.objects.remove(joint);
                        return;
                    }
                } else {
                    return; // try again later
                }
            }
            if (!entityB) {
                entityB = Bento.objects.get(b);
                if (entityB) {
                    // console.log('Found ', b);
                    bodyB = entityB.getComponent('body');
                    if (!bodyB) {
                        Utils.log('Can\'t create joint because ' + b + ' doesn\'t have a body');
                        Bento.objects.remove(joint);
                        return;
                    }
                } else {
                    return; // try again later
                }
            }
            var bA = bodyA.getB2Body();
            var bB = bodyB.getB2Body();

            switch (tiled.type) {
                case 'revoluteJoint':
                    jointDef = new Box2D.b2RevoluteJointDef();

                    if (motorSpeed) {
                        if (maxMotorTorque) {
                            jointDef.maxMotorTorque = maxMotorTorque;
                        }
                        jointDef.motorSpeed = motorSpeed;
                        jointDef.enableMotor = true;
                    }
                    break;
                case 'weldJoint':
                    jointDef = new Box2D.b2WeldJointDef();
                    break;
                case 'distanceJoint':
                    jointDef = new Box2D.b2DistanceJointDef();
                    var dX = anchorB.x - anchorA.x;
                    var dY = anchorB.y - anchorA.y;
                    jointDef.length = Math.sqrt(dX * dX + dY * dY);
                    jointDef.dampingRatio = 0.99;
                    jointDef.frequencyHz = 5.0;
                    break;
                case 'lineJoint':
                case 'prismaticJoint':
                    if (tiled.type === 'lineJoint') {
                        jointDef = new Box2D.b2LineJointDef();
                    } else {
                        jointDef = new Box2D.b2PrismaticJointDef();
                    }
                    // the 2nd polyline point is repurposed to be an axis specifier
                    var p1 = Box2D.toVector2(anchorA);
                    var p2 = Box2D.toVector2(anchorB);
                    axis = Box2D.toB2Vec2(p1.subtract(p2).normalize());
                    break;
                default:
                    Utils.log('Unknown joint type ' + tiled.type);
                    Bento.objects.remove(joint);
                    return;
            }
            // Common setup for all joints?
            jointDef.bodyA = bA;
            jointDef.bodyB = bB;
            jointDef.localAnchorA.SetV(bA.GetLocalPoint(anchorA));
            if (axis) {
                jointDef.localAnchorB.SetV(bB.GetLocalPoint(anchorA));
                jointDef.localAxisA = axis;
                jointDef.enableLimit = enableLimit;
                jointDef.lowerTranslation = lowerTranslation;
                jointDef.upperTranslation = upperTranslation;
            } else {
                jointDef.localAnchorB.SetV(bB.GetLocalPoint(anchorB));
            }
            jointDef.collideConnected = false;
            joint.b2Joint = Box2D.box2DWorld.CreateJoint(jointDef);
            setAngularDamping();
            // console.log(entityA.name, entityB.name);
            // console.log(bA.GetWorldCenter(), bB.GetWorldCenter());
            // console.log(anchorA, anchorB);
        };

        /**
         * When one of the bodies belonging to this joint is sliced, it becomes
         *  two bodies (which I call primary and secondary).
         * If the joint is touching the secondary body / not touching the primary
         *  body, then we should destory it and make a new joint connecting the
         *  non-sliced body to the secondary body.
         */
        var onBodySlice = function onBodySlice(_ref) {
            var _ref2 = _slicedToArray(_ref, 2),
                primaryBody = _ref2[0],
                secondaryBody = _ref2[1];

            var otherBody = void 0; // the body belonging to this joint that was not sliced
            var localAnchor = void 0; // the anchor attached to the primary body
            var localAnchorOther = void 0; // the anchor attached to the non-sliced body

            var _jointDef = jointDef,
                localAnchorA = _jointDef.localAnchorA,
                localAnchorB = _jointDef.localAnchorB;


            if (primaryBody === bodyA) {
                localAnchor = localAnchorA;
                localAnchorOther = localAnchorB;
                otherBody = bodyB;
            } else if (primaryBody === bodyB) {
                localAnchor = localAnchorB;
                localAnchorOther = localAnchorA;
                otherBody = bodyA;
            } else {
                return; // this slice is not relevant to this joint
            }

            var primary = primaryBody.getB2Body();
            var worldAnchor = primary.GetWorldPoint(localAnchor);
            var anchoredToPrimary = false;
            var anchoredToSecondary = false;
            primaryBody.fixtures.forEach(function (f) {
                if (f.box2DFixture.TestPoint(worldAnchor)) {
                    anchoredToPrimary = true;
                }
            });
            secondaryBody.fixtures.forEach(function (f) {
                if (f.box2DFixture.TestPoint(worldAnchor)) {
                    anchoredToSecondary = true;
                }
            });
            console.log('Anchored:', anchoredToPrimary, anchoredToSecondary);

            if (anchoredToSecondary || !anchoredToPrimary) {
                if (otherBody === bodyB) {
                    bodyA = secondaryBody;
                    localAnchorA = localAnchor;
                    localAnchorB = localAnchorOther;
                } else {
                    bodyB = secondaryBody;
                    localAnchorA = localAnchorOther;
                    localAnchorB = localAnchor;
                }
                entityA = bodyA.parent;
                entityB = bodyB.parent;

                Box2D.box2DWorld.DestroyJoint(joint.b2Joint);

                // Destroy and rebuild the joint
                // Note: using the same jointDef from before, just updating some of the properties
                jointDef.bodyA = bodyA.getB2Body();
                jointDef.bodyB = bodyB.getB2Body();
                jointDef.localAnchorA.SetV(localAnchorA);
                jointDef.localAnchorB.SetV(localAnchorB);

                joint.b2Joint = Box2D.box2DWorld.CreateJoint(jointDef);
                setAngularDamping();
            }
        };

        var setAngularDamping = function setAngularDamping() {
            if (bodyA.getType() === 'dynamic') {
                bodyA.setAngularDamping(0.25);
            }
            if (bodyB.getType() === 'dynamic') {
                bodyB.setAngularDamping(0.25);
            }
        };

        var joint = {
            name: 'joint',
            b2Joint: null,
            start: function start() {
                EventSystem.on('bodySlice', onBodySlice);
                EventSystem.on('setupJoints', setup);
            },
            destroy: function destroy() {
                EventSystem.off('bodySlice', onBodySlice);
                EventSystem.off('setupJoints', setup);
            }
        };
        var jointEntity = new Entity({
            z: Globals.Layers.JOINTS,
            name: 'joint',
            position: position.add(tiled.polyline[0]),
            scale: new Vector2(0.1, 0.1),
            components: [joint]
        });

        if (showJoint) {
            if (tiled.type === 'distanceJoint') {
                jointEntity.scale.x = 0.14;
                jointEntity.scale.y = 0.14;
                var pixiChain = void 0,
                    pixiChainEnd = void 0;
                pixiChain = new window.PIXI.extras.TilingSprite(Utils.newTexture('boss/chain'), 44, 0);
                pixiChain.pivot.x = 0.5 * pixiChain.width;
                pixiChain.pivot.y = 1.0 * pixiChain.height;
                jointEntity.z = Globals.Layers.CHAIN;
                jointEntity.attach({
                    name: 'behavior',
                    draw: function draw(_ref3) {
                        var renderer = _ref3.renderer;

                        // draw chain
                        var p1 = jointEntity.toLocalPosition(Box2D.toBento(joint.b2Joint.GetAnchorA()));
                        var p2 = jointEntity.toLocalPosition(Box2D.toBento(joint.b2Joint.GetAnchorB()));
                        // renderer.drawLine([0,0,0,1.0], p1.x, p1.y, p2.x, p2.y, 4.0);
                        pixiChain.rotation = p2.angleBetween(p1) + Math.PI * 0.5;
                        pixiChain.height = p2.distance(p1);
                        var v = new Vector2(0, pixiChain.height);
                        v.rotateRadian(pixiChain.rotation);
                        pixiChain.x = v.x + p1.x;
                        pixiChain.y = v.y + p1.y;
                        pixiChain.pivot.y = 1.0 * pixiChain.height;
                        renderer.drawPixi(pixiChain);
                    }
                });
            } else {
                jointEntity.attach(new Sprite({
                    imageName: 'screw',
                    originRelative: new Vector2(0.5, 0.5)
                }));
            }
        } else {
            joint.visible = false;
        }

        return jointEntity;
    };

    /**
     * Special joint
     * a <--revoluteJoint--> mid
     * mid <--prismaticJoint--> b
     */
    // Joint.Hinge = function (settings) {

    //     let {
    //         a='', b='',
    //         position,
    //         tiled,
    //         showJoint,
    //         motorSpeed,
    //         maxMotorTorque,
    //         enableLimit = false,
    //         lowerTranslation = -0.01,
    //         upperTranslation = 0.01,
    //     } = settings;

    //     let entityA, entityB;
    //     let bodyA, bodyB;      // entity body components

    //     // initial anchor settings (Box2D world coordinates, not useful after the bodies have moved)
    //     let anchorA = new Box2D.b2Vec2(
    //         Box2D.toMeters(tiled.x + tiled.polyline[0].x),
    //         Box2D.toMeters(tiled.y + tiled.polyline[0].y)
    //     );
    //     let anchorB = new Box2D.b2Vec2(
    //         Box2D.toMeters(tiled.x + tiled.polyline[1].x),
    //         Box2D.toMeters(tiled.y + tiled.polyline[1].y)
    //     );
    //     let axis; // axis unit vector, for line joints and prismatic joints

    //     let midBody = new Box2D.Body({
    //         type: 'dynamic',
    //     });
    //     let midEntity = new Entity({
    //         name: `midBody-${midUid++}`,
    //         position: Box2D.toBento(anchorA),
    //         components: [ midBody ]
    //     });

    //     // let jointAMid = new Joint({
    //     //     a: a,
    //     //     b: midEntity.name,
    //     //     position,

    //     // });

    //     let started = false;
    //     let obj = {
    //         name: 'hinge',
    //         start () {
    //             if (!started) {
    //                 started = true;
    //                 Bento.objects.attach(midEntity);
    //                 Bento.objects.attach(jointAMid);
    //                 Bento.objects.attach(jointMidB);
    //             }
    //         },
    //         destroy () {

    //         }
    //     };
    // };

    var midUid = 0;

    return Joint;
});