'use strict';

window.VERSION = '0.0.0';
window.BUILD = 0;
window.VERSION = "1.2.6";
window.BUILD = 75;

// undefine require
if (window.require) {
    window.require = undefined;
}
if (window.define) {
    window.define = undefined;
}

if (bento.require.config) {
    // requirejs config
    bento.require.config({
        baseUrl: 'js',
        waitSeconds: 0
    });
}

window.startGame = function () {
    bento.require(['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/tween', 'bento/autoresize', 'utils', 'init', 'modules/migratesavestate'], function (Bento, Vector2, Rectangle, Tween, AutoResize, Utils, Init, MigrateSaveState) {
        // setup game
        MigrateSaveState({
            id: 'goslice/'
        });
        Bento.saveState.setId('goslice/');
        Bento.setup({
            name: 'Go Slice',
            canvasId: 'canvas',
            renderer: 'pixi',
            subPixel: true,
            pixelSize: Utils.isTablet() ? 6 : 3,
            responsiveResize: {
                landscape: false,
                minWidth: 180,
                maxWidth: 180,
                minHeight: 240,
                maxHeight: 400
            },
            useDeltaT: true,
            autoThrottle: true,
            antiAlias: true,
            preventContextMenu: true,
            reload: {
                // DEV ONLY! right mouse click refreshes the current screen with new javascript code
                simple: 'mouseDown-right',
                assets: 'buttonDown-1',
                jump: 'buttonDown-2'
            },
            // DEV ONLY! q downloads a screenshot
            screenshot: 'buttonDown-q',
            // set to false with build script
            dev: false
        }, function () {
            Init();
        });
    });
};

// window.patchUrl = 'http://localhost/slice/misc/patch';
window.patchUrl = 'https://d3q8vwl9xlq0g.cloudfront.net/goslice/data';
window.patchGame = function () {
    var hasStarted = false;
    window.patcher(window.patchUrl, 'v' + window.VERSION, function (res) {
        if (res) {
            console.log('Ended with', res);
        }
        if (!hasStarted) {
            window.startGame();
            hasStarted = true;
        }
    });
};

// entry points for cordova apps
document.addEventListener('deviceready', function () {
    window.patchGame();
}, false);

// other entry point
(function () {
    if (navigator.isCocoonJS) {
        // CocoonJs is cordova
        return;
    }
    if (window.cordova) {
        // wait for device ready
        return;
    }

    // playable ads
    if (window.mraid) {
        var hasStarted = false;
        if (window.mraid.getState() === 'loading') {
            // note: how long does mraid need for startup? if it takes long, 
            // we should make a loading screen in an html div and remove it when loading is complete
            window.mraid.addEventListener("ready", function () {
                if (!hasStarted) {
                    window.startGame();
                    hasStarted = true;
                }
            });
        } else {
            window.startGame();
            hasStarted = true;
        }
        return;
    }
    window.startGame();
})();