'use strict';

bento.define('globals', ['bento', 'bento/utils'], function (Bento, Utils) {

    // Game Events
    // -----------
    // 'guiLoaded'   - gui module finished adding all ui elements for this screen
    // 'levelLoaded' - tiled reader finished adding all objects
    // 'mainLoaded'  - both gui loader and tiled reader are ready
    // 'win'         - player hit goal flag. Or pass data `{skipped = true}` if the player skipped.
    // 'fail'        - player has likely failed the level (e.g. due to outOfSlices event, or failOnTurn flag)
    // 'pullLever'   - player reached lever in boss-fight. Cage will break, 'win' will be fired shortly after.
    // 'slice'       - player sliced one or more terrain bodies
    // 'bodySlice'   - fired for each body that is sliced. data[0] is the original body, data[1] is the newly created sibling body
    // 'firstSlice'  - player made their first slice on this level
    // 'outOfSlices' - player used their last available slice
    // 'nextLevel'   - player pressed next at the victory screen (game should start the next level)

    // 'beginSlicing' - player pressed their finger somewhere in the level
    // 'endSlicing'   - player released their finger (doesn't necessarily mean any terrain was actually sliced)

    // 'updateCoins'  - change the value displayed by the coin counter
    // 'buyCharacter' - player purchased a character in the shop
    // 'characterChanged' - the player was assigned a new character and the changes should take effect immediately

    // 'saveToCloud'


    /**
     * Global values
     * @moduleName Globals
     */
    var n = -2; // layer counter, should be set so that CAMERA_START = 0

    var Globals = {

        assetsLoaded: false,

        simulationSpeed: 1.3, // affects movement speed but not UI

        camera: null,
        maxSlices: 5,
        sliceCount: 0,
        paths: null, // mapping of path names to path data
        theme: null, // string, if set, forces the level to have a certain theme

        hasWon: false, // is the level currently complete?
        starsEarned: 0, // score for the level, set upon completion
        retryCount: 0, // how many times has the current level been restarted?
        failCount: 0, // how many times has the user run out of slices on this level?
        failOnTurn: false, // if true, this level should be beatable without the player turning around (use to suggest retry at appropriate time)

        debugInfo: false, // show FPS counter, level name, etc.

        hintsAvailable: 0, // how many hints are in this level
        hintsRevealed: 0, // how many hints are we showing?

        oldCoins: 0, // how many coins did the player have before clearing the level?
        alreadyCleared: false, // if true, the player has beaten this level before

        cageCharacter: null, // if this is a boss level, which character ID is unlocked when we beat it? (maybe we already have it, in which case we should ignore it)

        crossPromo: null, // global reference to crosspromo entity, can be reused between screens

        levelTimer: 0, // how much time in total player has spend on a single level (for Coda)

        /* @snippet Globals.Layers.*/
        Layers: {
            BACKGROUND: n++,
            CAMERA_START: n++,
            GRID: n++,
            CHAIN: n++,
            TERRAIN_SHADOW: n++,
            TERRAIN: n++,
            CAGE: n++,
            WALL: n++,
            JOINTS: n++,
            GOAL: n++,
            COLLECTIBLE: n++,
            LEMMING: n++,
            SLICER: n++,
            HINT_LINE: n++,
            BOX2D_DEBUG: n++,
            ANIM_SCISSOR: n++,
            CAMERA_END: n++,
            GUI_FILL: n++,
            GUI: n++,
            CROSS_PROMO: n++,
            SHOP_UNLOCK_FX: n++,
            GUI_COIN_ANIM: n++,
            DEV_MENU: n++,
            CONFETTI: n++,
            NOTIFICATIONS: n++,
            DIALOGS: n++,
            DIALOG_CONFETTI: n++,
            IAP_OVERLAY: n++,
            IPHONE_X_OVERLAY: n++,
            END: n++
        },
        PauseLevels: {
            NONE: 0,
            PAUSE: 1,
            SYSTEM: 2
        },
        /* @snippet Globals.Categories.*/
        Categories: {
            ALL: 0xffff,
            DEFAULT: 0x0001,
            WALL: 0x0002,
            PLAYER: 0x0004,
            GOAL: 0x0008,
            SLICEABLE: 0x0010,
            COLLECTIBLE: 0x0020,
            REGION: 0x0040, // e.g. fail sensor
            CAGE: 0x0080
        },

        freeGiftTime: function freeGiftTime(gift) {
            // return 10 * 1000; // test: 10 seconds
            switch (gift) {
                case 0:
                    return 0;
                case 1:
                    return 4 * 60 * 60 * 1000;
                default:
                    return 4 * 60 * 60 * 1000;
            }
        }
    };

    return Globals;
});