'use strict';

/**
 * Clickable app icon
 * @moduleName AppIcon
 * @snippet AppIcon|constructor
AppIcon({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/appicon', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'modules/localization', 'modules/textstyles', 'components/debug', 'components/iconbutton'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, Globals, Localization, TextStyles, Debug, IconButton) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var sprite = new Sprite({
            imageName: 'appicon',
            originRelative: new Vector2(0.5, 0.5)
        });
        var behavior = new Object({
            name: 'behavior',
            update: function update(data) {
                appIcon.rotation = Math.PI / 50 * Math.sin(appIcon.timer / 20);
            }
        });
        var appIcon = new Entity({
            name: 'appIcon',
            position: new Vector2(0, 0),
            scale: new Vector2(0.1, 0.1),
            components: [sprite, behavior]
        });
        var installNow = new Text({
            text: Localization.getText('playnow'),
            position: new Vector2(0, 0),
            fontSettings: TextStyles.get('dialogTitle'),
            align: 'center',
            textBaseline: 'middle',
            scale: new Vector2(3, 3),
            maxWidth: 140,
            linebreaks: false,
            // shadowColor: '#9e75b3',
            // shadow: true,
            // shadowOffset: new Vector2(1, 1),
            strokeStyle: '#8f421d',
            lineWidth: 4
        });
        var clickable = new Clickable({
            onClick: function onClick(data) {
                // Utils.openStoreUrl();
            }
        });
        var iconButton = new IconButton({
            position: new Vector2(0, 32),
            imageName: 'buttons/wide_yellow',
            frameCountX: 1,
            frameCountY: 3,
            // icon options
            icon: installNow,
            // icon transparency when inactive
            iconAlpha: 0.5,
            components: [new Object({
                name: 'wiggle',
                timer: 0,
                update: function update() {
                    this.timer += 0.1;
                    iconButton.rotation = Math.PI / 60 * Math.sin(this.timer);
                }
            })],
            excited: true,
            onClick: function onClick() {
                Utils.openStoreUrl();
            }
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'appIcon',
            position: position,
            boundingBox: new Rectangle(-32, -32, 64, 64),
            float: true,
            components: [
            // appIcon,
            iconButton, clickable]
        });
        return entity;
    };
});