'use strict';

bento.define('gui/cageprogressbar', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/eventlistener', 'components/simplesprite', 'modules/savemanager', 'modules/levelmanager', 'gui/darkenbar', 'gui/gui', 'modules/localization', 'modules/textstyles'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, Globals, EventListener, SimpleSprite, SaveManager, LevelManager, DarkenBar, Gui, Localization, TextStyles) {

    var CageProgressBar = function CageProgressBar(settings) {
        var position = settings.position;


        var totalLevels = LevelManager.getTotalLevels();
        var startLevel = Math.max(0, Utils.currentLevel - 1);
        var endLevel = Math.min(totalLevels - 1, Utils.currentLevel);
        var foundBoss = false;
        while (startLevel > 0 && startLevel > Utils.currentLevel - 12 && !LevelManager.isBossLevel(startLevel)) {
            startLevel--;
        }
        if (startLevel !== 0) {
            startLevel++;
        }
        while (endLevel < totalLevels && endLevel < Utils.currentLevel + 12) {
            if (LevelManager.isBossLevel(endLevel)) {
                foundBoss = true;
                break;
            }
            endLevel++;
        }
        var numStages = endLevel - startLevel + 1;
        var currentStage = Utils.currentLevel - startLevel;

        if (numStages >= 12 || !foundBoss) {
            // assume something went wrong, or we ran out of levels
            return {
                name: 'nilCageProgressBar'
            };
        }

        var fillWidth = 8;
        var spacing = 18;

        var widthPx = spacing * numStages - 4;

        // let barFill = new SimpleSprite({
        //     imageName: 'gui/cage/progression-bar-fill',
        //     position: new Vector2(0, 0),
        // });
        var barFill = new Entity({
            name: 'barFill',
            // position: new Vector2(-widthPx / (fillWidth*3), 0),
            position: new Vector2(-widthPx / 2 - 35 * 0.14 / 2 + 2.5, 0),
            scale: new Vector2(3 * widthPx / fillWidth, 0.14),
            components: [new Sprite({
                imageName: 'gui/cage/progression-bar-fill',
                // originRelative: new Vector2(0.5, 0.5),
                originRelative: new Vector2(0, 0.5),
                frameCountX: 3,
                animations: {
                    default: { speed: 0, frames: [1] // take the middle to prevent blurring
                    } }
            })]
        });
        var barCap1 = new SimpleSprite({
            imageName: 'gui/cage/progression-bar-cap',
            position: new Vector2(-widthPx / 2 - 35 * 0.14 / 2, 0),
            scale: new Vector2(0.14, 0.14)
        });
        var barCap2 = new SimpleSprite({
            imageName: 'gui/cage/progression-bar-cap',
            position: new Vector2(widthPx / 2 + 35 * 0.14 / 2, 0),
            scale: new Vector2(-0.14, 0.14)
        });

        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'cageProgressBar',
            float: true,
            position: position,
            components: [barFill, barCap1, barCap2]
        });

        entity.scale.scale(Math.min(1.0, 5 / numStages)).scale(Gui.scaleFactor);
        if (Utils.isTablet()) {
            position.y -= 5;
        }

        for (var i = 0; i < numStages; i++) {
            var y = 0;
            var x = i * spacing - spacing * (numStages - 1) / 2;
            if (i < currentStage) {
                entity.attach(new SimpleSprite({
                    imageName: 'gui/cage/level-done-new',
                    position: new Vector2(x, y),
                    scale: new Vector2(0.14, 0.14)
                }));
            } else if (LevelManager.isBossLevel(startLevel + i)) {
                if (i === currentStage) {
                    (function () {
                        var tada = new Entity({
                            name: 'fxTada',
                            position: new Vector2(x, y),
                            scale: new Vector2(0.13, 0.13),
                            components: [new Object({
                                name: 'behavior',
                                update: function update() {
                                    tada.rotation += 0.01;
                                }
                            }), new Sprite({
                                imageName: 'gui/cage/tada',
                                originRelative: new Vector2(0.5, 0.5)
                            })]
                        });
                        entity.attach(tada);
                    })();
                }
                entity.attach(new SimpleSprite({
                    imageName: 'gui/cage/silhouette',
                    position: new Vector2(x + 1.2, y),
                    scale: new Vector2(0.14, 0.13)
                }));
            } else {
                entity.attach(new SimpleSprite({
                    imageName: 'gui/cage/level',
                    position: new Vector2(x, y),
                    scale: new Vector2(0.1, 0.1)
                }));
                if (i === currentStage) {
                    entity.attach(new SimpleSprite({
                        imageName: 'gui/cage/player-position',
                        position: new Vector2(x, y - 7),
                        scale: new Vector2(0.16, 0.16)
                    }));
                }
            }
        }

        // fade out on level win
        entity.attach(new EventListener({
            eventName: 'win',
            onEvent: function onEvent() {
                Tween({
                    delay: 40,
                    from: 1.0, to: 0.0, in: 30,
                    ease: 'linear',
                    onStart: function onStart() {},
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                    }
                });
            }
        }));

        // show intro if this is a boss level and we just reached it.
        if (Globals.cageCharacter && Globals.retryCount === 0) {
            Utils.timeout(20, function () {
                bossFightIntro();
            });
        }

        return entity;
    };

    var bossFightIntro = function bossFightIntro() {

        var container = new Entity({
            name: 'bossFightIntroContainer',
            position: new Vector2(0, -40),
            alpha: 0
        });

        var textScale = Utils.approach(Gui.scaleFactorTall, 1.0, 0.2);
        var starScale = 0.18;

        var darkenBar = new DarkenBar({
            position: new Vector2(0, 0),
            height: 56 * textScale,
            time: 30,
            color: [0, 0.02, 0.51, 0.3]
        });
        container.attach(darkenBar);

        var bossFightText = new Text({
            text: Localization.getText('bossFight').toUpperCase(),
            position: new Vector2(0, 0),
            scale: new Vector2(textScale, textScale),
            fontSettings: TextStyles.get('bossFight'),
            align: 'center',
            maxWidth: 90,
            maxHeight: 60,
            linebreaks: true
            // drawDebug: true,
        });
        container.attach(bossFightText);

        Tween({
            from: 0, to: textScale, in: 60,
            ease: 'elastic',
            decay: 9,
            oscillations: 3,
            onStart: function onStart() {},
            onUpdate: function onUpdate(v, t) {
                bossFightText.scale.x = v;
                bossFightText.scale.y = v;
            },
            onComplete: function onComplete() {}
        });

        Tween({
            from: 0.0, to: 1.0, in: 20,
            onStart: function onStart() {
                Gui.attach(container);
            },
            onUpdate: function onUpdate(v) {
                container.alpha = v;
            }
        });

        var star1 = new Entity({
            name: 'star',
            position: new Vector2(-66, 0),
            scale: new Vector2(starScale, starScale),
            components: [new Sprite({
                imageName: 'gui/cage/star-bossfight',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });
        var star2 = new Entity({
            name: 'star',
            position: new Vector2(66, 0),
            scale: new Vector2(starScale, starScale),
            components: [new Sprite({
                imageName: 'gui/cage/star-bossfight',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        new Tween({
            from: 0, to: 1, in: 40,
            ease: 'easeOutBack',
            onStart: function onStart() {
                container.attach(star1);
                container.attach(star2);
            },
            onUpdate: function onUpdate(v, t) {
                star1.scale.x = v * 0.18;
                star1.scale.y = v * 0.18;
                star2.scale.x = v * 0.18;
                star2.scale.y = v * 0.18;
            }
        });

        new Tween({
            from: 0.3, to: 0, in: 140,
            ease: 'easeOutQuad',
            onUpdate: function onUpdate(v, t) {
                star1.rotation -= v;
                star2.rotation += v;
            }
        });

        new Tween({
            delay: 110,
            from: 1.0, to: 0, in: 30,
            ease: 'easeInQuad',
            onUpdate: function onUpdate(v, t) {
                // container.scale.y = v;
                darkenBar.scale.y = v;
                bossFightText.scale.x = textScale * v;
                bossFightText.scale.y = textScale * v;
                star1.position.x -= (1 - v) * 8;
                star2.position.x += (1 - v) * 8;
                star1.scale.x = starScale * v;
                star1.scale.y = starScale * v;
                star2.scale.x = starScale * v;
                star2.scale.y = starScale * v;
                // bossFightText.scale.y = v;
            },
            onComplete: function onComplete() {
                container.removeSelf();
            }
        });

        Utils.timeout(14, function () {
            Bento.audio.setVolume(0.5, 'sfx_boss_intro');
            Bento.audio.playSound('sfx_boss_intro');
        });
    };

    return CageProgressBar;
});