'use strict';

bento.define('gui/challengepopup', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/simplesprite', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'modules/admanager', 'modules/levelmanager', 'components/debug', 'bento/components/modal', 'modules/tapticengine', 'modules/localization', 'modules/savemanager', 'gui/iapoverlay', 'components/mask', 'modules/timemanager', 'gui/timerlabel', 'gui/dialog'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, SimpleSprite, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, Gui, GuiCommon, AdManager, LevelManager, Debug, Modal, TapticEngine, Localization, SaveManager, IapOverlay, Mask, TimeManager, TimerLabel, Dialog) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var onComplete = void 0;
    var canClick = false;

    var ChallengePopup = function ChallengePopup(settings) {

        onComplete = settings.onComplete;
        canClick = false;

        var viewport = Bento.getViewport();

        var fill = new Fill({
            name: 'fill',
            dimension: Bento.getViewport(true).getSize(),
            color: [0, 0, 0, 0.90], // [1, 1, 1, 1] is pure white
            originRelative: new Vector2(0, 0)
        });

        var challengePopupRoot = new Entity({
            name: 'challengePopupRoot',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            scale: new Vector2(Gui.scaleFactorTall, Gui.scaleFactorTall)
        });

        Gui.load({
            parent: challengePopupRoot,
            assetName: 'ui/challengepopup',
            handlers: GuiChallengePopup, onComplete: function onComplete() {
                // fade in overlay
                new Tween({
                    from: 0, to: 1, in: 20,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        canClick = true;
                    }
                });
            }
        });

        var entity = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'challengePopup',
            position: new Vector2(0, 0),
            float: true,
            components: [new Modal({}), fill, challengePopupRoot]
        });

        entity.extend({
            dismiss: function dismiss() {
                canClick = false;
                new Tween({
                    from: 1, to: 0, in: 20,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                    }
                });
            }
        });

        entity.alpha = 0;

        return entity;
    };

    var GuiChallengePopup = Utils.copyObject(GuiCommon);

    var currentChallenge = void 0;
    var levelsComplete = 0;
    var totalLevels = 7;
    var scheduledTime = 0;

    var canPlay = function canPlay() {
        return TimeManager.getTime() >= scheduledTime;
    };

    var playChallengeLevel = function playChallengeLevel() {
        Utils.currentLevelGroup = currentChallenge;
        Utils.currentLevel = LevelManager.getNextUncleared(currentChallenge);
        Bento.screens.show('screens/main');
    };

    Utils.extend(GuiChallengePopup, {
        init: function init(obj) {
            currentChallenge = LevelManager.getCurrentChallenge();
            levelsComplete = LevelManager.getNumberCleared(currentChallenge);
            totalLevels = LevelManager.getTotalLevels(currentChallenge);
            scheduledTime = SaveManager.load('scheduledChallenge');
        },
        challengeWindow: function challengeWindow(obj) {
            // hide the popup when clicked outside
            obj.attach(new Clickable({
                onClickMiss: function onClickMiss(data) {
                    obj.parent.parent.dismiss();
                }
            }));
        },
        challengeTitle: function challengeTitle(obj) {
            makeText(obj, {
                text: Localization.getText('challengeTitle').replace('{NUMBER}', totalLevels).toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        challengeDesc: function challengeDesc(obj) {
            makeText(obj, {
                text: Localization.getText('challengeDesc').replace('{NUMBER}', totalLevels).toUpperCase(),
                fontSettings: TextStyles.get('dialogText'),
                align: 'center'
            });
        },
        challengeProgress: function challengeProgress(obj) {
            var challengeNumber = TimeManager.getTime() < scheduledTime ? levelsComplete : levelsComplete + 1;
            makeText(obj, {
                text: '( ' + challengeNumber + ' / ' + totalLevels + ' )',
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        challengeWait: function challengeWait(obj) {
            if (canPlay()) {
                obj.removeSelf();
            }
        },
        challengeWaitText: function challengeWaitText(obj) {
            makeText(obj, {
                text: Localization.getText('nextChallengeTime').toUpperCase(),
                fontSettings: TextStyles.get('dialogText'),
                align: 'center'
            });
        },
        challengeWaitTimer: function challengeWaitTimer(obj) {
            // not really a button, just looks like one
            var timer = new TimerLabel({
                position: new Vector2(0, 0),
                scale: new Vector2(1.2, 1.2),
                useLocal: false,
                time: scheduledTime,
                onComplete: function onComplete() {
                    Gui.hide('challengeWait');
                    Gui.hide('buttonChallengeNextForAd');
                    Gui.show('buttonChallengePlay');
                }
            });
            var hoursRemaining = (scheduledTime - TimeManager.getTime()) / (60 * 60 * 1000);
            if (hoursRemaining < 10) {
                // slight horizontal adjustment
                timer.position.x -= 2.5 * timer.scale.x;
            }
            obj.attach(timer);
        },
        buttonChallengeNextForAd: function buttonChallengeNextForAd(obj) {
            if (canPlay()) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    AdManager.tryShowRewarded({
                        onReward: function onReward() {
                            playChallengeLevel();
                        },
                        onComplete: function onComplete(error) {
                            if (error) {
                                Bento.objects.attach(new Dialog({
                                    name: '7DayPopupAdFailed',
                                    text: Localization.getText('noAd'),
                                    type: 'ok'
                                }));
                            }
                        },

                        tag: '7Day_Popup_Next_Level',
                        rewardType: '7day_extra_level',
                        rewardAmount: 1
                    });
                }
            });
        },
        challengeNextText: function challengeNextText(obj) {
            makeText(obj, {
                text: Localization.getText('nextChallenge').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        buttonChallengePlay: function buttonChallengePlay(obj) {
            if (!canPlay()) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    playChallengeLevel();
                }
            });
        },
        challengePlayText: function challengePlayText(obj) {
            makeText(obj, {
                text: Localization.getText('play').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        challengeTapToClose: function challengeTapToClose(obj) {
            makeText(obj, {
                text: Localization.getText('tapToClose').toUpperCase(),
                fontSettings: TextStyles.get('mainWhite'),
                align: 'center'
            });
            obj.alpha = 0.5;
        }
    });

    return ChallengePopup;
});