'use strict';

bento.define('gui/characterunlock', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/hdsprite', 'modules/textstyles', 'modules/confetti', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'modules/localization', 'gui/dialog', 'gui/gui', 'bento/components/modal', 'entities/lemming', 'modules/skinmanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, HDSprite, TextStyles, Confetti, SimpleSprite, Debug, SaveManager, Localization, Dialog, Gui, Modal, Lemming, SkinManager) {
    return function (settings) {
        var characterId = settings.characterId,
            _onComplete = settings.onComplete;


        SkinManager.unlockCharacter(characterId); // immediately unlock

        var viewport = Bento.getViewport();

        var titleText = new Text({
            text: Localization.getText('newCharacter').toUpperCase(),
            position: new Vector2(0, -0.2 * viewport.height),
            scale: new Vector2(0.8, 0.8),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 180,
            maxHeight: 32
            // drawDebug: true
        });

        var characterDummy = new Entity({
            name: 'characterUnlockDummy',
            position: new Vector2(0, 10),
            scale: new Vector2(2.0, 2.0),
            components: [new Sprite(Lemming.getAnimations(characterId)['walk'])]
        });

        var buttonOk = new Entity({
            name: 'characterUnlockButtonOk',
            position: new Vector2(0, 0.2 * viewport.height),
            scale: new Vector2(0.27, 0.27),
            components: [new Sprite({
                imageName: 'gui/button-small-green',
                originRelative: new Vector2(0.5, 0.5)
            }), new Text({
                text: Localization.getText('okay').toUpperCase(),
                position: new Vector2(0, 0),
                scale: new Vector2(2.3, 2.3),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 42,
                maxHeight: 32,
                drawDebug: false
            })]
        });

        var container = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'characterUnlock',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            float: true,
            components: [new Modal({}), // not needed but happens to keep the confetti which is cool?
            new Object({
                name: 'behavior',
                start: function start() {

                    new Tween({
                        delay: 0,
                        from: 0, to: titleText.scale.x, in: 60,
                        ease: 'elastic',
                        decay: 9,
                        oscillations: 3,
                        onStart: function onStart() {
                            container.attach(titleText);
                        },
                        onUpdate: function onUpdate(v, t) {
                            titleText.scale.x = v;
                            titleText.scale.y = v;
                        }
                    });

                    new Tween({
                        delay: 12,
                        from: 0, to: characterDummy.scale.x, in: 60,
                        ease: 'elastic',
                        decay: 9,
                        oscillations: 3,
                        onStart: function onStart() {
                            container.attach(characterDummy);
                        },
                        onUpdate: function onUpdate(v, t) {
                            characterDummy.scale.x = v;
                            characterDummy.scale.y = v;
                        }
                    });

                    new Tween({
                        delay: 24,
                        from: 0, to: buttonOk.scale.x, in: 60,
                        ease: 'elastic',
                        decay: 9,
                        oscillations: 3,
                        onStart: function onStart() {
                            container.attach(buttonOk);
                        },
                        onUpdate: function onUpdate(v, t) {
                            buttonOk.scale.x = v;
                            buttonOk.scale.y = v;
                        }
                    });
                }
            })]
        });

        var _canClick = true;

        Gui.makeButton(buttonOk, {
            canClick: function canClick() {
                return _canClick;
            },
            onClick: function onClick() {
                _canClick = false;
                var titleTextScale = titleText.scale.x;
                var characterDummyScale = characterDummy.scale.x;
                var buttonOkScale = buttonOk.scale.x;
                new Tween({
                    from: 1, to: 0, in: 20,
                    ease: 'easeInBack',
                    onStart: function onStart() {},
                    onUpdate: function onUpdate(v, t) {
                        titleText.scale.x = v * titleTextScale;
                        titleText.scale.y = v * titleTextScale;
                        characterDummy.scale.x = v * characterDummyScale;
                        characterDummy.scale.y = v * characterDummyScale;
                        buttonOk.scale.x = v * buttonOkScale;
                        buttonOk.scale.y = v * buttonOkScale;
                    },
                    onComplete: function onComplete() {
                        container.removeSelf();
                        Utils.timeout(8, function () {
                            if (_onComplete) {
                                _onComplete();
                            }
                            SaveManager.save('character', characterId);
                        });
                    }
                });
            }
        });

        return container;
    };
});