'use strict';

bento.define('gui/characterunlockdialog', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/hdsprite', 'modules/textstyles', 'modules/confetti', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'modules/localization', 'gui/dialog', 'gui/gui', 'bento/components/modal', 'entities/lemming', 'modules/skinmanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, HDSprite, TextStyles, Confetti, SimpleSprite, Debug, SaveManager, Localization, Dialog, Gui, Modal, Lemming, SkinManager) {
    return function (settings) {
        var characterId = settings.characterId,
            _onComplete = settings.onComplete;


        var viewport = Bento.getViewport();

        var characterDummy = new Entity({
            name: 'characterUnlockDummy',
            position: new Vector2(0, 10),
            scale: new Vector2(2.0, 2.0),
            components: [new Sprite(Lemming.getAnimations(characterId)['walk'])]
        });

        var container = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'characterUnlock',
            float: true,
            components: [new Object({
                name: 'behavior',
                start: function start() {
                    new Tween({
                        delay: 12,
                        from: 0, to: characterDummy.scale.x, in: 60,
                        ease: 'elastic',
                        decay: 9,
                        oscillations: 3,
                        onStart: function onStart() {
                            container.attach(characterDummy);
                        },
                        onUpdate: function onUpdate(v, t) {
                            characterDummy.scale.x = v;
                            characterDummy.scale.y = v;
                        }
                    });
                }
            })]
        });

        var useEn = Localization.getLanguage() === 'en';

        var dialog = new Dialog({
            // title: Localization.getText('newCharacter').toUpperCase(),
            title: useEn ? 'NEW CHARACTER!' : Localization.getText('yougot').toUpperCase(),
            type: 'ok',
            onComplete: function onComplete() {
                SkinManager.unlockCharacter(characterId);
                SaveManager.save('character', characterId);
                EventSystem.fire('characterChanged', characterId);
                if (_onComplete) {
                    _onComplete();
                }
            }
        });

        dialog.getComponent('window', function (w) {
            w.attach(container);
        });
        dialog.attach({
            name: 'behavior',
            start: function start() {
                Bento.objects.attach(new Confetti({
                    speedY: 0.7,
                    particleScale: 0.08,
                    z: Globals.Layers.DIALOGS
                }));
                Bento.audio.playSound('sfx_victory_jingle_cheer');
            }
        });

        return dialog;
    };
});