'use strict';

/**
 * Ask player if they would like to load from cloud.
 * @moduleName CloudLoadDialog
 */
bento.define('gui/cloudloaddialog', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'modules/cloudsave', 'modules/localization', 'gui/dialog', 'gui/loadingspinner', 'entities/gui/gametext', 'gui/gui', 'bento/components/sprite', 'modules/textstyles', 'bento/gui/text'], function (Bento, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, CloudSave, Localization, Dialog, LoadingSpinner, GameText, Gui, Sprite, TextStyles, Text) {
    'use strict';

    return function (settings) {
        var onSuccess = settings.onSuccess,
            onDeclined = settings.onDeclined;


        var cloudLoadSuccess = false;

        var done = false;

        var entity = new Dialog({
            name: 'cloudLoadDialog',
            text: Localization.getText('loadFromCloud'),
            autoDismiss: false,
            type: 'yesno',
            onYes: function onYes() {
                var loadingSpinner = new LoadingSpinner({
                    scale: new Vector2(0.15, 0.15)
                });

                var buttonOk = new Entity({
                    name: 'cloudLoadDialogButtonOk',
                    position: new Vector2(0, 0),
                    scale: new Vector2(0.27, 0.27),
                    components: [new Sprite({
                        imageName: 'gui/button-small-green',
                        originRelative: new Vector2(0.5, 0.5)
                    }), new Text({
                        text: Localization.getText('okay').toUpperCase(),
                        position: new Vector2(0, 0),
                        scale: new Vector2(2.3, 2.3),
                        fontSettings: TextStyles.get('main'),
                        align: 'center',
                        maxWidth: 42,
                        maxHeight: 32,
                        drawDebug: false
                    })]
                });
                Gui.makeButton(buttonOk, {
                    canClick: function canClick() {
                        return !done;
                    },
                    onClick: function onClick() {
                        done = true;
                        entity.dismiss();
                    }
                });

                btnYes.removeSelf();
                btnNo.removeSelf();

                textText.visible = false;

                container.attach(loadingSpinner);

                Utils.timeout(40, function () {

                    CloudSave.loadFromCloud(function (success, reason) {
                        console.log('loadFromCloud result:', success, reason);
                        if (success) {

                            textText.setText(Localization.getText('cloudLoaded'));
                            textText.visible = true;
                            container.remove(loadingSpinner);

                            if (onSuccess) {
                                onSuccess();
                            }

                            bento.require(['screens/preloader', 'modules/levelmanager', 'modules/skinmanager'], function (preloader, LevelManager, SkinManager) {
                                // Make sure to refresh any cached save values, etc.
                                LevelManager.init();
                                SkinManager.init();
                                preloader.initSaveData();
                                Utils.timeout(30, function () {
                                    Bento.screens.show('screens/titlescreen');
                                });
                            });
                        } else {
                            if (reason === 'cloudSaveNone') {
                                textText.setText(Localization.getText('cloudNoSaveFile'));
                            } else if (reason === 'notLoggedIn') {
                                // should handle this case differently?
                                // in reality this dialog should never have been shown if the player wasn't logged in.
                                textText.setText(Localization.getText('error'));
                            } else {
                                textText.setText(Localization.getText('error'));
                            }

                            textText.visible = true;
                            container.remove(loadingSpinner);

                            // scale in 'continue' button
                            Tween({
                                from: 0, to: 0.3, in: 20,
                                ease: 'easeOutBack',
                                onStart: function onStart() {
                                    buttonContainer.attach(buttonOk);
                                },
                                onUpdate: function onUpdate(v, t) {
                                    buttonOk.scale.x = v;
                                    buttonOk.scale.y = v;
                                }
                            });
                        }
                    });
                });
            },
            onNo: function onNo() {
                entity.dismiss();
                if (onDeclined) {
                    onDeclined();
                }
            },
            onClose: function onClose() {}
        });

        var container = entity.getComponent('window');
        var buttonContainer = container.getComponent('buttonContainer');
        var btnYes = buttonContainer.getComponent('cloudLoadDialogButtonYes');
        var btnNo = buttonContainer.getComponent('cloudLoadDialogButtonNo');
        var textText = container.getComponent('text');

        return entity;
    };
});