'use strict';

bento.define('gui/coincollectanim', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'entities/gui/gamecounter', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/eventlistener', 'components/simplesprite', 'modules/savemanager', 'gui/gui'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, GameCounter, Utils, Screen, Tween, Globals, EventListener, SimpleSprite, SaveManager, Gui) {
    return function (settings) {
        var _settings$z = settings.z,
            z = _settings$z === undefined ? Globals.Layers.GUI_COIN_ANIM : _settings$z,
            from = settings.from,
            to = settings.to,
            _settings$amount = settings.amount,
            amount = _settings$amount === undefined ? 10 : _settings$amount,
            _settings$interval = settings.interval,
            interval = _settings$interval === undefined ? 4 : _settings$interval,
            _settings$duration = settings.duration,
            duration = _settings$duration === undefined ? 40 : _settings$duration,
            onCoinSpawn = settings.onCoinSpawn,
            onCoinCollect = settings.onCoinCollect,
            _onComplete = settings.onComplete;


        var sfxTimer = {
            name: 'sfxTimer',
            cooldown: 0,
            updateWhenPaused: 999,
            update: function update(_ref) {
                var speed = _ref.speed;

                if (this.cooldown > 0) {
                    this.cooldown -= speed;
                }
            },
            play: function play() {
                if (this.cooldown <= 0) {
                    Bento.audio.playSound('sfx_coin');
                    this.cooldown += interval / 2;
                }
            }
        };
        Bento.objects.attach(sfxTimer);

        Utils.repeat(amount, function (i) {
            var offset = new Vector2(Utils.getRandomRangeFloat(-1.0, 1.0), Utils.getRandomRangeFloat(-1.0, 1.0)).scale(16);

            var coin = new Entity({
                z: z,
                name: 'coinAnim',
                components: [new SimpleSprite({
                    imageName: 'gui/coin',
                    position: new Vector2(0, 0),
                    scale: new Vector2(0.11 * Gui.scaleFactor, 0.11 * Gui.scaleFactor)
                })]
            });

            var tween = Tween({
                delay: i * interval,
                from: 0, to: 1, in: duration,
                ease: 'linear',
                onStart: function onStart() {
                    Bento.objects.attach(coin);
                    if (onCoinSpawn) {
                        onCoinSpawn(i + 1);
                    }
                    sfxTimer.play();
                },
                onUpdate: function onUpdate(v, t) {
                    coin.position.x = from.x + (to.x - from.x) * v + offset.x * Math.sin(v * Math.PI);
                    coin.position.y = from.y + (to.y - from.y) * v + offset.y * Math.sin(v * Math.PI);
                },
                onComplete: function onComplete() {
                    Bento.objects.remove(coin);
                    if (onCoinCollect) {
                        onCoinCollect(i + 1);
                    }
                    sfxTimer.play();
                    if (i + 1 === amount) {
                        if (_onComplete) {
                            _onComplete();
                        }
                        Bento.objects.remove(sfxTimer);
                    }
                }
            });
            tween.name = 'coinCollectTween'; // GUI should not restore this tween
        });
    };
});