'use strict';

bento.define('gui/coincounter', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'entities/gui/gamecounter', 'bento/utils', 'bento/screen', 'bento/tween', 'globals', 'bento/components/eventlistener', 'components/simplesprite', 'modules/savemanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, GameCounter, Utils, Screen, Tween, Globals, EventListener, SimpleSprite, SaveManager) {
    return function (settings) {
        var position = settings.position,
            entity = settings.entity,
            _settings$value = settings.value,
            value = _settings$value === undefined ? SaveManager.load('coins') : _settings$value,
            _settings$numbersImag = settings.numbersImage,
            numbersImage = _settings$numbersImag === undefined ? 'numbers-jua-104-black' : _settings$numbersImag;


        if (!entity) {
            entity = new Entity({
                name: 'coinCounter',
                position: position
            });
        }

        var updateCounter = function updateCounter(v) {
            value = v;
            // v = v.toString();
            // let offset = v.length * 4.5;
            // icon.position.x = -offset;
            // counter.position.x = 10-offset;
            counter.setValue(v);
        };

        var icon = new SimpleSprite({
            imageName: 'gui/coin-icon',
            position: new Vector2(0, 0.5),
            scale: new Vector2(0.12, 0.12)
        });
        var iconScale = icon.scale.x;

        var counter = new GameCounter({
            name: 'coinCounterCounter',
            value: '',
            imageName: numbersImage,
            frameCountX: 1,
            frameCountY: 10,
            padding: 0,
            align: 'right',
            spacing: new Vector2(0, 0),
            position: new Vector2(-10, 0),
            updateWhenPaused: 0,
            scale: new Vector2(0.093, 0.093)
        });
        var counterScale = counter.scale.x;

        entity.attach(icon);
        entity.attach(counter);

        entity.attach(new EventListener({
            eventName: 'updateCoins',
            onEvent: function onEvent(v) {
                updateCounter(v === undefined ? SaveManager.load('coins') : v);
                new Tween({
                    from: 1.14, to: 1.0, in: 16,
                    ease: 'easeOutQuart',
                    onUpdate: function onUpdate(s) {
                        counter.scale.x = counterScale * s;
                        counter.scale.y = counterScale * s;
                    }
                });
            }
        }));

        var initialized = false;
        entity.attach({
            name: 'behavior',
            start: function start() {
                if (!initialized) {
                    // first time this instance is shown, we want to use the passed value for animation purposes
                    initialized = true;
                    updateCounter(value);
                } else {
                    // on subsequent times, assume the animation was finished or interrupted, so we show the real coin value
                    updateCounter(SaveManager.load('coins'));
                }
            }
        });

        return entity;
    };
});