'use strict';

bento.define('gui/coindialog', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/hdsprite', 'modules/textstyles', 'modules/confetti', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'modules/localization', 'gui/dialog', 'gui/coincounter', 'gui/gui', 'gui/coincollectanim', 'modules/cloudsave'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, HDSprite, TextStyles, Confetti, SimpleSprite, Debug, SaveManager, Localization, Dialog, CoinCounter, Gui, CoinCollectAnim, CloudSave) {
    return function (settings) {
        var amount = settings.amount,
            _settings$imageName = settings.imageName,
            imageName = _settings$imageName === undefined ? 'gui/icon-1000' : _settings$imageName,
            _settings$iconScale = settings.iconScale,
            iconScale = _settings$iconScale === undefined ? new Vector2(0.15, 0.15) : _settings$iconScale,
            _settings$iconOffset = settings.iconOffset,
            iconOffset = _settings$iconOffset === undefined ? new Vector2(0, 0) : _settings$iconOffset;


        var oldCoins = SaveManager.load('coins');
        var newCoins = oldCoins + amount;
        SaveManager.save('coins', newCoins);

        CloudSave.saveToCloud(function (success, errorMsg) {
            if (!success) {
                console.log(errorMsg);
                return;
            }
        });

        var dialog = new Dialog({
            // position: new Vector2(-viewport.width/2, -viewport.height/2),
            title: Localization.getText('yougot').toUpperCase(),
            type: 'ok',
            onComplete: function onComplete() {
                Gui.get('coinCounter', function (cc) {
                    Gui.get('starCounter', function (sc) {
                        new Tween({
                            from: 0.0, to: 1.0, in: 20,
                            ease: 'easeOutQuad',
                            onUpdate: function onUpdate(v, t) {
                                cc.alpha = v;
                                sc.alpha = v;
                            }
                        });
                    });
                });
                EventSystem.fire('updateCoins', newCoins);
            }
        });

        Gui.get('coinCounter', function (cc) {
            Gui.get('starCounter', function (sc) {
                dialog.attach(Tween.TweenBehavior({
                    from: 1.0, to: 0.0, in: 30,
                    ease: 'easeOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        cc.alpha = v;
                        sc.alpha = v;
                    }
                }));
            });
            var pos = cc.getWorldPosition();
            var cc2 = new CoinCounter({
                position: dialog.toLocalPosition(pos),
                numbersImage: 'numbers-jua-104-white'
            });
            dialog.attach(cc2);

            var w = dialog.getComponent('window');
            dialog.attach({
                name: 'behavior',
                start: function start() {
                    var productIcon = new Entity({
                        name: 'productIcon',
                        position: Vector2(-25, 0).add(iconOffset),
                        scale: Vector2(1.3, 1.3).multiply(iconScale),
                        components: [new Sprite({
                            imageName: imageName,
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    });
                    var amountText = new Text({
                        text: '+' + amount,
                        position: new Vector2(10, 0),
                        scale: new Vector2(0.8, 0.8),
                        fontSettings: TextStyles.get('main'),
                        align: 'left',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    });

                    w.attach(productIcon);
                    w.attach(amountText);

                    var animAmount = 10;
                    var interval = void 0,
                        duration = void 0;
                    if (amount < 1000) {
                        interval = 4;
                        duration = 26;
                        animAmount = 10;
                    } else if (amount < 5000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 20;
                    } else if (amount < 11000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 30;
                    } else if (amount < 24000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 40;
                    } else {
                        interval = 4;
                        duration = 26;
                        animAmount = 50;
                    }

                    CoinCollectAnim({
                        z: Globals.Layers.DIALOG_CONFETTI,
                        from: productIcon.getWorldPosition(),
                        to: cc.getWorldPosition(),
                        amount: animAmount,
                        interval: interval,
                        duration: duration,
                        onCoinSpawn: function onCoinSpawn(i) {
                            new Tween({
                                from: 1.2, to: 1, in: 6,
                                ease: 'easeOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    // hdSprite.scale.x = hdSpriteScale * v;
                                    // hdSprite.scale.y = hdSpriteScale * v;
                                    // shadowSprite.scale.x = hdSpriteScale * v;
                                    // shadowSprite.scale.y = hdSpriteScale * v;
                                }
                            });
                        },
                        onCoinCollect: function onCoinCollect(i) {
                            EventSystem.fire('updateCoins', Math.floor(oldCoins + amount * i / animAmount));
                        }
                    });
                }
            });
        });

        return dialog;
    };
});