'use strict';

bento.define('gui/darkenbar', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'components/simplesprite'], function (Bento, Vector2, Rectangle, Sprite, Entity, EventSystem, Utils, Tween, Globals, Fill, SimpleSprite) {

    var viewport = Bento.getViewport();

    /**
     *  
     */
    var DarkenBar = function DarkenBar(settings) {
        var position = settings.position,
            _settings$height = settings.height,
            height = _settings$height === undefined ? 50 : _settings$height,
            _settings$time = settings.time,
            time = _settings$time === undefined ? 30 : _settings$time,
            _settings$color = settings.color,
            color = _settings$color === undefined ? [0, 0, 0, 0.5] : _settings$color;


        var targetAlpha = color[3];

        var fill = new Fill({
            color: color,
            originRelative: new Vector2(0.5, 0.5),
            dimension: new Rectangle(0, 0, viewport.width, 0)
        });

        var behavior = {
            name: 'behavior',
            start: function start() {
                new Tween({
                    from: 0, to: 1.0, in: time,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        fill.color[3] = v * targetAlpha;
                        fill.dimension.y = -v * height / 2;
                        fill.dimension.height = v * height;
                    }
                });
            }
        };

        var entity = new Entity({
            name: 'darkenBar',
            position: position,
            components: [fill, behavior]
        }).extend({
            hide: function hide() {
                new Tween({
                    from: 1.0, to: 0.0, in: 20,
                    ease: 'easeInBack',
                    onUpdate: function onUpdate(v, t) {
                        fill.color[3] = v * targetAlpha;
                        fill.dimension.y = -v * height / 2;
                        fill.dimension.height = v * height;
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                    }
                });
            }
        });

        return entity;
    };

    return DarkenBar;
});