'use strict';

/**
 * Generic dialog with title and text in the middle
 * @moduleName Dialog
 * @snippet Dialog.snippet
Dialog({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/dialog', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/modal', 'bento/components/fill', 'bento/components/nineslice', 'modules/textstyles', 'gui/gui', 'components/hdsprite', 'components/simplesprite', 'modules/localization'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, Globals, Modal, Fill, NineSlice, TextStyles, Gui, HDSprite, SimpleSprite, Localization) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var name = settings.name || 'dialog';
        var position = settings.position;
        var _onComplete = settings.onComplete;
        var title = settings.title || '';
        var text = settings.text || '';
        var width = settings.width || 144;
        var height = settings.height || 144;
        var type = settings.type || 'ok'; // ok, yesno
        var analytics = settings.analytics || false;
        var autoDismiss = settings.autoDismiss !== false;
        var canClose = settings.canClose || false;

        var done = false;

        var buttonOk = void 0,
            buttonYes = void 0,
            buttonNo = void 0,
            buttonClose = void 0;

        if (type === 'ok') {
            buttonOk = new Entity({
                name: name + 'ButtonOk',
                position: new Vector2(0, 0),
                scale: new Vector2(0.27, 0.27),
                components: [new Sprite({
                    imageName: 'gui/button-small-green',
                    originRelative: new Vector2(0.5, 0.5)
                }), new Text({
                    text: Localization.getText('okay').toUpperCase(),
                    position: new Vector2(0, 0),
                    scale: new Vector2(2.3, 2.3),
                    fontSettings: TextStyles.get('main'),
                    align: 'center',
                    maxWidth: 42,
                    maxHeight: 32,
                    drawDebug: false
                })]
            });
            Gui.makeButton(buttonOk, {
                canClick: function canClick() {
                    return !done;
                },
                onClick: function onClick() {
                    done = true;
                    if (settings.onOk) {
                        settings.onOk();
                    }
                    if (autoDismiss) {
                        entity.dismiss();
                    }
                }
            });
        } else if (type === 'yesno') {

            buttonNo = new Entity({
                name: name + 'ButtonNo',
                position: new Vector2(-26, 0),
                components: [HDSprite({
                    gamepixelDimension: new Rectangle(0, 0, 42, 42), // desired dimension in pixels
                    imageName: 'gui/button-pink-empty',
                    originRelative: new Vector2(0.5, 0.5)
                }), SimpleSprite({
                    imageName: 'buttons/cross-white',
                    scale: new Vector2(0.12, 0.12)
                })]
            });
            Gui.makeButton(buttonNo, {
                canClick: function canClick() {
                    return !done;
                },
                onClick: function onClick() {
                    done = true;
                    if (settings.onNo) {
                        settings.onNo();
                    }
                    if (autoDismiss) {
                        entity.dismiss();
                    }
                }
            });
            buttonYes = new Entity({
                name: name + 'ButtonYes',
                position: new Vector2(26, 0),
                components: [HDSprite({
                    gamepixelDimension: new Rectangle(0, 0, 42, 42), // desired dimension in pixels
                    imageName: 'gui/button-mint-blank',
                    originRelative: new Vector2(0.5, 0.5)
                }), SimpleSprite({
                    imageName: 'buttons/tick-white',
                    scale: new Vector2(0.12, 0.13)
                })]
            });
            Gui.makeButton(buttonYes, {
                canClick: function canClick() {
                    return !done;
                },
                onClick: function onClick() {
                    done = true;
                    if (settings.onYes) {
                        settings.onYes();
                    }
                    if (autoDismiss) {
                        entity.dismiss();
                    }
                }
            });
        } else {
            Utils.log('Unknown dialog type \'' + type + '\'');
        }

        if (canClose) {
            buttonClose = new Entity({
                name: 'buttonClose',
                position: new Vector2(60, -70 - height / 2),
                scale: new Vector2(0.15, 0.15),
                components: [new Sprite({
                    imageName: 'gui/close-button',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });
            Gui.makeButton(buttonClose, {
                canClick: function canClick() {
                    return !done;
                },
                onClick: function onClick() {
                    done = true;
                    entity.dismiss();
                }
            });
        }

        if (type === 'yesno') {}
        // let button = settings.button || new IconButton({
        //     name: 'button',
        //     imageName: 'gui/buttons/button_yellow_wide',
        //     iconName: 'gui/icons/tick',
        //     iconAlpha: 0.5,
        //     sfx: 'sfx_click',
        //     onClick: function () {
        //         if (settings.onYes) {
        //             settings.onYes();
        //             if (analytics) {
        //                 EventSystem.fire('Analytics', {
        //                     category: 'Dialogs',
        //                     action: name,
        //                     value: 1
        //                 });
        //                 EventSystem.fire('GameAnalytics-addDesignEvent', {
        //                     eventId: "Dialogs:" + name, // max 5 segments, each 32 length max
        //                     value: 1
        //                 });
        //             }
        //         }
        //         if (autoDismiss) {
        //             entity.dismiss();
        //         }
        //     }
        // });
        // let buttonNo = settings.buttonNo || new IconButton({
        //     name: 'button',
        //     imageName: 'gui/buttons/small_green',
        //     iconName: 'gui/icons/cross_green',
        //     iconAlpha: 0.5,
        //     sfx: 'sfx_click',
        //     onClick: function () {
        //         if (settings.onNo) {
        //             settings.onNo();
        //             if (analytics) {
        //                 EventSystem.fire('Analytics', {
        //                     category: 'Dialogs',
        //                     action: name,
        //                     value: 0
        //                 });
        //                 EventSystem.fire('GameAnalytics-addDesignEvent', {
        //                     eventId: "Dialogs:" + name, // max 5 segments, each 32 length max
        //                     value: 0
        //                 });
        //             }
        //         }
        //         if (autoDismiss) {
        //             entity.dismiss();
        //         }
        //     }
        // });
        var backgroundFill = new Entity({
            name: 'background',
            components: [new Fill({
                name: 'fill',
                dimension: Bento.getViewport(true).getSize(),
                color: [0, 0, 0.1, 0.7]
            })]
        });
        var titleText = new Text({
            position: new Vector2(0, -height / 2 + 15),
            text: title,
            fontSettings: settings.titleStyle || TextStyles.get('dialogTitle'),
            align: 'center',
            textBaseline: 'middle',
            maxWidth: width * 0.9,
            maxHeight: 16,
            linebreaks: false,
            shadowColor: '#7ed6ff',
            shadow: settings.titleShadow !== false,
            shadowOffset: new Vector2(1, 1)
            // scale: new Vector2(0.148, 0.148),
        });
        titleText.name = 'titleText';
        var textText = new Text({
            position: new Vector2(0, 0),
            text: text,
            fontSettings: settings.textStyle || TextStyles.get('dialogText'),
            align: 'center',
            textBaseline: 'middle',
            maxWidth: width * 0.9 / 1.6,
            maxHeight: height * 0.9 - 80,
            linebreaks: true,
            // drawDebug: true,
            scale: new Vector2(1.6, 1.6)
        });
        textText.name = 'text';
        var nineSlice = new Entity({
            name: 'sliceContainer',
            scale: new Vector2(0.148, 0.148),
            components: [new NineSlice({
                imageName: 'gui/frame-iap',
                originRelative: new Vector2(0.5, 0.5),
                padding: 1,
                width: width / 0.148,
                height: height / 0.148
            })]
        });
        var buttonContainer = new Entity({
            name: 'buttonContainer',
            position: new Vector2(0, height / 2 - 0)
        });
        if (buttonOk) buttonContainer.attach(buttonOk);
        if (buttonYes) buttonContainer.attach(buttonYes);
        if (buttonNo) buttonContainer.attach(buttonNo);
        if (buttonClose) buttonContainer.attach(buttonClose);

        var windowEntity = new Entity({
            name: 'window',
            position: viewport.getExtents(),
            components: [nineSlice, titleText, textText, buttonContainer]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function start(data) {
                buttonContainer.components.forEach(function (btn) {
                    btn.active = false;
                });
                new Tween({
                    from: 0,
                    to: Gui.scaleFactor,
                    in: 20,
                    ease: 'elastic',
                    decay: 6,
                    oscillations: 1,
                    onUpdate: function onUpdate(v, t) {
                        windowEntity.scale.x = v;
                        windowEntity.scale.y = v;
                    },
                    onComplete: function onComplete() {
                        buttonContainer.components.forEach(function (btn) {
                            btn.active = true;
                        });
                    }
                });
            },
            destroy: function destroy(data) {},
            update: function update(data) {},
            draw: function draw(data) {}
        };
        var modal = new Modal({});
        var entity = new Entity({
            z: settings.z || Globals.Layers.DIALOGS,
            name: name,
            position: position,
            float: true,
            dimension: new Rectangle(0, 0, width, height),
            components: [modal, backgroundFill, windowEntity, behavior]
        }).extend({
            dismiss: function dismiss() {
                buttonContainer.components.forEach(function (btn) {
                    btn.active = false;
                });
                new Tween({
                    from: windowEntity.scale.x,
                    to: 0,
                    in: 15,
                    ease: 'easeInBack',
                    onUpdate: function onUpdate(v, t) {
                        windowEntity.scale.x = v;
                        windowEntity.scale.y = v;
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                        if (_onComplete) {
                            _onComplete();
                        }
                    }
                });
            }
        });

        // if (type === 'yesno') {
        //     buttonNo.position.x = -45;
        //     button.position.x = 24;
        //     buttonContainer.attach(buttonNo);
        //     analytics = Utils.getDefault(settings.analytics, true);
        //     if (name === 'dialog') {
        //         Utils.log("Warning: give dialog a name");
        //     }
        // }
        return entity;
    };
});