'use strict';

bento.define('gui/freegift', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'entities/gui/gamecounter', 'bento/utils', 'globals', 'bento/tween', 'modules/savemanager', 'modules/timemanager', 'components/simplesprite', 'gui/timerlabel', 'components/hdsprite', 'bento/gui/text', 'modules/textstyles', 'gui/gui', 'gui/coincollectanim', 'modules/localization', 'modules/notifications'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, GameCounter, Utils, Globals, Tween, SaveManager, TimeManager, SimpleSprite, TimerLabel, HDSprite, Text, TextStyles, Gui, CoinCollectAnim, Localization, Notifications) {
    return function (settings) {
        var entity = settings.entity,
            _settings$style = settings.style,
            style = _settings$style === undefined ? 'small' : _settings$style;


        var giftIcon = void 0,
            giftText = void 0,
            nextGiftText = void 0,
            giftTimer = void 0,
            dimension = void 0;

        var scheduledTime = SaveManager.load('scheduledFreeGift');
        // let scheduledTime = TimeManager.getTime() + 4 * 1000;
        var canClaim = TimeManager.getTime() > scheduledTime;

        var showTimer = function showTimer() {
            if (giftTimer) {
                giftTimer.removeSelf();
            }
            if (style === 'small') {
                giftTimer = new TimerLabel({
                    position: new Vector2(0, 10),
                    scale: new Vector2(0.6, 0.6),
                    useLocal: false,
                    time: scheduledTime,
                    onComplete: function onComplete() {
                        return repaint();
                    }
                });
                var hoursRemaining = (scheduledTime - TimeManager.getTime()) / (60 * 60 * 1000);
                if (hoursRemaining < 10) {
                    // slight horizontal adjustment
                    giftTimer.position.x -= 1.5;
                }
            } else {
                giftTimer = new TimerLabel({
                    position: new Vector2(16, 10),
                    scale: new Vector2(1, 1),
                    useLocal: false,
                    time: scheduledTime,
                    onComplete: function onComplete() {
                        return repaint();
                    }
                });
            }
            entity.attach(giftTimer);
        };
        var hideTimer = function hideTimer() {
            if (giftTimer) {
                giftTimer.removeSelf();
            }
        };

        if (style === 'small') {
            dimension = new Rectangle(-16, -16, 32, 32);
            giftIcon = new SimpleSprite({
                imageName: 'gui/gift-icon',
                position: new Vector2(0, -4),
                scale: new Vector2(0.065, 0.065)
            });
            giftText = new Text({
                text: Localization.getText('freeGift').toUpperCase(),
                position: new Vector2(0, 10),
                scale: new Vector2(0.5, 0.5),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 54,
                maxHeight: 11
            });
        } else {
            // big style
            dimension = new Rectangle(-60, -30, 120, 60);
            giftIcon = new SimpleSprite({
                imageName: 'gui/gift-icon',
                position: new Vector2(-30, 0),
                scale: new Vector2(0.12, 0.12)
            });
            giftText = new Text({
                text: function () {
                    var lang = Localization.getLanguage();
                    var s = Localization.getText('freeGift');
                    if (lang === 'ja') {
                        s = s.replace('無料', '無料\n'); // linebreak after 'free'
                    } else {
                        s = s.toUpperCase();
                    }
                    return s;
                }(),
                position: new Vector2(18, 0),
                scale: new Vector2(0.82, 0.82),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                linebreaks: false,
                maxWidth: 64,
                maxHeight: 32
            });
            nextGiftText = new Text({
                text: function () {
                    var lang = Localization.getLanguage();
                    var s = Localization.getText('nextFreeGiftTimer');
                    if (lang === 'ja') {
                        s = s.replace('フリーギフト', 'フリーギフト\n'); // forced linebreak after 'free gift'
                    }
                    return s;
                }(),
                position: new Vector2(18, -8),
                scale: new Vector2(0.6, 0.6),
                fontSettings: TextStyles.get('main'),
                textBaseline: 'bottom',
                align: 'center',
                maxWidth: 90,
                maxHeight: 32
                // drawDebug: true,
            });
        }

        var getImageName = function getImageName() {
            var btnStyle = style === 'small' ? 'small-square' : 'big';
            var btnColor = canClaim ? 'green' : 'grey';
            return 'gui/button-' + btnStyle + '-' + btnColor;
        };
        var hdSprite = HDSprite({
            name: 'buttonBg',
            imageName: getImageName(),
            gamepixelDimension: dimension,
            originRelative: new Vector2(0.5, 0.5)
        });

        var repaint = function repaint() {
            canClaim = TimeManager.getTime() > scheduledTime;
            // make button available or unavailable
            if (canClaim) {
                giftText.visible = true;
                hideTimer();
                if (nextGiftText) nextGiftText.visible = false;
            } else {
                giftText.visible = false;
                showTimer();
                if (nextGiftText) nextGiftText.visible = true;
            }
            // set image green or grey
            hdSprite.getComponent('buttonBg', function (sprite) {
                sprite.setup({
                    imageName: getImageName()
                });
            });
        };

        if (!entity) {
            entity = new Entity({
                name: 'freeGiftContainer',
                position: new Vector2(0, 0),
                dimension: dimension,
                components: [
                    // hdSprite,
                    // {
                    //     draw({renderer}) {
                    //         renderer.fillCircle(Utils.color('ff0000'), 0, 0, 200);
                    //     }
                    // }
                ]
            });
        }

        Gui.makeButton(entity, {
            shadow: 'gui/button-' + ('small' ? 'small-square' : 'big') + '-shadow',
            canClick: function canClick() {
                return TimeManager.getTime() > scheduledTime;
            },
            onClick: function onClick() {
                collect(50);
            }
        });
        entity.attach(hdSprite);

        if (giftIcon) entity.attach(giftIcon);
        if (giftText) entity.attach(giftText);
        if (nextGiftText) entity.attach(nextGiftText);

        repaint();

        var collect = function collect(amount) {
            var oldCoins = SaveManager.load('coins');
            var newCoins = oldCoins + amount;
            var diff = newCoins - oldCoins;
            SaveManager.save('coins', newCoins);

            var claimedFreeGifts = SaveManager.load('claimedFreeGifts');
            SaveManager.save('claimedFreeGifts', claimedFreeGifts + 1);
            scheduledTime = TimeManager.getTime() + Globals.freeGiftTime(claimedFreeGifts + 1);
            SaveManager.save('scheduledFreeGift', scheduledTime);

            Notifications.resetBadgeNumber();

            if (SaveManager.load('notificationsAllowed') && SaveManager.load('notificationsFreeGift')) {
                EventSystem.fire('scheduleNotification', {
                    id: 'freeGiftReminder',
                    title: Localization.getText('notifications').freeGiftTitle,
                    body: Localization.getText('notifications').freeGiftBody,
                    time: scheduledTime - TimeManager.getTime()
                });
            }

            var originalScale = new Vector2(entity.scale.x, entity.scale.y);
            var iconScaleAmount = style === 'small' ? 0.5 : 1;
            var coinCollectIcon = new Entity({
                name: 'coinCollectIcon',
                position: entity.position.clone(),
                scale: new Vector2(0, 0),
                components: [new SimpleSprite({
                    imageName: 'gui/button-coin-c',
                    position: new Vector2(-16, 0),
                    scale: new Vector2(0.16, 0.16)
                }), new Text({
                    text: '+' + amount,
                    position: new Vector2(10, 0),
                    scale: new Vector2(0.7, 0.7),
                    fontSettings: TextStyles.get('main'),
                    align: 'left',
                    maxWidth: 64,
                    maxHeight: 32,
                    drawDebug: false
                })]
            });
            entity.parent.attach(coinCollectIcon);
            entity.parent.moveComponentTo(coinCollectIcon, 0);

            Tween({
                from: 1.0, to: 0.0, in: 20,
                ease: 'easeInBack',
                onUpdate: function onUpdate(v, t) {
                    entity.scale.x = originalScale.x * v;
                    entity.scale.y = originalScale.y * v;
                    coinCollectIcon.scale.x = originalScale.x * (1 - v) * iconScaleAmount;
                    coinCollectIcon.scale.y = originalScale.y * (1 - v) * iconScaleAmount;
                },
                onComplete: function onComplete() {
                    CoinCollectAnim({
                        from: entity.getWorldPosition().add(Vector2(0, 0)),
                        to: Gui.get('coinCounter').getWorldPosition(),
                        amount: 25,
                        interval: 3,
                        duration: 26,
                        onCoinCollect: function onCoinCollect(i) {
                            EventSystem.fire('updateCoins', Math.floor(oldCoins + diff * i / 25));
                        },
                        onComplete: function onComplete() {
                            repaint();
                            Tween({
                                delay: 30,
                                from: 0, to: 1.0, in: 20,
                                ease: 'easeOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    entity.scale.x = originalScale.x * v;
                                    entity.scale.y = originalScale.y * v;
                                    coinCollectIcon.scale.x = originalScale.x * (1 - v) * iconScaleAmount;
                                    coinCollectIcon.scale.y = originalScale.y * (1 - v) * iconScaleAmount;
                                },
                                onComplete: function onComplete() {
                                    coinCollectIcon.removeSelf();
                                }
                            });
                        }
                    });
                }
            });
        };

        return entity;
    };
});