'use strict';

bento.define('gui/guichallengevictory', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'components/sparkles', 'modules/tapticengine', 'modules/levelmanager', 'modules/admanager', 'modules/savemanager', 'modules/timemanager', 'gui/darkenbar', 'components/debug', 'components/simplesprite', 'gui/dialog', 'gui/coincollectanim', 'entities/gui/tutorialhand', 'entities/gui/gamecounter', 'gui/timerlabel', 'gui/freegift', 'modules/notifications', 'modules/localization', 'components/mask', 'modules/skinmanager', 'gui/characterunlockdialog', 'entities/lemming'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, Gui, GuiCommon, Sparkles, TapticEngine, LevelManager, AdManager, SaveManager, TimeManager, DarkenBar, Debug, SimpleSprite, Dialog, CoinCollectAnim, TutorialHand, GameCounter, TimerLabel, FreeGift, Notifications, Localization, Mask, SkinManager, CharacterUnlockDialog, Lemming) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiChallengeVictory = Utils.copyObject(GuiCommon);

    var clearTextScale = Utils.approach(Gui.scaleFactorTall, 1.0, 0.2);
    var canShowRewardedAds = false;
    var totalLevels = 0;
    var giveCharacter = false;

    Utils.extend(GuiChallengeVictory, {
        init: function init(settings) {
            canShowRewardedAds = settings.canShowRewardedAds;
            totalLevels = LevelManager.getTotalLevels(Utils.currentLevelGroup);
            giveCharacter = Utils.currentLevel + 1 === totalLevels;
        },
        root: function root(obj) {
            GuiCommon.root(obj);

            Utils.timeout(20, function () {
                Gui.show('coinsEarnedIcon');
            });
        },
        challengeTitle: function challengeTitle(obj) {
            Gui.adjust(obj, {
                y: 100 * (1 - Gui.scaleFactorTall) + 10 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor
            });
        },
        challengeTitleText: function challengeTitleText(obj) {
            makeText(obj, {
                text: Localization.getText('challengeTitle').replace('{NUMBER}', totalLevels).toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        victoryContainer: function victoryContainer(obj) {
            obj.removeSelf();
            Utils.timeout(1, function () {
                Gui.show('victoryContainer');
                var y = 100 * (1 - Gui.scaleFactorTall);
                y += 20 * Gui.scaleFactorTall;
                Gui.adjust(obj, { y: y });
            });
            obj.attach({
                name: 'behavior',
                start: function start() {
                    if (!giveCharacter) {
                        Gui.show('challengeWait');
                        Gui.show('buttonNext');
                    }
                    if (Globals.starsEarned < 3) {
                        Gui.show('buttonRetryWide');
                    }

                    var starsEarned = new StarsEarned({
                        // position: new Vector2(0, -94 + 60*(1-Gui.scaleFactor))
                        position: new Vector2(0, -128 + 60 * (1 - Gui.scaleFactorTall))
                    });
                    starsEarned.scale.scale(Gui.scaleFactorTall);
                    obj.attach(starsEarned);
                    obj.remove(this);

                    Bento.audio.playSound('sfx_clear');
                }
            });
        },
        clearContainer: function clearContainer(obj) {
            // contains the 'COMPLETED!' text,

            var darkenBar = new DarkenBar({
                position: new Vector2(0, 1),
                height: 40 * clearTextScale,
                time: 30,
                color: [0, 0.02, 0.51, 0.2]
            });
            obj.attach(darkenBar);
            obj.moveComponentTo(darkenBar, 0);

            var clearText = Gui.get('clearText');
            var clearTextStartPos = clearText && clearText.position.clone();

            obj.attach(new Sparkles({
                offset: new Vector2(0, 0),
                radius: new Vector2(70, 12),
                spawnRate: 1.0,
                speed: 0.8,
                scale: new Vector2(0.12, 0.12)
            }));
        },
        clearText: function clearText(obj) {
            makeText(obj, {
                text: Localization.getText('levelCompleted').toUpperCase(),
                fontSettings: TextStyles.get('heading'),
                align: 'center',
                linebreaks: false
                // drawDebug: true
            });
            obj.scale.x = obj.scale.y = 0;
            new Tween({
                delay: 8,
                from: 0, to: clearTextScale, in: 50,
                ease: 'elastic',
                decay: 9,
                oscillations: 3,
                applyOnDelay: true,
                onUpdate: function onUpdate(v, t) {
                    obj.scale.x = v;
                    obj.scale.y = v;
                }
            });
        },
        challengeWait: function challengeWait(obj) {
            var showPos = void 0,
                hidePos = void 0;
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        delay: 4,
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        onStart: function onStart() {
                            return parent.remove(obj);
                        }
                    });
                }
            });

            Gui.adjust(obj, {
                x: 0,
                y: 60 * (1 - Gui.scaleFactorTall) - 30 * (1 - Gui.scaleFactor),
                scale: Math.min(Gui.scaleFactorTall + 0.2, 1.0),
                onAdjust: function onAdjust() {
                    showPos = obj.position.clone();
                    hidePos = new Vector2(0, viewport.height + 20);
                    obj.attach(showable);
                }
            });
        },
        challengeWaitText: function challengeWaitText(obj) {
            makeText(obj, {
                text: Localization.getText('nextChallengeTime').toUpperCase(),
                fontSettings: TextStyles.get('dialogText'),
                align: 'center'
            });
        },
        challengeWaitTimer: function challengeWaitTimer(obj) {
            // not really a button, just looks like one

            var scheduledTime = SaveManager.load('scheduledChallenge');

            var timer = new TimerLabel({
                position: new Vector2(0, 0),
                scale: new Vector2(1.2, 1.2),
                useLocal: false,
                time: scheduledTime,
                onComplete: function onComplete() {
                    Gui.hide('challengeWait');
                    Gui.hide('buttonNext');
                    Gui.hide('buttonRetryWide');
                    Gui.show('buttonChallengePlay');
                }
            });
            var hoursRemaining = (scheduledTime - TimeManager.getTime()) / (60 * 60 * 1000);
            if (hoursRemaining < 10) {
                // slight horizontal adjustment
                timer.position.x -= 2.5 * timer.scale.x;
            }
            obj.attach(timer);
        },
        buttonNext: function buttonNext(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                onClick: function onClick() {
                    AdManager.tryShowRewarded({
                        onReward: function onReward() {
                            Utils.currentLevel = LevelManager.getNextUncleared(Utils.currentLevelGroup);
                            Bento.screens.show('screens/main');
                        },
                        onComplete: function onComplete(error) {
                            if (error) {
                                Bento.objects.attach(new Dialog({
                                    name: '7DayVictoryAdFailed',
                                    text: Localization.getText('noAd'),
                                    type: 'ok'
                                }));
                            }
                        },

                        tag: '7Day_Victory_Next_Level',
                        rewardType: '7day_extra_level',
                        rewardAmount: 1
                    });
                }
            });
            var showPos = void 0,
                hidePos = void 0;
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        onStart: function onStart() {
                            return parent.remove(obj);
                        }
                    });
                }
            });

            Gui.adjust(obj, {
                x: 0,
                y: 40 * (1 - Gui.scaleFactorTall) - 40 * (1 - Gui.scaleFactor),
                scale: Math.min(Gui.scaleFactorTall + 0.2, 1.0),
                onAdjust: function onAdjust() {
                    showPos = obj.position.clone();
                    hidePos = new Vector2(0, viewport.height + 20);
                    obj.attach(showable);
                }
            });
        },
        buttonNextText: function buttonNextText(obj) {
            makeText(obj, {
                text: Localization.getText('nextChallenge').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        buttonRetryWide: function buttonRetryWide(obj) {
            makeButton(obj, {
                canClick: function canClick() {
                    return showable.isShown;
                },
                onClick: function onClick() {
                    Globals.retryCount++;
                    Bento.screens.show('screens/main');
                }
            });
            var showPos = obj.position.clone();
            var hidePos = new Vector2(0, 40);
            var showable = Gui.FadeShowable({
                showTime: 80,
                hideTime: 0,
                isShown: false
            });
            obj.attach(showable);
            Utils.timeout(2, function () {
                obj.dimension.width += 70 * Gui.scaleFactorTall;
            });

            Gui.adjust(obj, {
                x: 22 * (1 - Gui.scaleFactorTall),
                y: 20 * (1 - Gui.scaleFactorTall) - 50 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactorTall
            });
        },
        buttonRetryWideText: function buttonRetryWideText(obj) {
            makeText(obj, {
                text: Localization.getText('retry').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'left',
                linebreaks: false
            });
            obj.transform.y -= 2;
        },
        buttonChallengePlay: function buttonChallengePlay(obj) {
            obj.removeSelf();
            makeButton(obj, {
                onClick: function onClick() {
                    Utils.currentLevel = LevelManager.getNextUncleared(Utils.currentLevelGroup);
                    Bento.screens.show('screens/main');
                }
            });
            obj.attach(new Wiggle({
                amplitude: 0.01
            }));
        },
        challengePlayText: function challengePlayText(obj) {
            makeText(obj, {
                text: Localization.getText('nextChallenge').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        victoryLevelName: function victoryLevelName(obj) {
            makeText(obj, {
                text: Localization.getText('levelN').replace('{NUMBER}', Utils.currentLevel + 1 + ' / ' + totalLevels).toUpperCase(),
                fontSettings: TextStyles.get('levelName'),
                align: 'center'
            });
            Utils.timeout(1, function () {
                Gui.adjust(obj, {
                    y: 20 * (1 - Gui.scaleFactorTall),
                    scale: clearTextScale
                });
            });
        },
        coinCounter: function coinCounter(obj) {
            obj.removeSelf();
            GuiCommon.coinCounter(obj, {
                value: Globals.oldCoins
            });
            obj.position.y -= 20 * (1 - Gui.scaleFactorTall);
            if (Gui.scaleFactorTall < 1.0) {
                obj.position.y -= 8;
            }
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(40, 0));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 30,
                        ease: 'easeOutQuart',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, t / 30, 1);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 20,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, 1 - t / 20, 1);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
            Utils.timeout(1, function () {
                Gui.show('coinCounter');
            });
        },
        starCounter: function starCounter(obj) {
            obj.removeSelf();
            GuiCommon.starCounter(obj);

            obj.position.y -= 14 * (1 - Gui.scaleFactorTall);
            if (Gui.scaleFactorTall < 1.0) {
                obj.position.y -= 8;
            }
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(40, 0));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 30,
                        ease: 'easeOutQuart',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, t / 30, 1);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 20,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, 1 - t / 20, 1);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
            Utils.timeout(1, function () {
                Gui.show('starCounter');
            });
        },
        coinsEarnedIcon: function coinsEarnedIcon(obj) {
            obj.removeSelf();

            var diff = SaveManager.load('coins') - Globals.oldCoins;

            // don't show if we didn't earny any new coins
            if (diff === 0) {
                obj.attach(new Gui.Showable({
                    show: function show() {
                        return false;
                    }
                }));
                return;
            }

            Gui.adjust(obj, {
                x: 18 * (1 - Gui.scaleFactor),
                y: -22 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor,
                onAdjust: function onAdjust() {}
            });

            obj.attach(new Text({
                text: '+' + diff,
                position: new Vector2(12, 0),
                scale: new Vector2(0.6, 0.6),
                fontSettings: TextStyles.get('main'),
                align: 'left',
                maxWidth: 50,
                maxHeight: 32
                // drawDebug: true
            }));

            var hdSprite = obj.getComponent('coinsEarnedHdSprite');
            var hdSpriteScale = hdSprite.scale.x;

            var shadowSprite = new HDSprite({
                name: 'shadow',
                imageName: 'gui/coin-shadow',
                gamepixelDimension: obj.dimension.clone(),
                originRelative: new Vector2(0.5, 0.5)
            });
            shadowSprite.position.x = 1;
            shadowSprite.position.y = 1;
            obj.attach(shadowSprite);
            obj.moveComponentTo(shadowSprite, 1);

            var initialDelay = 36;
            var amount = void 0; // how many coins to show in the animation
            var interval = void 0; // how much time between each coin spawn
            if (diff <= 5) {
                initialDelay += 20;
                amount = 5;
                interval = 6;
            } else if (diff <= 10) {
                initialDelay += 40;
                amount = 7;
                interval = 5.5;
            } else {
                initialDelay += 60;
                amount = 10;
                interval = 5;
            }

            var showable = new Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    Utils.timeout(20, function () {
                        var fromCoins = SaveManager.load('coins') - diff; // reload coins in case they changed in the meantime.
                        CoinCollectAnim({
                            from: obj.getWorldPosition(),
                            to: Gui.get('coinCounter').getWorldPosition(),
                            amount: amount,
                            interval: interval,
                            duration: 26,
                            onCoinSpawn: function onCoinSpawn(i) {
                                new Tween({
                                    from: 1.2, to: 1, in: 6,
                                    ease: 'easeOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        hdSprite.scale.x = hdSpriteScale * v;
                                        hdSprite.scale.y = hdSpriteScale * v;
                                        shadowSprite.scale.x = hdSpriteScale * v;
                                        shadowSprite.scale.y = hdSpriteScale * v;
                                    }
                                });
                            },
                            onCoinCollect: function onCoinCollect(i) {
                                EventSystem.fire('updateCoins', Math.floor(fromCoins + diff * i / amount));
                            }
                        });
                    });
                    return Tween({
                        from: 0, to: 1, in: 20,
                        delay: 0,
                        applyOnDelay: 0,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.transform.y = 16 * -(1 - v);
                            obj.alpha = v;
                        }
                    });
                },
                hide: function hide(parent) {
                    parent.remove(obj);
                }
            });
            obj.attach(showable);
        },
        characterUnlock: function characterUnlock(obj) {
            if (!giveCharacter) {
                obj.removeSelf();
                return;
            }

            var characterId = SkinManager.getCharacterByChallenge(Utils.currentLevelGroup);

            // immediately unlock and set as current character
            SkinManager.unlockCharacter(characterId);
            SaveManager.save('character', characterId);

            var titleText = new Text({
                text: Localization.getText('newCharacter').toUpperCase(),
                position: new Vector2(0, -30),
                scale: new Vector2(0.8, 0.8),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 180,
                maxHeight: 32
            });

            var characterDummy = new Entity({
                name: 'characterUnlockDummy',
                position: new Vector2(0, 24),
                scale: new Vector2(2.0, 2.0),
                components: [new Sprite(Lemming.getAnimations(characterId)['walk'])]
            });

            // animate in title text
            new Tween({
                delay: 30,
                from: 0, to: titleText.scale.x, in: 60,
                ease: 'elastic',
                decay: 9,
                oscillations: 3,
                onStart: function onStart() {
                    obj.attach(titleText);
                },
                onUpdate: function onUpdate(v, t) {
                    titleText.scale.x = v;
                    titleText.scale.y = v;
                }
            });

            // animate in character
            new Tween({
                delay: 42,
                from: 0, to: characterDummy.scale.x, in: 60,
                ease: 'elastic',
                decay: 9,
                oscillations: 3,
                onStart: function onStart() {
                    obj.attach(characterDummy);
                },
                onUpdate: function onUpdate(v, t) {
                    characterDummy.scale.x = v;
                    characterDummy.scale.y = v;
                }
            });

            Utils.timeout(50, function () {
                Bento.audio.playSound('sfx_victory_jingle');
            });

            Gui.adjust(obj, {
                y: 74 * (1 - Gui.scaleFactorTall) - 50 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactorTall,
                onAdjust: function onAdjust() {}
            });
        },
        tapToClose: function tapToClose(obj) {
            makeText(obj, {
                text: Localization.getText('tapToClose').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });
            obj.attach({
                name: 'behavior',
                start: function start() {},
                update: function update() {
                    obj.alpha = 0.6 + Math.sin(obj.timer * 0.06) * 0.3;
                }
            });
            Gui.adjust(obj, {
                x: 0 * (1 - Gui.scaleFactor),
                y: 40 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactor,
                onAdjust: function onAdjust() {}
            });
        }
    });

    return GuiChallengeVictory;
});