'use strict';

bento.define('gui/guicommon', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'modules/levelmanager', 'entities/gui/starsearned', 'entities/gridbg', 'gui/gui', 'gui/guicommon', 'gui/guinavbar', 'gui/coincounter', 'gui/starcounter', 'modules/localization', 'modules/crosspromo/crosspromo'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, LevelManager, StarsEarned, GridBg, Gui, GuiCommon, GuiNavbar, CoinCounter, StarCounter, Localization, CrossPromo) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    return {
        root: function root(obj) {
            obj.z = Globals.Layers.GUI;
            obj.float = true;
        },
        buttonHome: function buttonHome(obj) {
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });
            // Gui.adjust(obj, {y, scale: Gui.scaleFactor });

            // let y = -20 * (1-Gui.scaleFactorTall) - 10 * (1-Gui.scaleFactor); 
            // if (Gui.scaleFactorTall < 1.0) {
            //     y -= 8;  // there's no notch so we can move up a little
            // }
            // Gui.adjust(obj, {y, scale: Gui.scaleFactor });
        },
        buttonPlay: function buttonPlay(obj) {
            makeButton(obj, {
                shadow: true,
                onClick: function onClick() {
                    Utils.currentLevelGroup = 'default';
                    Utils.currentLevel = LevelManager.getNextUncleared();
                    Bento.screens.show('screens/main');
                }
            });
            obj.attach(new Text({
                text: Localization.getText('play').toUpperCase(),
                position: new Vector2(0, 0),
                scale: new Vector2(1, 1).scale(obj.dimension.width * 0.015),
                fontSettings: TextStyles.get('buttonLabel'),
                align: 'center',
                maxWidth: 64,
                maxHeight: 32
            }));
        },
        coinCounter: function coinCounter(obj) {
            var settings = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            settings.entity = obj;
            CoinCounter(settings);
            var y = -20 * (1 - Gui.scaleFactorTall) - 10 * (1 - Gui.scaleFactor);
            if (Gui.scaleFactorTall < 1.0) {
                y -= 8; // there's no notch so we can move up a little
            }
            Gui.adjust(obj, { y: y, scale: Gui.scaleFactor });
        },
        starCounter: function starCounter(obj) {
            var settings = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            settings.entity = obj;
            StarCounter(settings);
            var y = -14 * (1 - Gui.scaleFactorTall);
            if (Gui.scaleFactorTall < 1.0) {
                y -= 8;
            }
            Gui.adjust(obj, { y: y, scale: Gui.scaleFactor });
        },
        navbar: function navbar(obj) {

            Gui.load({
                parent: obj,
                assetName: CrossPromo.canShow() ? 'ui/navbar2x2' : 'ui/navbar4x1',
                handlers: GuiNavbar,
                onComplete: function onComplete() {
                    var showPos = obj.position.clone();
                    var hidePos = new Vector2(showPos.x, viewport.height + 20);
                    var showable = Gui.Showable({
                        isShown: true,
                        show: function show(parent) {
                            return Tween({
                                delay: 8,
                                from: hidePos, to: showPos, in: 36,
                                ease: 'easeOutQuad',
                                onStart: function onStart() {
                                    parent.attach(obj);
                                },
                                onUpdate: function onUpdate(v, t) {
                                    v.copyInto(obj.position);
                                    obj.alpha = Utils.clamp(0, t / 30, 1);
                                }
                            });
                        },
                        hide: function hide(parent) {
                            return Tween({
                                from: obj.position.clone(), to: hidePos, in: 30,
                                ease: 'easeInQuad',
                                onUpdate: function onUpdate(v, t) {
                                    v.copyInto(obj.position);
                                    obj.alpha = Utils.clamp(0, 1 - t / 30, 1);
                                },
                                onComplete: function onComplete() {
                                    parent.remove(obj);
                                }
                            });
                        }
                    });
                    obj.attach(showable);

                    var behavior = {
                        name: 'behavior',
                        center: function center() {
                            obj.position.x = 0;
                            showPos.x = 0;
                            hidePos.x = 0;
                        }
                    };
                    obj.attach(behavior);

                    if (CrossPromo.canShow()) {
                        Gui.adjust(obj, {
                            x: -20 * (1 - Gui.scaleFactor),
                            y: 40 * (1 - Gui.scaleFactor),
                            scale: Gui.scaleFactor
                        });
                    } else {
                        Gui.adjust(obj, {
                            y: 80 * (1 - Gui.scaleFactor),
                            onAdjust: function onAdjust() {
                                behavior.center();
                            }
                        });
                    }

                    // Utils.timeout(10, () => {
                    //     Gui.hide('navbar');
                    // });
                    // Utils.timeout(100, () => {
                    //     Gui.show('navbar');
                    // });
                }
            });
        }
    };
});