'use strict';

bento.define('gui/guiiapshop', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'bento/gui/scrollinglist', 'gui/gui', 'gui/guicommon', 'gui/darkenbar', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'entities/gui/tutorialhand', 'components/mask', 'modules/localization', 'modules/admanager', 'gui/dialog', 'gui/coindialog', 'gui/coincounter', 'gui/iapoverlay', 'gui/vippopup', 'modules/timemanager', 'modules/cloudsave', 'modules/vipmanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, ScrollingList, Gui, GuiCommon, DarkenBar, SimpleSprite, Debug, SaveManager, TutorialHand, Mask, Localization, AdManager, Dialog, CoinDialog, CoinCounter, IapOverlay, VipPopup, TimeManager, CloudSave, VipManager) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiShop = Utils.copyObject(GuiCommon);

    var fromScreen = void 0,
        showBackButton = void 0;

    Utils.extend(GuiShop, {
        init: function init(settings) {
            fromScreen = settings.fromScreen;

            console.log({ fromScreen: fromScreen });
            showBackButton = fromScreen === 'main' || fromScreen === 'victoryscreen' || fromScreen === 'shop';
        },
        buttonBack: function buttonBack(obj) {
            if (!showBackButton) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    if (fromScreen === 'main' || fromScreen === 'victoryscreen') {
                        Bento.screens.show('screens/victoryscreen', { restore: true });
                    } else {
                        Bento.screens.show('screens/' + fromScreen, { restore: true });
                    }
                }
            });
            var backText = Text({
                text: Localization.getText('back').toUpperCase(),
                position: new Vector2(0, 14),
                scale: new Vector2(0.45, 0.45),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 64,
                maxHeight: 18,
                drawDebug: false
            });
            obj.attach(backText);
        },
        buttonHome: function buttonHome(obj) {
            if (showBackButton) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });
        },
        iapContainer: function iapContainer(obj) {

            var dummy = new Entity({
                name: 'dummy'
            });
            var container = new Entity({
                name: 'container',
                dimensions: new Rectangle(0, 0, 100, 0)
            });

            var vSpacingReduction = 24 * Math.min(1 - Gui.scaleFactorTall, 0.2);

            var y = 56;

            if (!VipManager.isVip()) {
                y += 22;
                container.attach(new VipPanel({
                    position: new Vector2(0, y)
                }));
                y += 90 - vSpacingReduction;
            }

            container.attach(new FreeCoins({
                position: new Vector2(0, y)
            }));

            y += 84 - vSpacingReduction;

            container.attach(new IapItem({
                type: 'coin',
                productId: 'goslice_coins1',
                amount: 1000,
                dummyPrice: '$1.99',
                imageName: 'gui/icon-1000',
                isConsumable: true,
                iconScale: new Vector2(0.15, 0.15),
                iconOffset: new Vector2(0, 0),
                position: new Vector2(-40, y)
            }));
            container.attach(new IapItem({
                type: 'coin',
                productId: 'goslice_coins2',
                amount: 5000,
                dummyPrice: '$4.99',
                imageName: 'gui/icon-5000',
                isConsumable: true,
                iconScale: new Vector2(0.15, 0.15),
                iconOffset: new Vector2(0, 0),
                position: new Vector2(40, y)
            }));

            y += 107 - vSpacingReduction;
            container.attach(new IapItem({
                type: 'coin',
                productId: 'goslice_coins3',
                amount: 11000,
                dummyPrice: '$9.99',
                imageName: 'gui/icon-11000',
                isConsumable: true,
                iconScale: new Vector2(0.15, 0.15),
                iconOffset: new Vector2(0, 0),
                position: new Vector2(-40, y)
            }));
            container.attach(new IapItem({
                type: 'coin',
                productId: 'goslice_coins4',
                amount: 24000,
                dummyPrice: '$19.99',
                imageName: 'gui/icon-24000',
                isConsumable: true,
                iconScale: new Vector2(0.15, 0.15),
                iconOffset: new Vector2(0, 0),
                position: new Vector2(40, y)
            }));

            var bottom = y;
            container.dimension.height = 400;

            var area = new Rectangle(-viewport.width / 2, 20, viewport.width, viewport.height + 410 * (1 - Gui.scaleFactor));
            var grabArea = area.clone();
            grabArea.width += 100;
            grabArea.x -= 50;

            var list = new ScrollingList({
                z: Globals.Layers.GUI,
                position: new Vector2(0, 0),
                updateWhenPaused: 0,
                grabArea: grabArea, // area that user can grab
                direction: 'y', // 'x' or 'y'
                spacing: 0, // additional spacing between items
                maxOffset: bottom - viewport.height + 150, // additional max scrolling distance
                minOffset: 0, // additional min scrolling distance
                snap: false, // snap items (cancels maxOffset and minOffset)
                snapNextOffset: 0, onSnap: function onSnap(item, index) {},
                onLoseFocus: function onLoseFocus(index) {},
                onScrollStart: function onScrollStart() {},
                // callback when scrolling has started
                snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
                snapSpeed: 0.3, // speed of snapping
                damping: 0.95 // speed decrease,
            });
            list.addItem(dummy);
            list.addItem(container);
            Mask.applyToContainer(list, area);
            obj.attach(list);
            obj.area = area; // make area accessible to children

            Gui.adjust(obj, {
                y: -120 * (1 - Gui.scaleFactor) - 8 - (Gui.notch ? 0 : 12),
                scale: Utils.approach(Gui.scaleFactorTall, 1.0, 0.1)
            });
        }
    });

    var VipPanel = function VipPanel(settings) {
        var position = settings.position;


        var bg = new Entity({
            name: 'iapBg',
            position: new Vector2(0, 0),
            scale: new Vector2(0.155, 0.155),
            components: [new Sprite({
                imageName: 'gui/vip-box',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var vipTitle = new Text({
            // text: Localization.getText('vipAccess').toUpperCase(),  // TODO
            text: 'VIP',
            position: new Vector2(0, -48),
            scale: new Vector2(0.75, 0.75),
            fontSettings: TextStyles.get('vipTitle'),
            align: 'center',
            maxWidth: 82,
            maxHeight: 12
            // drawDebug: true
        });

        var makeIcon = function makeIcon(_ref) {
            var imageName = _ref.imageName,
                pos = _ref.pos,
                text = _ref.text,
                _ref$maxWidth = _ref.maxWidth,
                maxWidth = _ref$maxWidth === undefined ? 180 : _ref$maxWidth;

            return new Entity({
                name: 'noAdsIcon',
                position: pos,
                scale: new Vector2(0.155, 0.155),
                components: [new Sprite({
                    imageName: imageName,
                    originRelative: new Vector2(0.5, 0.5)
                }), new Text({
                    text: text,
                    position: new Vector2(0, 100),
                    scale: new Vector2(1.9, 1.9),
                    fontSettings: TextStyles.get('vipLabel'),
                    align: 'center',
                    maxWidth: maxWidth,
                    maxHeight: 40
                    // drawDebug: true
                })]
            });
        };

        var noAdsIcon = makeIcon({
            imageName: 'gui/no-ads',
            pos: new Vector2(-26, -20),
            text: Localization.getText('removeAds').toUpperCase()
        });
        var coinsIcon = makeIcon({
            imageName: 'gui/coins',
            pos: new Vector2(-26, 20),
            text: function () {
                var lang = Localization.getLanguage();
                var s = Localization.getText('nCoinsPerDay');
                if (lang === 'ja') {
                    s = s.replace('{NUMBER}', '\n100');
                } else {
                    s = s.replace('{NUMBER}', '100');
                }
                return s.toUpperCase();
            }(),
            maxWidth: 130
        });

        var vipCharsIcon = makeIcon({
            imageName: 'gui/characters',
            pos: new Vector2(26, -20),
            text: Localization.getText('vipCharacters').toUpperCase()
        });
        var hintsIcon = makeIcon({
            imageName: 'gui/hints',
            pos: new Vector2(26, 20),
            text: function () {
                var lang = Localization.getLanguage();
                var s = Localization.getText('nFreeHintsPerDay');
                if (lang === 'ja') {
                    s = s.replace('{NUMBER}', '2\n');
                } else {
                    s = s.replace('{NUMBER}', '2');
                }
                return s.toUpperCase();
            }(),
            maxWidth: 130
        });

        // let priceText = new Text({
        //     text: Localization.getText('freeTrial')
        //             .replace('{NUMBER}', '3')
        //             .replace('{CURRENCY}', IapOverlay.getPrice('goslice_vip', '$7.99'))
        //             .toUpperCase(),
        //     position: new Vector2(0, -37),
        //     scale: new Vector2(0.36, 0.36),
        //     fontSettings: TextStyles.get('vipLabel'),
        //     align: 'center',
        //     maxWidth: 200,
        //     maxHeight: 50,
        //     // drawDebug: true
        // });
        // noAdsIcon.scale.scale(0.85);
        // coinsIcon.scale.scale(0.85);
        // vipCharsIcon.scale.scale(0.85);
        // hintsIcon.scale.scale(0.85);

        // noAdsIcon.position.y += 13;
        // vipCharsIcon.position.y += 13;
        // coinsIcon.position.y += 6;
        // hintsIcon.position.y += 6;


        var entity = new Entity({
            name: 'vipPanel',
            position: position,
            dimension: new Rectangle(-60, -50, 120, 106),
            components: [bg, vipTitle, hintsIcon, coinsIcon, noAdsIcon, vipCharsIcon]
        });

        makeButton(entity, {
            canClick: function canClick(data) {
                var iapContainer = Gui.get('iapContainer');
                if (!iapContainer) return false;
                var area = iapContainer.area;
                var pos = iapContainer.toLocalPosition(data.worldPosition);
                return !Utils.didScroll(entity) && area.hasPosition(pos);
            },
            onClick: function onClick() {
                var dialog = new VipPopup({});
                Bento.objects.attach(dialog);
            }
        });

        return entity;
    };

    var FreeCoins = function FreeCoins(settings) {
        var position = settings.position;


        var bg = new Entity({
            name: 'iapBg',
            position: new Vector2(0, 0),
            scale: new Vector2(0.16, 0.16),
            components: [new Sprite({
                imageName: 'gui/coin-for-ads',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var rewardText = new Text({
            text: Localization.getText('freeNCoins').replace('{NUMBER}', '100').toUpperCase(),
            position: new Vector2(0, 0),
            scale: new Vector2(0.75, 0.75),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 64,
            maxHeight: 32,
            drawDebug: false
        });

        var entity = new Entity({
            name: 'freeCoins',
            position: position,
            dimension: new Rectangle(-72, -21, 144, 42),
            components: [bg, rewardText]
        });

        makeButton(entity, {
            canClick: function canClick(data) {
                var iapContainer = Gui.get('iapContainer');
                if (!iapContainer) return false;
                var area = iapContainer.area;
                var pos = iapContainer.toLocalPosition(data.worldPosition);
                return !Utils.didScroll(entity) && area.hasPosition(pos);
            },
            onClick: function onClick() {
                AdManager.tryShowRewarded({
                    onReward: function onReward() {
                        Bento.objects.attach(new CoinDialog({
                            amount: 100
                        }));
                    },
                    onComplete: function onComplete(error) {},

                    tag: 'Iap_Shop_Free_Coins',
                    rewardType: 'coins',
                    rewardAmount: 100
                });
            }
        });

        return entity;
    };

    var IapItem = function IapItem(settings) {
        var productId = settings.productId,
            type = settings.type,
            amount = settings.amount,
            dummyPrice = settings.dummyPrice,
            imageName = settings.imageName,
            _settings$iconScale = settings.iconScale,
            iconScale = _settings$iconScale === undefined ? new Vector2(1, 1) : _settings$iconScale,
            _settings$iconOffset = settings.iconOffset,
            iconOffset = _settings$iconOffset === undefined ? new Vector2(0, 0) : _settings$iconOffset,
            position = settings.position,
            _settings$isConsumabl = settings.isConsumable,
            isConsumable = _settings$isConsumabl === undefined ? true : _settings$isConsumabl;


        var price = IapOverlay.getPrice(productId, dummyPrice);

        var bg = new Entity({
            name: 'iapBg',
            position: new Vector2(0, 0),
            scale: new Vector2(0.156, 0.156),
            components: [new Sprite({
                imageName: 'gui/frame-iap',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var icon = new Entity({
            name: 'icon',
            position: iconOffset,
            scale: iconScale,
            components: [new Sprite({
                imageName: imageName,
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var rewardText = new Text({
            text: '' + amount,
            position: new Vector2(-4.5, -35),
            scale: new Vector2(0.42, 0.42),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 64,
            maxHeight: 32,
            drawDebug: false
        });

        var rewardIcon = void 0;
        if (type === 'coin') {
            rewardIcon = new Entity({
                name: 'rewardIcon',
                position: new Vector2(rewardText.position.x + 8 + Math.floor(Math.log10(amount)) * 3, rewardText.position.y),
                scale: new Vector2(0.08, 0.08),
                components: [new Sprite({
                    imageName: 'gui/coin-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });
        }

        var costText = new Text({
            text: price,
            position: new Vector2(0, 34),
            scale: new Vector2(0.52, 0.52),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 64,
            maxHeight: 32,
            drawDebug: false
        });

        var entity = new Entity({
            name: 'iapItem',
            position: position,
            dimension: new Rectangle(-35, -45, 70, 90),
            components: [bg, icon, rewardText, costText]
        });

        if (rewardIcon) {
            entity.attach(rewardIcon);
        }

        makeButton(entity, {
            canClick: function canClick(data) {
                var iapContainer = Gui.get('iapContainer');
                if (!iapContainer) return false;
                var area = iapContainer.area;
                var pos = iapContainer.toLocalPosition(data.worldPosition);
                return !Utils.didScroll(entity) && area.hasPosition(pos);
            },
            onClick: function onClick() {
                var onSuccess = function onSuccess() {
                    // coins are save immediately upon creating the dialog
                    Bento.objects.attach(new CoinDialog({
                        amount: amount,
                        imageName: imageName,
                        iconScale: iconScale,
                        iconOffset: iconOffset
                    }));
                };
                if (Utils.isDev()) {
                    onSuccess();
                    return;
                }
                IapOverlay.purchase({
                    z: Globals.Layers.IAP_OVERLAY,
                    id: productId,
                    isConsumable: isConsumable,
                    resolveAutomatically: true,
                    fontSettings: TextStyles.get('iapOverlay'),
                    onPurchaseComplete: function onPurchaseComplete(success, id, purchaseInfo) {
                        console.log('onPurchaseComplete', success, id, purchaseInfo);
                        if (success && id === productId) {
                            onSuccess();
                        }
                    }
                });
            }
        });

        return entity;
    };

    return GuiShop;
});