'use strict';

bento.define('gui/guilevelselect', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/savemanager', 'modules/levelmanager', 'modules/skinmanager', 'bento/gui/scrollinglist', 'gui/gui', 'gui/guicommon', 'components/mask', 'modules/localization', 'components/simplesprite', 'components/debug'], function (Bento, Vector2, Rectangle, Sprite, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, SaveManager, LevelManager, SkinManager, ScrollingList, Gui, GuiCommon, Mask, Localization, SimpleSprite, Debug) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var isTablet = function isTablet() {
        return Gui.scaleFactor <= 0.75;
    };
    var viewport = Bento.getViewport();

    var GuiLevelSelect = Utils.copyObject(GuiCommon);

    Utils.extend(GuiLevelSelect, {
        worldList: function worldList(obj) {

            var worldListScale = Gui.scaleFactor * (isTablet() ? 0.9 : 1.0);

            Gui.adjust(obj, {
                // x: 0 * (1-Gui.scaleFactor),
                // y: 0 * (1-Gui.scaleFactor),
                y: Gui.notch ? 0 : -20 * (1 - worldListScale),
                scale: worldListScale,
                onAdjust: function onAdjust() {
                    console.log(Gui.scaleFactor);
                }
            });

            // Worlds scrolling list:

            var columns = isTablet() ? 2 : 1;
            var xSpacing = (viewport.width - 30) / columns + (isTablet() ? 35 : 0);
            var ySpacing = 132;

            var dummy = new Entity({
                name: 'dummy'
            });
            var container = new Entity({
                name: 'container',
                dimensions: new Rectangle(0, 0, 100, 500)
            });

            var x = void 0,
                y = void 0,
                tile = void 0;

            var initialScrollY = 0;

            var numWorlds = LevelManager.getNumWorlds();
            var currentLevel = Utils.currentLevel;
            var lastUnlockedWorldY = 0;
            for (var i = 0; i < numWorlds; i++) {
                x = i % columns - (columns / 2 - 0.5);
                y = Math.floor(i / columns);
                var worldInfo = LevelManager.getWorldInfo(i);
                container.attach(new WorldTile({
                    worldInfo: worldInfo,
                    position: new Vector2(x * xSpacing, y * ySpacing + 80)
                }));
                if (LevelManager.getTotalStars() >= worldInfo.starsToUnlock) {
                    lastUnlockedWorldY = y;
                }
                // if (currentLevel >= worldInfo.fromLevel && currentLevel < worldInfo.toLevel) {
                //     // initialScrollY = y * ySpacing - viewport.height/2;
                //     initialScrollY = y * ySpacing;
                // }
            }
            initialScrollY = lastUnlockedWorldY * ySpacing - 80 * Gui.scaleFactor;

            var bottom = y * ySpacing + 80;
            container.dimension.height = ySpacing;

            var area = new Rectangle(-viewport.width * 0.65, 0, viewport.width * 1.3, viewport.height / Gui.scaleFactor);
            var list = new ScrollingList({
                z: Globals.Layers.GUI,
                position: new Vector2(0, 0),
                updateWhenPaused: 0,
                area: area, // area that user can grab
                direction: 'y', // 'x' or 'y'
                spacing: 0, // additional spacing between items
                maxOffset: bottom - viewport.height + ySpacing + 50, // additional max scrolling distance
                minOffset: 0, // additional min scrolling distance
                snap: false, // snap items (cancels maxOffset and minOffset)
                snapNextOffset: 0, // distance to scroll to snap to next, in screen pixels. Used with snap:true, best with damping:0
                onSnap: function onSnap(item, index) {}, // called when snap focus changed
                onLoseFocus: function onLoseFocus(index) {}, // called when snap focus changed
                onScrollStart: function onScrollStart() {}, // callback when scrolling has started
                snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
                snapSpeed: 0.3, // speed of snapping
                damping: 0.95 // speed decrease,
            });
            list.area = area;
            list.addItem(dummy);
            list.addItem(container);
            list.attach({
                name: 'behavior',
                started: false,
                start: function start() {
                    if (!this.started) {
                        this.started = true;
                        Mask.applyToContainer(list, area);
                        list.forceTo(-initialScrollY);
                    }
                },
                update: function update() {
                    var scrollY = list.position.y - list.getScroll();
                    var minY = scrollY - ySpacing;
                    var maxY = scrollY + viewport.height + ySpacing;
                    container.components.forEach(function (item) {
                        var itemY = item.position.y;
                        item.visible = itemY > minY && itemY < maxY;
                        item.active = item.visible;
                    });
                }
            });

            obj.attach(list);
        },
        levelList: function levelList(obj) {
            // to be populated once a level is pressed.
            obj.removeSelf();
            Gui.adjust(obj, {
                y: Gui.notch ? 0 : -50 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactor
            });
        },
        buttonBack: function buttonBack(obj) {
            obj.removeSelf();
            makeButton(obj, {
                onClick: function onClick() {
                    Gui.hide('buttonBack');
                    Gui.hide('levelList');
                    Gui.show('buttonHome');
                    Gui.show('worldList');
                }
            });
        }
    });

    var WorldTile = function WorldTile(settings) {
        var worldNumber = settings.worldNumber,
            worldInfo = settings.worldInfo,
            position = settings.position;
        var fromLevel = worldInfo.fromLevel,
            toLevel = worldInfo.toLevel,
            starsToUnlock = worldInfo.starsToUnlock,
            starsEarned = worldInfo.starsEarned;


        var totalStars = LevelManager.getTotalStars(); // how many stars does the player have overall
        var levelDiff = toLevel - fromLevel; // how many levels are in this world
        var isLocked = totalStars < starsToUnlock; // do we have enough stars overall to play the world
        var hasAllStars = starsEarned === levelDiff * 3; // has the player gotten every star in this world 

        var worldTileBgShadow = new Sprite({
            imageName: 'gui/levelselect/world-select-shadow',
            originRelative: new Vector2(0.5, 0.5)
        });
        worldTileBgShadow.origin.x -= 9;
        worldTileBgShadow.origin.y -= 9;
        var worldTileBg = new Entity({
            name: 'worldTileBg',
            position: new Vector2(0, 0),
            scale: new Vector2(0.15, 0.15),
            components: [worldTileBgShadow, new Sprite({
                imageName: 'gui/levelselect/world-select',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var worldRangeText = new Text({
            text: fromLevel + 1 + '-' + toLevel,
            position: new Vector2(0, -50),
            scale: new Vector2(0.6, 0.6),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 92,
            maxHeight: 32
            // drawDebug: true,
        });

        var thumbLevel = Utils.clamp(fromLevel, fromLevel + 3, toLevel - 1);
        var thumb = new Entity({
            name: 'thumb',
            position: new Vector2(0, 0),
            // scale: new Vector2(0.32, 0.28).scale(1.3),
            scale: new Vector2(0.28, 0.28).scale(1.3),
            components: [new Sprite({
                imageName: 'thumbs/' + LevelManager.getName(thumbLevel),
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var starIcon = new Entity({
            name: 'starIcon',
            scale: new Vector2(0.09, 0.09),
            components: [new Sprite({
                imageName: 'gui/star-icon',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var entity = new Entity({
            name: 'worldTile',
            position: position,
            dimension: new Rectangle(-50, -40, 100, 100),
            components: [worldTileBg, worldRangeText, thumb, starIcon]
        });

        makeButton(entity, {
            depress: false,
            canClick: function canClick(data) {
                return (!isLocked || Utils.isDev()) && !Utils.didScrollOrMissedArea(entity, data);
            },
            onClick: function onClick() {
                Gui.get('levelList', function (levelList) {
                    levelList.removeByName('levelList');
                    levelList.attach(new LevelList({
                        fromLevel: fromLevel,
                        toLevel: toLevel
                    }));
                });
                Gui.hide('buttonHome');
                Gui.hide('worldList');
                Gui.show('buttonBack');
                Gui.show('levelList');
            }
        });

        if (isLocked) {
            var tileDarken = new Entity({
                name: 'worldTileDarken',
                scale: worldTileBg.scale.clone(),
                alpha: 0.6,
                components: [new Sprite({
                    imageName: 'gui/levelselect/world-select-darken',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });
            entity.attach(tileDarken);
            entity.moveComponentTo(tileDarken, starIcon.rootIndex);

            starIcon.position = new Vector2(-12, 30);
            var starsText = new Text({
                text: starsToUnlock,
                position: new Vector2(2, 30),
                scale: new Vector2(0.6, 0.6),
                fontSettings: TextStyles.get('mainWhite'),
                align: 'left',
                maxWidth: 200,
                maxHeight: 32
                // drawDebug: true
            });
            entity.attach(starsText);

            var subText = new Text({
                text: 'TO UNLOCK',
                position: new Vector2(0, 46),
                scale: new Vector2(0.4, 0.4),
                fontSettings: TextStyles.get('mainWhite'),
                align: 'center',
                maxWidth: 200,
                maxHeight: 32
                // drawDebug: true
            });
            entity.attach(subText);

            entity.attach(new Entity({
                name: 'lockIcon',
                position: new Vector2(0, -4),
                scale: new Vector2(0.14, 0.14),
                components: [new Sprite({
                    imageName: 'gui/lock-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            }));
        } else {
            starIcon.position = new Vector2(-17, 43);
            var _starsText = new Text({
                text: Utils.zeroPad(starsEarned, 2) + '/' + levelDiff * 3,
                position: new Vector2(10, 43),
                scale: new Vector2(0.54, 0.54),
                fontSettings: TextStyles.get(hasAllStars ? 'mainWhite' : 'main'),
                align: 'center',
                maxWidth: 200,
                maxHeight: 32
                // drawDebug: true
            });
            entity.attach(_starsText);

            if (hasAllStars) {
                entity.attach(new Entity({
                    name: 'medalCompleted',
                    position: new Vector2(36, -32),
                    scale: new Vector2(0.16, 0.16),
                    components: [new Sprite({
                        imageName: 'gui/levelselect/medal-completed',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                }));
            }
        }

        return entity;
    };

    /**
     * A single level tile, with thumbnail and stars.
     */
    var LevelTile = function LevelTile(settings) {
        var index = settings.index,
            level = settings.level,
            position = settings.position,
            maskArea = settings.maskArea;


        var starsEarned = LevelManager.getStars(index);
        var isLocked = LevelManager.isLocked(index);

        var tileBgShadowSprite = new Sprite({
            imageName: 'gui/levelselect/level-select-shadow',
            originRelative: new Vector2(0.5, 0.5)
        });
        tileBgShadowSprite.origin.x -= 8;
        tileBgShadowSprite.origin.y -= 8;
        var tileBg = new Entity({
            name: 'levelTileBg',
            position: new Vector2(0, 0),
            scale: new Vector2(0.16, 0.16),
            components: [tileBgShadowSprite, new Sprite({
                imageName: 'gui/levelselect/level-select',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });
        var badgeBg = new Entity({
            name: 'badgeBg',
            position: new Vector2(-18, -30),
            scale: new Vector2(0.15, 0.17),
            components: [new Sprite({
                imageName: 'gui/levelselect/level-badge',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var str = index + 1 + '';
        var text = new Text({
            text: str,
            position: new Vector2(-18, -30.5),
            scale: new Vector2(0.5, 0.5),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 48,
            maxHeight: 32
            // drawDebug: true
        });

        var thumb = new Entity({
            name: 'thumb',
            position: new Vector2(0, -8),
            scale: new Vector2(0.28, 0.28),
            components: [new Sprite({
                imageName: 'thumbs/' + level.name,
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var behavior = {
            name: 'behavior',
            start: function start() {
                Gui.makeButton(entity, {
                    depress: false,
                    canClick: function canClick(data) {
                        return (!isLocked || Utils.isDev()) && !Utils.didScrollOrMissedArea(entity, data);
                    },
                    onClick: function onClick() {
                        Utils.currentLevelGroup = 'default';
                        Utils.currentLevel = index;
                        Bento.screens.show('screens/main');
                    }
                });
            }
        };

        var entity = new Entity({
            name: 'levelTile-' + level,
            position: position,
            dimension: new Rectangle(-28, -35, 56, 70),
            components: [tileBg, thumb, badgeBg, behavior, text]
        });

        for (var i = 0; i < 3; i++) {
            var imageName = void 0,
                alpha = void 0,
                scale = void 0;
            if (i + 1 <= starsEarned) {
                imageName = 'gui/star-icon';
                alpha = 1.0;
                scale = 0.072;
            } else {
                imageName = 'gui/levelselect/star-icon-empty';
                alpha = 0.75;
                scale = 0.07;
            }
            entity.attach(new Entity({
                name: 'star',
                position: new Vector2((i - 1) * 15.5, 24),
                scale: new Vector2(scale, scale),
                alpha: alpha,
                components: [new Sprite({
                    imageName: imageName,
                    originRelative: new Vector2(0.5, 0.5)
                })]
            }));
        }

        if (isLocked) {
            var tileDarken = new Entity({
                name: 'levelTileDarken',
                scale: tileBg.scale.clone(),
                alpha: 0.6,
                components: [new Sprite({
                    imageName: 'gui/levelselect/level-select-darken',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });
            entity.attach(tileDarken);
            entity.moveComponentTo(tileDarken, badgeBg.rootIndex);

            entity.attach(new Entity({
                name: 'lockIcon',
                position: new Vector2(0, -8),
                scale: new Vector2(0.12, 0.12),
                components: [new Sprite({
                    imageName: 'gui/lock-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            }));
        }

        var bossNumber = LevelManager.getBossNumber(index);
        var bossCharacterId = SkinManager.getCharacterIdByBossNumber(bossNumber);
        if (!isLocked && bossCharacterId && !SaveManager.load('hasChar-' + bossCharacterId)) {
            entity.attach(new Entity({
                name: 'clearedBadge',
                position: new Vector2(23, -30),
                scale: new Vector2(0.10, 0.10),
                components: [new Sprite({
                    imageName: 'gui/notification-badge',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            }));
        } else if (starsEarned >= 1) {
            entity.attach(new Entity({
                name: 'clearedBadge',
                position: new Vector2(24, -30),
                scale: new Vector2(0.14, 0.14),
                components: [new Sprite({
                    imageName: 'gui/cage/level-done',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            }));
        }

        return entity;
    };

    var LevelList = function LevelList(settings) {
        var fromLevel = settings.fromLevel,
            toLevel = settings.toLevel;


        var starTotal = 0;
        var clearTotal = 0;
        var levelDiff = toLevel - fromLevel;

        for (var i = fromLevel; i < toLevel; i++) {
            var stars = LevelManager.getStars(i);
            starTotal += stars;
            clearTotal += stars > 0 ? 1 : 0; // don't count skips?
        }

        var levelsText = new Text({
            text: Localization.getText('levelsRange').replace('{A}', fromLevel + 1).replace('{B}', toLevel).toUpperCase(),
            position: new Vector2(0, 0),
            scale: new Vector2(0.7, 0.7),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 200,
            maxHeight: 32
            // drawDebug: true
        });

        var starIcon = new SimpleSprite({
            imageName: 'gui/star-icon',
            position: new Vector2(-18, 20),
            scale: new Vector2(0.08, 0.08)
        });
        var starsText = new Text({
            text: Utils.zeroPad(starTotal, 2) + '/' + levelDiff * 3,
            position: new Vector2(11, 20),
            scale: new Vector2(0.52, 0.52),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 200,
            maxHeight: 32
            // drawDebug: true
        });

        var shadowSprite = new Sprite({
            imageName: 'gui/levelselect/progression-shadow',
            originRelative: new Vector2(0.5, 0.5)
        });
        shadowSprite.origin.x -= 8;
        shadowSprite.origin.y -= 8;
        var emptySprite = new Sprite({
            imageName: 'gui/levelselect/progression00',
            originRelative: new Vector2(0.5, 0.5)
        });
        var fullSprite = new Sprite({
            imageName: 'gui/levelselect/progression10',
            originRelative: new Vector2(0.5, 0.5)
        });
        var progressText = new Text({
            text: Utils.zeroPad(clearTotal, 2) + '/' + levelDiff,
            scale: new Vector2(2.5, 2.5),
            fontSettings: TextStyles.get('main'),
            align: 'center',
            maxWidth: 200,
            maxHeight: 32
            // drawDebug: true
        });
        var progressBarContents = new Entity({
            name: 'progressBarContents',
            components: [shadowSprite, emptySprite, fullSprite, progressText]
        });
        var maskRect = progressBarContents.dimension.clone();
        maskRect.width *= clearTotal / levelDiff;
        progressBarContents.attach(new Mask({
            rectangle: maskRect,
            sprite: fullSprite
        }));
        var progressBar = new Entity({
            name: 'progressBar',
            position: new Vector2(0, 42),
            scale: new Vector2(0.162, 0.162),
            components: [progressBarContents]
        });

        // Create the scrolling list
        var columns = isTablet() ? 3 : 2;
        var xSpacing = (viewport.width - 30) / columns + (isTablet() ? 14 : 0);
        var ySpacing = 82;

        var dummy = new Entity({
            name: 'dummy'
        });
        var container = new Entity({
            name: 'container',
            dimensions: new Rectangle(0, 0, 100, 500)
        });

        var x = void 0,
            y = void 0,
            tile = void 0;

        for (var _i = fromLevel; _i < toLevel; _i++) {
            var level = LevelManager.getInfo(_i);
            var j = _i - fromLevel;
            x = j % columns - (columns / 2 - 0.5);
            y = Math.floor(j / columns);
            tile = new LevelTile({
                level: level,
                index: _i,
                position: new Vector2(x * xSpacing, y * ySpacing + 50)
            });
            container.attach(tile);
        }

        var bottom = y * ySpacing + 100;
        container.dimension.height = ySpacing;

        var area = new Rectangle(-viewport.width * 0.6, 0, viewport.width * 1.2, viewport.height);
        var list = new ScrollingList({
            z: Globals.Layers.GUI,
            position: new Vector2(0, 0),
            updateWhenPaused: 0,
            area: area, // area that user can grab
            direction: 'y', // 'x' or 'y'
            spacing: 0, // additional spacing between items
            maxOffset: bottom - viewport.height + ySpacing + 50, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: false, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 0, // distance to scroll to snap to next, in screen pixels. Used with snap:true, best with damping:0
            onSnap: function onSnap(item, index) {}, // called when snap focus changed
            onLoseFocus: function onLoseFocus(index) {}, // called when snap focus changed
            onScrollStart: function onScrollStart() {}, // callback when scrolling has started
            snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
            snapSpeed: 0.3, // speed of snapping
            damping: 0.95 // speed decrease,
        });
        list.area = area;
        list.addItem(dummy);
        list.addItem(container);
        list.attach({
            name: 'behavior',
            start: function start() {
                Mask.applyToContainer(list, area);
            },
            update: function update() {
                var scrollY = list.position.y - list.getScroll();
                var minY = scrollY - ySpacing;
                var maxY = scrollY + viewport.height + ySpacing / 2;
                container.components.forEach(function (item) {
                    var itemY = item.position.y;
                    item.visible = itemY > minY && itemY < maxY;
                    item.active = item.visible;
                });
            }
        });

        var listContainer = new Entity({
            name: 'listContainer',
            position: new Vector2(0, 60),
            components: [list]
        });

        return new Entity({
            name: 'levelList',
            position: new Vector2(0, 0),
            components: [levelsText, starIcon, starsText, progressBar, listContainer]
        });
    };

    return GuiLevelSelect;
});