'use strict';

bento.define('gui/guimain', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'entities/gui/slicecounter', 'gui/gui', 'gui/guicommon', 'components/sparkles', 'modules/tapticengine', 'modules/levelmanager', 'modules/admanager', 'modules/savemanager', 'gui/darkenbar', 'modules/localization', 'components/debug'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, SliceCounter, Gui, GuiCommon, Sparkles, TapticEngine, LevelManager, AdManager, SaveManager, DarkenBar, Localization, Debug) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiMain = Utils.copyObject(GuiCommon);

    // how fast do the top and bottom gameplay UI elements slide in
    var anchorSpeed = 16;

    Utils.extend(GuiMain, {
        anchorTopMain: function anchorTopMain(obj) {
            var shownY = obj.position.y;
            var hiddenY = shownY - 60;
            if (Utils.isTablet()) {
                shownY -= 5;
            }

            var isShown = true;
            obj.position.y = isShown ? shownY : hiddenY;

            var showable = Gui.Showable({
                isShown: isShown,
                show: function show(parent) {
                    return new Tween({
                        from: obj.position.y, to: shownY, in: anchorSpeed,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.position.y = v;
                            obj.alpha = t / anchorSpeed;
                        }
                    });
                },
                hide: function hide(parent) {
                    return new Tween({
                        from: obj.position.y, to: hiddenY, in: anchorSpeed,
                        ease: 'easeOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            obj.position.y = v;
                            obj.alpha = t / anchorSpeed;
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        anchorBottomMain: function anchorBottomMain(obj) {
            var shownY = obj.position.y;
            var hiddenY = shownY + 60;

            var isShown = true;
            obj.position.y = isShown ? shownY : hiddenY;

            var showable = Gui.Showable({
                isShown: isShown,
                show: function show(parent) {
                    return new Tween({
                        from: obj.position.y, to: shownY, in: anchorSpeed,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.position.y = v;
                            obj.alpha = t / anchorSpeed;
                        }
                    });
                },
                hide: function hide(parent) {
                    return new Tween({
                        from: obj.position.y, to: hiddenY, in: anchorSpeed,
                        ease: 'easeOutQuad',
                        onUpdate: function onUpdate(v, t) {
                            obj.position.y = v;
                            obj.alpha = t / anchorSpeed;
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        buttonSkip: function buttonSkip(obj) {
            var isShown = !LevelManager.hasCleared(Utils.currentLevel);
            if (!isShown) {
                obj.removeSelf();
            }
            makeButton(obj, {
                shadow: 'gui/button-hintskip-shadow',
                onClick: function onClick() {
                    AdManager.tryShowRewarded({
                        onReward: function onReward() {
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'game:skip:rewardSuccess',
                                value: Utils.currentLevel + 1
                            });
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'game:skip:level:' + (Utils.currentLevel + 1)
                            });
                            EventSystem.fire('win', {
                                skipped: true
                            });
                        },
                        onComplete: function onComplete(error) {
                            if (error) {
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'game:skip:rewardFail',
                                    value: Utils.currentLevel + 1
                                });
                            }
                        },

                        tag: 'Skip_Level',
                        rewardType: 'skip_level',
                        rewardAmount: 1
                    });
                }
            });
        },
        buttonSkipText: function buttonSkipText(obj) {
            makeText(obj, {
                text: Localization.getText('skip').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                linebreaks: false
            });
        },
        buttonRetry: function buttonRetry(obj) {
            var shown = false;
            var shownY = obj.position.y;
            var hiddenY = -40;
            // obj.position.y = hiddenY;
            makeButton(obj, {
                onClick: function onClick() {
                    Globals.retryCount++;
                    Bento.screens.show('screens/main');
                    LevelManager.levelRetryAnalytics(Utils.currentLevel);
                }
            });
            // let showable = Gui.Showable({
            //     show (parent) {
            //         return new Tween({
            //             from: obj.position.y, to: shownY, in: 20,
            //             ease: 'easeOutQuad',
            //             onStart () {
            //                 parent.attach(obj);
            //             },
            //             onUpdate (v, t) {
            //                 obj.position.y = v;
            //             }
            //         });
            //     },
            //     hide (parent) {
            //         return new Tween({
            //             from: obj.position.y, to: hiddenY, in: 20,
            //             ease: 'easeInQuad',
            //             onUpdate (v, t) {
            //                 obj.position.y = v;
            //             },
            //             onComplete () {
            //                 parent.remove(obj);
            //             }
            //         });
            //     }
            // });
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return new Tween({
                        from: 0, to: 1, in: 8,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        }
                    });
                },
                hide: function hide(parent) {
                    return new Tween({
                        from: 1, to: 0, in: 8,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        buttonRetryWide: function buttonRetryWide(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                canClick: function canClick() {
                    return showable.isShown;
                },
                onClick: function onClick() {
                    Globals.retryCount++;
                    Bento.screens.show('screens/main');
                    LevelManager.levelRetryAnalytics(Utils.currentLevel);
                }
            });
            var showPos = obj.position.clone();
            var hidePos = new Vector2(0, 40);
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
            obj.attach(new Wiggle({
                amplitude: 0.006
            }));
        },
        buttonRetryWideText: function buttonRetryWideText(obj) {
            makeText(obj, {
                text: Localization.getText('retry').toLowerCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
            obj.transform.y -= 2;
        },
        buttonHintContainer: function buttonHintContainer(obj) {
            var isShown = Globals.hintsRevealed < Globals.hintsAvailable;
            obj.attach(new Gui.FadeShowable({
                isShown: isShown,
                showTime: 8,
                fadeTime: 8
            }));
        },
        buttonHint: function buttonHint(obj) {
            makeButton(obj, {
                shadow: 'gui/button-hintskip-shadow',
                canClick: function canClick() {
                    return Gui.isShown('buttonHintContainer');
                },
                onClick: function onClick() {
                    var showHint = function showHint() {
                        // Globals.hintsRevealed++;
                        Globals.hintsRevealed = Globals.hintsAvailable; // reveal all hints
                        if (Globals.hintsRevealed >= Globals.hintsAvailable) {
                            Gui.hide('buttonHintContainer');
                        }
                    };
                    var hints = SaveManager.load('hints');
                    if (hints > 0) {
                        SaveManager.save('hints', --hints);
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'game:hint:spendHint',
                            value: Utils.currentLevel + 1
                        });
                        showHint();
                        // redraw badge
                        Gui.get('hintAdBadge', GuiMain.hintAdBadge);
                    } else {
                        AdManager.tryShowRewarded({
                            onReward: function onReward() {
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'game:hint:rewardSuccess',
                                    value: Utils.currentLevel + 1
                                });
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'game:hint:level:' + (Utils.currentLevel + 1)
                                });
                                showHint();
                            },
                            onComplete: function onComplete(error) {
                                if (error) {
                                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                                        eventId: 'game:hint:rewardFail',
                                        value: Utils.currentLevel + 1
                                    });
                                }
                            },

                            tag: 'Get_Hint',
                            rewardType: 'hint',
                            rewardAmount: 1
                        });
                    }
                }
            });
            if (Globals.retryCount >= 1) {
                obj.attach(new Wiggle({}));
            }
        },
        hintText: function hintText(obj) {
            makeText(obj, {
                text: Localization.getText('hint').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                linebreaks: false
            });
        },
        hintAdBadge: function hintAdBadge(obj) {
            obj.removeByName('text');
            obj.getComponent('hdsprite16', function (hdSprite) {
                hdSprite.getComponent('hdsprite16', function (sprite) {
                    var hints = SaveManager.load('hints');
                    if (hints === 0) {
                        // show ad badge
                        sprite.setup({
                            imageName: 'gui/ad-badge'
                        });
                    } else {
                        // show numerical badge (how many free hints are remaining)
                        sprite.setup({
                            imageName: 'gui/hint-count-badge'
                        });
                        var text = new Text({
                            text: hints + '',
                            position: new Vector2(-0.05, -0.4),
                            scale: new Vector2(0.35, 0.35),
                            fontSettings: TextStyles.get('main'),
                            align: 'center',
                            maxWidth: 64,
                            maxHeight: 32,
                            drawDebug: false
                        });
                        obj.attach(text);
                    }
                });
            });
        },
        buttonFastForward: function buttonFastForward(obj) {
            var active = false;
            var clickable = true;

            var fastForwardRing = new window.PIXI.Graphics();
            fastForwardRing.lineStyle(1, 0xffffff);
            fastForwardRing.drawCircle(0, 0, obj.dimension.width / 2);

            makeButton(obj, {
                onClick: function onClick() {
                    if (!clickable) return;
                    if (active) {
                        active = false;
                        Bento.setGameSpeed(1);
                    } else {
                        active = true;
                        Bento.setGameSpeed(3);
                    }
                }
            });
            obj.attach({
                name: 'behavior',
                draw: function draw(_ref) {
                    var renderer = _ref.renderer;

                    if (active) {
                        fastForwardRing.alpha = 0.6 + Math.sin(obj.ticker * 0.1) * 0.4;
                    } else {
                        fastForwardRing.alpha = 1.0;
                    }
                    renderer.drawPixi(fastForwardRing);
                },
                destroy: function destroy() {
                    Bento.setGameSpeed(1);
                }
            });
            obj.attach(new EventListener({
                eventName: 'win',
                onEvent: function onEvent() {
                    Bento.setGameSpeed(1);
                    active = false;
                    clickable = false;
                    Tween({
                        from: obj.position.y,
                        to: obj.position.y + 40,
                        in: 26,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            obj.position.y = v;
                        },
                        onComplete: function onComplete() {
                            obj.removeSelf();
                        }
                    });
                }
            }));
        },
        buttonHome: function buttonHome(obj) {

            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });

            obj.dimension.grow(-12);

            var showable = Gui.Showable({
                show: function show(parent) {
                    return new Tween({
                        from: 0, to: 1, in: 8,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        }
                    });
                },
                hide: function hide(parent) {
                    return new Tween({
                        from: 1, to: 0, in: 8,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        sliceCounter: function sliceCounter(obj) {
            var sliceCounter = new SliceCounter({
                position: obj.position.clone()
            });
            obj.parent.attach(sliceCounter);
            obj.removeSelf();
        },
        levelName: function levelName(obj) {

            var group = LevelManager.getGroupInfo(Utils.currentLevelGroup);

            var levelNumber = Utils.currentLevel + 1 + '';
            if (group.isChallenge) {
                levelNumber += ' / ' + group.levels.length;
            }

            makeText(obj, {
                text: Localization.getText('levelN').replace('{NUMBER}', levelNumber).toUpperCase(),
                fontSettings: TextStyles.get('levelName'),
                align: 'center'
            });
        }
    });

    return GuiMain;
});