'use strict';

bento.define('gui/guinavbar', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/eventlistener', 'components/wiggle', 'components/simplesprite', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'gui/gui', 'gui/guicommon', 'components/sparkles', 'modules/tapticengine', 'modules/levelmanager', 'modules/savemanager', 'modules/vipmanager', 'components/debug', 'gui/dialog', 'modules/localization', 'gui/iapoverlay', 'modules/skinmanager', 'modules/admanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, EventListener, Wiggle, SimpleSprite, HDSprite, UILoader, TextStyles, Gui, GuiCommon, Sparkles, TapticEngine, LevelManager, SaveManager, VipManager, Debug, Dialog, Localization, IapOverlay, SkinManager, AdManager) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText,
        scaleFactor = Gui.scaleFactor;

    var viewport = Bento.getViewport();

    var GuiNavBar = Utils.copyObject(GuiCommon);

    var hiddenNames = {};

    /**
     * Nav bar, intended to be used as a sub-gui.
     */
    Utils.extend(GuiNavBar, {
        buttonNoAds: function buttonNoAds(obj) {
            if (hiddenNames[obj.name]) {
                obj.removeSelf();
                return;
            }
            // let hasNoAds = true;
            var hasNoAds = SaveManager.load('noAds') || VipManager.isVip();
            console.log('Checking NoAds: ', SaveManager.load('noAds'), VipManager.isVip());
            makeButton(obj, {
                shadow: 'gui/button-shadow',
                canClick: function canClick() {
                    return !hasNoAds;
                },
                onClick: function onClick() {
                    IapOverlay.purchase({
                        z: Globals.Layers.IAP_OVERLAY,
                        id: 'goslice_noads',
                        isConsumable: false,
                        resolveAutomatically: true,
                        fontSettings: TextStyles.get('iapOverlay'),
                        onPurchaseComplete: function onPurchaseComplete(success, id, purchaseInfo) {
                            // init.js handles saving, just draw a checkmark for now and prevent clicking again.
                            if (success && id === 'goslice_noads') {
                                if (!hasNoAds) {
                                    hasNoAds = true;
                                    obj.attach(new SimpleSprite({
                                        imageName: 'gui/selected-icon-navbar',
                                        position: new Vector2(0, 0),
                                        scale: new Vector2(0.12, 0.12)
                                    }));
                                    Gui.hide('buttonNoAdsText');
                                    AdManager.hideBanner();
                                    EventSystem.fire('bannerVisibilityChanged');
                                }
                            }
                        }
                    });
                }
            });
            if (hasNoAds) {
                obj.attach(new SimpleSprite({
                    imageName: 'gui/selected-icon-navbar',
                    position: new Vector2(0, 6.5),
                    scale: new Vector2(0.09, 0.09)
                }));
                Gui.get('buttonNoAdsText', function (txt) {
                    Gui.adjust(txt, {
                        y: -4.5
                    });
                });
            }
        },
        buttonNoAdsText: function buttonNoAdsText(obj) {
            var hasNoAds = SaveManager.load('noAds') || VipManager.isVip();
            if (hasNoAds) {
                obj.dimension.height -= 10;
            }
            if (Localization.getLanguage() !== 'en') {
                obj.dimension.width += 6;
            }

            makeText(obj, {
                text: Localization.getText('removeAds').toUpperCase().replace(' ', '\n'),
                fontSettings: TextStyles.get('buttonLabel'),
                align: 'center',
                linebreaks: false
                // drawDebug:true
            });
        },
        buttonLevelSelect: function buttonLevelSelect(obj) {
            if (hiddenNames[obj.name]) {
                obj.removeSelf();
                return;
            }
            makeButton(obj, {
                shadow: 'gui/button-shadow',
                onClick: function onClick() {
                    Bento.screens.show('screens/levelselect');
                }
            });
        },
        buttonIapShop: function buttonIapShop(obj) {
            if (hiddenNames[obj.name]) {
                obj.removeSelf();
                return;
            }
            makeButton(obj, {
                shadow: 'gui/button-shadow',
                onClick: function onClick() {
                    Bento.screens.show('screens/iapshop', {
                        fromScreen: Utils.currentScreenName()
                    });
                }
            });
        },
        buttonCharacterSelect: function buttonCharacterSelect(obj) {
            if (hiddenNames[obj.name]) {
                obj.removeSelf();
                return;
            }
            makeButton(obj, {
                shadow: 'gui/button-shadow',
                onClick: function onClick() {
                    Bento.screens.show('screens/shop', {
                        fromScreen: Utils.currentScreenName()
                    });
                }
            });

            var coins = SaveManager.load('coins');
            var cheapestItemCost = SkinManager.getCheapestItemCost();

            // show a notification badge if the player can afford any character or scissor in the shop
            // Note: the badge won't go away unless the button is rebuilt. It's assumed that money won't decrease while the button is visible
            var canAfford = false;
            var checkCanAfford = function checkCanAfford() {
                if (canAfford) {
                    return false;
                }
                if (coins >= cheapestItemCost) {
                    canAfford = true;
                    obj.attach(new Wiggle({
                        amplitude: 0.018
                    }));
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/notification-badge',
                        position: new Vector2(14, -11),
                        scale: new Vector2(0.088, 0.088)
                    }));
                }
            };

            checkCanAfford();

            obj.attach(new EventListener({
                eventName: 'updateCoins',
                onEvent: function onEvent(val) {
                    coins = val;
                    checkCanAfford();
                }
            }));
        }
    });

    return GuiNavBar;
});