'use strict';

bento.define('gui/guisettingsmenu', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'bento/gui/scrollinglist', 'gui/gui', 'gui/guicommon', 'gui/darkenbar', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'entities/gui/tutorialhand', 'components/mask', 'modules/localization', 'modules/admanager', 'gui/dialog', 'gui/coindialog', 'gui/coincounter', 'gui/iapoverlay', 'modules/tapticengine', 'bento/components/modal', 'entities/luckykatlogo', 'modules/cloudsavechecker'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, ScrollingList, Gui, GuiCommon, DarkenBar, SimpleSprite, Debug, SaveManager, TutorialHand, Mask, Localization, AdManager, Dialog, CoinDialog, CoinCounter, IapOverlay, TapticEngine, Modal, LuckyKatLogo, CloudSaveChecker) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiSettingsMenu = Utils.copyObject(GuiCommon);

    var fromScreen = void 0,
        showBackButton = void 0;

    var creditsText = 'Made by Lucky Kat\n\n-- PRODUCER --\nHerdjie Zhou\n\n-- CODE --\nJeremy Clarke\nHernan Zhou\n\n-- ART --\nGregory Parisi\n\n-- LEVEL DESIGN --\nJeremy Clarke\nHernan Zhou\nRowan Brak\nEmi Monserrate\nGregory Parisi\n\n-- MUSIC --\nMaxo';

    Utils.extend(GuiSettingsMenu, {
        init: function init(settings) {
            fromScreen = settings.fromScreen;

            console.log({ fromScreen: fromScreen });
            showBackButton = fromScreen === 'main' || fromScreen === 'victoryscreen' || fromScreen === 'shop';
        },
        buttonHome: function buttonHome(obj) {
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });
        },
        buttonToggleMusic: function buttonToggleMusic(obj) {
            makeButton(obj, {
                shadow: true,
                onClick: function onClick() {
                    var muted = !Bento.saveState.load('muteMusic', false);
                    Bento.audio.muteMusic(muted, true);
                    Bento.saveState.save('muteMusic', muted);
                    obj.alpha = muted ? 0.5 : 1.0;
                }
            });

            var muted = Bento.saveState.load('muteMusic', false);
            obj.alpha = muted ? 0.5 : 1.0;
        },
        buttonToggleSound: function buttonToggleSound(obj) {
            makeButton(obj, {
                shadow: true,
                onClick: function onClick() {
                    var muted = !Bento.saveState.load('muteSound', false);
                    Bento.audio.muteSound(muted);
                    Bento.saveState.save('muteSound', muted);
                    obj.alpha = muted ? 0.5 : 1.0;
                }
            });

            var muted = Bento.saveState.load('muteSound', false);
            obj.alpha = muted ? 0.5 : 1.0;
        },
        buttonToggleHaptics: function buttonToggleHaptics(obj) {
            makeButton(obj, {
                shadow: true,
                onClick: function onClick() {
                    var muted = !Bento.saveState.load('disableTapticEngine', false);
                    TapticEngine.enable(!muted);
                    Bento.saveState.save('disableTapticEngine', muted); // TODO -- clean up
                    obj.alpha = muted ? 0.5 : 1.0;
                }
            });

            var muted = Bento.saveState.load('disableTapticEngine', false);
            obj.alpha = muted ? 0.5 : 1.0;
        },
        buttonRestorePurchases: function buttonRestorePurchases(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                onClick: function onClick() {
                    var onSuccess = function onSuccess() {
                        Utils.timeout(60, function () {
                            Bento.screens.show('screens/titlescreen');
                        });
                        cleanUp();
                    };
                    var cleanUp = function cleanUp() {
                        EventSystem.off('restore_success', onSuccess);
                        EventSystem.off('restore_error', cleanUp);
                        EventSystem.off('purchase_error', cleanUp);
                    };

                    // Allow to ask to restore from cloudsave

                    CloudSaveChecker.enableQuickStart();
                    SaveManager.save('cloudLoadDeclined', false);

                    EventSystem.on('restore_success', onSuccess);
                    EventSystem.on('restore_error', cleanUp);
                    EventSystem.on('purchase_error', cleanUp);
                    IapOverlay.restore({
                        z: Globals.Layers.IAP_OVERLAY,
                        fontSettings: TextStyles.get('iapOverlay')
                    });
                }
            });
            // Gui.adjust(obj, {
            //     y: 20
            // });
        },
        buttonRestorePurchasesText: function buttonRestorePurchasesText(obj) {
            makeText(obj, {
                text: Localization.getText('restore').toUpperCase(),
                fontSettings: TextStyles.get('settingsButton'),
                align: 'center'
            });
        },
        buttonCredits: function buttonCredits(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                onClick: function onClick() {
                    var fill = new Fill({
                        name: 'fill',
                        dimension: Bento.getViewport(true).getSize(),
                        color: [0, 0, 0, 0.90],
                        originRelative: new Vector2(0, 0)
                    });
                    var behavior = {
                        name: 'behavior',
                        start: function start() {
                            // fade in overlay
                            new Tween({
                                from: 0, to: 1, in: 30,
                                ease: 'easeInOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    overlay.alpha = v;
                                }
                            });
                        }
                    };
                    var clickable = new Clickable({
                        onClick: function onClick(data) {
                            overlay.removeByName('clickable');
                            new Tween({
                                from: 1, to: 0, in: 30,
                                ease: 'easeInOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    overlay.alpha = v;
                                },
                                onComplete: function onComplete() {
                                    overlay.removeSelf();
                                }
                            });
                        }
                    });
                    var text = new Text({
                        text: creditsText,
                        scale: new Vector2(1.25, 1.25),
                        fontSettings: TextStyles.get('credits'),
                        align: 'center',
                        maxWidth: 120,
                        maxHeight: 200,
                        drawDebug: false
                    });
                    var logo = new SimpleSprite({
                        imageName: 'luckykat-credits-logo',
                        scale: new Vector2(0.25, 0.25)
                    });
                    logo.position.y -= 126;
                    text.position.y += 46;

                    var overlay = new Entity({
                        float: true,
                        z: Globals.Layers.DIALOGS,
                        name: 'creditsOverlay',
                        dimension: viewport.clone(),
                        components: [new Modal({}), fill, behavior, clickable, new Entity({
                            name: 'creditsInnerContainer',
                            position: viewport.getExtents(),
                            scale: new Vector2(Gui.scaleFactorTall, Gui.scaleFactorTall),
                            components: [text, logo]
                        })]
                    });
                    Bento.objects.attach(overlay);
                }
            });
        },
        buttonCreditsText: function buttonCreditsText(obj) {
            makeText(obj, {
                text: Localization.getText('credits').toUpperCase(),
                fontSettings: TextStyles.get('settingsButton'),
                align: 'center'
            });
        },
        versionText: function versionText(obj) {
            makeText(obj, {
                text: 'v' + window.VERSION + '-' + window.BUILD + '.' + (window.PATCHVERSION || 0) + (Bento.isDev() || Bento.saveState.load('isDev', false) ? ' DEV' : '') + (Utils.isFB() ? 'FB' : ''),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        }
    });

    return GuiSettingsMenu;
});