'use strict';

bento.define('gui/guishop', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'bento/gui/scrollinglist', 'gui/gui', 'gui/guicommon', 'gui/darkenbar', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'entities/gui/tutorialhand', 'components/mask', 'modules/localization', 'bento/components/nineslice', 'modules/vipmanager', 'gui/vippopup', 'gui/dialog', 'modules/skinmanager', 'modules/levelmanager', 'modules/admanager', 'gui/shopunlockfxoverlay', 'modules/xorshift', 'gui/challengepopup'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, ScrollingList, Gui, GuiCommon, DarkenBar, SimpleSprite, Debug, SaveManager, TutorialHand, Mask, Localization, NineSlice, VipManager, VipPopup, Dialog, SkinManager, LevelManager, AdManager, ShopUnlockFxOverlay, XorShift, ChallengePopup) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiShop = Utils.copyObject(GuiCommon);

    var PIXI = window.PIXI;

    var selectedCharacterTile = void 0;
    var selectedScissorTile = void 0;
    var selectedThemeTile = void 0;

    var characterCost = 0;

    var fromScreen = null;
    var showBackButton = false;
    var showTutMessage = false;

    var currentScroll = 0;
    var currentTabIndex = 0;
    var tabs = [{
        button: 'tabCharacters',
        container: 'characterContainer',
        hdSprite: 'hdsprite4'
    }, {
        button: 'tabScissors',
        container: 'scissorContainer',
        hdSprite: 'hdsprite1'
    }, {
        button: 'tabTheme',
        container: 'themeContainer',
        hdSprite: 'hdsprite5'
    }];
    var isSwitchingTabs = false;

    var setCurrentTab = function setCurrentTab(buttonName) {
        if (tabs[currentTabIndex].button === buttonName) {
            return; // nothing to do
        }
        if (isSwitchingTabs) {
            return;
        }
        isSwitchingTabs = true;
        var oldTabIndex = currentTabIndex;
        var oldTab = tabs[oldTabIndex];
        var oldContainer = Gui.get(oldTab.container);
        for (var i = 0; i < tabs.length; i++) {
            var tab = tabs[i];
            if (tab.button === buttonName) {
                (function () {
                    currentTabIndex = i;
                    var container = Gui.get(tab.container);
                    var sign = Utils.sign(i - oldTabIndex);
                    Tween({
                        from: viewport.width * sign, to: 0, in: 10,
                        ease: 'easeInOutQuad',
                        onStart: function onStart() {
                            Gui.show(container);
                        },
                        onUpdate: function onUpdate(v, t) {
                            container.position.x = v;
                            oldContainer.position.x = v - viewport.width * sign;
                        },
                        onComplete: function onComplete() {
                            Gui.hide(oldContainer);
                            isSwitchingTabs = false;
                        }
                    });
                })();
            } else {
                // immediately hide any tab container except for the old one, since that will be animated away
                if (i !== oldTabIndex) {
                    Gui.hide(tab.container);
                }
            }
        }

        var _loop = function _loop(_i) {
            Gui.get(tabs[_i].button, function (tabBtn) {
                paintTabButton(tabBtn, tabBtn.getComponent(tabs[_i].hdSprite));
            });
        };

        for (var _i = 0; _i < tabs.length; _i++) {
            _loop(_i);
        }
    };

    var getTabByButton = function getTabByButton(buttonName) {
        for (var i = 0; i < tabs.length; i++) {
            if (tabs[i].button === buttonName) {
                return tabs[i];
            }
        }
    };
    var getTabByContainer = function getTabByContainer(containerName) {
        for (var i = 0; i < tabs.length; i++) {
            if (tabs[i].container === containerName) {
                return tabs[i];
            }
        }
    };

    // highlight and raise tab button if it is currently selected
    var paintTabButton = function paintTabButton(button) {
        var tab = getTabByButton(button.name);
        var isCurrent = tabs[currentTabIndex].button === button.name;
        button.position.y = isCurrent ? -2 : 2;
        button.getComponent(tab.hdSprite, function (hdSprite) {
            hdSprite.getComponent(tab.hdSprite, function (sprite) {
                sprite.setup({
                    imageName: isCurrent ? 'gui/button-shop-tab' : 'gui/button-shop-tab-dark'
                });
            });
        });
        updateNotification(tab);
    };

    // show badge for a given tab, if applicable
    var updateNotification = function updateNotification(tab) {
        var cheapestCost = Infinity;
        switch (tab.button) {
            case 'tabCharacters':
                cheapestCost = SkinManager.getCharacterCost();
                break;
            case 'tabScissors':
                cheapestCost = SkinManager.getCheapestScissorCost();
                break;
            case 'tabTheme':
                cheapestCost = SkinManager.getCheapestThemeCost();
                break;
        }
        Gui.get(tab.button, function (button) {
            if (SaveManager.load('coins') >= cheapestCost) {
                if (!button.getComponent('notificationBadge')) {
                    var badge = new Entity({
                        name: 'notificationBadge',
                        position: new Vector2(17, -13),
                        scale: new Vector2(0.08, 0.08),
                        components: [new Sprite({ imageName: 'gui/notification-badge' })]
                    });
                    button.attach(badge);
                }
            } else {
                button.removeByName('notificationBadge');
            }
        });
    };

    Utils.extend(GuiShop, {
        init: function init(settings) {
            var restore = settings.restore;

            if (!restore) {
                currentScroll = 0;
                fromScreen = settings.fromScreen;
                showBackButton = fromScreen === 'main' || fromScreen === 'victoryscreen';
            }
            characterCost = SkinManager.getCharacterCost();
            // currentTabIndex = 0;  // don't set this, so the last visited tab is remembered.
            showTutMessage = false;
            if (SkinManager.getNumBoughtCharacters() === 0 && characterCost <= SaveManager.load('coins')) {
                currentTabIndex = 0;
                showTutMessage = true;
            }
        },
        tabContainer: function tabContainer(obj) {
            setCurrentTab(tabs[currentTabIndex].button);

            Gui.adjust(obj, {
                x: 0 * (1 - Gui.scaleFactor),
                y: -90 * (1 - Gui.scaleFactorTall) + 60 * (1 - Gui.scaleFactor),
                scale: (1 + Gui.scaleFactor) / 2
            });

            Gui.get(tabs[currentTabIndex].container, function (cont) {
                cont.getComponent('scrollingList', function (list) {
                    console.log(list);
                    list.forceTo(currentScroll);
                });
            });
        },
        tabCharacters: function tabCharacters(obj) {
            makeButton(obj, {
                shadow: 'gui/button-shop-tab-shadow',
                onClick: function onClick() {
                    setCurrentTab(obj.name);
                }
            });
            obj.attach({
                name: 'behavior',
                start: function start() {
                    paintTabButton(obj);
                }
            });
        },
        tabScissors: function tabScissors(obj) {
            paintTabButton(obj);
            makeButton(obj, {
                shadow: 'gui/button-shop-tab-shadow',
                onClick: function onClick() {
                    setCurrentTab(obj.name);
                }
            });
            obj.attach({
                name: 'behavior',
                start: function start() {
                    paintTabButton(obj);
                }
            });
        },
        tabTheme: function tabTheme(obj) {
            paintTabButton(obj);
            makeButton(obj, {
                shadow: 'gui/button-shop-tab-shadow',
                onClick: function onClick() {
                    setCurrentTab(obj.name);
                }
            });
            obj.attach({
                name: 'behavior',
                start: function start() {
                    paintTabButton(obj);
                }
            });
        },
        tabCharactersText: function tabCharactersText(obj) {
            makeText(obj, {
                text: Localization.getText('shopTabCharacter').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        tabScissorsText: function tabScissorsText(obj) {
            makeText(obj, {
                text: Localization.getText('shopTabScissors').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        tabThemeText: function tabThemeText(obj) {
            makeText(obj, {
                text: Localization.getText('shopTabTheme').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        tutMessage: function tutMessage(obj) {
            if (showTutMessage) {
                Gui.hide('tabContainer');
                makeText(obj, {
                    text: Localization.getText('shopTutorial'),
                    fontSettings: TextStyles.get('main'),
                    align: 'center'
                });
                Gui.adjust(obj, {
                    y: -80 * (1 - Gui.scaleFactorTall) + 60 * (1 - Gui.scaleFactor),
                    scale: (1 + Gui.scaleFactor) / 2
                });
                // move characters down to make room for message
                // Gui.get('characterContainer', cc => {
                //     Gui.adjust(cc, { y: 25 - 80*(1-Gui.scaleFactor)});
                // });
                obj.attach(new EventListener({
                    eventName: 'buyCharacter',
                    onEvent: function onEvent() {
                        obj.getComponent('text', function (text) {
                            text.setText(Localization.getText('awesome') + '\n');
                        });

                        Gui.get('buttonBack', function (buttonBack) {
                            if (buttonBack.parent) {

                                var tutorialHand = new TutorialHand({
                                    position: buttonBack.position.add(Vector2(0, 22)),
                                    innerScale: 0.12
                                });
                                tutorialHand.float = true;
                                buttonBack.parent.attach(tutorialHand);
                            }
                        });
                    }
                }));
            }
        },
        buttonBack: function buttonBack(obj) {
            if (!showBackButton) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/victoryscreen', { restore: true });
                }
            });
            var backText = Text({
                text: Localization.getText('back').toUpperCase(),
                position: new Vector2(0, 14),
                scale: new Vector2(0.45, 0.45),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 64,
                maxHeight: 18,
                drawDebug: false
            });
            obj.attach(backText);
        },
        buttonHome: function buttonHome(obj) {
            if (showBackButton) {
                obj.removeSelf();
            }
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/titlescreen');
                }
            });
        },
        characterContainer: function characterContainer(obj) {

            var extraItems = []; // unused?

            var ownedChars = [];
            var adChars = [];
            var coinChars = [];
            var bossChars = [];
            var challengeChars = [];
            var vipChars = [];

            // Get a uniquely seeded RNG for this user.
            var deviceId = SaveManager.load('identifier');
            var deviceIdParts = deviceId.match(/.{1,6}/g);
            var rng = new XorShift([Utils.stringHash(deviceIdParts[0] || ''), Utils.stringHash(deviceIdParts[1] || ''), Utils.stringHash(deviceIdParts[2] || ''), Utils.stringHash(deviceIdParts[3] || '')]);

            SkinManager.getShopCharacters().forEach(function (c) {
                var r = rng.random();
                if (SaveManager.load('hasChar-' + c.id)) {
                    ownedChars.push(c);
                } else if (c.ad) {
                    adChars.splice(Math.floor(r * adChars.length), 0, c);
                } else if (c.coins) {
                    coinChars.splice(Math.floor(r * coinChars.length), 0, c);
                } else if (Utils.isDefined(c.bossNumber)) {
                    bossChars.push(c);
                } else if (c.challenge) {
                    var levelsComplete = LevelManager.getNumberCleared(c.challenge);
                    var totalLevels = LevelManager.getTotalLevels(c.challenge);
                    if (levelsComplete === totalLevels) {
                        // last resort in case the game crashed before giving the character??
                        Utils.log('Player somehow missed unlocking a challenge character?');
                        ownedChars.push(c);
                        SkinManager.unlockCharacter(c.id);
                    } else {
                        challengeChars.push(c);
                    }
                } else if (c.vip) {
                    if (VipManager.isVip()) {
                        ownedChars.unshift(c);
                    } else {
                        vipChars.push(c);
                    }
                } else {
                    Utils.log('Don\'t know what to do with shop character ' + c.id);
                }
            });

            var items = null;

            var sections = [{
                items: ownedChars
            }, {
                // name: 'FREE',
                name: Localization.getText('free').toUpperCase(),
                items: adChars
            }, {
                name: Localization.getText('shop').toUpperCase(),
                items: coinChars
            }, {
                name: 'BOSS',
                items: bossChars
            }, {
                name: 'VIP',
                items: vipChars
            }, {
                name: Localization.getText('challenge').toUpperCase(),
                items: challengeChars
            }];

            if (showTutMessage) {
                // only show purchaseable characters.
                items = coinChars;
                sections = null;
                // sections = [
                //     {
                //         name: 'SHOP',
                //         items: coinChars,
                //     }
                // ];
            }

            GuiShop.genericContainer(obj, {
                makeTile: CharacterTile,
                items: items,
                sections: sections,
                extraItems: extraItems
            });
        },
        scissorContainer: function scissorContainer(obj) {
            GuiShop.genericContainer(obj, {
                items: SkinManager.getShopScissors(),
                makeTile: ScissorTile
            });
        },
        themeContainer: function themeContainer(obj) {
            GuiShop.genericContainer(obj, {
                items: SkinManager.getShopThemes(),
                makeTile: ThemeTile
            });
        },
        genericContainer: function genericContainer(obj, settings) {
            var makeTile = settings.makeTile,
                items = settings.items,
                sections = settings.sections,
                _settings$extraItems = settings.extraItems,
                extraItems = _settings$extraItems === undefined ? [] : _settings$extraItems;


            var tabIndex = void 0;
            for (var i = 0; i < tabs.length; i++) {
                if (tabs[i].container === obj.name) {
                    tabIndex = i;
                    break;
                }
            }
            if (tabIndex !== currentTabIndex) {
                obj.removeSelf();
            }

            obj.selectedIcon = new Entity({
                name: 'selectedIcon',
                position: new Vector2(0, 24),
                scale: new Vector2(0.16, 0.16),
                components: [new Sprite({
                    imageName: 'gui/selected-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });

            var columns = 4;
            var xSpacing = (viewport.width - 20) / columns;
            var ySpacing = 70;

            var dummy = new Entity({
                name: 'dummy'
                // components: Debug({}),
            });
            var container = new Entity({
                name: 'container',
                dimensions: new Rectangle(0, 0, 100, 0)
                // components: Debug({}),
            });

            var x = void 0,
                y = void 0,
                tile = void 0;

            if (sections) {
                x = 0;
                y = 0;
                sections.forEach(function (section) {
                    if (section.items.length === 0) {
                        return; // skip empty section
                    }

                    // assume items in an unnamed section can be spaced closer together vertically
                    var condensedRows = !section.name;

                    if (section.name) {
                        y += 0.9;
                        container.attach(new Text({
                            text: section.name,
                            position: new Vector2(0, y * ySpacing + 54),
                            scale: new Vector2(0.8, 0.8),
                            fontSettings: TextStyles.get('main'),
                            align: 'center',
                            maxWidth: 140,
                            maxHeight: 32,
                            drawDebug: false
                        }));
                        y += 0.65;
                    } else {
                        y += 0.2;
                    }
                    section.items.forEach(function (item, i) {
                        x = i % columns - (columns / 2 - 0.5);

                        tile = makeTile({
                            index: i,
                            item: item,
                            position: new Vector2(x * xSpacing, y * ySpacing + 50)
                        });
                        container.attach(tile);

                        if ((i + 1) % 4 === 0 && i !== section.items.length - 1) {
                            y += condensedRows ? 0.8 : 1.0;
                        }
                    });
                    if (condensedRows) {
                        y -= 0.25;
                    }
                });
            } else {
                for (var _i2 = 0; _i2 < items.length; _i2++) {
                    x = _i2 % columns - (columns / 2 - 0.5);
                    y = Math.floor(_i2 / columns);
                    tile = makeTile({
                        index: _i2,
                        item: items[_i2],
                        position: new Vector2(x * xSpacing, y * ySpacing + 50)
                    });
                    container.attach(tile);
                }
            }

            var bottom = y * ySpacing;
            container.dimension.height = ySpacing;

            var area = new Rectangle(-viewport.width / 2, 20, viewport.width, viewport.height - 100 * (1 - Gui.scaleFactorTall));

            var list = new ScrollingList({
                z: Globals.Layers.GUI,
                position: new Vector2(0, 0),
                updateWhenPaused: 0,
                area: area, // area that user can grab
                direction: 'y', // 'x' or 'y'
                spacing: 0, // additional spacing between items
                maxOffset: -viewport.height + 120 * Gui.scaleFactorTall, // additional max scrolling distance
                minOffset: 0, // additional min scrolling distance
                snap: false, // snap items (cancels maxOffset and minOffset)
                snapNextOffset: 0, // distance to scroll to snap to next, in screen pixels. Used with snap:true, best with damping:0
                onSnap: function onSnap(item, index) {}, // called when snap focus changed
                onLoseFocus: function onLoseFocus(index) {}, // called when snap focus changed
                onScrollStart: function onScrollStart() {}, // callback when scrolling has started
                snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
                snapSpeed: 0.3, // speed of snapping
                damping: 0.95 // speed decrease,
            });
            container.dimension.height = bottom + ySpacing + 24;
            list.addItem(container);
            extraItems.forEach(function (item) {
                list.addItem(item);
            });
            list.addItem(dummy);

            list.attach({
                name: 'behavior',
                hasInit: false,
                start: function start() {
                    if (!this.hasInit) {
                        Mask.applyToContainer(list, area);
                        this.hasInit = true;
                    }
                },
                update: function update() {
                    var scrollY = list.position.y - list.getScroll();
                    var minY = scrollY - ySpacing * 2;
                    var maxY = scrollY + viewport.height + ySpacing / 2;
                    container.components.forEach(function (item) {
                        if (item.isEntity) {
                            var itemY = item.position.y;
                            item.visible = itemY > minY && itemY < maxY;
                            item.active = item.visible;
                        }
                    });

                    if (tabs[currentTabIndex].container === obj.name) {
                        currentScroll = list.getScroll();
                    }
                }
            });
            obj.attach(list);

            area.height += 100 * (1 - Gui.scaleFactor);
            Gui.adjust(obj, {
                y: -100 * (1 - Gui.scaleFactorTall) + 60 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor,
                onAdjust: function onAdjust() {
                    obj.removeByName('positioner');
                }
            });

            obj.area = area;

            // to mask a sprite that was added later.
            obj.applyMask = function (sprite) {
                list.attach(new Mask({
                    sprite: sprite,
                    rectangle: area
                }));
            };
        }
    });

    var CharacterTile = function CharacterTile(settings) {
        var index = settings.index,
            character = settings.item,
            position = settings.position;


        var dimension = new Rectangle(-18, -16, 36, 50);

        var isSelected = SaveManager.load('character') === character.id;
        var isLocked = (character.ad || characterCost > 0 || character.bossNumber != null) && !SaveManager.load('hasChar-' + character.id);
        if (character.vip) {
            isLocked = !VipManager.isVip();
        }
        var canAfford = !isLocked || characterCost <= SaveManager.load('coins');
        var isGachaBall = isLocked && !character.vip && character.bossNumber == null && character.challenge == null;

        var sprite = void 0,
            spriteObj = void 0;

        var setupSprite = function setupSprite(autoAttach) {
            if (spriteObj) {
                spriteObj.removeSelf();
            }
            if (isGachaBall && character.ball) {
                sprite = new Sprite({
                    imageName: 'gacha/' + character.ball,
                    originRelative: new Vector2(0.5, 0.94)
                });
                spriteObj = new Entity({
                    name: 'spriteObj',
                    position: new Vector2(0, 12),
                    scale: new Vector2(0.09, 0.09),
                    components: [sprite]
                });
            } else if (character.challenge && isLocked) {
                sprite = new Sprite({
                    imageName: 'gui/cage/silhouette',
                    originRelative: new Vector2(0.5, 0.5)
                });
                spriteObj = new Entity({
                    name: 'spriteObj',
                    position: new Vector2(1, 0),
                    scale: new Vector2(0.275, 0.275),
                    components: [sprite]
                });
            } else {
                sprite = new Sprite({
                    imageName: 'character' + character.id + '-idle',
                    originRelative: new Vector2(0.5, 0.5),
                    frameCountX: 5,
                    frameCountY: 1,
                    animations: {
                        default: { speed: 0, frames: [0] },
                        idle: { speed: 0.125, frames: [0, 1, 2, 3, 4] }
                    }
                });
                spriteObj = new Entity({
                    name: 'spriteObj',
                    position: new Vector2(-2, -12),
                    scale: new Vector2(1.6, 1.6),
                    components: [sprite]
                });
            }
            if (autoAttach) {
                entity.attach(spriteObj);

                // if entity is already added, we should set up our own mask because the container won't do it for us.
                if (entity.parent) {
                    Gui.get('characterContainer', function (container) {
                        container.applyMask(sprite);
                    });
                }
            }
        };

        var costButton = void 0;
        var unlockLevel = void 0;

        if (isLocked) {
            // spriteObj.alpha = 0.6; // TODO: greyscale?
            if (character.ad) {
                costButton = new Entity({
                    name: 'costButton',
                    position: new Vector2(0, 26),
                    dimension: new Rectangle(-19, -10, 38, 20),
                    components: [new Entity({
                        name: 'costButtonBg',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165),
                        components: [new Sprite({
                            imageName: 'gui/button-small-blue',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Entity({
                        name: 'adBadge',
                        // position: new Vector2(9, 0),
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.09, 0.09),
                        components: [new Sprite({
                            imageName: 'gui/ad-badge',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    })]
                });
            } else if (character.bossNumber != null) {
                unlockLevel = LevelManager.getLevelByBossNumber(character.bossNumber);
                costButton = new Entity({
                    name: 'costButtonBoss',
                    position: new Vector2(0, 26),
                    dimension: new Rectangle(-19, -10, 38, 20),
                    components: [new Entity({
                        name: 'costButtonBg',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165),
                        components: [new Sprite({
                            imageName: 'gui/button-small-yellow',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Text({
                        text: Localization.getText('levelN').replace('{NUMBER}', unlockLevel + 1).toUpperCase(),
                        // text: `LV ${unlockLevel+1}`,
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.35, 0.35),
                        fontSettings: TextStyles.get('main'),
                        align: 'center',
                        maxWidth: 80,
                        maxHeight: 32
                        // drawDebug: true
                    })]
                });
            } else if (character.vip) {
                costButton = new Entity({
                    name: 'costButtonVip',
                    position: new Vector2(0, 26),
                    dimension: new Rectangle(-19, -10, 38, 20),
                    components: [new Entity({
                        name: 'costButtonBg',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165),
                        components: [new Sprite({
                            imageName: 'gui/button-small-vip',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Text({
                        text: 'VIP',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.4, 0.4),
                        fontSettings: TextStyles.get('mainVip'),
                        align: 'center',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    })]
                });
            } else if (character.challenge) {

                var levelsComplete = LevelManager.getNumberCleared(character.challenge);
                var totalLevels = LevelManager.getTotalLevels(character.challenge);

                costButton = new Entity({
                    name: 'costButtonBoss',
                    position: new Vector2(0, 26),
                    dimension: new Rectangle(-19, -10, 38, 20),
                    components: [new Entity({
                        name: 'costButtonBg',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165),
                        components: [new Sprite({
                            imageName: 'gui/button-small-yellow',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Text({
                        text: levelsComplete + ' / ' + totalLevels,
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.4, 0.4),
                        fontSettings: TextStyles.get('main'),
                        align: 'center',
                        maxWidth: 80,
                        maxHeight: 32
                        // drawDebug: true
                    })]
                });
            } else {
                costButton = new Entity({
                    name: 'costButton',
                    position: new Vector2(0, 26),
                    dimension: new Rectangle(-19, -10, 38, 20),
                    components: [new Entity({
                        name: 'costButtonBg',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165),
                        components: [new Sprite({
                            imageName: 'gui/button-small-green',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Entity({
                        name: 'coinIcon',
                        position: new Vector2(9, 0),
                        scale: new Vector2(0.10, 0.10),
                        components: [new Sprite({
                            imageName: 'gui/coin-icon',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    }), new Text({
                        text: characterCost,
                        position: new Vector2(-7, 0),
                        scale: new Vector2(0.35, 0.35),
                        fontSettings: TextStyles.get('main'),
                        align: 'center',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    })]
                });
                costButton.attach(new EventListener({
                    name: 'costButtonUpdater',
                    eventName: 'revealCharacter',
                    ignorePause: true,
                    onEvent: function onEvent(data) {
                        costButton.getComponent('text', function (text) {
                            text.setText(characterCost);
                        });
                    }
                }));
            }
            Gui.makeButton(costButton, {
                canClick: function canClick(data) {
                    var container = Gui.get('characterContainer');
                    if (!container) return false;
                    var area = container.area;
                    var pos = container.toLocalPosition(data.worldPosition);
                    return isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
                },
                onClick: function onClick() {

                    EventSystem.fire('FirebaseAnalytics-logEvent', {
                        name: 'shop_item_clicked',
                        properties: { item_id: 'character-' + character.id }
                    });

                    // Unlock character if we have enough coins
                    var coins = SaveManager.load('coins');

                    var doUnlock = function doUnlock() {
                        SkinManager.unlockCharacter(character.id);
                        characterCost = SkinManager.getCharacterCost();
                        costButton.removeSelf();
                        isLocked = false;

                        setupSprite(false); // remake sprite (to get rid of mask)
                        entity.attach(spriteObj);

                        entity.removeByName('wiggleBehavior');

                        Bento.objects.attach(new ShopUnlockFxOverlay({
                            ballSpriteObj: spriteObj,
                            characterTile: entity,
                            onReveal: function onReveal() {
                                confettiCheer();
                                isGachaBall = false;
                                spriteObj.alpha = 1.0;
                                setupSprite(false);
                                entity.attach(spriteObj);
                                EventSystem.fire('revealCharacter', character.id); // update cost texts
                            },
                            onComplete: function onComplete() {
                                setupSprite(true); // remake sprite with mask
                                setSelected(true);
                                EventSystem.fire('buyCharacter', character.id); // update tutorial text
                            }
                        }));

                        EventSystem.fire('FirebaseAnalytics-logEvent', {
                            name: 'shop_item_unlocked',
                            properties: {
                                'item_id': 'character-' + character.id,
                                'total_num_item_unlocked': SkinManager.getNumBoughtCharacters()
                            }
                        });
                    };

                    if (character.bossNumber != null) {
                        Bento.objects.attach(new Dialog({
                            type: 'ok',
                            text: Localization.getText('unlockAtLv').replace('{NUMBER}', unlockLevel + 1)
                        }));
                    } else if (character.challenge) {
                        Bento.objects.attach(new ChallengePopup({}));
                    } else if (character.ad) {
                        AdManager.tryShowRewarded({
                            onReward: function onReward() {
                                doUnlock();
                            },
                            onComplete: function onComplete(error) {
                                if (error) {
                                    // ?
                                }
                            },

                            tag: 'Shop_Free_Character',
                            rewardType: 'character',
                            rewardAmount: 1
                        });
                    } else if (characterCost <= coins) {
                        spendCoinAnimation(characterCost);
                        SaveManager.save('coins', coins - characterCost);
                        doUnlock();
                        tabs.forEach(function (tab) {
                            return updateNotification(tab);
                        });
                    } else if (character.vip) {
                        Bento.objects.attach(new VipPopup({}));
                    } else {
                        Bento.screens.show('screens/iapshop', {
                            fromScreen: Utils.currentScreenName()
                        });
                    }
                }
            });
        }

        var setSelected = function setSelected(val) {
            isSelected = val;
            if (selectedCharacterTile && selectedCharacterTile !== entity) {
                selectedCharacterTile.setSelected(false);
            }
            if (val) {
                Gui.get('characterContainer', function (cc) {
                    cc.selectedIcon.removeSelf();
                    entity.attach(cc.selectedIcon);
                });
                if (!isGachaBall) {
                    sprite.setAnimation('idle');
                }
                selectedCharacterTile = entity;
                SaveManager.save('character', character.id);
            } else {
                if (!isGachaBall) {
                    sprite.setAnimation('default');
                }
            }
        };

        var entity = new Entity({
            name: 'characterTile',
            position: position,
            dimension: dimension
        }).extend({
            setSelected: setSelected
        });

        setupSprite(true);

        Gui.makeButton(entity, {
            haptics: false,
            canClick: function canClick(data) {
                var container = Gui.get('characterContainer');
                if (!container) return false;
                var area = container.area;
                var pos = container.toLocalPosition(data.worldPosition);
                return !isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
            },
            onClick: function onClick() {
                setSelected(true);
                EventSystem.fire('FirebaseAnalytics-logEvent', {
                    name: 'shop_item_equipped',
                    properties: {
                        'item_id': 'character-' + character.id,
                        'total_num_item_unlocked': SkinManager.getNumBoughtCharacters()
                    }
                });
            }
        });

        if (costButton) {
            entity.attach(costButton);
        }

        if (isSelected) {
            setSelected(true);
        }

        if (isGachaBall) {
            var nextWiggleTime = entity.timer + Utils.getRandomRange(60, 200);
            entity.attach({
                name: 'wiggleBehavior',
                update: function update() {
                    if (isGachaBall && entity.timer >= nextWiggleTime) {
                        nextWiggleTime += Utils.getRandomRange(60, 120);
                        if (Math.random() < 0.05) {
                            Tween({
                                from: 0,
                                to: Utils.pickRandom(-0.14, 0.14),
                                in: 8,
                                ease: 'easeOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    spriteObj.rotation = v;
                                },
                                onComplete: function onComplete() {
                                    Tween({
                                        from: spriteObj.rotation,
                                        to: 0,
                                        in: 60,
                                        ease: 'elastic',
                                        decay: 6,
                                        oscillations: 4,
                                        onUpdate: function onUpdate(v, t) {
                                            spriteObj.rotation = v;
                                        }
                                    });
                                }
                            });
                        }
                    }
                }
            });
        }

        return entity;
    };

    var ScissorTile = function ScissorTile(settings) {
        var index = settings.index,
            scissor = settings.item,
            position = settings.position;


        var dimension = new Rectangle(-18, -16, 36, 50);

        var isSelected = SaveManager.load('scissor') === scissor.id;
        var isLocked = !SaveManager.load('hasScissor-' + scissor.id);

        var sprite = new Sprite({
            imageName: 'scissors/scissors-' + scissor.id,
            originRelative: new Vector2(0.5, 0.5),
            frameCountX: 2,
            frameCountY: 1,
            animations: {
                default: { speed: 0, frames: [1] },
                snip: { speed: 0.22, frames: [0, 1, 1, 0, 1], loop: false }
            }
        });
        var spriteObj = new Entity({
            name: 'spriteObj',
            position: new Vector2(0, -4),
            scale: new Vector2(0.1, 0.1),
            components: [sprite]
        });

        var canAfford = scissor.cost <= SaveManager.load('coins');

        var costButton = void 0;

        if (isLocked) {
            spriteObj.alpha = 0.6;
            costButton = new Entity({
                name: 'costButton',
                position: new Vector2(0, 26),
                dimension: new Rectangle(-19, -10, 38, 20),
                components: [new Entity({
                    name: 'buttonSmallGreen',
                    position: new Vector2(0, 0),
                    scale: new Vector2(0.165, 0.165),
                    components: [new Sprite({
                        imageName: 'gui/button-small-green',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                }), new Entity({
                    name: 'coinIcon',
                    position: new Vector2(9, 0),
                    scale: new Vector2(0.10, 0.10),
                    components: [new Sprite({
                        imageName: 'gui/coin-icon',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                }), new Text({
                    text: scissor.cost,
                    position: new Vector2(-7, 0),
                    scale: new Vector2(0.35, 0.35),
                    fontSettings: TextStyles.get('main'),
                    align: 'center',
                    maxWidth: 64,
                    maxHeight: 32,
                    drawDebug: false
                })]
            });
            Gui.makeButton(costButton, {
                canClick: function canClick(data) {
                    var container = Gui.get('scissorContainer');
                    if (!container) return false;
                    var area = container.area;
                    var pos = container.toLocalPosition(data.worldPosition);
                    return isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
                },
                onClick: function onClick() {

                    EventSystem.fire('FirebaseAnalytics-logEvent', {
                        name: 'shop_item_clicked',
                        properties: { item_id: 'scissor-' + scissor.id }
                    });

                    // Unlock scissor if we have enough coins
                    var coins = SaveManager.load('coins');
                    if (scissor.cost <= coins) {
                        spendCoinAnimation(scissor.cost);
                        confettiCheer();
                        SaveManager.save('coins', coins - scissor.cost);
                        SkinManager.unlockScissor(scissor.id);
                        isLocked = false;
                        spriteObj.alpha = 1.0;
                        costButton.removeSelf();
                        setSelected(true);
                        EventSystem.fire('buyScissor', scissor.id);
                        tabs.forEach(function (tab) {
                            return updateNotification(tab);
                        });
                        // snip animation
                        var scale = spriteObj.scale.clone();
                        Tween({
                            delay: 8,
                            from: 0, to: Math.PI * 3, in: 38,
                            ease: 'linear',
                            onUpdate: function onUpdate(v, t) {
                                spriteObj.scale.x = scale.x * (1.0 - Math.sin(v) * 0.25 * (1 - t / 38));
                                spriteObj.scale.y = scale.y * (1.0 + Math.sin(v) * 0.20 * (1 - t / 38));
                            }
                        });
                        Utils.timeout(12, function () {
                            sprite.setAnimation('snip');
                        });

                        EventSystem.fire('FirebaseAnalytics-logEvent', {
                            name: 'shop_item_unlocked',
                            properties: {
                                'item_id': 'scissor-' + scissor.id,
                                'total_num_item_unlocked': SkinManager.getNumBoughtScissors()
                            }
                        });
                    } else {
                        Bento.screens.show('screens/iapshop', {
                            fromScreen: Utils.currentScreenName()
                        });
                    }
                }
            });
        }

        var setSelected = function setSelected(val) {
            isSelected = val;
            if (selectedScissorTile && selectedScissorTile !== entity) {
                selectedScissorTile.setSelected(false);
            }
            if (val) {
                Gui.get('scissorContainer', function (sc) {
                    sc.selectedIcon.removeSelf();
                    entity.attach(sc.selectedIcon);
                });
                // sprite.setAnimation('idle');
                selectedScissorTile = entity;
                SaveManager.save('scissor', scissor.id);
            } else {
                // sprite.setAnimation('default');
            }
        };

        var entity = new Entity({
            name: 'scissorTile',
            position: position,
            dimension: dimension,
            components: [spriteObj]
        }).extend({
            setSelected: setSelected
        });

        Gui.makeButton(entity, {
            haptics: false,
            canClick: function canClick(data) {
                var container = Gui.get('scissorContainer');
                if (!container) return false;
                var area = container.area;
                var pos = container.toLocalPosition(data.worldPosition);
                return !isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
            },
            onClick: function onClick() {
                setSelected(true);
                EventSystem.fire('FirebaseAnalytics-logEvent', {
                    name: 'shop_item_equipped',
                    properties: { item_id: 'scissor-' + scissor.id }
                });
            }
        });

        if (costButton) {
            entity.attach(costButton);
        }

        if (isSelected) {
            setSelected(true);
        }

        return entity;
    };

    var ThemeTile = function ThemeTile(settings) {
        var index = settings.index,
            theme = settings.item,
            position = settings.position;


        var dimension = new Rectangle(-18, -16, 36, 50);

        var isSelected = SaveManager.load('theme') === theme.id;
        var isLocked = !SaveManager.load('hasTheme-' + theme.id);

        var sprite = new Sprite({
            imageName: 'theme-previews/' + theme.id,
            originRelative: new Vector2(0.5, 0.5)
        });
        var spriteObj = new Entity({
            name: 'spriteObj',
            position: new Vector2(0, -4),
            scale: new Vector2(0.25, 0.25),
            components: [sprite]
        });

        var canAfford = theme.cost <= SaveManager.load('coins');

        var costButton = void 0;

        if (isLocked) {
            // spriteObj.alpha = 0.8;
            costButton = new Entity({
                name: 'costButton',
                position: new Vector2(0, 26),
                dimension: new Rectangle(-19, -10, 38, 20),
                components: [new Entity({
                    name: 'buttonSmallGreen',
                    position: new Vector2(0, 0),
                    scale: new Vector2(0.165, 0.165),
                    components: [new Sprite({
                        imageName: 'gui/button-small-green',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                }), new Entity({
                    name: 'coinIcon',
                    position: new Vector2(9, 0),
                    scale: new Vector2(0.10, 0.10),
                    components: [new Sprite({
                        imageName: 'gui/coin-icon',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                }), new Text({
                    text: theme.cost,
                    position: new Vector2(-7, 0),
                    scale: new Vector2(0.35, 0.35),
                    fontSettings: TextStyles.get('main'),
                    align: 'center',
                    maxWidth: 64,
                    maxHeight: 32,
                    drawDebug: false
                })]
            });
            Gui.makeButton(costButton, {
                canClick: function canClick(data) {
                    var container = Gui.get('themeContainer');
                    if (!container) return false;
                    var area = container.area;
                    var pos = container.toLocalPosition(data.worldPosition);
                    return isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
                },
                onClick: function onClick() {

                    EventSystem.fire('FirebaseAnalytics-logEvent', {
                        name: 'shop_item_clicked',
                        properties: { item_id: 'theme-' + theme.id }
                    });

                    // Unlock theme if we have enough coins
                    var coins = SaveManager.load('coins');
                    if (theme.cost <= coins) {
                        spendCoinAnimation(theme.cost);
                        confettiCheer();
                        SaveManager.save('coins', coins - theme.cost);
                        SkinManager.unlockTheme(theme.id);
                        isLocked = false;
                        spriteObj.alpha = 1.0;
                        costButton.removeSelf();
                        setSelected(true);
                        EventSystem.fire('buyTheme', theme.id);
                        tabs.forEach(function (tab) {
                            return updateNotification(tab);
                        });

                        EventSystem.fire('FirebaseAnalytics-logEvent', {
                            name: 'shop_item_unlocked',
                            properties: {
                                'item_id': 'theme-' + theme.id,
                                'total_num_item_unlocked': SkinManager.getNumBoughtThemes()
                            }
                        });
                    } else {
                        Bento.screens.show('screens/iapshop', {
                            fromScreen: Utils.currentScreenName()
                        });
                    }
                }
            });
        }

        var setSelected = function setSelected(val) {
            isSelected = val;
            if (selectedThemeTile && selectedThemeTile !== entity) {
                selectedThemeTile.setSelected(false);
            }
            if (val) {
                Gui.get('themeContainer', function (tc) {
                    tc.selectedIcon.removeSelf();
                    entity.attach(tc.selectedIcon);
                });
                // sprite.setAnimation('idle');
                selectedThemeTile = entity;
                SaveManager.save('theme', theme.id);
            } else {
                // sprite.setAnimation('default');
            }
        };

        var entity = new Entity({
            name: 'themeTile',
            position: position,
            dimension: dimension,
            components: [spriteObj]
        }).extend({
            setSelected: setSelected
        });

        Gui.makeButton(entity, {
            haptics: false,
            canClick: function canClick(data) {
                var container = Gui.get('themeContainer');
                if (!container) return false;
                var area = container.area;
                var pos = container.toLocalPosition(data.worldPosition);
                return !isLocked && area.hasPosition(pos) && !Utils.didScroll(entity);
            },
            onClick: function onClick() {
                setSelected(true);
                EventSystem.fire('FirebaseAnalytics-logEvent', {
                    name: 'shop_item_equipped',
                    properties: {
                        'item_id': 'theme-' + theme.id,
                        'total_num_item_unlocked': SkinManager.getNumBoughtThemes()
                    }
                });
            }
        });

        if (costButton) {
            entity.attach(costButton);
        }

        if (isSelected) {
            setSelected(true);
        }

        return entity;
    };

    /**
     * Gradually decrease coin counter, show confetti, cheer sfx
     */
    var spendCoinAnimation = function spendCoinAnimation(cost) {
        var coins = SaveManager.load('coins');
        // gradually decrease coin counter
        var n = 0;
        Tween({
            from: coins, to: coins - cost, in: 30,
            onUpdate: function onUpdate(v, t) {
                if (n++ % 3 === 0) {
                    EventSystem.fire('updateCoins', Math.floor(v));
                }
            },
            onComplete: function onComplete() {
                EventSystem.fire('updateCoins', coins);
            }
        });
        coins -= cost;
    };

    var confettiCheer = function confettiCheer() {
        Bento.objects.attach(new Confetti({
            speedY: 0.7,
            particleScale: 0.08,
            z: Globals.Layers.CONFETTI
        }));
        Bento.audio.playSound('sfx_victory_jingle_cheer');
    };

    return GuiShop;
});