'use strict';

bento.define('gui/guititlescreen', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'gui/darkenbar', 'modules/levelmanager', 'modules/savemanager', 'gui/freegift', 'modules/localization', 'components/debug', 'gui/vippopup', 'modules/vipmanager', 'components/sparkles', 'modules/crosspromo/crosspromo', 'gui/challengepopup', 'modules/timemanager', 'gui/dialog', 'modules/notifications', 'components/simplesprite'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, StarsEarned, Gui, GuiCommon, DarkenBar, LevelManager, SaveManager, FreeGift, Localization, Debug, VipPopup, VipManager, Sparkles, CrossPromo, ChallengePopup, TimeManager, Dialog, Notifications, SimpleSprite) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiTitleScreen = Utils.copyObject(GuiCommon);

    var currentChallenge = null;
    var showChallengeButton = false;

    Utils.extend(GuiTitleScreen, {
        init: function init(settings) {
            currentChallenge = LevelManager.getCurrentChallenge();
            showChallengeButton = currentChallenge && LevelManager.getNumberCleared() >= 4;
        },
        logoTitle: function logoTitle(obj) {

            var darkenBar = new DarkenBar({
                position: obj.position,
                height: 76 * Gui.scaleFactor,
                speed: 30,
                color: [0, 0.02, 0.51, 0.2]
            });

            Utils.timeout(15, function () {
                Gui.show('logoTitle');
            });

            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    parent.attach(darkenBar);
                    return new Tween({
                        from: 0, to: 1, in: 20,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        }
                    });
                },
                hide: function hide(parent) {
                    darkenBar.hide();
                    return new Tween({
                        from: 1, to: 0, in: 8,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            obj.alpha = v;
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);

            obj.attach(new Wiggle({
                frequency: 0.09,
                amplitude: 0.01
            }));

            Gui.adjust(obj, {
                // x: 0 * (1-Gui.scaleFactor),
                y: 66 * (1 - Gui.scaleFactorTall) + 24 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor,
                onAdjust: function onAdjust() {}
            });

            var pointers = 0;
            obj.attach(new Clickable({
                onClick: function onClick(data) {
                    pointers += 1;
                },
                onClickUp: function onClickUp(data) {
                    pointers -= 1;
                }
            }));
            obj.attach({
                name: 'behavior',
                t: 0,
                update: function update(data) {
                    if (pointers >= 2) {
                        this.t += 1;
                    } else {
                        this.t = 0;
                    }
                    if (this.t === 240) {
                        Utils.prompt('', '?', function (input) {
                            if (input === window.atob('aW1hbG92ZXI=')) {
                                Bento.saveState.save('isDev', true);
                                Utils.setDev(true);
                            }
                        });
                    }
                }
            });
        },
        logoTitleHdSprite: function logoTitleHdSprite(obj) {

            var postfix = {
                'de': 'EN',
                'en': 'EN',
                'es': 'EN',
                'fr': 'EN',
                'it': 'EN',
                'ja': 'JA',
                'ko': 'KO',
                'pt': 'EN',
                'ru': 'RU',
                'zh': 'ZH',
                'zt': 'ZH'
            }[Localization.getLanguage()];

            var image = Bento.assets.getImage('logos/logo-' + postfix);
            if (image) {
                obj.getComponent('logoTitleHdSprite', function (sprite) {
                    sprite.setup({
                        image: image,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                });
            }
        },
        buttonPlay: function buttonPlay(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                onClick: function onClick() {
                    Utils.timeout(15, function () {
                        Utils.currentLevelGroup = 'default';
                        Utils.currentLevel = SaveManager.load('nextLevel');
                        Bento.screens.show('screens/main');
                    });
                }
            });
            obj.attach({
                name: 'adjuster',
                start: function start() {
                    obj.scale.x *= Gui.scaleFactor;
                    obj.scale.y *= Gui.scaleFactor;
                    obj.position.y *= Gui.scaleFactor;
                }
            });
        },
        buttonPlayText: function buttonPlayText(obj) {
            makeText(obj, {
                text: Localization.getText('play').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        buttonSettingsMenu: function buttonSettingsMenu(obj) {
            makeButton(obj, {
                onClick: function onClick() {
                    Bento.screens.show('screens/settingsmenu');
                }
            });
        },
        buttonFreeGift: function buttonFreeGift(obj) {

            if (SaveManager.load('claimedFreeGifts') === 0) {
                obj.removeSelf();
                return;
            }

            // ignore the UI designer, draw our own button graphic insted
            obj.components.forEach(function (c) {
                obj.remove(c);
            });

            FreeGift({
                entity: obj
            });

            if (!showChallengeButton) {
                Gui.adjust(obj, {
                    y: 8 + 20 * (1 - Gui.scaleFactor),
                    scale: Gui.scaleFactor
                });
            } else {
                Gui.adjust(obj, {
                    y: 20 * (1 - Gui.scaleFactor),
                    scale: Gui.scaleFactor
                });
            }
        },
        buttonVip: function buttonVip(obj) {
            var isVip = VipManager.isVip();

            if (isVip) {
                obj.getComponent('hdsprite2', function (hdSprite) {
                    hdSprite.getComponent('hdsprite2', function (sprite) {
                        sprite.setup({
                            imageName: 'gui/vip-check-button'
                        });
                    });
                });
            } else {
                makeButton(obj, {
                    onClick: function onClick() {
                        Bento.objects.attach(new VipPopup({}));
                    }
                });
            }

            if (!showChallengeButton) {
                Gui.adjust(obj, {
                    y: 8 + 20 * (1 - Gui.scaleFactor),
                    scale: Gui.scaleFactor
                });
            } else {
                Gui.adjust(obj, {
                    y: 20 * (1 - Gui.scaleFactor),
                    scale: Gui.scaleFactor
                });
            }

            obj.attach(new Sparkles({
                offset: new Vector2(0, 6),
                radius: new Vector2(16, 16),
                spawnRate: 1.0,
                speed: 0.8,
                scale: new Vector2(0.12, 0.12)
            }));
        },
        buttonChallenge: function buttonChallenge(obj) {
            if (!showChallengeButton) {
                obj.removeSelf();
                return;
            }
            var scheduledTime = SaveManager.load('scheduledChallenge');
            var currentTime = TimeManager.getTime();

            var levelsComplete = LevelManager.getNumberCleared(currentChallenge);
            var totalLevels = LevelManager.getTotalLevels(currentChallenge);

            var levelText = new Text({
                text: (currentTime < scheduledTime ? levelsComplete : levelsComplete + 1) + '/' + totalLevels,
                position: new Vector2(0, 5),
                scale: new Vector2(0.6, 0.6),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 64,
                maxHeight: 32,
                drawDebug: false
            });
            obj.attach(levelText);

            if (Localization.getLanguage() !== 'en') {
                // don't force the popup in other languages while we have placeholder translations. :(
                SaveManager.save('shownChallengePopup', true);
            }

            Gui.get('buttonChallengeHdSprite', function (buttonChallengeHdSprite) {
                if (!SaveManager.load('shownChallengePopup')) {
                    // wiggle the button then show the popup
                    Tween({
                        delay: 20,
                        from: 1, to: 0, in: 30,
                        ease: 'linear',
                        onUpdate: function onUpdate(v, t) {
                            buttonChallengeHdSprite.rotation = Math.sin(t * 0.5) * v * 0.2;
                            levelText.rotation = buttonChallengeHdSprite.rotation;
                        },
                        onComplete: function onComplete() {
                            Bento.objects.attach(new ChallengePopup({}));
                            SaveManager.save('shownChallengePopup', true);
                        }
                    });
                } else if (currentTime >= scheduledTime) {
                    // just do a small wiggle
                    Tween({
                        delay: 20,
                        from: 1, to: 0, in: 30,
                        ease: 'linear',
                        onUpdate: function onUpdate(v, t) {
                            buttonChallengeHdSprite.rotation = Math.sin(t * 0.5) * v * 0.08;
                            levelText.rotation = buttonChallengeHdSprite.rotation;
                        }
                    });
                    // and a second wiggle a moment later
                    Tween({
                        delay: 100,
                        from: 1, to: 0, in: 30,
                        ease: 'linear',
                        onUpdate: function onUpdate(v, t) {
                            buttonChallengeHdSprite.rotation = Math.sin(t * 0.5) * v * 0.08;
                            levelText.rotation = buttonChallengeHdSprite.rotation;
                        }
                    });
                } else if (levelsComplete > 0) {
                    // show completion badge for today's challenge
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/cage/level-done-new',
                        position: new Vector2(9.5, 10),
                        scale: new Vector2(0.07, 0.07)
                    }));
                    // levelText.position.x -= 0.5;
                    levelText.position.y -= 0.5;
                }

                Gui.adjust(obj, {
                    y: 20 * (1 - Gui.scaleFactor),
                    scale: Gui.scaleFactor
                });
            });

            makeButton(obj, {
                onClick: function onClick() {
                    Bento.objects.attach(new ChallengePopup({}));
                    Notifications.resetBadgeNumber();
                }
            });

            var allowChallengeNotifications = function allowChallengeNotifications() {
                Notifications.init(function (success) {
                    if (success) {
                        SaveManager.save('notificationsAllowed', true);
                        SaveManager.save('notifications7DayChallenge', true);
                        EventSystem.fire('allowedNotifications');
                        EventSystem.fire('scheduleNotification', {
                            id: 'challengeReminder',
                            title: Localization.getText('challengeTitle').replace('{NUMBER}', totalLevels),
                            body: Localization.getText('notificationChallengeBody'),
                            time: 24 * 60 * 60 * 1000
                        });
                    }
                });
            };
            if (levelsComplete === 1 && currentTime < scheduledTime && SaveManager.load('notificationsAsked') <= 1 && !SaveManager.load('notifications7DayChallenge')) {
                if (SaveManager.load('notificationsAllowed')) {
                    // If we already accepted the free gift, assume challenge notifications are OK too.
                    allowChallengeNotifications();
                } else {
                    // Ask to be remindeded of challenges
                    Utils.timeout(8, function () {
                        var dialog = new Dialog({
                            name: 'askChallengeNotifications',
                            type: 'yesno',
                            text: Localization.getText('askChallengeReminder'),
                            onYes: function onYes() {
                                allowChallengeNotifications();
                            },
                            onComplete: function onComplete() {
                                Gui.show('buttonNext');
                                SaveManager.add('notificationsAsked', 1);
                            }
                        });
                        dialog.getComponent('window', function (w) {
                            w.attach(new SimpleSprite({
                                imageName: '7days/calendar-smile',
                                position: new Vector2(0, -35),
                                scale: new Vector2(0.14, 0.14)
                            }));
                            w.getComponent('text', function (text) {
                                text.position.y += 20;
                            });
                        });
                        Bento.objects.attach(dialog);
                    });
                }
            }
        },
        buttonChallengeText: function buttonChallengeText(obj) {
            if (!currentChallenge) {
                obj.removeSelf();
                return;
            }
            var totalLevels = LevelManager.getTotalLevels(currentChallenge);
            makeText(obj, {
                text: function () {
                    var lang = Localization.getLanguage();
                    var s = Localization.getText('challengeTitle');
                    if (lang === 'ja') s = s.replace('の', 'の\n');
                    return s.replace('{NUMBER}', totalLevels).toUpperCase();
                }(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        crossPromo: function crossPromo(obj) {
            if (!Globals.crossPromo) {
                if (CrossPromo.canShow()) {
                    Globals.crossPromo = new CrossPromo({});
                } else {
                    return;
                }
            }
            if (Globals.crossPromo.parent) {
                Globals.crossPromo.removeSelf();
            }
            Globals.crossPromo.rotation = Utils.toRadian(-7);
            Globals.crossPromo.setSize(new Rectangle(0, 0, 92, 92));
            obj.attach(Globals.crossPromo);

            Gui.adjust(obj, {
                x: 20 * (1 - Gui.scaleFactorTall),
                y: 38 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall
            });
        }
    });

    return GuiTitleScreen;
});