'use strict';

bento.define('gui/guivictory', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'entities/gui/slicecounter', 'gui/gui', 'gui/guicommon', 'components/sparkles', 'modules/tapticengine', 'modules/levelmanager', 'modules/admanager', 'modules/savemanager', 'modules/timemanager', 'gui/darkenbar', 'components/debug', 'components/simplesprite', 'gui/dialog', 'gui/coincollectanim', 'entities/gui/tutorialhand', 'entities/gui/gamecounter', 'gui/timerlabel', 'gui/freegift', 'modules/notifications', 'modules/localization', 'gui/chestprogress', 'components/mask', 'gui/spinwheeloverlay', 'modules/vipmanager', 'gui/vippopup', 'modules/crosspromo/crosspromo', 'modules/skinmanager', 'gui/characterunlockdialog'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, SliceCounter, Gui, GuiCommon, Sparkles, TapticEngine, LevelManager, AdManager, SaveManager, TimeManager, DarkenBar, Debug, SimpleSprite, Dialog, CoinCollectAnim, TutorialHand, GameCounter, TimerLabel, FreeGift, Notifications, Localization, ChestProgress, Mask, SpinWheelOverlay, VipManager, VipPopup, CrossPromo, SkinManager, CharacterUnlockDialog) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var viewport = Bento.getViewport();

    var GuiVictory = Utils.copyObject(GuiCommon);

    var clearTextScale = Utils.approach(Gui.scaleFactorTall, 1.0, 0.2);

    // 'none', 'coins', 'hints', 'chest', 'gift', 'character'
    var rewardType = 'none';

    // when first free gift:
    // - hide next level and retry buttons
    // - point hand to free gift button
    // - ask notifications
    // - point hand to shop
    var firstFreeGift = false;

    var canShowRewardedAds = false;
    var showCrossPromo = false;
    var hasClaimedSocialReward = false; // for safety in case user declines to open twitter etc.
    var levelsCleared = void 0;
    var characterToUnlock = void 0; // id of random unlockable character (only used when rewardType === 'character')

    Utils.extend(GuiVictory, {

        savedState: null, init: function init(settings) {
            canShowRewardedAds = settings.canShowRewardedAds;

            levelsCleared = LevelManager.getNumberCleared();
            firstFreeGift = false;
            hasClaimedSocialReward = false;

            ChestProgress.incrementProgress();
            if (levelsCleared < 4) {
                rewardType = 'none';
            } else if (ChestProgress.hasChestAvailable()) {
                rewardType = 'chest'; // Note: should check rewarded ad availability??
            } else if (SaveManager.load('claimedFreeGifts') === 0) {
                firstFreeGift = true;
                rewardType = 'gift';
            } else {
                var possibleRewards = [];
                if (canShowRewardedAds) {
                    possibleRewards.push('hints');
                    possibleRewards.push('coins');
                }
                characterToUnlock = SkinManager.getRandomUnlockableCharacter();
                if (characterToUnlock) {
                    possibleRewards.push('character');
                }
                var checkSocialReward = function checkSocialReward(key) {
                    if (Utils.canOpenUrl(key) && !SaveManager.load('claimedSocial-' + key)) {
                        possibleRewards.push(key);
                    }
                };
                if (levelsCleared >= 6 && Math.random() < 0.5 && Localization.getLanguage() !== 'zh') {
                    checkSocialReward('twitter');
                    checkSocialReward('facebook');
                    checkSocialReward('instagram');
                }

                rewardType = Utils.pickRandomFrom(possibleRewards) || 'none';

                // testing character unlocks
                // if (characterToUnlock) {
                //     rewardType = 'character';
                // }
            }

            // showCrossPromo = levelsCleared >= 5;
            showCrossPromo = true;
            var showVipPopup = !VipManager.isVip() && levelsCleared >= 10 && (levelsCleared - 10) % 7 === 0 && !Globals.alreadyCleared && Globals.starsEarned === 3 && !SaveManager.load('noAds') && Utils.currentLevelGroup === 'default';

            if (showVipPopup) {
                Utils.timeout(1, function () {
                    Bento.objects.attach(new VipPopup({}));
                });
            }
        },
        root: function root(obj) {
            GuiCommon.root(obj);
            obj.attach({
                name: 'destroyHandler',
                start: function start() {
                    // save the state of this screen and all tweens
                    Bento.objects.attach({
                        z: -1,
                        name: 'victoryStateSaver',
                        destroy: function destroy() {
                            GuiVictory.savedState = Gui.getState();
                        }
                    });
                }
            });
        },
        buttonRetryWide: function buttonRetryWide(obj) {
            makeButton(obj, {
                canClick: function canClick() {
                    return showable.isShown;
                },
                onClick: function onClick() {
                    Globals.retryCount++;
                    Bento.screens.show('screens/main');
                }
            });
            var showPos = obj.position.clone();
            var hidePos = new Vector2(0, 40);
            var showable = Gui.FadeShowable({
                showTime: 80,
                isShown: false
            });
            obj.attach(showable);
            Utils.timeout(2, function () {
                obj.dimension.width += 70 * Gui.scaleFactorTall;
                // obj.attach(Debug({}));
            });

            if (rewardType === 'none') {
                Gui.adjust(obj, {
                    x: 22 * (1 - Gui.scaleFactorTall),
                    y: -30 * (1 - Gui.scaleFactorTall),
                    scale: Gui.scaleFactorTall
                });
            } else {
                Gui.adjust(obj, {
                    x: 22 * (1 - Gui.scaleFactorTall),
                    y: -22 * (1 - Gui.scaleFactorTall),
                    scale: Gui.scaleFactorTall
                });
            }
        },
        buttonRetryWideText: function buttonRetryWideText(obj) {
            makeText(obj, {
                text: Localization.getText('retry').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'left',
                linebreaks: false
            });
            obj.transform.y -= 2;
        },
        victoryContainer: function victoryContainer(obj) {
            obj.removeSelf();
            Utils.timeout(1, function () {
                Gui.show('victoryContainer');
                var y = 100 * (1 - Gui.scaleFactorTall);
                if (rewardType === 'none') {
                    y += 20 * Gui.scaleFactorTall;
                }
                Gui.adjust(obj, { y: y });
            });
            obj.attach({
                name: 'behavior',
                start: function start() {
                    if (rewardType !== 'none') {
                        Gui.show('buttonVictoryReward');
                    }
                    if (rewardType !== 'chest' && !firstFreeGift) {
                        Gui.show('buttonNext');
                        Gui.show('buttonRetryWide');
                    }

                    var starsEarned = new StarsEarned({
                        position: new Vector2(0, -94 + 60 * (1 - Gui.scaleFactor))
                    });
                    starsEarned.scale.scale(Gui.scaleFactor);
                    obj.attach(starsEarned);
                    obj.remove(this);

                    Bento.audio.playSound('sfx_clear');
                }
            });
        },
        clearContainer: function clearContainer(obj) {
            // masked container, initially contains 'COMPLETED!' text,
            // which then slides out to make room for ChestProgress animation
            // unless the level has bean cleared before

            var darkenBar = new DarkenBar({
                position: new Vector2(0, 1),
                height: 40 * clearTextScale,
                time: 30,
                color: [0, 0.02, 0.51, 0.2]
            });
            obj.attach(darkenBar);
            obj.moveComponentTo(darkenBar, 0);

            // TEMP commented
            // if (Globals.alreadyCleared) {
            //     return;
            // }

            var clearText = Gui.get('clearText');
            var clearTextStartPos = clearText && clearText.position.clone();

            var chestProgress = new ChestProgress({
                position: new Vector2(0, -42),
                scale: new Vector2(clearTextScale, clearTextScale),
                onProgressed: function onProgressed() {
                    Gui.show('coinsEarnedIcon');
                    if (rewardType === 'chest') {
                        Bento.audio.playSound('sfx_clear'); // TODO: unique sound?
                        Gui.get('buttonVictoryReward', function (btn) {
                            btn.colorify();
                        });
                        Gui.show('buttonNext');
                        Gui.show('buttonRetryWide');
                    }
                }
            });
            var chestProgressStartPos = chestProgress.position.clone();

            var showChestProgress = function showChestProgress() {
                // if level skipped, show chest progress immediately
                if (!Globals.hasWon) {
                    if (clearText) clearText.removeSelf();
                    obj.attach(chestProgress);
                    chestProgress.position.y += 42;
                    return;
                }

                Tween({
                    delay: 80,
                    from: 0, to: 42, in: 30,
                    ease: 'easeInOutQuad',
                    onStart: function onStart() {
                        var maskRect = new Rectangle(-viewport.width * 0.75, -20 * clearTextScale, viewport.width * 1.5, 40 * clearTextScale);
                        maskRect.y += obj.position.y + 1;

                        obj.attach(chestProgress);
                        Mask.applyToContainer(obj, maskRect);
                    },
                    onUpdate: function onUpdate(v, t) {
                        if (clearText) {
                            clearText.position.y = clearTextStartPos.y + v;
                            chestProgress.position.y = chestProgressStartPos.y + v;
                        }
                    },
                    onComplete: function onComplete() {
                        if (clearText) clearText.removeSelf();
                    }
                });
            };

            showChestProgress();
        },
        clearText: function clearText(obj) {
            makeText(obj, {
                text: Localization.getText('levelCompleted').toUpperCase(),
                fontSettings: TextStyles.get('heading'),
                align: 'center',
                linebreaks: false
                // drawDebug: true
            });
            obj.scale.x = obj.scale.y = 0;
            new Tween({
                delay: 8,
                from: 0, to: clearTextScale, in: 50,
                ease: 'elastic',
                decay: 9,
                oscillations: 3,
                applyOnDelay: true,
                onUpdate: function onUpdate(v, t) {
                    obj.scale.x = v;
                    obj.scale.y = v;
                }
            });
        },
        buttonVictoryReward: function buttonVictoryReward(obj) {
            if (rewardType === 'none') {
                obj.removeSelf();
            }
            var wiggle = new Wiggle({
                amplitude: 0.008
            });
            obj.attach(wiggle);

            if (rewardType === 'chest') {
                wiggle.amplitude = 0;
            }

            var tutorialHand = void 0;

            var buttonSprite = void 0;
            obj.getComponent('hdsprite3', function (hdSprite) {
                hdSprite.getComponent('hdsprite3', function (sprite) {
                    buttonSprite = sprite;
                });
            });

            // change button color depending on reward type
            var imageName = void 0;
            switch (rewardType) {
                case 'coins':
                    break;
                case 'gift':
                    buttonSprite.setup({ imageName: 'gui/button-big-green' });
                    break;
                case 'hints':
                    break;
                case 'chest':
                    buttonSprite.setup({ imageName: 'gui/button-big-grey' });
                    break;
                case 'twitter':
                    buttonSprite.setup({ imageName: 'gui/socialmedia-button-twitter' });
                    break;
                case 'facebook':
                    buttonSprite.setup({ imageName: 'gui/socialmedia-button-fb' });
                    break;
                case 'instagram':
                    buttonSprite.setup({ imageName: 'gui/socialmedia-button-instagram' });
                    break;
                case 'character':
                    buttonSprite.setup({ imageName: 'gui/button-big-yellow' });
                    break;
            }

            // exposed function for when the player icon jumps onto on the small chest icon
            obj.colorify = function () {
                var fromScale = obj.scale.x;
                var toScale = obj.scale.x * 1.08;
                // grow/pulse button
                Tween({
                    from: fromScale,
                    to: toScale,
                    in: 5,
                    ease: 'easeInOutQuad',
                    onStart: function onStart() {},
                    onUpdate: function onUpdate(v, t) {
                        obj.scale.x = v;
                        obj.scale.y = v;
                    },
                    onComplete: function onComplete() {
                        // change all images to colored versions
                        buttonSprite.setup({ imageName: 'gui/button-big-green' });
                        Gui.get('victoryRewardIcon', function (icon) {
                            if (icon.colorify) {
                                icon.colorify();
                            }
                        });
                        // shrink back to original size
                        Tween({
                            from: toScale,
                            to: fromScale,
                            in: 26,
                            ease: 'easeInOutQuad',
                            onUpdate: function onUpdate(v, t) {
                                obj.scale.x = v;
                                obj.scale.y = v;
                            }
                        });
                    }
                });
                // start wiggling
                Tween({
                    from: 0, to: 0.008, in: 60,
                    onUpdate: function onUpdate(v, t) {
                        wiggle.amplitude = v;
                    }
                });
            };

            var claimedChest = false;
            var checkSkippedChest = function checkSkippedChest() {
                if (!claimedChest) {
                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                        eventId: 'game:chest:ignored'
                    });
                }
                EventSystem.off('screenHidden', checkSkippedChest);
            };
            if (rewardType === 'chest') {
                EventSystem.on('screenHidden', checkSkippedChest);
            }

            var hasClicked = false;
            makeButton(obj, {
                shadow: 'gui/button-big-shadow',
                canClick: function canClick() {
                    return !hasClicked;
                },
                onClick: function onClick() {

                    var wiggleButtonNext = function wiggleButtonNext() {
                        obj.removeByName('wiggle');
                        Gui.get('buttonNext', function (buttonNext) {
                            var btnNextWiggle = new Wiggle({
                                amplitude: 0
                            });
                            buttonNext.attach(btnNextWiggle);
                            Tween({
                                from: 0, to: 0.01, in: 60,
                                ease: 'linear',
                                onUpdate: function onUpdate(v, t) {
                                    btnNextWiggle.amplitude = v;
                                }
                            });
                        });
                    };

                    var giveCoinReward = function giveCoinReward(amount) {
                        var oldCoins = SaveManager.load('coins');
                        var newCoins = oldCoins + amount;
                        var diff = newCoins - oldCoins;
                        SaveManager.save('coins', newCoins);

                        // returned function can be used to animate the coin reward when ready
                        return function () {
                            var delay = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

                            obj.attach(new SimpleSprite({
                                imageName: 'gui/button-coin-c',
                                position: new Vector2(-16, 0),
                                scale: new Vector2(0.16, 0.16)
                            }));
                            obj.attach(new Text({
                                text: '+' + amount,
                                position: new Vector2(10, 0),
                                scale: new Vector2(0.7, 0.7),
                                fontSettings: TextStyles.get('main'),
                                align: 'left',
                                maxWidth: 64,
                                maxHeight: 32,
                                drawDebug: false
                            }));
                            Utils.timeout(delay, function () {
                                CoinCollectAnim({
                                    from: obj.getWorldPosition().add(Vector2(-10, 0)),
                                    to: Gui.get('coinCounter').getWorldPosition(),
                                    amount: 25,
                                    interval: 3,
                                    duration: 26,
                                    onCoinCollect: function onCoinCollect(i) {
                                        EventSystem.fire('updateCoins', Math.floor(oldCoins + diff * i / 25));
                                    }
                                });
                            });
                            wiggleButtonNext();
                        };
                    };

                    var giveHintReward = function giveHintReward(amount) {
                        var oldHints = SaveManager.load('hints');
                        var hints = oldHints + amount;
                        SaveManager.save('hints', hints);

                        // returned function can be used to animate gaining hints
                        return function () {
                            var delay = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;


                            var makeBulb = function makeBulb(n) {
                                var bulb = new Entity({
                                    z: Globals.Layers.GUI,
                                    name: 'bulb',
                                    position: obj.toWorldPosition(new Vector2((n + 0.5 - amount / 2) * 30, 10 - (1 - Gui.scaleFactor) * 40)),
                                    scale: new Vector2(0.10, 0.10),
                                    components: [new Sprite({
                                        imageName: 'gui/hint-icon',
                                        originRelative: new Vector2(0.5, 0.5)
                                    })]
                                });
                                var scale = bulb.scale.x;

                                // grow bulb
                                new Tween({
                                    delay: n * 8,
                                    from: 0, to: scale, in: 30,
                                    ease: 'easeInOutQuad',
                                    onStart: function onStart() {
                                        Bento.objects.attach(bulb);
                                    },
                                    onUpdate: function onUpdate(v, t) {
                                        bulb.scale.x = v;
                                        bulb.scale.y = v;
                                    },
                                    onComplete: function onComplete() {
                                        var moveDelay = 24 + n * 8;

                                        // adjust bulb scale to match hint counter icon
                                        Tween({
                                            delay: moveDelay - 8,
                                            from: bulb.scale.x,
                                            to: hintIcon.scale.x,
                                            in: 30,
                                            ease: 'easeOutBack',
                                            onUpdate: function onUpdate(v, t) {
                                                bulb.scale.x = v;
                                                bulb.scale.y = v;
                                            }
                                        });

                                        var hintIconScale = hintIcon.scale.x; // main counter icon scale

                                        // move bulb to hint counter
                                        Tween({
                                            delay: moveDelay,
                                            from: bulb.position.clone(),
                                            to: Gui.get('root').position.add(showPos),
                                            in: 30,
                                            ease: 'easeInOutQuad',
                                            onUpdate: function onUpdate(v, t) {
                                                bulb.position.x = v.x;
                                                bulb.position.y = v.y;
                                            },
                                            onComplete: function onComplete() {
                                                bulb.removeSelf();
                                                // increase and pulse hint counter value
                                                Tween({
                                                    from: 1.1, to: 1.0, in: 12,
                                                    ease: 'easeOutQuad',
                                                    onStart: function onStart() {
                                                        counter.setValue(oldHints + (n + 1));
                                                    },
                                                    onUpdate: function onUpdate(v, t) {
                                                        counter.scale.x = scale * v;
                                                        counter.scale.y = scale * v;
                                                        hintIcon.scale.x = hintIconScale * v;
                                                        hintIcon.scale.y = hintIconScale * v;
                                                    }
                                                });
                                            }
                                        });
                                    }
                                });
                            };

                            Utils.timeout(delay, function () {
                                Utils.repeat(amount, makeBulb);
                            });

                            var starCounter = Gui.get('starCounter');
                            var coinCounter = Gui.get('coinCounter');
                            Gui.hide('starCounter');
                            Gui.hide('coinCounter');
                            var pos1 = starCounter ? starCounter.position : new Vector2(73, -viewport.height * 0.5 + 25);
                            var pos2 = coinCounter ? coinCounter.position : new Vector2(73, -viewport.height * 0.5 + 46);
                            var showPos = pos1.add(pos2).scale(0.5);
                            var hidePos = showPos.add(Vector2(30, 0));
                            var counter = new GameCounter({
                                name: 'coinCounterCounter',
                                value: oldHints,
                                align: 'right',
                                position: new Vector2(-10, 2),
                                scale: new Vector2(0.095, 0.095)
                            });
                            var hintIcon = new SimpleSprite({
                                imageName: 'gui/hint-icon',
                                scale: new Vector2(0.06, 0.06)
                            });
                            var hintCounter = new Entity({
                                name: 'hintCounter',
                                position: hidePos.clone(),
                                components: [counter, hintIcon]
                            });
                            Tween({
                                delay: 20,
                                from: hidePos,
                                to: showPos,
                                in: 28,
                                ease: 'easeOutBack',
                                onStart: function onStart() {
                                    Gui.attach(hintCounter);
                                },
                                onUpdate: function onUpdate(v, t) {
                                    hintCounter.position.x = v.x;
                                    hintCounter.position.y = v.y;
                                },
                                onComplete: function onComplete() {
                                    wiggleButtonNext();
                                }
                            });
                        };
                    };

                    switch (rewardType) {
                        case 'coins':
                            AdManager.tryShowRewarded({
                                onReward: function onReward() {
                                    // clear the button and give reward
                                    obj.components.forEach(function (comp) {
                                        obj.remove(comp);
                                    });
                                    giveCoinReward(50)(); // animate immediately
                                },
                                onComplete: function onComplete(error) {
                                    if (error) {
                                        // clear the button and give error
                                        obj.components.forEach(function (comp) {
                                            obj.remove(comp);
                                        });
                                        obj.attach(new Text({
                                            text: Localization.getText('error'),
                                            position: new Vector2(0, 0),
                                            scale: new Vector2(0.6, 0.6),
                                            fontSettings: TextStyles.get('main'),
                                            align: 'center',
                                            maxWidth: 180,
                                            maxHeight: 64,
                                            drawDebug: false
                                        }));
                                    }
                                },

                                tag: 'Victory_Reward_Coins',
                                rewardType: 'coins',
                                rewardAmount: 50
                            });
                            break;

                        case 'hints':
                            AdManager.tryShowRewarded({
                                onReward: function onReward() {
                                    obj.components.forEach(function (comp) {
                                        obj.remove(comp);
                                    });
                                    giveHintReward(2)(); // animate immediately
                                },
                                onComplete: function onComplete(error) {
                                    if (error) {
                                        obj.components.forEach(function (comp) {
                                            obj.remove(comp);
                                        });
                                        obj.attach(new Text({
                                            text: Localization.getText('error'),
                                            position: new Vector2(0, 0),
                                            scale: new Vector2(0.6, 0.6),
                                            fontSettings: TextStyles.get('main'),
                                            align: 'center',
                                            maxWidth: 180,
                                            maxHeight: 64,
                                            drawDebug: false
                                        }));
                                    }
                                },

                                tag: 'Victory_Reward_Hints',
                                rewardType: 'hint',
                                rewardAmount: 2
                            });
                            break;

                        // First free gift:

                        case 'gift':

                            // clear the button and give reward
                            var doCoinAnimation = giveCoinReward(9999999);
                            var originalScale = void 0;
                            Tween({
                                from: 1.0, to: 0.0, in: 20,
                                ease: 'easeInBack',
                                onStart: function onStart() {
                                    originalScale = obj.scale.x;
                                },
                                onUpdate: function onUpdate(v, t) {
                                    obj.scale.x = v * originalScale;
                                    obj.scale.y = v * originalScale;
                                },
                                onComplete: function onComplete() {
                                    obj.components.forEach(function (comp) {
                                        obj.remove(comp);
                                    });
                                    doCoinAnimation();
                                    Tween({
                                        from: 0.0, to: 1.0, in: 20,
                                        ease: 'easeOutQuad',
                                        onUpdate: function onUpdate(v, t) {
                                            obj.scale.x = v * originalScale;
                                            obj.scale.y = v * originalScale;
                                        }
                                    });
                                    if (tutorialHand) {
                                        tutorialHand.hide();
                                    }

                                    // Direct player to skin shop:

                                    // remove all navbar buttons except buttonCharSelect
                                    Gui.hide('buttonNoAds');
                                    Gui.hide('buttonLevelSelect');
                                    Gui.hide('buttonIapShop');

                                    Gui.show('navbar');

                                    Gui.get('buttonCharacterSelect', function (btn) {
                                        // center navbar
                                        Gui.get('navbar', function (navbar) {
                                            navbar.getComponent('behavior', function (navbarBehavior) {
                                                navbarBehavior.center();
                                            });
                                        });
                                        // center shop button within navbar
                                        Gui.adjust(btn, {
                                            x: -btn.position.x,
                                            y: 8
                                        });

                                        Utils.timeout(100, function () {
                                            // show tutorial hand
                                            var shopHand = new TutorialHand({
                                                position: btn.position.add(Vector2(0, -16)),
                                                innerScale: 0.14
                                            });
                                            if (btn.parent) {
                                                btn.parent.attach(shopHand);
                                            }
                                            shopHand.rotation += Math.PI;
                                        });
                                    });

                                    var claimedFreeGifts = SaveManager.load('claimedFreeGifts');
                                    SaveManager.save('claimedFreeGifts', claimedFreeGifts + 1);
                                    SaveManager.save('scheduledFreeGift', TimeManager.getTime() + Globals.freeGiftTime(claimedFreeGifts + 1));

                                    var nextGiftButton = new FreeGift({
                                        dimension: obj.dimension.clone(),
                                        style: 'big'
                                    });
                                    nextGiftButton.position.x += 200 / originalScale;

                                    obj.attach(Tween.TweenBehavior({
                                        delay: 110,
                                        from: obj.position.x,
                                        to: obj.position.x - nextGiftButton.position.x * originalScale,
                                        in: 34,
                                        applyOnDelay: 0,
                                        ease: 'easeInOutQuad',
                                        onStart: function onStart() {
                                            obj.attach(nextGiftButton);
                                        },
                                        onUpdate: function onUpdate(v, t) {
                                            obj.position.x = v;
                                        }
                                    }));

                                    // When the player returns to this GUI from the shop:
                                    Gui.get('root').attach({
                                        timesResumed: -1,
                                        start: function start() {
                                            if (++this.timesResumed === 1) {

                                                // move back up 
                                                obj.position.y -= 20;

                                                // reset navbar
                                                Gui.get('navbar', function (navbar) {
                                                    navbar.components.forEach(function (c) {
                                                        return navbar.remove(c);
                                                    });
                                                    GuiCommon.navbar(navbar);
                                                });
                                                // Ask to be remindeded 
                                                if (SaveManager.load('notificationsAsked') === 0 && (!SaveManager.load('notificationsAllowed') || !SaveManager.load('notificationsFreeGift'))) {
                                                    Utils.timeout(8, function () {
                                                        Bento.objects.attach(new Dialog({
                                                            name: 'askNotifications',
                                                            type: 'yesno',
                                                            text: Localization.getText('askFreeGiftReminder'),
                                                            onYes: function onYes() {
                                                                Notifications.init(function (success) {
                                                                    if (success) {
                                                                        SaveManager.save('notificationsAllowed', true);
                                                                        SaveManager.save('notificationsFreeGift', true);
                                                                        EventSystem.fire('allowedNotifications');
                                                                        EventSystem.fire('scheduleNotification', {
                                                                            id: 'freeGiftReminder',
                                                                            title: Localization.getText('notifications').freeGiftTitle,
                                                                            body: Localization.getText('notifications').freeGiftBody,
                                                                            time: SaveManager.load('scheduledFreeGift') - TimeManager.getTime()
                                                                        });
                                                                    }
                                                                });
                                                            },
                                                            onComplete: function onComplete() {
                                                                Gui.show('buttonNext');
                                                                SaveManager.add('notificationsAsked', 1);
                                                            }
                                                        }));
                                                    });
                                                } else {
                                                    Gui.show('buttonNext');
                                                }
                                            }
                                        },
                                        destroy: function destroy() {}
                                    });
                                }
                            });
                            break;

                        case 'chest':
                            AdManager.tryShowRewarded({
                                onReward: function onReward() {
                                    obj.components.forEach(function (comp) {
                                        obj.remove(comp);
                                    });
                                    claimedChest = true;
                                    EventSystem.fire('GameAnalytics-addDesignEvent', {
                                        eventId: 'game:chest:rewardSuccess'
                                    });
                                    ChestProgress.resetProgress();
                                    var overlay = new SpinWheelOverlay({
                                        onComplete: function onComplete(item) {
                                            // console.log('Got Item: ', item);
                                            switch (item.type) {
                                                case 'coins':
                                                    giveCoinReward(item.amount)(20);
                                                    break;
                                                case 'hints':
                                                    giveHintReward(item.amount)(20);
                                                    break;
                                                default:
                                                    Utils.log('Unknown chest reward type.');
                                            }
                                            Tween({
                                                from: 0.0, to: obj.scale.x, in: 30,
                                                ease: 'easeOutBack',
                                                onStart: function onStart() {
                                                    originalScale = obj.scale.x;
                                                },
                                                onUpdate: function onUpdate(v, t) {
                                                    obj.scale.x = v;
                                                    obj.scale.y = v;
                                                }
                                            });
                                        }
                                    });
                                    Bento.objects.attach(overlay);
                                },
                                onComplete: function onComplete(error) {
                                    if (error) {
                                        obj.components.forEach(function (comp) {
                                            obj.remove(comp);
                                        });
                                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                                            eventId: 'game:chest:rewardFail'
                                        });
                                        obj.attach(new Text({
                                            text: Localization.getText('error'),
                                            position: new Vector2(0, 0),
                                            scale: new Vector2(0.6, 0.6),
                                            fontSettings: TextStyles.get('main'),
                                            align: 'center',
                                            maxWidth: 180,
                                            maxHeight: 64,
                                            drawDebug: false
                                        }));
                                    }
                                },

                                tag: 'Victory_Reward_Chest',
                                rewardType: 'chest',
                                rewardAmount: 1
                            });
                            break;

                        case 'facebook':
                        case 'twitter':
                        case 'instagram':

                            var giveSocialReward = function giveSocialReward() {
                                // Only give this once per screen.
                                // Otherwise user can repeatedly decline the iOS prompt for opening Twitter
                                //  then suspend/resume the app to get a huge amount of coins.
                                if (!hasClaimedSocialReward) {
                                    hasClaimedSocialReward = true;
                                    // clear the button and give reward
                                    obj.components.forEach(function (comp) {
                                        return obj.remove(comp);
                                    });
                                    giveCoinReward(50)();
                                    SaveManager.save('claimedSocial-' + rewardType, true);
                                }
                            };

                            var onDocResume = function onDocResume() {
                                giveSocialReward();
                                _onRemove();
                            };

                            var _onRemove = function _onRemove() {
                                EventSystem.off('documentResume', onDocResume);
                                EventSystem.off('screenHidden', _onRemove);
                            };

                            EventSystem.on('documentResume', onDocResume);
                            EventSystem.on('screenHidden', _onRemove); // in case we somehow never got the resume event

                            if (Utils.isNativeIos() && Utils.canOpenUrl(rewardType)) {
                                Utils.openUrl(Utils.urlSchemes[rewardType]);
                            } else {
                                Utils.openUrl(Utils.links[rewardType]);
                                Utils.timeout(30, giveSocialReward);
                            }
                            break;

                        case 'character':
                            AdManager.tryShowRewarded({
                                onReward: function onReward() {
                                    obj.components.forEach(function (comp) {
                                        return obj.remove(comp);
                                    });
                                    Bento.objects.attach(new CharacterUnlockDialog({
                                        characterId: characterToUnlock,
                                        onComplete: function onComplete() {
                                            wiggleButtonNext();
                                        }
                                    }));
                                },
                                onComplete: function onComplete(error) {
                                    if (error) {
                                        obj.components.forEach(function (comp) {
                                            return obj.remove(comp);
                                        });
                                        obj.attach(new Text({
                                            text: Localization.getText('error'),
                                            position: new Vector2(0, 0),
                                            scale: new Vector2(0.6, 0.6),
                                            fontSettings: TextStyles.get('main'),
                                            align: 'center',
                                            maxWidth: 180,
                                            maxHeight: 64,
                                            drawDebug: false
                                        }));
                                    }
                                },

                                tag: 'Victory_Reward_Character',
                                rewardType: 'character',
                                rewardAmount: 1
                            });
                            break;
                    }
                }
            });

            // let giftTest = new FreeGift({
            //     dimension: obj.dimension.clone()
            // });
            // Gui.attach(giftTest);
            // let giftTest2 = new FreeGift({
            //     dimension: obj.dimension.clone(),
            //     style: 'big'
            // });
            // Gui.attach(giftTest2);
            // giftTest2.position.y += 140;

            var showPos = void 0,
                hidePos = void 0;
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);

                            if (firstFreeGift) {
                                showPos.y += 20;
                                Utils.timeout(90, function () {
                                    if (!obj.getComponent('clickable')) {
                                        return; // user already pressed the button
                                    }
                                    tutorialHand = new TutorialHand({
                                        position: obj.toWorldPosition(Vector2(0, 80 * obj.scale.x)),
                                        innerScale: 0.16 * obj.scale.x
                                    });
                                    Bento.objects.attach(tutorialHand);
                                });
                            }
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            Gui.adjust(obj, {
                y: 50 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall,
                onAdjust: function onAdjust() {
                    showPos = obj.position.clone();
                    hidePos = new Vector2(0, viewport.height + 20);
                    obj.attach(showable);
                }
            });
        },
        victoryRewardText: function victoryRewardText(obj) {
            var useEn = ['ru', 'zt', 'ko', 'ja'].indexOf(Localization.getLanguage()) === -1; // 'follow us'/'like us' localisation workaround

            switch (rewardType) {
                case 'coins':
                    makeText(obj, {
                        text: Localization.getText('levelRewardCoins').replace('{NUMBER}', '50').toUpperCase(),
                        fontSettings: TextStyles.get('main'),
                        align: 'center'
                    });
                    break;
                case 'gift':
                    makeText(obj, {
                        text: function () {
                            var lang = Localization.getLanguage();
                            var s = Localization.getText('freeGift');
                            if (lang === 'ja') {
                                s = s.replace('無料', '無料\n'); // linebreak after 'free'
                            }
                            return s.toUpperCase();
                        }(),
                        fontSettings: TextStyles.get('main'),
                        align: 'center'
                    });
                    break;
                case 'hints':
                    makeText(obj, {
                        text: Localization.getText('levelRewardHints').replace('{NUMBER}', '2').toUpperCase(),
                        fontSettings: TextStyles.get('main'),
                        align: 'center'
                    });
                    break;
                case 'chest':
                    makeText(obj, {
                        text: Localization.getText('levelRewardChest').toUpperCase(),
                        fontSettings: TextStyles.get('main'),
                        align: 'center'
                    });
                    break;
                case 'facebook':
                    obj.position.x -= 6;
                    obj.dimension.width += 10;
                    makeText(obj, {
                        text: useEn ? 'LIKE US!' : Localization.getText('facebook').toUpperCase(), // TODO: get like us localised
                        fontSettings: TextStyles.get('buttonLabel'),
                        align: 'center'
                    });
                    break;
                case 'twitter':
                    obj.position.x -= 6;
                    obj.dimension.width += 10;
                    makeText(obj, {
                        text: useEn ? 'FOLLOW US!' : Localization.getText('twitter').toUpperCase(), // TODO: get follow us localised
                        fontSettings: TextStyles.get('buttonLabel'),
                        align: 'center'
                    });
                    break;
                case 'instagram':
                    obj.position.x -= 6;
                    obj.dimension.width += 10;
                    makeText(obj, {
                        text: useEn ? 'FOLLOW US!' : Localization.getText('instagram').toUpperCase(), // TODO: get follow us localised
                        fontSettings: TextStyles.get('buttonLabel'),
                        align: 'center'
                    });
                    break;
                case 'character':
                    obj.dimension.width += 12;
                    makeText(obj, {
                        // text: Localization.getText('freeCharacter').toUpperCase(),
                        text: useEn ? 'FREE CHARACTER!' : Localization.getText('free').toUpperCase(), // TODO: get freeCharacter localised
                        fontSettings: TextStyles.get('main'),
                        align: 'center'
                    });
                    Gui.adjust(obj, { x: -2 });
                    break;
                default:
                    // none
                    break;
            }
        },
        victoryRewardIcon: function victoryRewardIcon(obj) {
            switch (rewardType) {
                case 'coins':
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/button-coin-c',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.165, 0.165)
                    }));
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/ad-badge',
                        position: new Vector2(17, 16),
                        scale: new Vector2(0.112, 0.112)
                    }));
                    break;
                case 'gift':
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/gift-icon',
                        position: new Vector2(0, 0),
                        scale: new Vector2(0.12, 0.12)
                    }));
                    break;
                case 'hints':
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/hint-icon',
                        position: new Vector2(0, -4),
                        scale: new Vector2(0.1, 0.1)
                    }));
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/ad-badge',
                        position: new Vector2(16, 11),
                        scale: new Vector2(0.112, 0.112)
                    }));
                    break;
                case 'chest':
                    var chestSpr = new SimpleSprite({
                        imageName: 'gui/chest-grey',
                        position: new Vector2(0, -1),
                        scale: new Vector2(0.12, 0.12)
                    });
                    var adBadgeSpr = new SimpleSprite({
                        imageName: 'gui/ad-badge-grey',
                        position: new Vector2(20, 15),
                        scale: new Vector2(0.112, 0.112)
                    });
                    obj.attach(chestSpr);
                    obj.attach(adBadgeSpr);
                    obj.colorify = function () {
                        chestSpr.setImage('gui/chest');
                        adBadgeSpr.setImage('gui/ad-badge');
                    };
                    break;
                case 'twitter':
                case 'facebook':
                case 'instagram':
                    obj.attach(new Text({
                        text: '50',
                        position: new Vector2(-8, 18),
                        scale: new Vector2(0.5, 0.5),
                        fontSettings: TextStyles.get('buttonLabel'),
                        align: 'center',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    }));
                    break;
                case 'character':
                    obj.attach(new Entity({
                        name: 'charIcon',
                        position: new Vector2(-10, -8),
                        scale: new Vector2(1.5, 1.5),
                        components: [new Sprite({
                            imageName: 'character' + characterToUnlock + '-idle',
                            originRelative: new Vector2(0.5, 0.5),
                            frameCountX: 5,
                            animations: {
                                default: { speed: 1, frames: [0] }
                            }
                        })]
                    }));
                    obj.attach(new SimpleSprite({
                        imageName: 'gui/ad-badge',
                        position: new Vector2(10, 12),
                        scale: new Vector2(0.112, 0.112)
                    }));
                    break;
                default:
                    // none

                    break;
            }
        },
        buttonNext: function buttonNext(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                onClick: function onClick() {
                    EventSystem.fire('nextLevel');
                }
            });
            var showPos = void 0,
                hidePos = void 0;
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });

            if (rewardType === 'chest') {
                // make it blue instead
                obj.getComponent('hdsprite1', function (hdSprite) {
                    hdSprite.getComponent('hdsprite1', function (sprite) {
                        sprite.setup({
                            imageName: 'gui/button-wide-blue'
                        });
                    });
                });
                // fade in instead of springing in
                showable = new Gui.FadeShowable({
                    isShown: false,
                    showTime: 80,
                    test: true
                });
            }

            if (rewardType === 'none') {
                obj.attach(new Wiggle({
                    amplitude: 0.02
                }));
                Gui.adjust(obj, {
                    x: 0,
                    y: -20 * Gui.scaleFactorTall,
                    scale: Math.min(Gui.scaleFactorTall + 0.2, 1.0),
                    onAdjust: function onAdjust() {
                        showPos = obj.position.clone();
                        hidePos = new Vector2(0, viewport.height + 20);
                        obj.attach(showable);
                    }
                });
            } else {
                var scale = Gui.scaleFactorTall;
                Gui.adjust(obj, {
                    x: 0,
                    y: 4 * (1 - Gui.scaleFactorTall),
                    scale: scale,
                    onAdjust: function onAdjust() {
                        showPos = obj.position.clone();
                        hidePos = new Vector2(0, viewport.height + 20);
                        obj.attach(showable);
                    }
                });
            }
            // Gui.adjust(obj, {
            //     x: 40 * (1-Gui.scaleFactorTall),
            //     y: -16 * (1-Gui.scaleFactorTall),
            //     scale: Gui.scaleFactorTall,
            //     onAdjust () {

            //     }
            // });
        },
        buttonNextText: function buttonNextText(obj) {
            makeText(obj, {
                text: Localization.getText('nextLevel').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        victoryLevelName: function victoryLevelName(obj) {
            makeText(obj, {
                text: Localization.getText('levelN').replace('{NUMBER}', Utils.currentLevel + 1).toUpperCase(),
                fontSettings: TextStyles.get('levelName'),
                align: 'center'
            });
            Utils.timeout(1, function () {
                Gui.adjust(obj, {
                    y: 20 * (1 - Gui.scaleFactorTall),
                    scale: clearTextScale
                });
            });
        },
        coinCounter: function coinCounter(obj) {
            obj.removeSelf();
            GuiCommon.coinCounter(obj, {
                value: Globals.oldCoins
            });
            obj.position.y -= 20 * (1 - Gui.scaleFactorTall);
            if (Gui.scaleFactorTall < 1.0) {
                obj.position.y -= 8;
            }
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(40, 0));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 30,
                        ease: 'easeOutQuart',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, t / 30, 1);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 20,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, 1 - t / 20, 1);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
            Utils.timeout(1, function () {
                Gui.show('coinCounter');
            });
        },
        starCounter: function starCounter(obj) {
            obj.removeSelf();
            GuiCommon.starCounter(obj);

            obj.position.y -= 14 * (1 - Gui.scaleFactorTall);
            if (Gui.scaleFactorTall < 1.0) {
                obj.position.y -= 8;
            }
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(40, 0));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 30,
                        ease: 'easeOutQuart',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, t / 30, 1);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 20,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, 1 - t / 20, 1);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
            Utils.timeout(1, function () {
                Gui.show('starCounter');
            });
        },
        coinsEarnedIcon: function coinsEarnedIcon(obj) {
            obj.removeSelf();

            var diff = SaveManager.load('coins') - Globals.oldCoins;

            // don't show if we didn't earny any new coins
            if (diff === 0) {
                obj.attach(new Gui.Showable({
                    show: function show() {
                        return false;
                    }
                }));
                return;
            }

            obj.attach(new Text({
                text: '+' + diff,
                position: new Vector2(12, 0),
                scale: new Vector2(0.5, 0.5),
                fontSettings: TextStyles.get('main'),
                align: 'left',
                maxWidth: 50,
                maxHeight: 32
                // drawDebug: true
            }));

            var hdSprite = obj.getComponent('coinsEarnedHdSprite');
            var hdSpriteScale = hdSprite.scale.x;

            var shadowSprite = new HDSprite({
                name: 'shadow',
                imageName: 'gui/coin-shadow',
                gamepixelDimension: obj.dimension.clone(),
                originRelative: new Vector2(0.5, 0.5)
            });
            shadowSprite.position.x = 1;
            shadowSprite.position.y = 1;
            obj.attach(shadowSprite);
            obj.moveComponentTo(shadowSprite, 1);

            var initialDelay = 36;
            var amount = void 0; // how many coins to show in the animation
            var interval = void 0; // how much time between each coin spawn
            if (diff <= 5) {
                initialDelay += 20;
                amount = 5;
                interval = 6;
            } else if (diff <= 10) {
                initialDelay += 40;
                amount = 7;
                interval = 5.5;
            } else {
                initialDelay += 60;
                amount = 10;
                interval = 5;
            }

            var showable = new Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    Utils.timeout(20, function () {
                        var fromCoins = SaveManager.load('coins') - diff; // reload coins in case they changed in the meantime.
                        CoinCollectAnim({
                            from: obj.getWorldPosition(),
                            to: Gui.get('coinCounter').getWorldPosition(),
                            amount: amount,
                            interval: interval,
                            duration: 26,
                            onCoinSpawn: function onCoinSpawn(i) {
                                new Tween({
                                    from: 1.2, to: 1, in: 6,
                                    ease: 'easeOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        hdSprite.scale.x = hdSpriteScale * v;
                                        hdSprite.scale.y = hdSpriteScale * v;
                                        shadowSprite.scale.x = hdSpriteScale * v;
                                        shadowSprite.scale.y = hdSpriteScale * v;
                                    }
                                });
                            },
                            onCoinCollect: function onCoinCollect(i) {
                                EventSystem.fire('updateCoins', Math.floor(fromCoins + diff * i / amount));
                            }
                        });
                    });
                    return Tween({
                        from: 0, to: 1, in: 20,
                        delay: 0,
                        applyOnDelay: 0,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            obj.transform.y = 16 * -(1 - v);
                            obj.alpha = v;
                        }
                    });
                },
                hide: function hide(parent) {
                    parent.remove(obj);
                }
            });
            obj.attach(showable);

            Gui.adjust(obj, {
                y: -18 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactor,
                onAdjust: function onAdjust() {
                    if (!firstFreeGift) {
                        Gui.show('crossPromo');
                    }
                }
            });
        },
        navbar: function navbar(obj) {
            Gui.adjust(obj, {
                y: 10 * (1 - Gui.scaleFactorTall),
                onAdjust: function onAdjust() {
                    GuiCommon.navbar(obj);
                    obj.attach({
                        name: 'navbarReady',
                        start: function start() {
                            obj.remove(this);

                            // instant hide
                            Gui.hide(obj);
                            obj.removeSelf();

                            if (!firstFreeGift) {
                                // animate in
                                Gui.show(obj);
                            }
                        }
                    });
                }
            });
        },
        crossPromo: function crossPromo(obj) {
            if (!showCrossPromo) {
                // CrossPromo not supposed to be shown on this screen (e.g. for tutorial reasons)
                obj.removeSelf();
                return;
            }
            if (!Globals.crossPromo) {
                if (CrossPromo.canShow()) {
                    Globals.crossPromo = new CrossPromo({});
                } else {
                    // CrossPromo not initialised or unable to be shown.
                    obj.removeSelf();
                    return;
                }
            }

            if (Globals.crossPromo.parent) {
                Globals.crossPromo.removeSelf();
            }
            Globals.crossPromo.rotation = Utils.toRadian(-5);
            Globals.crossPromo.setSize(new Rectangle(0, 0, 86, 86));

            var showPos = void 0,
                hidePos = void 0;
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        delay: 8,
                        from: hidePos, to: showPos, in: 36,
                        ease: 'easeOutQuad',
                        onStart: function onStart() {
                            parent.attach(obj);
                            obj.visible = true;
                            parent.moveComponentTo(obj, 0);

                            if (Globals.crossPromo && !Globals.crossPromo.parent && CrossPromo.canShow()) {
                                obj.attach(Globals.crossPromo); // begin crosspromo when shown
                            }
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                            obj.alpha = Utils.clamp(0, t / 30, 1);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInQuad',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });

            Gui.adjust(obj, {
                x: 10 * (1 - Gui.scaleFactorTall),
                y: 30 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall,
                onAdjust: function onAdjust() {
                    showPos = obj.position.clone();
                    hidePos = new Vector2(showPos.x, viewport.height + 20);
                    obj.attach(showable);
                    if (!firstFreeGift) {
                        Gui.show(obj);
                    }
                }
            });
        }
    });

    return GuiVictory;
});