'use strict';

/**
 * Overlay to show what the game will look like on an iPhone X, or potentially other 'notch' devices.
 * Attach this entity to the game once, it will persist between screens.
 * If autoHide is true, it will only be visibile when the aspect ratio matches the iPhone X
 * 
 * @moduleName IPhoneXOverlay
 * @snippet IPhoneXOverlay.snippet
IPhoneXOverlay({
    z: Globals.Layers.IPHONE_X_OVERLAY,
    orientation: 'portrait',
    autoHide: true
})
 */
bento.define('gui/iphonexoverlay', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Sprite, Entity, EventSystem, Utils, Tween) {
    'use strict';

    var ratioLandscape = 2436 / 1125;
    var ratioPortrait = 1125 / 2436;
    var isIPhoneXAspectRatio = function isIPhoneXAspectRatio(viewport) {
        var ratio = viewport.width / viewport.height;
        return Math.abs(ratio - ratioLandscape) < 0.01 || Math.abs(ratio - ratioPortrait) < 0.01;
    };

    return function (settings) {
        var viewport = Bento.getViewport();

        var fixedOrientation = settings.orientation != null;
        var isLandscape = settings.orientation && settings.orientation !== 'portrait';
        var autoHide = settings.autoHide || true;
        var rotation = 0;

        switch (settings.orientation) {
            case 'landscape':
            case 'landscapeLeft':
                rotation = Math.PI * -0.5;
                break;
            case 'landscapeRight':
                rotation = Math.PI * 0.5;
                break;
        }

        var sprite = new Sprite({
            imageName: 'iphone-x-overlay/iphone-x',
            originRelative: new Vector2(0.5, 0.5)
        });

        var behavior = {
            name: 'behavior',
            update: function update() {
                if (isLandscape) {
                    entity.scale.x = viewport.height / entity.dimension.width;
                    entity.scale.y = viewport.width / entity.dimension.height;
                } else {
                    entity.scale.x = viewport.width / entity.dimension.width;
                    entity.scale.y = viewport.height / entity.dimension.height;
                }

                if (settings.autoHide) {
                    entity.visible = isIPhoneXAspectRatio(viewport);
                }
            }
        };

        var entity = new Entity({
            z: settings.z,
            name: 'iPhoneXOverlay',
            updateWhenPaused: 0,
            float: true,
            global: true,
            rotation: rotation,
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5),
            components: [sprite, behavior]
        });

        // return entity;
        return {};
    };
});