'use strict';

/**
 * Notification popup
 * @moduleName Notification
 * @snippet Notification|FireEvent
EventSystem.fire('notification', {
    text: '', // notification body text
    textId: '', // alternatively: Localization text id
    icon: null // icon entity
});
 * @snippet Notification|manual
Notification.add('${1:textString}', ${2:iconEntity});
 */
bento.define('gui/notification', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'modules/localization', 'modules/textstyles', 'globals'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, Localization, TextStyles, Globals) {
    'use strict';

    var pauseLevel = 100;
    var Notification = function Notification(settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var width = 120;
        var height = 48;
        var icon = settings.icon;

        var text = new Text({
            fontSettings: TextStyles.get('main'),
            position: new Vector2(0, -2),
            align: 'center',
            text: settings.text,
            maxWidth: 130,
            maxHeight: 22
            // drawDebug: true
        });
        // var iconBack = new Entity({
        //     name: 'iconBack',
        //     position: new Vector2(-width / 2 + 22, 0),
        //     components: [
        //         new Sprite({
        //             imageName: 'UI/notificationtoast-icon',
        //             originRelative: new Vector2(0.5, 0.5)
        //         })
        //     ]
        // });
        // window
        // var nineSlice = new NineSlice({
        //     originRelative: new Vector2(0.5, 0.5),
        //     imageName: 'gui/notification-slice',
        //     width: width*4,
        //     height: height*4,
        // });
        var panel = new Entity({
            name: 'panel',
            position: new Vector2(0, 0),
            scale: new Vector2(0.3, 0.3),
            components: [new Sprite({
                imageName: 'gui/notification-slice',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });
        var windowBox = new Entity({
            name: 'windowBox',
            position: new Vector2(viewport.width / 2, 0),
            components: [panel,
            // iconBack,
            text]
        });

        var behavior = {
            name: 'behavior',
            start: function start(data) {
                var clickable = new Clickable({
                    onClick: function onClick() {
                        tweenOut();
                    }
                });
                var tweenIn = function tweenIn() {
                    // drop down
                    entity.attach(new Tween.TweenBehavior({
                        from: -height / 2 - 8,
                        to: height / 2 - 8,
                        in: 20,
                        ease: 'elastic',
                        decay: 8,
                        oscillations: 1,
                        ignoreGameSpeed: true,
                        onUpdate: function onUpdate(v, t) {
                            windowBox.position.y = v;
                            entity.position.y = v;
                        },
                        onComplete: function onComplete() {
                            entity.attach(clickable);
                            entity.attach(new Tween.TweenBehavior({
                                in: 120,
                                ignoreGameSpeed: true,
                                onComplete: tweenOut
                            }));
                        }
                    }));
                };
                var isTweeningOut = false;
                var tweenOut = function tweenOut(time) {
                    if (isTweeningOut) {
                        return;
                    }
                    isTweeningOut = true;
                    entity.attach(new Tween.TweenBehavior({
                        from: height / 2 - 8,
                        to: -height / 2 - 8,
                        in: time || 60,
                        ease: 'exponential',
                        decay: 1,
                        ignoreGameSpeed: true,
                        onUpdate: function onUpdate(v, t) {
                            windowBox.position.y = v;
                            entity.position.y = v;
                        },
                        onComplete: function onComplete() {
                            // remove self
                            entity.removeSelf();
                            currentPopup = null;

                            // call next popup
                            showNext();
                        }
                    }));
                };
                tweenIn();
            }
        };
        var entity = new Entity({
            z: Globals.Layers.NOTIFICATIONS,
            name: 'notificationPopup',
            position: settings.position,
            float: true,
            global: true,
            updateWhenPaused: pauseLevel,
            boundingBox: new Rectangle(0, -height, width, height),
            components: [behavior, windowBox]
        });

        // if (!icon) {
        //     // no icon
        //     iconBack.removeSelf();
        // } else {
        //     // icon.position.x -= 4;
        //     iconBack.attach(icon);
        // }

        return entity;
    };

    // create popup and schedule it to show
    Notification.add = function (text, icon, width) {
        var popup = new Notification({
            width: width,
            text: text,
            icon: icon
        });
        popups.push(popup);

        if (!currentPopup) {
            showNext();
        }
        return popup;
    };

    // show next popup in line
    var showNext = function showNext() {
        if (currentPopup) {
            return;
        }

        var popup = popups.shift();
        if (popup) {
            currentPopup = popup;
            Bento.objects.attach(currentPopup);
        }
    };
    var popups = [];
    var currentPopup = null;

    // catch events for notifications
    EventSystem.on('notification', function (data) {
        if (data.textId) {
            data.text = Localization.getText(data.textId);
        }
        Notification.add(data.text, data.icon, data.width);
    });

    return Notification;
});