'use strict';

/**
 * Playable end card (open chest)
 * @moduleName PlayableEndCard
 * @snippet PlayableEndCard|constructor
PlayableEndCard({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/playablecard', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'entities/gridbg', 'globals', 'bento/components/modal', 'bento/components/eventlistener', 'modules/localization', 'modules/textstyles', 'gui/taphand'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, GridBg, Globals, Modal, EventListener, Localization, TextStyles, TapHand) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var titleLogoSprite = new Sprite({
            imageName: 'gui/logotitle-' + Localization.getLanguage(),
            originRelative: new Vector2(0.5, 0.5)
        });
        var titleLogo = new Entity({
            name: 'title',
            scale: new Vector2(0.15, 0.15),
            components: [titleLogoSprite, new Object({
                name: 'wiggle',
                timer: 0,
                update: function update() {
                    this.timer += 0.1;
                    titleLogo.scale.x = 0.15 + 0.005 * Math.sin(this.timer);
                    titleLogo.scale.y = 0.15 + 0.005 * Math.sin(this.timer);
                }
            })]
        });
        var behavior = new Object({
            name: 'behavior',
            start: function start(data) {
                var gridOverlay = new GridBg({
                    width: 900
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    onStart: function onStart() {
                        gridOverlay.alpha = 0;
                        chestIcon.alpha = 0;
                        entity.attach(gridOverlay);
                    },
                    onUpdate: function onUpdate(v, t) {
                        gridOverlay.alpha = v;
                    },
                    onComplete: function onComplete() {
                        entity.attach(titleLogo);
                        entity.attach(chestIcon);
                        entity.attach(button);
                        align();
                        Bento.audio.playSound('sfx_star_single');

                        new Tween({
                            from: 0,
                            to: 1,
                            in: 20,
                            onUpdate: function onUpdate(v, t) {
                                chestIcon.alpha = v;
                            }
                        });
                        new Tween({
                            from: chestIcon.position.y - 64,
                            to: chestIcon.position.y,
                            in: 60,
                            ease: 'easeOutBounce',
                            onUpdate: function onUpdate(v, t) {
                                chestIcon.position.y = v;
                            },
                            onComplete: function onComplete() {
                                chestIcon.attach(new Object({
                                    name: 'wiggle',
                                    timer: 0,
                                    update: function update() {
                                        this.timer += 0.1;
                                        chestIcon.rotation = Math.PI / 40 * Math.sin(this.timer);
                                    }
                                }));
                            }
                        });
                    }
                });
            },
            destroy: function destroy(data) {},
            update: function update(data) {},
            draw: function draw(data) {}
        });
        var chestSprite = new Sprite({
            imageName: 'gui/chest',
            originRelative: new Vector2(0.5, 0.5)
        });
        var chestIcon = new Entity({
            name: 'chestIcon',
            scale: new Vector2(0.20, 0.20),
            components: [chestSprite]
        });
        var entity = new Entity({
            z: Globals.Layers.CONFETTI + 1,
            name: 'endCard',
            position: position,
            float: true,
            components: [new Modal({}), behavior, new EventListener({
                name: 'eventListener',
                eventName: 'orientationChanged',
                ignorePause: false,
                onEvent: function onEvent(data) {
                    align();
                }
            })]
        });
        var tapHand = new TapHand({
            position: new Vector2(0, 48),
            rotation: Math.PI,
            scale: new Vector2(0.5, 0.5),
            upPosition: new Vector2(0, -30),
            speed: 1,
            startInUpPosition: false,
            onTap: function onTap() {}
        });
        var button = new ClickButton({
            name: 'open',
            imageName: 'gui/button-skip-blank',
            frameCountX: 1,
            frameCountY: 1,
            scale: new Vector2(0.4, 0.4),
            components: [new Text({
                text: Localization.getText('openNow'),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                scale: new Vector2(2, 2),
                maxWidth: 300 / 2,
                maxHeight: 100 / 2
            }), tapHand],
            onClick: function onClick() {
                Utils.openStoreUrl();
            }
        });
        var align = function align() {
            var isIpad = false;
            if (viewport.height > viewport.width) {
                isIpad = viewport.width / viewport.height > 0.7;
            } else {
                isIpad = viewport.height / viewport.width > 0.7;
            }
            if (isIpad) {
                chestIcon.scale.x = 0.1;
                chestIcon.scale.y = 0.1;
            } else {
                chestIcon.scale.x = 0.2;
                chestIcon.scale.y = 0.2;
            }

            titleLogo.position = new Vector2(viewport.width / 2, viewport.height * 1 / 6 + 16);
            chestIcon.position = new Vector2(viewport.width / 2, viewport.height * 3 / 6);
            button.position = new Vector2(viewport.width / 2, viewport.height * 4 / 6 + 16);
        };
        align();

        return entity;
    };
});