'use strict';

/**
 * Progress of playable
 * @moduleName PlayableProgress
 * @snippet PlayableProgress|constructor
PlayableProgress({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/playableprogress', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'bento/gui/centeredlist', 'globals', 'bento/components/eventlistener'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, CenteredList, Globals, EventListener) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var behavior = new Object({
            name: 'behavior',
            start: function start(data) {},
            destroy: function destroy(data) {},
            update: function update(data) {},
            draw: function draw(data) {}
        });
        var makeSegment = function makeSegment(isFilled, shouldAnimate) {
            var sprite = new Sprite({
                imageName: 'gui/progression-' + (isFilled ? 'full' : 'empty'),
                originRelative: new Vector2(0.5, 0.5)
            });
            var segment = new Entity({
                name: 'progressSegment',
                scale: new Vector2(0.16, 0.16),
                components: [sprite]
            });
            entity.addItem(segment);

            if (shouldAnimate) {
                entity.attach(new EventListener({
                    eventName: 'winLevel',
                    onEvent: function onEvent() {
                        new Tween({
                            from: segment.scale.x * 0.75,
                            to: segment.scale.x,
                            in: 60,
                            ease: 'elastic',
                            decay: 5,
                            oscillations: 3,
                            onStart: function onStart() {
                                sprite.setup({
                                    imageName: 'gui/progression-full',
                                    originRelative: new Vector2(0.5, 0.5)
                                });
                            },
                            onUpdate: function onUpdate(v, t) {
                                segment.scale.x = v;
                                segment.scale.y = v;
                            },
                            onComplete: function onComplete() {}
                        });
                    }
                }));
            }
        };
        var chestSprite = new Sprite({
            imageName: 'gui/chest-icon',
            originRelative: new Vector2(0.5, 0.5),
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                default: {
                    speed: 0,
                    frames: [0]
                }
            }
        });
        var chestIcon = new Entity({
            name: 'chestIcon',
            // scale: new Vector2(0.05, 0.05),
            scale: new Vector2(0.15, 0.15),
            components: [chestSprite, new Object({
                name: 'wiggle',
                update: function update(data) {
                    if (Bento.objects.ticker % 180 === 60) {
                        new Tween({
                            from: Math.PI / 10,
                            to: 0,
                            in: 120,
                            ease: 'elastic',
                            decay: 8,
                            oscillations: 6,
                            onUpdate: function onUpdate(v, t) {
                                chestIcon.rotation = v;
                            }
                        });
                    }
                }
            })]
        });
        var entity = new CenteredList({
            spacing: 32, // in pixels
            useItemDimensions: false, // note: uses last item's dimension
            direction: 'x',
            position: position,
            items: []
        });
        entity.z = Globals.Layers.GUI;
        entity.float = true;

        Utils.repeat(Globals.targetLevels, function (i, l) {
            makeSegment(Globals.clearedLevels > i, Globals.clearedLevels === i);
        });
        entity.addItem(chestIcon);

        return entity;
    };
});