'use strict';

bento.define('gui/shopunlockfxoverlay', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'modules/confetti', 'gui/dialog', 'bento/components/modal'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, Confetti, Dialog, Modal) {

    /**
     * Animation for opening a character gacha ball in the shop.
     * This takes a character tile and moves it into the foreground.
     */
    var ShopUnlockFxOverlay = function ShopUnlockFxOverlay(settings) {
        var ballSpriteObj = settings.ballSpriteObj,
            characterTile = settings.characterTile,
            onReveal = settings.onReveal,
            _onComplete = settings.onComplete;


        var viewport = Bento.getViewport(true);

        var fill = new Entity({
            name: 'fill',
            position: viewport.getExtents(),
            scale: new Vector2((viewport.width + 2) / 16, (viewport.height + 2) / 16),
            components: [new Sprite({
                imageName: 'texture/black',
                originRelative: new Vector2(0.5, 0.5)
            })]
        });

        var behavior = {
            name: 'behavior',
            start: function start() {

                var charStartParent = characterTile.parent;
                var charStartPos = characterTile.position.clone();
                var charNewPos = overlay.toLocalPosition(characterTile.getWorldPosition());

                var fxContainer = new Entity({
                    name: 'fxContainer'
                });
                overlay.attach(fxContainer); // bloom and rays should go behind character

                characterTile.removeSelf();
                characterTile.position.x = charNewPos.x;
                characterTile.position.y = charNewPos.y;
                overlay.attach(characterTile);

                var bloom = new Entity({
                    name: 'bloom',
                    position: characterTile.position.add(Vector2(0, -4)),
                    scale: new Vector2(0, 0),
                    components: [new Sprite({
                        imageName: 'gui/fx/bloom-hard',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                });
                fxContainer.attach(bloom);

                var spawnGibs = function spawnGibs() {
                    var sprite = ballSpriteObj.getComponent('sprite');
                    var gibSpriteSettings = {
                        image: sprite.spriteImage,
                        originRelative: new Vector2(0.5, 0.5),
                        frameCountX: 3,
                        frameCountY: 3,
                        animations: {
                            '0': { speed: 0, frames: [0] },
                            '1': { speed: 0, frames: [1] },
                            '2': { speed: 0, frames: [2] },
                            '3': { speed: 0, frames: [3] },
                            '4': { speed: 0, frames: [4] },
                            '5': { speed: 0, frames: [5] },
                            '6': { speed: 0, frames: [6] },
                            '7': { speed: 0, frames: [7] },
                            '8': { speed: 0, frames: [8] }
                        }
                    };
                    var pos = overlay.toLocalPosition(ballSpriteObj.getWorldPosition());

                    var _loop = function _loop(i) {
                        var x = i % 3 - 1;
                        var y = Math.floor(i / 3) - 1;
                        var gibSprite = new Sprite(gibSpriteSettings);
                        gibSprite.setAnimation('' + i);
                        var vel = new Vector2(Utils.getRandomRangeFloat(-1.5, 1.5) + x * 1.5, Utils.getRandomRangeFloat(-2.0, -3.0) + y * 0.5);
                        var accel = new Vector2(0, 0.2);
                        var rotVel = Utils.getRandomRangeFloat(-0.2, 0.2);
                        var gibBehavior = {
                            name: 'behavior',
                            update: function update() {
                                vel.addTo(accel);
                                gib.position.addTo(vel);
                                gib.rotation += rotVel;
                            }
                        };
                        var gib = new Entity({
                            name: 'gachaBallGib',
                            position: pos.add(Vector2(x * 14, y * 14 - 18)),
                            scale: ballSpriteObj.scale.scalarMultiply(characterTile.scale.x),
                            components: [gibSprite, gibBehavior]
                        });
                        overlay.attach(gib);
                    };

                    for (var i = 0; i < 9; i++) {
                        _loop(i);
                    }
                };

                var makeRay = function makeRay() {
                    var scaleX = 0.6;
                    var ray = new Entity({
                        name: 'ray',
                        position: characterTile.position.add(Vector2(0, -3)),
                        scale: new Vector2(0, 1),
                        rotation: Utils.getRandomFloat(Math.PI * 2),
                        components: [new Sprite({
                            imageName: 'gui/fx/ray',
                            originRelative: new Vector2(0.5, 1.0)
                        }), new Tween.TweenBehavior({
                            from: 0, to: 1.0, in: 10,
                            ease: 'easeOutQuad',
                            onStart: function onStart() {},
                            onUpdate: function onUpdate(v, t) {
                                ray.alpha = v;
                                ray.scale.x = v * scaleX;
                            }
                        }), new Tween.TweenBehavior({
                            delay: 20,
                            from: 1.00, to: 0.0, in: 30,
                            ease: 'easeInQuad',
                            onStart: function onStart() {},
                            onUpdate: function onUpdate(v, t) {
                                ray.alpha = v;
                                ray.scale.x = v * scaleX;
                            },
                            onComplete: function onComplete() {
                                ray.removeSelf();
                            }
                        })]
                    });
                    fxContainer.attach(ray);
                    return ray;
                };

                for (var t = 0; t < 5; t++) {
                    Utils.timeout(30 + t * 10, makeRay);
                }

                // darken screen
                overlay.attach(new Tween.TweenBehavior({
                    delay: 0,
                    from: 0, to: 0.5, in: 30,
                    ease: 'easeInOutQuad',
                    onStart: function onStart() {},
                    onUpdate: function onUpdate(v, t) {
                        fill.alpha = v;
                    },
                    onComplete: function onComplete() {}
                }));

                // grow and shrink bloom
                Utils.sequenceTweens({
                    from: 0, to: 0.06, in: 30,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        bloom.scale.x = v;
                        bloom.scale.y = v;
                    }
                }, {
                    to: 0.05, in: 20,
                    ease: 'easeInOutQuad'
                }, {
                    to: 0.08, in: 26,
                    ease: 'easeInOutQuad'
                }, {
                    to: 0.07, in: 26,
                    ease: 'easeInOutQuad'
                }, {
                    to: 0.092, in: 26,
                    ease: 'easeInQuad'
                }, {
                    to: 0.10, in: 8,
                    ease: 'easeOutQuad'
                }, {
                    to: 0.0, in: 60,
                    ease: 'easeInQuad'
                });
                // fade bloom in/out
                Utils.sequenceTweens({
                    from: 0, to: 1.0, in: 20,
                    ease: 'easeInQuad',
                    onUpdate: function onUpdate(v, t) {
                        bloom.alpha = v;
                    }
                }, {
                    ease: 'easeOutQuad',
                    delay: 130, to: 0.0, in: 30
                });

                var charTileScale = characterTile.scale.x;
                // wiggle gacha ball rapidly with increasing intensity
                // then swap out the sprite from gacha ball to character
                overlay.attach(new Tween.TweenBehavior({
                    from: 0, to: 1,
                    in: 130,
                    onUpdate: function onUpdate(v) {
                        characterTile.rotation = Math.sin(overlay.timer) * (v * 0.08 + 0.02);
                    },
                    onComplete: function onComplete() {
                        spawnGibs();
                        characterTile.rotation = 0;
                        characterTile.scale.x = charTileScale;
                        characterTile.scale.y = charTileScale;
                        characterTile.alpha = 1.0;
                        if (onReveal) onReveal();

                        Tween({
                            delay: 5,
                            from: 1.0, to: 0.0, in: 36,
                            ease: 'easeOutQuad',
                            onUpdate: function onUpdate(v, t) {
                                characterTile.scale.x = charTileScale - Math.sin(Math.PI * 2 * t / 20) * v * 0.2;
                                characterTile.scale.y = charTileScale + Math.sin(Math.PI * 2 * t / 20) * v * 0.2;
                            }
                        });
                    }
                }));

                // slightly grow gacha ball
                overlay.attach(new Tween.TweenBehavior({
                    from: charTileScale, to: charTileScale + 0.15,
                    in: 40,
                    ease: 'easeOutQuad',
                    onUpdate: function onUpdate(v) {
                        characterTile.scale.x = v;
                        characterTile.scale.y = v;
                    }
                }));

                // make gacha ball slightly transparent
                overlay.attach(new Tween.TweenBehavior({
                    from: 1.0, to: 0.8,
                    in: 40,
                    ease: 'easeOutQuad',
                    onUpdate: function onUpdate(v) {
                        characterTile.alpha = v;
                    }
                }));

                // fade to normal
                // put the character back in the shop container.
                overlay.attach(new Tween.TweenBehavior({
                    delay: 160,
                    from: 0.5, to: 0, in: 30,
                    ease: 'easeInOutQuad',
                    onStart: function onStart() {},
                    onUpdate: function onUpdate(v, t) {
                        fill.alpha = v;
                    },
                    onComplete: function onComplete() {
                        characterTile.removeSelf();
                        characterTile.position.x = charStartPos.x;
                        characterTile.position.y = charStartPos.y;
                        charStartParent.attach(characterTile);
                        overlay.removeSelf();
                        if (_onComplete) _onComplete();
                    }
                }));
            },
            update: function update() {}
        };

        var overlay = new Entity({
            z: Globals.Layers.SHOP_UNLOCK_FX,
            float: true,
            position: new Vector2(0, 0),
            components: [new Modal({}), fill, behavior]
        });

        return overlay;
    };

    return ShopUnlockFxOverlay;
});