'use strict';

bento.define('gui/spinwheeloverlay', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/simplesprite', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'modules/admanager', 'modules/levelmanager', 'components/debug', 'bento/components/modal', 'modules/tapticengine', 'modules/localization', 'modules/savemanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, SimpleSprite, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, Gui, GuiCommon, AdManager, LevelManager, Debug, Modal, TapticEngine, Localization, SaveManager) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var _onComplete = void 0;
    var _canClick = false;

    var SpinWheel = function SpinWheel(settings) {

        _onComplete = settings.onComplete;
        _canClick = false;

        var viewport = Bento.getViewport();

        var fill = new Fill({
            name: 'fill',
            dimension: Bento.getViewport(true).getSize(),
            color: [0, 0, 0, 0.90], // [1, 1, 1, 1] is pure white
            originRelative: new Vector2(0, 0)
        });

        var spinWheelRoot = new Entity({
            name: 'spinWheelRoot',
            position: new Vector2(viewport.width / 2, viewport.height / 2)
        });

        Gui.load({
            parent: spinWheelRoot,
            assetName: 'ui/spinwheel',
            handlers: GuiSpinWheel, onComplete: function onComplete() {
                // fade in overlay
                new Tween({
                    from: 0, to: 1,
                    in: 30,
                    delay: 0,
                    applyOnDelay: 0,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        _canClick = true;
                    }
                });
            }
        });

        var entity = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'spinWheelOverlay',
            position: new Vector2(0, 0),
            float: true,
            components: [new Modal({}), fill, spinWheelRoot]
        });

        spinWheelRoot.scale.scale(Gui.scaleFactorTall);

        entity.alpha = 0;

        return entity;
    };

    var GuiSpinWheel = Utils.copyObject(GuiCommon);

    var items = void 0;
    var totalWeight = 0.0;
    var chosenItemIndex = void 0;

    Utils.extend(GuiSpinWheel, {
        init: function init(obj) {
            // choose which item to land on
            // choose an offset within that item e.g. 0.05 to 0.95
            // if close to the edge, slide back a little?
            items = [{ type: 'coins', amount: 100, weight: 0.30, icon: null, label: null }, { type: 'hints', amount: 3, weight: 0.05, icon: null, label: null }, { type: 'coins', amount: 200, weight: 0.05, icon: null, label: null }, { type: 'coins', amount: 100, weight: 0.30, icon: null, label: null }, { type: 'hints', amount: 1, weight: 0.15, icon: null, label: null }, { type: 'coins', amount: 150, weight: 0.15, icon: null, label: null }];
        },
        wheelChest: function wheelChest(obj) {
            obj.getComponent('hdsprite3', function (hdSprite) {
                // hdSprite.position.x += 60
                obj.remove(hdSprite);
                var chestTop = new Entity({
                    name: 'chestTop',
                    position: new Vector2(0, -6),
                    scale: hdSprite.scale.clone(),
                    components: [new Sprite({
                        imageName: 'gui/chest-top',
                        originRelative: new Vector2(0.5, 0.5)
                    }), new Object({
                        name: 'oscBehavior',
                        n: 0,
                        oscAmount: 0,
                        update: function update() {
                            chestTop.transform.y = Math.sin(this.n++ * 0.1) * this.oscAmount;
                        }
                    })]
                });
                var chestBottom = new Entity({
                    name: 'chestBottom',
                    position: new Vector2(0, 12),
                    scale: hdSprite.scale.clone(),
                    components: [new Sprite({
                        imageName: 'gui/chest-bottom',
                        originRelative: new Vector2(0.5, 0.5)
                    })]
                });
                obj.attach(chestBottom);
                obj.attach(chestTop);
            });
        },
        wheelArrow: function wheelArrow(obj) {},
        wheel: function wheel(obj) {
            totalWeight = 0.0;
            for (var i = 0; i < items.length; i++) {
                totalWeight += items[i].weight;
            }
            // for (let i = 0; i < items.length; i++) {
            //     let item = items[i];
            //     console.log(i, item.type, item.amount, "percentage = " + (100*item.weight/totalWeight).toFixed(2) + "%");
            // }
            for (var _i = 0; _i < items.length; _i++) {
                var item = items[_i];
                var angle = (_i + 0.5) / items.length * Math.PI * 2;
                var itemEntity = new Entity({
                    name: 'wheelItem'
                });
                switch (item.type) {
                    case 'coins':
                        itemEntity.position.y -= 28;
                        itemEntity.scale.x = 0.15;
                        itemEntity.scale.y = 0.15;
                        itemEntity.attach(new Sprite({
                            imageName: 'gui/coin-icon',
                            originRelative: new Vector2(0.5, 0.5)
                        }));
                        itemEntity.rotation = angle;
                        break;
                    case 'hints':
                        itemEntity.position.y -= 30;
                        itemEntity.scale.x = 0.075;
                        itemEntity.scale.y = 0.075;
                        itemEntity.attach(new Sprite({
                            imageName: 'gui/hint-icon',
                            originRelative: new Vector2(0.5, 0.5)
                        }));
                        itemEntity.rotation = angle;
                        break;
                }
                itemEntity.position.rotateRadian(angle);
                obj.attach(itemEntity);
                var label = new Text({
                    alpha: 0.4,
                    text: '' + item.amount,
                    position: new Vector2(0, -52).rotateRadian(angle),
                    rotation: angle,
                    scale: new Vector2(0.75, 0.75),
                    fontSettings: TextStyles.get('main'),
                    align: 'center',
                    maxWidth: 64,
                    maxHeight: 32,
                    drawDebug: false
                });
                obj.attach(label);
                item.icon = itemEntity;
                item.label = label;
            }

            // like rotation, but from 0 to `items.length` rather than 0 to `PI*2`
            var n = 0;

            var vel = 0.0;
            var targetN = void 0;

            var prevItemIndex = 0;
            var currentItemIndex = 0;

            var getIndex = function getIndex(nVal) {
                return Math.floor(Utils.modulo(-nVal, items.length));
            };

            var wheelArrow = Gui.get('wheelArrow');

            var behavior = {
                name: 'behavior',
                start: function start() {
                    SaveManager.add('wheelTimes', 1);

                    targetN = 0;
                    vel = 0;
                    // accelerate to max spin velocity
                    Tween({
                        from: 0, to: 0.2, in: 60,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            vel = v;
                        },
                        onComplete: function onComplete() {}
                    });
                },
                update: function update() {
                    if (!targetN) {
                        n += vel;
                    }

                    obj.rotation = n / items.length * Math.PI * 2;

                    prevItemIndex = currentItemIndex;
                    currentItemIndex = getIndex(n);

                    if (currentItemIndex !== prevItemIndex) {
                        var _item = items[currentItemIndex];
                        var prevItem = items[prevItemIndex];
                        prevItem.label.alpha = 0.4;
                        _item.label.alpha = 1.0;
                        if (targetN) {
                            wheelArrow.rotation = -0.08;
                            TapticEngine.impact({ type: 'medium' });
                        }
                    }

                    wheelArrow.rotation *= 0.9;
                },
                stopWheel: function stopWheel() {
                    // let chosenItemIndex = 2; // TEMP: choose #2 every time
                    chosenItemIndex = 0;
                    var rand = Math.random() * totalWeight;
                    if (SaveManager.load('wheelTimes') === 1) {
                        // pick a good item
                        chosenItemIndex = Utils.pickRandom(1, 2, 5);
                    } else {
                        // pick item according to weight/probability
                        for (var _i2 = 0; _i2 < items.length; _i2++) {
                            rand -= items[_i2].weight;
                            if (rand <= 0) {
                                chosenItemIndex = _i2;
                                break;
                            }
                        }
                    }
                    targetN = n + 16;
                    for (var _i3 = 0; _i3 < 100; _i3++) {
                        if (getIndex(targetN) === chosenItemIndex) {
                            break;
                        }
                        targetN += 1;
                    }
                    // targetN = Math.floor(targetN) + 1 - Math.pow((targetN - Math.floor(targetN)), 2);
                    targetN = Utils.clamp(Math.floor(targetN) + 0.05, targetN, Math.floor(targetN) + 0.975);
                    Tween({
                        from: n,
                        to: targetN,
                        in: 340,
                        ease: 'easeOutCubic',
                        onStart: function onStart() {},
                        onUpdate: function onUpdate(v, t) {
                            n = v;
                        },
                        onComplete: function onComplete() {
                            var whole = Math.floor(targetN);
                            var frac = targetN - whole;
                            var adjustAmount = Math.abs(frac - 0.5) * 0.10;
                            if (adjustAmount > 0.025 && frac > 0.5) /*|| (adjustAmount > 0.035 && frac < 0.5)*/{
                                    frac = Utils.approach(frac, 0.5, adjustAmount);
                                    targetN = whole + frac;
                                    Tween({
                                        from: n, to: targetN,
                                        in: 40 + adjustAmount * 400,
                                        ease: 'easeInOutQuad',
                                        onStart: function onStart() {},
                                        onUpdate: function onUpdate(v, t) {
                                            n = v;
                                        },
                                        onComplete: function onComplete() {
                                            behavior.highlightPrize();
                                        }
                                    });
                                } else {
                                behavior.highlightPrize();
                            }
                        }
                    });
                },
                highlightPrize: function highlightPrize() {
                    // prize is ready, player can choose to collect it or spin again
                    Bento.audio.playSound('sfx_victory_jingle_cheer');

                    var confetti = new Confetti({
                        speedY: 0.7,
                        particleScale: 0.08,
                        z: Globals.Layers.DIALOG_CONFETTI
                    });
                    Bento.objects.attach(confetti);

                    Gui.show('buttonWheelCollect');
                    Gui.show('buttonWheelSpinAgain');
                    Gui.get('wheelChest', function (chest) {
                        chest.getComponent('chestTop', function (chestTop) {
                            chestTop.getComponent('oscBehavior', function (oscBehavior) {
                                Tween({
                                    from: 0, to: 1, in: 60,
                                    ease: 'easeInOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        oscBehavior.oscAmount = v;
                                    }
                                });
                            });
                            Tween({
                                from: chestTop.position.clone(),
                                to: chestTop.position.add(Vector2(0, -10)),
                                in: 60,
                                ease: 'easeInOutQuad',
                                onStart: function onStart() {},
                                onUpdate: function onUpdate(v, t) {
                                    chestTop.position.x = v.x;
                                    chestTop.position.y = v.y;
                                },
                                onComplete: function onComplete() {}
                            });
                        });
                        var halo = new Entity({
                            name: 'halo',
                            position: new Vector2(0, -2),
                            scale: new Vector2(0, 0),
                            components: [new Sprite({
                                imageName: 'gui/halo',
                                originRelative: new Vector2(0.5, 0.5)
                            })]
                        });
                        new Tween({
                            from: 0, to: 1, in: 60,
                            delay: 0,
                            ease: 'easeInOutQuad',
                            onStart: function onStart() {
                                chest.attach(halo);
                                chest.moveComponentTo(halo, 0);
                            },
                            onUpdate: function onUpdate(v, t) {
                                halo.alpha = v * 0.8;
                                halo.scale.x = v * 0.16;
                                halo.scale.y = v * 0.16;
                            }
                        });
                    });
                },
                reroll: function reroll() {
                    behavior.start();
                    Gui.get('wheelChest', function (chest) {
                        chest.getComponent('chestTop', function (chestTop) {
                            chestTop.getComponent('oscBehavior', function (oscBehavior) {
                                Tween({
                                    from: oscBehavior.oscAmount, to: 0, in: 60,
                                    ease: 'easeInOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        oscBehavior.oscAmount = v;
                                    }
                                });
                            });
                            Tween({
                                from: chestTop.position.clone(),
                                to: chestTop.position.add(Vector2(0, 10)),
                                in: 30,
                                ease: 'easeInQuad',
                                onUpdate: function onUpdate(v, t) {
                                    chestTop.position.x = v.x;
                                    chestTop.position.y = v.y;
                                }
                            });
                        });
                        chest.getComponent('halo', function (halo) {
                            new Tween({
                                from: 1, to: 0, in: 60,
                                delay: 0,
                                ease: 'easeInOutQuad',
                                onUpdate: function onUpdate(v, t) {
                                    halo.alpha = v * 0.8;
                                    halo.scale.x = v * 0.16;
                                    halo.scale.y = v * 0.16;
                                },
                                onComplete: function onComplete() {
                                    halo.removeSelf();
                                }
                            });
                        });
                    });
                }
            };
            obj.attach(behavior);
        },
        buttonWheelCollect: function buttonWheelCollect(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                canClick: function canClick() {
                    return Gui.isShown(obj) && _canClick;
                },
                onClick: function onClick() {
                    Gui.hide('buttonWheelCollect');
                    Gui.hide('buttonWheelSpinAgain');

                    Bento.objects.get('spinWheelOverlay', function (spinWheelOverlay) {
                        new Tween({
                            from: 1, to: 0, in: 30,
                            ease: 'easeInOutQuad',
                            onStart: function onStart() {
                                _canClick = false;
                            },
                            onUpdate: function onUpdate(v, t) {
                                spinWheelOverlay.alpha = v;
                            },
                            onComplete: function onComplete() {
                                spinWheelOverlay.removeSelf();
                                if (_onComplete) {
                                    _onComplete(items[chosenItemIndex]); // prev screen is responsible for handling reward
                                }
                            }
                        });
                    });
                }
            });
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(0, 60));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        buttonWheelCollectText: function buttonWheelCollectText(obj) {
            makeText(obj, {
                text: Localization.getText('collect').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        buttonWheelSpinAgain: function buttonWheelSpinAgain(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                canClick: function canClick() {
                    return Gui.isShown(obj) && _canClick;
                },
                onClick: function onClick() {
                    AdManager.tryShowRewarded({
                        onReward: function onReward() {
                            Gui.hide('buttonWheelCollect');
                            Gui.hide('buttonWheelSpinAgain');
                            Gui.get('wheel', function (wheel) {
                                wheel.getComponent('behavior', function (b) {
                                    b.reroll();
                                });
                            });
                            Utils.timeout(30, function () {
                                Gui.show('buttonWheelStop');
                            });
                        },
                        onComplete: function onComplete(error) {
                            if (error) {
                                // ?
                            }
                        },

                        tag: 'Victory_Reward_Chest_Reroll',
                        rewardType: 'reroll',
                        rewardAmount: 1
                    });
                }
            });
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(0, 60));
            var showable = Gui.Showable({
                isShown: false,
                show: function show(parent) {
                    return Tween({
                        delay: 10,
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        buttonWheelSpinAgainText: function buttonWheelSpinAgainText(obj) {
            makeText(obj, {
                text: Localization.getText('reroll').toUpperCase(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        },
        buttonWheelStop: function buttonWheelStop(obj) {
            makeButton(obj, {
                shadow: 'gui/button-wide-shadow',
                canClick: function canClick() {
                    return Gui.isShown(obj) && _canClick;
                },
                onClick: function onClick() {
                    Gui.hide('buttonWheelStop');
                    Gui.get('wheel', function (wheel) {
                        wheel.getComponent('behavior', function (b) {
                            b.stopWheel();
                        });
                    });
                }
            });
            var showPos = obj.position.clone();
            var hidePos = obj.position.add(Vector2(0, 60));
            var showable = Gui.Showable({
                isShown: true,
                show: function show(parent) {
                    return Tween({
                        from: hidePos, to: showPos, in: 60,
                        ease: 'elastic',
                        decay: 12,
                        oscillations: 2,
                        onStart: function onStart() {
                            parent.attach(obj);
                        },
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        }
                    });
                },
                hide: function hide(parent) {
                    return Tween({
                        from: obj.position.clone(), to: hidePos, in: 22,
                        ease: 'easeInBack',
                        onUpdate: function onUpdate(v, t) {
                            v.copyInto(obj.position);
                        },
                        onComplete: function onComplete() {
                            parent.remove(obj);
                        }
                    });
                }
            });
            obj.attach(showable);
        },
        buttonWheelStopText: function buttonWheelStopText(obj) {
            makeText(obj, {
                text: function () {
                    var s = Localization.getText('stopWheel');
                    var lang = Localization.getLanguage();
                    if (lang === 'ja') {
                        s = s.replace('ウィール', 'ウィール\n');
                    }
                    return s.toUpperCase();
                }(),
                fontSettings: TextStyles.get('main'),
                align: 'center'
            });
        }
    });

    return SpinWheel;
});