'use strict';

bento.define('gui/starsforadoverlay', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/simplesprite', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'modules/admanager', 'modules/levelmanager', 'components/debug', 'modules/localization'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, SimpleSprite, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, Gui, GuiCommon, AdManager, LevelManager, Debug, Localization) {

    var Star = function Star(settings) {
        var position = settings.position,
            earned = settings.earned;


        var sprite = new Sprite({
            imageName: earned ? 'gui/star-icon' : 'gui/star-icon-empty',
            originRelative: new Vector2(0.5, 0.5)
        });

        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'star',
            position: position,
            // scale: new Vector2(0.092, 0.092),
            scale: new Vector2(0.11, 0.11),
            components: [sprite]
        });

        return entity;
    };

    return function (settings) {
        var _onComplete = settings.onComplete;


        var _canClick = false;

        var viewport = Bento.getViewport();

        var fill = new Fill({
            name: 'fill',
            dimension: Bento.getViewport(true).getSize(),
            color: [0, 0, 0, 0.75], // [1, 1, 1, 1] is pure white
            originRelative: new Vector2(0, 0)
        });

        var stars = new Entity({
            name: 'starsEarned',
            position: new Vector2(0, -80)
        });

        var textAskCollect = new Text({
            text: Localization.getText('starsForAd'),
            position: new Vector2(0, -40),
            scale: new Vector2(0.5, 0.5),
            fontSettings: TextStyles.get('buttonLabel'),
            align: 'center',
            maxWidth: 400,
            maxHeight: 32,
            drawDebug: false
        });
        var buttonSkip = new Entity({
            name: 'buttonStarsForAdSkip',
            position: new Vector2(0, 24),
            dimension: new Rectangle(-28, -12, 60, 30),
            components: [new Text({
                text: Localization.getText('skip').toUpperCase(),
                position: new Vector2(0, 0),
                scale: new Vector2(0.6, 0.6),
                fontSettings: TextStyles.get('buttonLabel'),
                align: 'center',
                maxWidth: 68,
                maxHeight: 32,
                linebreaks: false
            })]
        });

        var buttonCollect = new Entity({
            name: 'buttonStarsForAdCollect',
            position: new Vector2(0, -8),
            dimension: new Rectangle(-65, -16, 130, 32),
            components: [new SimpleSprite({
                imageName: 'gui/button-wide-green',
                position: new Vector2(0, 0),
                scale: new Vector2(0.16, 0.14)
            }), new SimpleSprite({
                imageName: 'gui/ad-silhouette',
                position: new Vector2(-42, 0),
                scale: new Vector2(0.07, 0.07)
            }), new Text({
                text: Localization.getText('collect').toUpperCase(),
                position: new Vector2(14, 0),
                scale: new Vector2(0.8, 0.8),
                fontSettings: TextStyles.get('main'),
                align: 'center',
                maxWidth: 100,
                maxHeight: 32
            })]
        });
        Gui.makeButton(buttonCollect, {
            canClick: function canClick() {
                return _canClick;
            },
            onClick: function onClick() {
                _canClick = false;
                var extraStars = 3 - Globals.starsEarned;
                AdManager.tryShowRewarded({
                    onReward: function onReward() {
                        Globals.starsEarned = 3;
                        LevelManager.levelWin(Utils.currentLevel, Globals.starsEarned, Utils.currentLevelGroup);
                    },
                    onComplete: function onComplete(error) {
                        var delay = 0;
                        if (error) {
                            buttonCollect.removeSelf();
                            buttonSkip.removeSelf();
                            textAskCollect.removeSelf();
                            var textOops = new Text({
                                text: Localization.getText('error'), // TODO: don't error if user cancelled
                                position: new Vector2(0, -20),
                                scale: new Vector2(0.6, 0.6),
                                fontSettings: TextStyles.get('buttonLabel'),
                                align: 'center',
                                maxWidth: 200,
                                maxHeight: 32,
                                drawDebug: false
                            });
                            container.attach(textOops);
                            delay = 60;
                        }
                        new Tween({
                            delay: delay,
                            from: 1, to: 0,
                            in: 30,
                            ease: 'easeInOutQuad',
                            onUpdate: function onUpdate(v, t) {
                                entity.alpha = v;
                            },
                            onComplete: function onComplete() {
                                _onComplete();
                            }
                        });
                    },

                    tag: 'Get_3_Stars',
                    rewardType: 'stars',
                    rewardAmount: extraStars
                });
            }
        });
        Gui.makeButton(buttonSkip, {
            canClick: function canClick() {
                return _canClick;
            },
            onClick: function onClick() {
                _canClick = false;
                new Tween({
                    from: 1, to: 0,
                    in: 30,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        _onComplete();
                    }
                });
            }
        });

        for (var i = 0; i < 3; i++) {
            stars.attach(new Star({
                earned: i < Globals.starsEarned,
                position: new Vector2((i - 1) * 30, i === 1 ? -4 : 2)
            }));
        }

        var idealSlices = Globals.maxSlices - 2;

        var textSlicesFor3Stars = new Text({
            text: idealSlices === 1 ? Localization.getText('sliceFor3Stars').toUpperCase() : Localization.getText('slicesFor3Stars').replace('{NUMBER}', idealSlices).toUpperCase(),
            position: new Vector2(0, viewport.height / 2 - 24),
            scale: new Vector2(0.38, 0.38),
            alpha: 0.75,
            fontSettings: TextStyles.get('buttonLabel'),
            align: 'center',
            maxWidth: 420,
            maxHeight: 80
            // drawDebug: true
        });

        var container = new Entity({
            name: 'container',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            components: [stars, textAskCollect, buttonCollect, buttonSkip, textSlicesFor3Stars]
        });

        var entity = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'starsForAdOverlay',
            position: new Vector2(0, 0),
            float: true,
            components: [fill, container]
        });

        entity.alpha = 0;

        new Tween({
            from: 0, to: 1,
            in: 30,
            delay: 0,
            applyOnDelay: 0,
            ease: 'easeInOutQuad',
            onUpdate: function onUpdate(v, t) {
                entity.alpha = v;
            },
            onComplete: function onComplete() {
                _canClick = true;
            }
        });

        return entity;
    };
});