'use strict';

/**
 * Tutorial hand - shows a tapping gesture
 * @moduleName TapHand
 * @snippet TapHand()|constructor
TapHand({
    z: 0,
    position: new Vector2(0, 0),
    rotation: 0,
    imageName: 'gui/taphand',
    upPosition: new Vector2(0, -30),
    speed: 1,
    startInUpPosition: false,
    onTap: function () {}
})
 */
bento.define('gui/taphand', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween) {
    'use strict';

    return function (settings) {
        var tapPosition = new Vector2(0, 0);
        var upPosition = settings.upPosition || new Vector2(0, -30);
        var speed = settings.speed || 1;
        var rotation = settings.rotation || 0;
        var startInUpPosition = settings.startInUpPosition || false;
        var imageName = settings.imageName || 'gui/taphand';
        var onTap = settings.onTap || function () {};
        var ignoreGameSpeed = settings.ignoreGameSpeed;

        var sprite = new Sprite({
            imageName: imageName,
            originRelative: new Vector2(0.5, 1)
        });

        var moveUp = function moveUp() {
            var completed = false;
            new Tween({
                from: 0,
                to: upPosition.y,
                'in': Math.floor(24 / speed),
                delay: Math.floor(12 / speed),
                ease: Tween.EASEINOUTQUAD,
                ignoreGameSpeed: ignoreGameSpeed,
                onUpdate: function onUpdate(v, t) {
                    entity.position.y = tapPosition.y + v;
                },
                onComplete: function onComplete() {
                    if (completed) {
                        return;
                    }
                    completed = true;
                    moveDown();
                }
            });

            new Tween({
                from: 0,
                to: upPosition.x,
                'in': Math.floor(24 / speed),
                delay: Math.floor(12 / speed),
                ease: Tween.EASEINOUTQUAD,
                ignoreGameSpeed: ignoreGameSpeed,
                onUpdate: function onUpdate(v, t) {
                    entity.position.x = tapPosition.x + v;
                }
            });
        };

        var moveDown = function moveDown() {
            var completed = false;
            new Tween({
                from: upPosition.y,
                to: 0,
                'in': Math.floor(8 / speed),
                delay: Math.floor(18 / speed),
                ease: Tween.EASEINQUAD,
                ignoreGameSpeed: ignoreGameSpeed,
                onUpdate: function onUpdate(v, t) {
                    entity.position.y = tapPosition.y + v;
                },
                onComplete: function onComplete() {
                    if (onTap) {
                        onTap();
                    }
                    if (completed) {
                        return;
                    }
                    completed = true;
                    moveUp();

                    new Tween({
                        from: -0.2,
                        to: 0,
                        'in': 16,
                        ease: Tween.EASEINOUTBACK,
                        ignoreGameSpeed: ignoreGameSpeed,
                        onUpdate: function onUpdate(v, t) {
                            entity.scale.y = 1 + v;
                            entity.scale.x = 1 - v;
                        },
                        onComplete: function onComplete() {}
                    });
                }
            });

            new Tween({
                from: upPosition.x,
                to: 0,
                'in': Math.floor(8 / speed),
                delay: Math.floor(18 / speed),
                ease: Tween.EASEINQUAD,
                ignoreGameSpeed: ignoreGameSpeed,
                onUpdate: function onUpdate(v, t) {
                    entity.position.x = tapPosition.x + v;
                }
            });
        };
        var hasStarted = false;
        var behavior = {
            name: 'behaviorComponent',
            start: function start(data) {
                if (hasStarted) {
                    return;
                }
                hasStarted = true;
                if (!startInUpPosition) {
                    moveUp();
                } else {
                    entity.position = upPosition.clone();
                    moveDown();
                }
            }
        };
        var entity = new Entity({
            name: 'tutorialhand',
            updateWhenPaused: 0,
            components: [sprite, behavior]
        });
        var container = new Entity({
            z: settings.z,
            name: 'tapHand',
            position: settings.position,
            scale: settings.scale,
            float: settings.float || false,
            rotation: rotation,
            components: [entity].concat(settings.components || [])
        }).extend({
            hide: function hide(onComplete) {
                if (entity.alpha !== 1) {
                    return;
                }

                new Tween({
                    from: 1,
                    to: 0,
                    'in': 20,
                    ease: Tween.LINEAR,
                    ignoreGameSpeed: ignoreGameSpeed,
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: onComplete
                });
            }
        });
        return container;
    };
});