'use strict';

bento.define('gui/timerlabel', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'entities/gui/gamecounter', 'bento/utils', 'bento/tween', 'modules/savemanager', 'modules/timemanager', 'components/simplesprite'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, GameCounter, Utils, Tween, SaveManager, TimeManager, SimpleSprite) {

    // let counterSettings = {
    //     scale: new Vector2(0.1, 0.1),
    //     align: 'right',
    // };

    var TimerLabel = function TimerLabel(settings) {
        var saveId = settings.saveId;
        var useLocal = settings.useLocal;
        var time = settings.time;

        var seconds = new GameCounter({
            scale: new Vector2(0.1, 0.1),
            align: 'left'
        });
        var minutes = new GameCounter({
            scale: new Vector2(0.1, 0.1),
            align: 'center'
        });
        var hours = new GameCounter({
            scale: new Vector2(0.1, 0.1),
            align: 'right'
        });

        seconds.position = new Vector2(10, 0);
        minutes.position = new Vector2(0, 0);
        hours.position = new Vector2(-10, 0);

        var colon1 = new SimpleSprite({
            imageName: 'colon-jua-black',
            position: new Vector2(-8.25, 0),
            scale: new Vector2(0.09, 0.09)
        });
        var colon2 = new SimpleSprite({
            imageName: 'colon-jua-black',
            position: new Vector2(8, 0),
            scale: new Vector2(0.09, 0.09)
        });

        var timeTab = new Entity({
            name: 'timeTab',
            position: new Vector2(0, 0),
            components: [colon1, colon2, hours, minutes, seconds]
        });

        var addDigit = function addDigit(value) {
            return ('00' + value).slice(-2);
        };
        var behavior = {
            name: 'behavior',
            start: function start() {
                entity.timer = 60; // immediately redraw
            },
            update: function update() {
                // update time every second
                if (entity.timer < 60) {
                    return;
                }
                entity.timer -= 60;

                var t;
                if (time) {
                    t = time;
                } else if (saveId) {
                    var saveData = SaveManager.load(saveId);
                    if (Utils.isUndefined(saveData.time)) {
                        t = saveData;
                    } else {
                        t = saveData.time;
                    }
                }
                var currentTime = (t || 0) - (useLocal ? TimeManager.getLocalTime() : TimeManager.getTime());
                if (currentTime > 0) {
                    var s = currentTime / 1000;
                    var m = s / 60;
                    var h = m / 60;

                    hours.setValue(Math.floor(h));
                    // hours.setValue(addDigit(Math.floor(h)));
                    minutes.setValue(addDigit(Math.floor(m) % 60));
                    seconds.setValue(addDigit(Math.floor(s) % 60));
                    entity.visible = true;
                } else {
                    entity.visible = false;
                    if (Utils.isDefined(settings.onComplete)) {
                        settings.onComplete();
                    }
                }
            }
        };
        var entity = new Entity({
            z: settings.z,
            name: 'timerLabel',
            position: settings.position,
            scale: settings.scale,
            components: [timeTab, behavior]
        });

        return entity;
    };

    return TimerLabel;
});