'use strict';

bento.define('gui/vipgiftdialog', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/hdsprite', 'modules/textstyles', 'modules/confetti', 'components/simplesprite', 'components/debug', 'modules/savemanager', 'modules/localization', 'gui/dialog', 'gui/coincounter', 'gui/gui', 'gui/coincollectanim', 'modules/cloudsave', 'modules/timemanager', 'entities/gui/gamecounter'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, HDSprite, TextStyles, Confetti, SimpleSprite, Debug, SaveManager, Localization, Dialog, CoinCounter, Gui, CoinCollectAnim, CloudSave, TimeManager, GameCounter) {

    var VipGiftDialog = function VipGiftDialog(settings) {
        var gift = settings.gift;


        var hints = gift.hints;
        var amount = gift.coins;

        var oldCoins = SaveManager.load('coins');
        var newCoins = oldCoins + amount;
        SaveManager.save('coins', newCoins);

        var oldHints = SaveManager.load('hints');
        var newHints = oldHints + hints;
        SaveManager.save('hints', newHints);

        CloudSave.saveToCloud(function (success, errorMsg) {
            if (!success) {
                console.log(errorMsg);
                return;
            }
        });

        var dialog = new Dialog({
            // position: new Vector2(-viewport.width/2, -viewport.height/2),
            //title: Localization.getText('dailyVipReward').toUpperCase(), // TODO
            title: Localization.getText('yougot').toUpperCase(),
            titleStyle: TextStyles.get('vipBannerTitle'),
            titleShadow: false,
            width: 132,
            type: 'ok',
            onComplete: function onComplete() {
                Gui.get('coinCounter', function (cc) {
                    Gui.get('starCounter', function (sc) {
                        new Tween({
                            from: 0.0, to: 1.0, in: 20,
                            ease: 'easeOutQuad',
                            onUpdate: function onUpdate(v, t) {
                                cc.alpha = v;
                                sc.alpha = v;
                            }
                        });
                    });
                });
                EventSystem.fire('updateCoins', newCoins);
            }
        });

        var w = dialog.getComponent('window');

        var vipHeadline = new SimpleSprite({
            imageName: 'gui/vip-headline',
            position: new Vector2(0, -80),
            scale: new Vector2(0.176, 0.176)
        });

        w.scale.scale(0.8);
        w.attach(vipHeadline);
        w.moveComponentTo(w.getComponent('titleText'), vipHeadline.rootIndex + 1);

        Gui.get('coinCounter', function (cc) {
            Gui.get('starCounter', function (sc) {
                dialog.attach(Tween.TweenBehavior({
                    from: 1.0, to: 0.0, in: 30,
                    ease: 'easeOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        cc.alpha = v;
                        sc.alpha = v;
                    }
                }));
            });
            var parent = cc.parent;
            var pos = cc.getWorldPosition();
            var cc2 = new CoinCounter({
                position: dialog.toLocalPosition(pos),
                numbersImage: 'numbers-jua-104-white'
            });
            cc2.scale.scale(Gui.scaleFactor);
            dialog.attach(cc2);

            dialog.attach({
                name: 'behavior',
                start: function start() {
                    var coinsIcon = new Entity({
                        name: 'coinsIcon',
                        position: Vector2(-22, -12),
                        scale: Vector2(0.16, 0.16),
                        components: [new Sprite({
                            imageName: 'gui/icon-1000',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    });
                    var coinAmountText = new Text({
                        text: '+' + amount,
                        position: new Vector2(8, -12),
                        scale: new Vector2(0.7, 0.7),
                        fontSettings: TextStyles.get('main'),
                        align: 'left',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    });
                    var hintsIcon = new Entity({
                        name: 'hintsIcon',
                        position: Vector2(-22, 28),
                        scale: Vector2(0.16, 0.16),
                        components: [new Sprite({
                            imageName: 'gui/2-hints',
                            originRelative: new Vector2(0.5, 0.5)
                        })]
                    });
                    var hintAmountText = new Text({
                        text: '+' + hints,
                        position: new Vector2(8, 30),
                        scale: new Vector2(0.7, 0.7),
                        fontSettings: TextStyles.get('main'),
                        align: 'left',
                        maxWidth: 64,
                        maxHeight: 32,
                        drawDebug: false
                    });

                    w.attach(coinsIcon);
                    w.attach(coinAmountText);
                    w.attach(hintsIcon);
                    w.attach(hintAmountText);

                    var animAmount = 10;
                    var interval = void 0,
                        duration = void 0;
                    if (amount < 1000) {
                        interval = 4;
                        duration = 26;
                        animAmount = 10;
                    } else if (amount < 5000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 20;
                    } else if (amount < 11000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 30;
                    } else if (amount < 24000) {
                        interval = 3;
                        duration = 26;
                        animAmount = 40;
                    } else {
                        interval = 4;
                        duration = 26;
                        animAmount = 50;
                    }

                    Utils.timeout(20, function () {
                        CoinCollectAnim({
                            z: Globals.Layers.DIALOG_CONFETTI,
                            from: coinsIcon.getWorldPosition(),
                            to: cc.getWorldPosition(),
                            amount: animAmount,
                            interval: interval,
                            duration: duration,
                            onCoinSpawn: function onCoinSpawn(i) {
                                new Tween({
                                    from: 1.2, to: 1, in: 6,
                                    ease: 'easeOutQuad',
                                    onUpdate: function onUpdate(v, t) {
                                        // hdSprite.scale.x = hdSpriteScale * v;
                                        // hdSprite.scale.y = hdSpriteScale * v;
                                        // shadowSprite.scale.x = hdSpriteScale * v;
                                        // shadowSprite.scale.y = hdSpriteScale * v;
                                    }
                                });
                            },
                            onCoinCollect: function onCoinCollect(i) {
                                EventSystem.fire('updateCoins', Math.floor(oldCoins + amount * i / animAmount));
                            }
                        });
                    });

                    // Show Hint counter
                    // let pos1 = sc ? sc.position : new Vector2(73, -viewport.height*0.5 + 25);
                    // let pos2 = cc ? cc.position : new Vector2(73, -viewport.height*0.5 + 46);
                    // let showPos = pos1.add(pos2).scale(0.5);
                    var showPos = pos.add(Vector2(0, 20));
                    var hidePos = showPos.add(Vector2(30, 0));
                    var counter = new GameCounter({
                        name: 'hintCounterCounter',
                        value: oldHints,
                        align: 'right',
                        position: new Vector2(-10, 2),
                        scale: new Vector2(0.095, 0.095),
                        imageName: 'numbers-jua-104-white'
                    });
                    var hintIcon = new SimpleSprite({
                        imageName: 'gui/hint-icon',
                        scale: new Vector2(0.06, 0.06)
                    });
                    var hintCounter = new Entity({
                        name: 'hintCounter',
                        position: hidePos.clone(),
                        scale: new Vector2(Gui.scaleFactor, Gui.scaleFactor),
                        components: [counter, hintIcon]
                    });
                    Tween({
                        delay: 20,
                        from: hidePos,
                        to: showPos,
                        in: 28,
                        ease: 'easeOutBack',
                        onStart: function onStart() {
                            dialog.attach(hintCounter);
                        },
                        onUpdate: function onUpdate(v, t) {
                            hintCounter.position.x = v.x;
                            hintCounter.position.y = v.y;
                        },
                        onComplete: function onComplete() {
                            var _loop = function _loop(i) {
                                // increase and pulse hint counter value
                                Tween({
                                    delay: 40 + i * 20,
                                    from: 1.12, to: 1.0, in: 12,
                                    ease: 'easeOutQuad',
                                    onStart: function onStart() {
                                        counter.setValue(oldHints + i + 1);
                                    },
                                    onUpdate: function onUpdate(v, t) {
                                        counter.scale.x = 0.095 * v;
                                        counter.scale.y = 0.095 * v;
                                        hintIcon.scale.x = 0.06 * v;
                                        hintIcon.scale.y = 0.06 * v;
                                    }
                                });
                            };

                            for (var i = 0; i < hints; i++) {
                                _loop(i);
                            }
                        }
                    });
                }
            });
        });

        return dialog;
    };

    // VIP reward checks based on thunderdogs.
    // Note there may be some issues with this approach
    // e.g. if I claim a gift after 47 hours, I should get my next gift in 1 hour, not in 24 hours.

    var getVipGiftDays = function getVipGiftDays() {
        // check difference between last time gift was claimed
        // --> this time should also set when vip becomes active?
        var timeDiff = TimeManager.getTime() - Bento.saveState.load('lastTimeVipClaimed', 0);
        var daysDiff = Math.floor(timeDiff / 1000 / 60 / 60 / 24);
        if (daysDiff >= 1) {
            return daysDiff;
        } else {
            return 0;
        }
    };
    var getGift = function getGift() {
        var days = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getVipGiftDays();

        if (days === 0) {
            return null;
        }
        if (days > 7) {
            // limit to 7 days in case there is some terrible exploit here
            days = 7;
        }
        return {
            coins: 100 * days,
            hints: 2 * days
        };
    };
    var hasGift = function hasGift() {
        return getVipGiftDays() > 0;
    };

    Utils.extend(VipGiftDialog, {
        getVipGiftDays: getVipGiftDays,
        getGift: getGift,
        hasGift: hasGift
    });

    return VipGiftDialog;
});