'use strict';

bento.define('gui/vippopup', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/text', 'bento/utils', 'bento/tween', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'components/wiggle', 'components/simplesprite', 'components/hdsprite', 'gui/uiloader', 'modules/textstyles', 'modules/confetti', 'entities/gridbg', 'entities/gui/starsearned', 'gui/gui', 'gui/guicommon', 'modules/admanager', 'modules/levelmanager', 'components/debug', 'bento/components/modal', 'modules/tapticengine', 'modules/localization', 'modules/savemanager', 'gui/iapoverlay', 'components/mask', 'modules/timemanager', 'modules/cloudsave', 'gui/vipgiftdialog'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, Text, Utils, Tween, Globals, Fill, EventListener, Wiggle, SimpleSprite, HDSprite, UILoader, TextStyles, Confetti, GridBg, StarsEarned, Gui, GuiCommon, AdManager, LevelManager, Debug, Modal, TapticEngine, Localization, SaveManager, IapOverlay, Mask, TimeManager, CloudSave, VipGiftDialog) {
    var makeButton = Gui.makeButton,
        makeText = Gui.makeText;

    var onComplete = void 0;
    var _canClick = false;

    var VipPopup = function VipPopup(settings) {

        onComplete = settings.onComplete;
        _canClick = false;

        var viewport = Bento.getViewport();

        var fill = new Fill({
            name: 'fill',
            dimension: Bento.getViewport(true).getSize(),
            color: [0, 0, 0, 0.90], // [1, 1, 1, 1] is pure white
            originRelative: new Vector2(0, 0)
        });

        var vipPopupRoot = new Entity({
            name: 'vipPopupRoot',
            position: new Vector2(viewport.width / 2, viewport.height / 2)
        });

        Gui.load({
            parent: vipPopupRoot,
            assetName: 'ui/vippopup',
            handlers: GuiVipPopup, onComplete: function onComplete() {
                // fade in overlay
                new Tween({
                    from: 0, to: 1,
                    in: 30,
                    delay: 0,
                    applyOnDelay: 0,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        _canClick = true;
                    }
                });
            }
        });

        var entity = new Entity({
            z: Globals.Layers.DIALOGS,
            name: 'vipPopup',
            position: new Vector2(0, 0),
            float: true,
            components: [new Modal({}), fill, vipPopupRoot]
        });

        entity.extend({
            dismiss: function dismiss() {
                _canClick = false;
                new Tween({
                    from: 1, to: 0, in: 30,
                    ease: 'easeInOutQuad',
                    onUpdate: function onUpdate(v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function onComplete() {
                        entity.removeSelf();
                    }
                });
            }
        });

        entity.alpha = 0;

        return entity;
    };

    var GuiVipPopup = Utils.copyObject(GuiCommon);

    var items = void 0;
    var totalWeight = 0.0;
    var chosenItemIndex = void 0;

    Utils.extend(GuiVipPopup, {
        init: function init(obj) {},
        vipWindow: function vipWindow(obj) {
            Gui.adjust(obj, {
                x: 0 * (1 - Gui.scaleFactorTall),
                y: -60 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall
            });
        },
        vipWindowCrown: function vipWindowCrown(obj) {
            Gui.adjust(obj, {
                x: 0 * (1 - Gui.scaleFactorTall),
                y: 80 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall * Gui.scaleFactorTall
            });
        },
        vipWindowClose: function vipWindowClose(obj) {
            Gui.adjust(obj, {
                x: -46 * (1 - Gui.scaleFactorTall),
                y: 80 * (1 - Gui.scaleFactorTall),
                scale: Gui.scaleFactorTall * Gui.scaleFactorTall
            });
            makeButton(obj, {
                canClick: function canClick() {
                    return _canClick;
                },
                onClick: function onClick() {
                    Bento.objects.get('vipPopup', function (vipPopup) {
                        vipPopup.dismiss();
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: 'Stats:subscriptionConversion',
                            value: 0
                        });
                    });
                }
            });
        },
        buttonSubscribe: function buttonSubscribe(obj) {

            var area = new Rectangle(-48, -17, 96, 34);
            var shine = new Entity({
                name: 'shine',
                position: new Vector2(0, 0),
                scale: new Vector2(0.25, 0.25),
                components: [new Sprite({
                    imageName: 'gui/shine',
                    originRelative: new Vector2(0.5, 0.5)
                })]
            });
            var maskContainer = new Entity({
                name: 'maskContainer',
                position: new Vector2(0, 0),
                dimension: area,
                components: [shine,
                // Debug({}),
                new Mask({
                    rectangle: area,
                    sprite: shine.getComponent('sprite')
                })]
            });
            obj.attach(maskContainer);
            obj.attach({
                name: 'behavior',
                shineCount: 0,
                start: function start() {
                    this.shineCount = 0;
                    shine.position.x = -400;
                },
                update: function update(_ref) {
                    var speed = _ref.speed;

                    shine.position.x += speed * 6;
                    if (shine.position.x > 160) {
                        shine.position.x = this.shineCount % 2 === 0 ? -100 : -900;
                        this.shineCount++;
                    }
                }
            });

            var maskOverlay = new HDSprite({
                imageName: 'gui/free-trial-button-overlay-light',
                gamepixelDimension: obj.dimension.clone(),
                originRelative: new Vector2(0.5, 0.5)
            });
            obj.attach(maskOverlay);
            // obj.moveComponentTo(maskOverlay, 4);

            Gui.adjust(obj, {
                x: 0 * (1 - Gui.scaleFactorTall),
                y: -150 * (1 - Gui.scaleFactorTall) + 1,
                scale: Gui.scaleFactorTall * 1.1
            });

            makeButton(obj, {
                onClick: function onClick() {
                    IapOverlay.purchase({
                        z: Globals.Layers.IAP_OVERLAY,
                        id: 'goslice_vip',
                        subscription: true,
                        isConsumable: false,
                        fontSettings: TextStyles.get('iapOverlay'),
                        onPurchaseComplete: function onPurchaseComplete(success, id, purchaseInfo) {
                            if (!success || id !== 'goslice_vip') {
                                return;
                            }

                            // close the popup, the onPurchase() handler in init.js should take care of the rest
                            Bento.objects.get('vipPopup', function (vipPopup) {
                                vipPopup.dismiss();
                            });

                            // perform cloud save
                            CloudSave.saveToCloud();

                            AdManager.hideBanner();
                            EventSystem.fire('bannerVisibilityChanged');

                            // title screen should handle giving out the reward for the first time.
                            Bento.screens.show('screens/titlescreen');

                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'Stats:subscriptionConversion',
                                value: 1
                            });
                        }
                    });
                }
            });

            obj.getComponent('hdsprite3', function (hdSprite) {
                hdSprite.getComponent('hdsprite3', function (sprite) {
                    sprite.setup({
                        imageName: 'gui/free-trial-button-light'
                    });
                });
            });
        },
        subscribeText1: function subscribeText1(obj) {
            // makeText(obj, {
            //     text: Localization.getText('tryForFree').toUpperCase(),
            //     fontSettings: TextStyles.get('buttonSubscribeText1'),
            //     align: 'center',
            // });
            obj.dimension.width += 12;
            obj.position.y += 8;
            obj.dimension.height += 12;
            makeText(obj, {
                text: Localization.getText('freeTrial').replace('{NUMBER}', '3').replace('{CURRENCY}', IapOverlay.getPrice('goslice_vip', '$7.99')).toUpperCase(),
                fontSettings: TextStyles.get('buttonSubscribeText1'),
                align: 'center'
                // drawDebug: true,
            });
        },
        subscribeText2: function subscribeText2(obj) {
            // makeText(obj, {
            //     text: Localization.getText('freeTrial')
            //         .replace('{NUMBER}', '3')
            //         .replace('{CURRENCY}', IapOverlay.getPrice('goslice_vip', '$7.99'))
            //         .toUpperCase(),
            //     fontSettings: TextStyles.get('buttonSubscribeText2'),
            //     align: 'center',
            // });
        },
        vipTitleGoSlice: function vipTitleGoSlice(obj) {
            makeText(obj, {
                text: Localization.getText('title').toUpperCase(),
                fontSettings: TextStyles.get('vipWindowTitle1'),
                align: 'center'
            });
        },
        vipTitleVip: function vipTitleVip(obj) {
            makeText(obj, {
                text: 'VIP',
                fontSettings: TextStyles.get('vipWindowTitle2'),
                align: 'center'
            });
        },
        vipNoAdsText: function vipNoAdsText(obj) {
            makeText(obj, {
                text: Localization.getText('removeAds').toUpperCase(),
                fontSettings: TextStyles.get('vipMain'),
                align: 'center'
            });
        },
        vipCoinsText: function vipCoinsText(obj) {
            makeText(obj, {
                text: Localization.getText('nCoinsPerDay').replace('{NUMBER}', '100').toUpperCase(),
                fontSettings: TextStyles.get('vipMain'),
                align: 'center'
            });
        },
        vipHintsText: function vipHintsText(obj) {
            makeText(obj, {
                text: Localization.getText('nHintsPerDay').replace('{NUMBER}', '2').toUpperCase(),
                fontSettings: TextStyles.get('vipMain'),
                align: 'center'
            });
        },
        vipCharsText: function vipCharsText(obj) {
            makeText(obj, {
                text: Localization.getText('exclusiveCharacters').toUpperCase(),
                fontSettings: TextStyles.get('vipChars'), // temp
                align: 'center'
            });
        },
        vipTerms: function vipTerms(obj) {
            var text = VipPopup.loadTerms();
            text.removeByName('behavior');
            text.removeSelf();

            obj.attach(text);

            // on iPad the terms end up considerably shrunken.
            // so we can move the popup window and other elements down,
            // depending on how much less vertical space the text occupies.
            text.attach({
                name: 'behavior',
                start: function start() {
                    var intendedHeight = 78;
                    var height = text.getBoundingBox().height;
                    var diff = intendedHeight - height;
                    text.position.y = -36 + diff;
                    Gui.get('vipWindow', function (e) {
                        return e.position.y += diff * 0.8;
                    });
                    Gui.get('vipWindowCrown', function (e) {
                        return e.position.y += diff * 0.8;
                    });
                    Gui.get('vipWindowClose', function (e) {
                        return e.position.y += diff * 0.8;
                    });
                    Gui.get('buttonSubscribe', function (e) {
                        return e.position.y += diff * 0.76;
                    });
                }
            });
        },
        vipPrivacyPolicy: function vipPrivacyPolicy(obj) {
            makeText(obj, {
                text: Localization.getText('privacyPolicy'),
                fontSettings: TextStyles.get('vipLink'),
                align: 'center',
                linebreaks: false
            });
            Gui.adjust(obj, {
                x: 60 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor
            });
            obj.attach(new Clickable({
                onClick: function onClick(data) {
                    switch (Localization.getLanguage()) {
                        case 'ja':
                            Utils.openUrl('https://www.lucky-kat.com/games-privacy-notice-ja');break;
                        case 'ko':
                            Utils.openUrl('https://www.lucky-kat.com/games-privacy-notice-ko');break;
                        default:
                            Utils.openUrl('https://www.lucky-kat.com/games-privacy-notice');break;
                    }
                }
            }));
        },
        vipTermsOfService: function vipTermsOfService(obj) {
            makeText(obj, {
                text: Localization.getText('termsOfService'),
                fontSettings: TextStyles.get('vipLink'),
                align: 'center',
                linebreaks: false
            });
            Gui.adjust(obj, {
                x: -60 * (1 - Gui.scaleFactor),
                scale: Gui.scaleFactor
            });
            obj.attach(new Clickable({
                onClick: function onClick(data) {
                    switch (Localization.getLanguage()) {
                        case 'ja':
                            Utils.openUrl('https://www.lucky-kat.com/terms-of-use-ja');break;
                        case 'ko':
                            Utils.openUrl('https://www.lucky-kat.com/terms-of-use-ko');break;
                        default:
                            Utils.openUrl('https://www.lucky-kat.com/terms-of-use');break;
                    }
                }
            }));
        }
    });

    var termsText = void 0;

    VipPopup.loadTerms = function () {
        if (termsText) {
            return termsText; // cached result
        }

        var viewport = Bento.getViewport();
        var serviceString = '';
        if (Utils.isNativeAndroid()) serviceString = 'Google Play';else if (Utils.isNativeIos()) serviceString = 'iTunes';

        var subscriptionName = void 0;
        var lang = Localization.getLanguage();
        if (lang === 'ja' || lang === 'zh' || lang === 'zt') {
            subscriptionName = Localization.getText('title') + 'VIP'; // no space for these langs
        } else {
            subscriptionName = Localization.getText('title') + ' VIP';
        }
        var termsString = Localization.getText('subscriptionTerms').replace('{SUBSCRIPTIONNAME}', subscriptionName).replace('{DAYS}', 3).replace('{PRICE}', IapOverlay.getPrice('goslice_vip', '$7.99')).replace('{FEATURES}', Localization.getText('subscriptionFeatures')).replace('{SERVICE}', serviceString);

        var termsWidth = viewport.width * 0.94 * Math.max(Gui.scaleFactor * Gui.scaleFactor, 0.93);
        var termsScale = 0.48 * Gui.scaleFactor * Gui.scaleFactor;

        termsText = new Text({
            position: new Vector2(termsWidth / -2, -36),
            text: termsString,
            font: 'Helvetica',
            fontSize: 9,
            fontColor: '#aaaaaa',
            align: 'left',
            textBaseline: 'center',
            maxWidth: termsWidth * (1 / termsScale),
            // maxHeight: 200,
            linebreaks: true,
            ySpacing: 2,
            // drawDebug: true,
            scale: new Vector2(termsScale, termsScale)
        });

        return termsText;
    };

    return VipPopup;
});