'use strict';

bento.define('modules/abtest', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/tween', 'modules/savemanager'], function (Bento, Vector2, Rectangle, Entity, EventSystem, Utils, Tween, SaveManager) {
    /**
     * AB test manager
     * @moduleName ABTest
     */
    var bootupAnalytics = function bootupAnalytics() {
        var now = Date.now();
        var day = Math.floor(now / 1000 / 60 / 60 / 24);

        // doing some bootup analytics
        if (SaveManager.load('bootups') === 0) {
            SaveManager.save('firstBootDate', now);
            EventSystem.fire('GameAnalytics-addDesignEvent', { eventId: 'firstBoot' });
        }
        SaveManager.add('bootups', 1);

        var firstBootDay = Math.floor(SaveManager.load('firstBootDate') / 1000 / 60 / 60 / 24);
        var dayDiff = day - firstBootDay;
        var retainedDays = SaveManager.load('retainedDays');
        retainedDays[dayDiff] = 1;
        SaveManager.save('retainedDays', retainedDays);

        if (dayDiff === 1) {
            EventSystem.fire('Tenjin', {
                action: 'D1_retention'
            });
        } else if (dayDiff === 7) {
            EventSystem.fire('Tenjin', {
                action: 'D7_retention'
            });
        }

        if (dayDiff === 1) {
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'game:levelsCleared:D1',
                value: SaveManager.load('levelsCleared')
            });
        } else if (dayDiff === 7) {
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'game:levelsCleared:D7',
                value: SaveManager.load('levelsCleared')
            });
        } else if (dayDiff === 14) {
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'game:levelsCleared:D14',
                value: SaveManager.load('levelsCleared')
            });
        } else if (dayDiff === 30) {
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'game:levelsCleared:D30',
                value: SaveManager.load('levelsCleared')
            });
        }

        // update total playtime every 5 seconds
        window.setInterval(function () {
            SaveManager.add('totalTimePlayed', 5000);
        }, 5000);
    };

    /**
     * For sanity, we always call this even if the custom dimension value didn't change since last boot.
     */
    var setCustomDimensions = function setCustomDimensions() {
        EventSystem.fire('GameAnalytics-setCustomDimension01', Bento.saveState.load('AB_InterstitialRate', ''));
        // EventSystem.fire('GameAnalytics-setCustomDimension02', Bento.saveState.load('AB_GameSpeed', ''));
        // EventSystem.fire('GameAnalytics-setCustomDimension03', Bento.saveState.load('AB_Theme', ''));
    };

    return {
        init: function init() {

            // Generate a UID for this user, which is also used for CloudSave
            if (SaveManager.load('identifier') === '') {
                var userId = void 0;
                // use default methods
                if (window.device) {
                    userId = 'dvc_' + window.device.uuid;
                } else if (navigator.isCocoonJS && window.Cocoon && window.Cocoon.Device) {
                    userId = 'ccd_' + window.Cocoon.Device.getDeviceId();
                } else {
                    // last resort: randomize number
                    userId = 'rnd_' + (1 + Utils.getRandom(100000000));
                }

                if (Utils.isDev()) {
                    userId = 'dev';
                }

                SaveManager.save('identifier', userId);
            }

            bootupAnalytics();

            // Set up AB Tests
            // ---------------
            // Note: Use Bento.saveState for AB test keys to reduce clutter in SaveManager.

            var keys = Bento.saveState.getKeys();
            for (var i = 0; i < keys.length; i++) {
                if (keys[i].indexOf('stars-') === 0) {
                    // Don't setup any new AB tests if the player has already beaten any level.
                    // In the future we can just use `if (bootups !== 1)`
                    // But we can't rely on that yet.
                    console.log('Skipping AB test setup');
                    setCustomDimensions();
                    return;
                }
            }

            // Testing alternative level order:
            if (!Bento.saveState.load('AB_InterstitialRate')) {
                var dimension = Utils.pickRandom('InterstitialPer3Levels', 'InterstitialPer5Levels');
                Bento.saveState.save('AB_InterstitialRate', dimension);
                console.log('AB01:', dimension);
            }

            // Testing slightly faster simulation speed
            // if (!Bento.saveState.load('AB_GameSpeed')) {
            //     let dimension = Utils.pickRandom('NormalSpeed', 'FasterSpeed');
            //     Bento.saveState.save('AB_GameSpeed', dimension);
            //     console.log('AB02:', dimension);
            // }

            // Testing a new level theme every 5 levels
            // if (!Bento.saveState.load('AB_Theme')) {
            //     let dimension = Utils.pickRandom('ConstantTheme', 'ChangingTheme');
            //     Bento.saveState.save('AB_Theme', dimension);
            //     console.log('AB03:', dimension);
            // }

            setCustomDimensions();
        }
    };
});