'use strict';

bento.define('modules/admanager', ['bento', 'utils', 'audia', 'modules/ironsource', 'bento/eventsystem', 'modules/vipmanager', 'modules/savemanager'], function (Bento, Utils, Audia, IronSource, EventSystem, VipManager, SaveManager) {

    var interstitialCounter = 1; // by starting from 1 always show interstitial unless we watch an RV.

    var incrementCounter = function incrementCounter() {
        return ++interstitialCounter;
    };
    var getCounter = function getCounter() {
        return interstitialCounter;
    };
    var resetCounter = function resetCounter() {
        interstitialCounter = 0;
    };

    var countViews = function countViews() {
        return SaveManager.load('firstPlayedVersion') >= Utils.versionToHex('1.0.6');
    };

    var canShowBanner = function canShowBanner() {
        return !(VipManager.isVip() || SaveManager.load('noAds'));
    };
    var canShowInterstitial = function canShowInterstitial(callback) {
        if (VipManager.isVip()) {
            callback(false, 'User is VIP');
        } else if (SaveManager.load('noAds')) {
            callback(false, 'User has noAds IAP');
        } else if (Bento.isDev()) {
            callback(true);
        } else {
            if (IronSource.canShowInterstitial()) {
                callback(true);
            } else {
                callback(false, '');
            }
        }
    };

    var tryShowInterstitial = function tryShowInterstitial(callback) {
        var tag = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
        var force = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (Bento.isDev() && !force) {
            callback(false, 'Interstitial ads not shown in Dev mode.');
        } else {
            canShowInterstitial(function (canShow, e) {
                if (canShow) {
                    IronSource.showInterstitial({
                        tag: tag,
                        muteAudio: true,
                        onComplete: function onComplete() {
                            EventSystem.fire('Tenjin', { action: 'interstitial_view' });
                            if (countViews()) {
                                var totalInterstitials = SaveManager.load('totalInterstitials');
                                SaveManager.save('totalInterstitials', ++totalInterstitials);
                                EventSystem.fire('GameAnalytics-addDesignEvent', {
                                    eventId: 'ads:interstitial:success',
                                    value: totalInterstitials
                                });
                                EventSystem.fire('FirebaseAnalytics-logEvent', {
                                    name: 'interstitial_impression'
                                });
                            }
                            callback(true);
                        }
                    });
                } else {
                    callback(false, e || '');
                }
            });
        }
    };

    var canShowRewarded = function canShowRewarded(callback) {
        if (Bento.isDev()) {
            callback(true);
        } else {
            if (IronSource.canShowRewarded()) {
                callback(true);
            } else {
                callback(false, '');
            }
        }
    };

    var tryShowRewarded = function tryShowRewarded(_ref) {
        var _onReward = _ref.onReward,
            _onComplete = _ref.onComplete,
            _ref$tag = _ref.tag,
            tag = _ref$tag === undefined ? '' : _ref$tag,
            rewardType = _ref.rewardType,
            rewardAmount = _ref.rewardAmount;

        if (Bento.isDev()) {
            Utils.timeout(4, function () {
                if (window.confirm('REWARDED AD')) {
                    _onReward();
                    _onComplete();
                    resetCounter();
                } else {
                    _onComplete('User cancelled');
                }
            });
        } else {

            var lvl = Utils.currentLevel + 1;
            var group = Utils.currentLevelGroup;
            var qualifiedLevelNumber = group === 'default' ? '' + lvl : group + '-' + lvl;

            canShowRewarded(function (canShow, e) {
                IronSource.showRewarded({
                    tag: tag,
                    muteAudio: true,
                    onReward: function onReward() {
                        EventSystem.fire('Tenjin', { action: 'video_view' });
                        if (countViews()) {
                            var totalRewarded = SaveManager.load('totalRewarded');
                            SaveManager.save('totalRewarded', ++totalRewarded);
                            if (totalRewarded === 10) {
                                EventSystem.fire('Tenjin', { action: '10RV_watched' });
                            } else if (totalRewarded === 15) {
                                EventSystem.fire('Tenjin', { action: '15RV_watched' });
                            } else if (totalRewarded === 20) {
                                EventSystem.fire('Tenjin', { action: '20RV_watched' });
                            }
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'ads:rewarded:success',
                                value: totalRewarded
                            });
                        }
                        if (tag) {
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'ads:' + tag + ':rewardSuccess'
                            });
                            EventSystem.fire('FirebaseAnalytics-logEvent', {
                                name: 'rewarded_video_complete',
                                properties: {
                                    reward_type_name: rewardType,
                                    received_reward_amount: rewardAmount
                                }
                            });
                        }
                        _onReward();
                        resetCounter();
                    },
                    onComplete: function onComplete(error) {
                        _onComplete(error);
                        if (error) {
                            EventSystem.fire('GameAnalytics-addDesignEvent', {
                                eventId: 'ads:' + tag + ':rewardFail'
                            });
                        } else {
                            EventSystem.fire('FirebaseAnalytics-logEvent', {
                                name: 'rewarded_video_impression',
                                properties: {
                                    reward_type_name: rewardType,
                                    impression_level: qualifiedLevelNumber
                                }
                            });
                        }
                    }
                });
            });
        }
    };

    // Coda stuff

    document.addEventListener('didClickBanner', function (data) {
        EventSystem.fire('FirebaseAnalytics-logEvent', {
            name: 'banner_click',
            properties: {}
        });
    }, false);

    document.addEventListener('bannerWillLeaveApplication', function (data) {
        EventSystem.fire('FirebaseAnalytics-logEvent', {
            name: 'banner_impression',
            properties: {}
        });
    }, false);

    var viewportReduction = 24;
    var viewportReduced = false;

    // Call this in init, before any call to showBanner/hideBanner.
    var initViewportHack = function initViewportHack() {
        var viewport = Bento.getViewport();
        var realViewport = viewport.clone();

        if (canShowBanner()) {
            viewport.height -= viewportReduction;
            viewportReduced = true;
        }

        Bento.getViewport = function (real) {
            return real ? realViewport : viewport;
        };
    };

    var showBanner = function showBanner() {
        if (IronSource.isBannerVisible()) {
            console.log('Banner already visible');
        } else if (!canShowBanner()) {
            console.log('Not allowed to show banner.');
        } else {
            IronSource.showBanner();
            if (!viewportReduced) {
                var viewport = Bento.getViewport();
                viewport.height -= viewportReduction;
                viewportReduced = true;
            }
            EventSystem.fire('bannerVisibilityChanged');
        }
    };

    var hideBanner = function hideBanner() {
        IronSource.hideBanner();
        if (viewportReduced) {
            var viewport = Bento.getViewport();
            viewport.height += viewportReduction;
            viewportReduced = false;
        }
        EventSystem.fire('bannerVisibilityChanged');
    };

    return {
        canShowRewarded: canShowRewarded,
        tryShowRewarded: tryShowRewarded,
        canShowInterstitial: canShowInterstitial,
        tryShowInterstitial: tryShowInterstitial,
        incrementCounter: incrementCounter,
        getCounter: getCounter,
        resetCounter: resetCounter,
        canShowBanner: canShowBanner,
        showBanner: showBanner,
        hideBanner: hideBanner,
        initViewportHack: initViewportHack
    };
});