'use strict';

/**
 * Confetti overlay
 * @moduleName Confetti
 */
bento.define('modules/confetti', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/utils', 'bento/tween'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, Utils, Tween) {
    'use strict';

    return function (settings) {
        var viewport = Bento.getViewport();
        var pauseLevel = settings.updateWhenPaused || Bento.objects.isPaused();
        var duration = settings.duration || 180;
        var scale = new Vector2(settings.particleScale || 1, settings.particleScale || 1);
        var entity = new Entity({
            z: settings.z || 0,
            name: 'confetti',
            position: settings.position || new Vector2(0, 0),
            float: true,
            updateWhenPaused: pauseLevel,
            global: settings.global,
            init: function init() {
                var self = this;
                var fadeIn = new Tween({
                    from: 0,
                    to: 1,
                    in: 10,
                    ease: 'linear',
                    updateWhenPaused: pauseLevel,
                    do: function _do(v, t) {
                        self.alpha = v;
                    },
                    onComplete: function onComplete() {
                        var delay = Utils.timeout(duration, function () {
                            var fadeOut = new Tween({
                                from: 1,
                                to: 0,
                                in: 30,
                                ease: 'linear',
                                updateWhenPaused: pauseLevel,
                                do: function _do(v, t) {
                                    self.alpha = v;
                                },
                                onComplete: function onComplete() {
                                    Bento.objects.remove(self);
                                }
                            });
                            fadeOut.global = settings.global || false;
                        });
                        delay.updateWhenPaused = pauseLevel;
                        delay.global = settings.global || false;
                    }
                });
                fadeIn.updateWhenPaused = pauseLevel;
                fadeIn.global = settings.global || false;
            }
        });
        var spawnConfetti = function spawnConfetti() {
            var confetti = new Entity({
                name: 'confetti',
                position: new Vector2(Utils.getRandom(viewport.width), Utils.getRandom(viewport.height)),
                scale: scale.clone(),
                updateWhenPaused: pauseLevel,
                components: [new Sprite({
                    image: Bento.assets.getImage(settings.imageName || 'confettis'),
                    originRelative: new Vector2(0.5, 0.5),
                    frameCountX: 3,
                    animations: {
                        'default': {
                            speed: 0,
                            frames: [Utils.getRandom(3)]
                        }
                    }
                })]
            }).attach({
                name: 'behavior',
                speedX: 0,
                speedY: settings.speedY || 1,
                rotationSpeed: Utils.getRandom(5),
                start: function start() {
                    confetti.timer = Utils.getRandomFloat(1000);
                },
                update: function update() {
                    var dimension = confetti.dimension,
                        position = confetti.position;
                    if (position.x < -dimension.width) {
                        position.x = viewport.width + dimension.width;
                    } else if (position.x > viewport.width + dimension.width) {
                        position.x = -dimension.width;
                    }
                    if (position.y < -dimension.height) {
                        position.y = viewport.height + dimension.height;
                    } else if (position.y > viewport.height + dimension.height) {
                        position.y = -dimension.height;
                    }

                    this.speedX += (Utils.getRandom(10) - Utils.getRandom(10)) / 10;
                    this.speedX = Math.min(Math.max(this.speedX, -2), 2);

                    position.x += this.speedX;
                    position.y += this.speedY;

                    // flip
                    confetti.scale.x = Math.sin(confetti.timer / (this.rotationSpeed + 5)) * settings.particleScale;
                }
            });

            entity.attach(confetti);
        };

        Utils.repeat(25, spawnConfetti);

        return entity;
    };
});