'use strict';

bento.define('modules/glowfilter', [], function () {
    // https://pixi-glow-v3.netlify.com

    var PIXI = window.PIXI;

    var GlowFilter = function GlowFilter(textureWidth, textureHeight, distance, outerStrength, innerStrength, color, quality, useMipmap) {
        PIXI.AbstractFilter.call(this);

        quality = Math.pow(quality, 1 / 2);
        this.quality = quality;
        distance *= quality;
        textureWidth *= quality;
        textureHeight *= quality;
        var lod = useMipmap ? Math.log(1 / quality) / Math.log(2) : 0;

        this.uniforms = {
            distance: { type: '1f', value: distance },
            outerStrength: { type: '1f', value: null },
            innerStrength: { type: '1f', value: null },
            glowColor: { type: '3f', value: null },
            pixelWidth: { type: '1f', value: null },
            pixelHeight: { type: '1f', value: null }
        };

        this.color = color;
        this.outerStrength = outerStrength;
        this.innerStrength = innerStrength;
        this.textureWidth = textureWidth;
        this.textureHeight = textureHeight;

        this.fragmentSrc = '\nprecision highp float;\nvarying vec2 vTextureCoord;\nuniform sampler2D texture;\nuniform float distance;\nuniform float outerStrength;\nuniform float innerStrength;\nuniform vec3 glowColor;\nuniform float pixelWidth;\nuniform float pixelHeight;\nvec2 px = vec2(pixelWidth, pixelHeight);\nvoid main(void) {\n    const float PI = 3.14159265358979323846264;\n    vec4 ownColor = texture2D(texture, vTextureCoord, 0.);\n    vec4 curColor;\n    float totalAlpha = 0.;\n    float maxTotalAlpha = 0.;\n    float cosAngle;\n    float sinAngle;\n    for (float angle = 0.; angle < PI * 2.; angle += ' + Math.min(0.5 / distance, Math.PI * 0.5).toFixed(7) + ') {\n       cosAngle = cos(angle);\n       sinAngle = sin(angle);\n       for (float curDistance = 0.; curDistance <= ' + Math.max(1.0, distance).toFixed(7) + '; curDistance++) {\n           curColor = texture2D(texture, vec2(vTextureCoord.x + cosAngle * curDistance * px.x, vTextureCoord.y + sinAngle * curDistance * px.y), ' + lod.toFixed(7) + ');\n           totalAlpha += (curDistance / distance) * curColor.a;\n           maxTotalAlpha += (curDistance / distance);\n       }\n    }\n    maxTotalAlpha = max(maxTotalAlpha, 0.0001);\n    ownColor.a = max(ownColor.a, 0.0001);\n    ownColor.rgb = ownColor.rgb / ownColor.a;\n    float outerGlowAlpha = (totalAlpha / maxTotalAlpha)  * outerStrength * (1. - ownColor.a);\n    float innerGlowAlpha = ((maxTotalAlpha - totalAlpha) / maxTotalAlpha) * innerStrength * ownColor.a;\n    float resultAlpha = (ownColor.a + outerGlowAlpha);\n    gl_FragColor = vec4(mix(mix(ownColor.rgb, glowColor, innerGlowAlpha / ownColor.a), glowColor, outerGlowAlpha / resultAlpha) * resultAlpha, resultAlpha);\n}';
    };

    GlowFilter.prototype = Object.create(PIXI.AbstractFilter.prototype);
    GlowFilter.prototype.constructor = GlowFilter;

    Object.defineProperty(GlowFilter.prototype, 'color', {
        set: function set(value) {
            this.uniforms.glowColor.value = PIXI.utils.hex2rgb(value);
            this.dirty = true;
        }
    });

    Object.defineProperty(GlowFilter.prototype, 'outerStrength', {
        set: function set(value) {
            this.uniforms.outerStrength.value = value;
            this.dirty = true;
        }
    });

    Object.defineProperty(GlowFilter.prototype, 'innerStrength', {
        set: function set(value) {
            this.uniforms.innerStrength.value = value;
            this.dirty = true;
        }
    });

    Object.defineProperty(GlowFilter.prototype, 'textureWidth', {
        set: function set(value) {
            this.uniforms.pixelWidth.value = 1 / value;
            this.dirty = true;
        }
    });

    Object.defineProperty(GlowFilter.prototype, 'textureHeight', {
        set: function set(value) {
            this.uniforms.pixelHeight.value = 1 / value;
            this.dirty = true;
        }
    });

    return GlowFilter;
});