'use strict';

bento.define('modules/levelmanager', ['bento', 'bento/utils', 'modules/savemanager', 'globals', 'bento/eventsystem'], function (Bento, Utils, SaveManager, Globals, EventSystem) {

    /**
     * This module is responsible for ordering the levels, and saving/loading stats for each level.
     * The exposed functions only deal with indexes (where 0 = first level),
     *  so the rest of the game doesn't need to know about level IDs.
     */

    var groups = {};
    var challenges = [];

    var defineGroup = function defineGroup(groupName, properties, levels) {
        var coinsPerStar = properties.coinsPerStar,
            _properties$isChallen = properties.isChallenge,
            isChallenge = _properties$isChallen === undefined ? false : _properties$isChallen;


        if (coinsPerStar == null) {
            coinsPerStar = isChallenge ? 10 : 5;
        }

        groups[groupName] = {
            name: groupName,
            infos: [],
            levels: levels,
            coinsPerStar: coinsPerStar,
            isChallenge: isChallenge,
            firstUncleared: 0,
            totalStars: -1,
            totalCleared: -1,
            bossLevels: [], // list of boss level indexes
            levelIndexToBossNumber: {} // inverse of the above array, e.g. level 5 is the 0th boss
        };

        if (properties.isChallenge) {
            challenges.push(groupName);
        }
    };

    defineGroup('default', {}, ['test2', // TUTORIAL: slice
    'test2b', 'testi2', 'testb2', // diamond!
    // 'testh8', // two rectangles (too precise?)
    'testc8b', // simple ramp [BOSS]
    'testf11', // TUTORIAL: push
    'testf3', // fall and push
    'testc3', // let's surfing!   (could actually 'special case' this one so that it always works?)
    'teste2', // nook backwards
    'teste5', // plug the gap [BOSS]
    // 'testh11', // boring sliding ramp, sucks?
    'test13', 'testh3', 'lva3', // knock the block into the hole
    'testh4', // french fries  <--- this and the preceding levels are cool
    'test6', // wedge push    <-- this one less-cool but it's ok because there's been a lot of excitement
    'testb4', // kinematic 1
    'testd2', // nook
    'testh7', // TUTORIAL: two slices
    'testd4', // dual nook
    'testf1', // finnicky 2-slice
    'teste3', // cut same block 2 ways
    'testc7', 'test7', // simple ramp over chasm
    'lva4', // cool sawtooth
    'test11a', // stalagtites
    'teste4', 'testh1', 'testd3', 'testc5', 'testh13', // floating cut then push   <-- serve as a reintro to pushing
    'teste1', 'testf4', 'testf6', // Vclimb1
    'testf5', // Vclimb2
    'test15', // kinematic, push the block into the back hole
    'test_b1', // topple bridge (move down)
    'testg1', 'test9', 'testh6', 'testd6', 'testc4', 'testd5', 'testd1', 'test14', 'testb6',
    // 'testf10', // boring, basically the same as Vclimb1 (testf6)
    'lvb3', // dominos
    'testb7', 'testb8', 'testc6', 'testf2', 'test12', 'lvf3', // balance boss [BOSS]
    'lva1', // flag on tower
    'greg19', // 2 slice drop down gate
    'greg14', // cool tunnels
    'greg17', // improv catapult
    'lva7', // windmill 1
    'row-a31', // cut elevator [BOSS]
    'test9b', // wedge climb above pit
    'lvb5', // just some sideways bars with timing
    'greg2', // ~ 40/50 ?
    'lva2', // flag adventure
    'greg15', // haha which one
    'lva5', // annoying floating block
    'greg8', // moving circles timing [BOSS]
    'row-b4', // rowan layers
    'greg3', // split big block over tiny diamond
    'greg10', // finding the right point to slice
    'greg9', // tricks you (don't cut the first platform)
    'lvb4', // three bars
    'lvb4b', // three bars inverted
    'lvf8', // three layer tower [BOSS]
    'lva9', // moving platforms 1
    'lva10', // moving platforms 2
    'greg6', // a piece goes on a moving platform journey
    // 'lvb6', // too basic? add hints
    'lvc1', // push 3 pieces over gaps
    'lvc3', 'lvc4', // diagonal chute removal
    'lvc5', // double diagonal chute removal [BOSS]
    'lvc6', 'lva12', // ball: indiana jones
    'lvc7', // ball: 3x order puzzle
    'lvb2', // ball: loop around

    'lvb8', // little ball 1
    'lvb9', // little ball 2

    'lvg4', // going under [BOSS]

    'hernan1', // cuttable circle intro
    'hernan2', // cut circle in half
    'hernan3', // cut circle sideways
    'hernan4', // cut circle in half
    'hernan12', // intro hinges
    'hernan19', // hinge bridge [BOSS]
    'hernan13', // 2 hinges
    'hernan15', // golf with hinges
    'hernan14', // 2 hinges trick
    'hernan20', // hinge bridge 2
    'hernan6', // hinge bridge 3
    'lvg2', // three chains [BOSS]
    'hernan7', // block hinge
    'hernan8', // double cut hinge
    'hernan9', // double cut hinge 2
    'hernan5', // circle hinge bridge
    'hernan11', // motorized hinge
    'hernan16', // motorized balancing [BOSS]
    'hernan17', // motorized golf
    'hernan18', // conveyor belt

    'row-a1', 'row-a2', 'row-a3', 'row-a4', 'row-a5', 'row-a6', 'row-a7', 'row-a8', 'row-a9', 'lvg5', // use cage to build stairs [BOSS]
    'row-a10', 'row-a11', 'row-a12', 'row-a13', 'row-a14', 'row-a15', 'row-a16', 'row-a17', 'row-a18', 'row-a19', 'row-a20', 'emi-a1', 'emi-a2', 'emi-a3', 'emi-a4', 'emi-a5', 'emi-a6', 'emi-a8', // topple then walk back [BOSS]
    'emi-a7', 'emi-a9', 'emi-a10', 'emi-a11', 'emi-a13', 'emi-a14', 'emi-a12', //   [BOSS]
    'emi-a15', 'emi-a16', 'emi-a17', 'emi-a18', 'emi-a19', 'emi-a20', // [BOSS]
    'lvd2', 'lvd3', 'lvd4', 'lve2', 'lve4', 'lve8', // [BOSS]  <----
    'lve9', 'lve5', 'lve6', 'lve7', 'lvf1', 'lvf2', 'hernan21', 'hernan22', 'hernan23', 'hernan24', 'hernan25', 'hernan26', 'hernan27', 'hernan28', 'hernan29', 'hernan30', 'hernan31', 'hernan32', 'hernan33', 'hernan34', 'hernan36', // [BOSS]
    'hernan35', 'hernan37', 'hernan38', 'hernan39', 'hernan40', 'row-b1', 'row-b2', 'row-b3', 'row-b5', 'row-b6', 'row-b7', 'row-b8', 'row-b9', 'row-b10', // This one would make a good boss level!
    'row-b11', 'row-b12', 'row-b13', 'row-b14', 'row-b15', 'row-b16',
    // 'row-b17',  // omitted due to lag interfering with path-following?
    'row-b18', 'row-b19', 'row-b20', 'hernan10', // knock ball upside down

    'hernan45', 'hernan46', 'hernan47', 'hernan48', 'row-a34', // awesome elevator!!

    'hernan41', 'hernan44', 'hernan42', 'hernan43', 'row-a36', // need screw for flag
    'row-a39', // awesome rollers
    'row-a40', // awesome rollers 2
    'row-a37', 'row-a38', // jesus ok this is actually good but wow

    'emi-c1', // easy exploit?    // could go earlier
    'emi-c2', // bugs
    'emi-c3',
    // 'emi-c4',  too similar to c5
    'emi-c5', 'emi-c6', // cool!
    'emi-c7',
    // 'emi-c8',  // cheese
    // 'emi-c8a', // .
    // 'emi-c8b', // .
    'emi-c8c', // non cheese version?
    // 'emi-c8d', // .
    'emi-c9', 'emi-c10', // good  -- oops I made emi-c8c too similar
    'emi-c11', // good
    'emi-c12', // minor fix?
    'emi-c13', // good
    'emi-c14', // cheese
    'emi-c15', 'emi-c16', 'emi-c17', 'emi-c18', 'emi-c19', // its cool but maybe needs incorporating into more of a puzzle?
    'emi-c19b', 'emi-c20', // doesn't always work out :\
    'emi-c21', // the moving part isn't really needed here?
    'emi-c22', // fixed cheese (can't over the top of the ball)
    'emi-c23', // boulder factory
    'emi-c23b', // block factory
    // 'emi-c24',  // lag
    // 'emi-c24b', // less lag but kinda janky
    // 'emi-c24c', // testing
    'emi-c25', 'emi-c26', 'emi-c27', 'emi-c28', // fixed with chain, would be nice to rotate flag but the joints are overriding it or something

    'emi-c29', // -- should set the character off first then release the boulder... Not sure how to make this work
    // 'emi-c29b',  // -- should set the character off first then release the boulder... Not sure how to make this work

    // zorbing levels
    'row-a21', // a bit too precise?  maybe suspend from ceiling instead
    // 'row-a22',   // too similar to above
    'row-a23', // a bit precise and unclear? but you can probably fudge your way through it
    'row-a24', // !! weird... the level solves itself after 1 slice if you wait for long enough
    'row-a25', 'row-a26', 'row-a27', // very precise and easily goes wrong
    'row-a28', 'row-a29', 'row-a30', 'lvg6', // slice corner of corner
    'lvg7', // a variation of that??
    'lvg8', // weight machine
    'lvg9', // cross (hard)
    'lvb7', // add hints
    // 'lvh1',  // dual dominos
    'lvh1b', // dual dominos alt.
    'lvh2',

    // elevator levels etc.
    'row-a32', // elevator: things can go wrong but it's not too bad?
    'row-a33', // elevator with more stuff in it: high chance of things clipping into walls :(
    'row-a35'] // great solution but the first slice is too precise

    // 'greg20',
    );

    defineGroup('challenge1', {
        isChallenge: true
    }, [
    // 'emi-b1',
    'emi-b3c',
    // 'emi-b3b',
    'emi-b5', 'emi-b2', 'emi-b4', 'emi-b6b', 'emi-b7', 'emi-b8']
    // 'emi-b9',  // not working yet
    );

    // defineGroup('challenge2', {
    //     isChallenge: true,
    // }, [
    //     'greg1',
    //     'greg4',
    //     'greg5',
    //     'greg11',
    //     'greg16',
    //     'greg18',
    //     'greg20',
    // ]);


    // levels = ([

    //     'lvc1b',    // no good :\

    //     'emi-b1',
    //     'emi-b2',
    //     'emi-b3',
    //     'emi-b4',
    //     'emi-b5',
    //     'emi-b6',
    //     'emi-b7',
    //     'emi-b8',
    //     'emi-b9',

    //     'greg1',
    //     'greg4',
    //     'greg5',
    //     'greg7',  // too similar to existing level
    //     'greg11',
    //     'greg12',  // too simple
    //     'greg13',  // too similar to existing level
    //     'greg16',
    //     'greg18',
    //     'greg20',
    // ]);


    // Dev/WIP levels:

    if (Utils.isDev()) {

        // ----------
        // to add:
        // - the double dagger split level?
        // todo:
        // - develop the windmill idea (lva8 sucks)
        // - fix the climbing tower idea (lvb10, b11)
        // - develop the moving flag idea
        // ----------

        var group = groups['default'];

        // allow developer to override the order by specifying a list of levels in the URL

        var urlLevels = Utils.getHashParameter('level') || Utils.getHashParameter('levels');
        if (urlLevels) {
            group.levels = urlLevels.split(',').map(function (s) {
                return s.trim();
            });
        }

        console.log('Levels:', group.levels.length);
        for (var i = 0; i < group.levels.length; i++) {
            var s = group.levels[i];
            for (var j = 0; j < i; j++) {
                if (s === group.levels[j]) {
                    Utils.log('WARNING: Levels ' + i + ' and ' + j + ' are duplicates (assetName ' + s + ')');
                }
            }
        }
    }

    var updateFirstUncleared = function updateFirstUncleared(group) {
        var infos = group.infos;
        for (var _i = 0; _i < infos.length; _i++) {
            var level = infos[_i];
            if (level.stars === -1) {
                group.firstUncleared = _i;
                return;
            }
        }
        // in case we beat every level, stay at the last level
        group.firstUncleared = infos.length - 1;
    };

    var saveLevelsCleared = function saveLevelsCleared() {
        // Note: only works for default group
        var n = LevelManager.getNumberCleared();
        if (n > SaveManager.load('levelsCleared')) {
            SaveManager.save('levelsCleared', n);
        }
    };

    var checkBossLevel = function checkBossLevel(name) {
        var tiled = Bento.assets.getJson(name);
        if (!tiled) return false;

        var props = tiled.properties;
        if (!props) return false;

        if (Utils.isArray(props)) {
            for (var _j = 0; _j < props.length; _j++) {
                if (props[_j].name === 'isBoss') {
                    return props[_j].value;
                }
            }
            return false;
        } else {
            return props['isBoss'];
        }
    };

    var initBossLevels = function initBossLevels() {
        var group = groups['default'];
        group.levels.forEach(function (levelName, i) {
            if (checkBossLevel(levelName)) {
                group.levelIndexToBossNumber[i] = group.bossLevels.length;
                group.bossLevels.push(i);
            }
        });
        if (Utils.isDev()) {
            bento.require(['modules/skinmanager'], function (SkinManager) {
                var lvls = group.bossLevels.length;
                var chars = SkinManager.getNumBossCharacters();
                if (lvls !== chars) {
                    Utils.log('WARNING: Group ' + group.name + ', found ' + lvls + ' boss levels but ' + chars + ' boss characters are available');
                }
            });
        }
    };

    // Worlds:
    // While the levels are arranged in one big list, they are made available to the player in groups of 10.
    // Each group requires a certain amount of stars before the player can play its levels.
    // If the player already has enough stars, they can skip ahead to the first level of the world.
    var levelsPerWorld = 10;
    var starsToUnlockWorld = function starsToUnlockWorld(w) {
        return Math.floor(w * levelsPerWorld * 3 * 0.8);
    };
    var worldIsLocked = function worldIsLocked(w, group) {
        return group.totalStars < starsToUnlockWorld(w);
    };
    var worldByLevelIndex = function worldByLevelIndex(i) {
        return Math.floor(i / levelsPerWorld);
    };
    var isFirstLevelInWorld = function isFirstLevelInWorld(i) {
        return i === worldByLevelIndex(i) * levelsPerWorld;
    };

    var LevelManager = {
        init: function init() {

            var registerLevel = function registerLevel(group, levelName, i) {
                var stars = SaveManager.load('stars-' + levelName);
                group.infos[i] = {
                    index: i,
                    name: levelName,
                    stars: stars
                };
                if (stars !== -1) {
                    group.totalStars += Utils.clamp(0, stars, 3);
                    group.totalCleared += 1;
                }
            };

            var _loop = function _loop(g) {
                var group = groups[g];
                group.totalStars = 0;
                group.totalCleared = 0;
                group.infos = [];
                group.levels.forEach(function (levelName, i) {
                    registerLevel(group, levelName, i);
                });
                updateFirstUncleared(group);
                if (group.name === 'default') {
                    saveLevelsCleared();
                }
            };

            for (var g in groups) {
                _loop(g);
            }

            // in order to know which levels are boss levels, the level JSONs need to be available
            if (Utils.assetsLoaded) initBossLevels();else EventSystem.on('assetsLoaded', initBossLevels);
        },
        getNextUncleared: function getNextUncleared() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            var group = groups[groupName];
            return group.firstUncleared;
        },
        getNext: function getNext(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return group.infos[i + 1] ? i + 1 : -1;
        },
        getPrevious: function getPrevious(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return group.infos[i - 1] ? i - 1 : -1;
        },
        levelWin: function levelWin(i, stars) {
            var groupName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'default';

            var group = groups[groupName];
            var level = group.infos[i];
            var wasCleared = level.stars !== -1;
            if (stars > level.stars) {
                group.totalStars += Utils.clamp(0, stars, 3) - Utils.clamp(0, level.stars, 3); // clamping to be safe cause values start as -1
                level.stars = stars;
                SaveManager.save('stars-' + level.name, stars);
                console.log('Saved level ' + groupName + ':' + i + ', ' + stars + ' stars');
                if (!wasCleared) {
                    group.totalCleared += 1;
                }
            }
            updateFirstUncleared(group);
            if (groupName === 'default') {
                saveLevelsCleared();
                SaveManager.add('winCount', 1);
            }
            EventSystem.fire('saveToCloud');
        },
        getName: function getName(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return group.levels[i];
        },
        getStars: function getStars(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return Utils.clamp(0, group.infos[i].stars, 3);
        },
        getNumberCleared: function getNumberCleared() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            var group = groups[groupName];
            return group.totalCleared;
        },
        hasCleared: function hasCleared(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return group.infos[i].stars !== -1;
        },
        isLocked: function isLocked(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            var w = worldByLevelIndex(i);
            return !(group.infos[i].stars >= 0 || i === 0) // must not be a level that we already beat (or the very first level)
            && (!isFirstLevelInWorld(i) || worldIsLocked(w, group)) // must not be the first level in a world that is unlocked
            && !(i > 0 && group.infos[i - 1].stars >= 0 && w === worldByLevelIndex(i - 1)); // must not come right after a level in the same world that we already beat
        },
        isValid: function isValid(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return i >= 0 && i < group.levels.length;
        },
        forEach: function forEach(f) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            group.infos.forEach(f);
        },
        getInfo: function getInfo(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            return group.infos[i];
        },
        getMaxStars: function getMaxStars() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            return groups[groupName].levels.length * 3;
            // if (groupName) {
            //     return groups[groupName].levels.length * 3;
            // } else {
            //     let totalLevels = 0;
            //     for (let g in groups) {
            //         totalLevels += groups[g].levels.length;
            //     }
            //     return totalLevels * 3;
            // }
        },
        getTotalStars: function getTotalStars() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            return groups[groupName].totalStars;
            // if (groupName) {
            //    return groups[groupName].totalStars;
            // } else {
            //     let totalStars = 0;
            //     for (let g in groups) {
            //         totalStars += groups[g].totalStars;
            //     }
            //     return totalStars;
            // }
        },
        isBossLevel: function isBossLevel(i) {
            return Utils.isDefined(groups['default'].levelIndexToBossNumber[i]);
        },
        getBossNumber: function getBossNumber(i) {
            return groups['default'].levelIndexToBossNumber[i];
        },
        getLevelByBossNumber: function getLevelByBossNumber(bossIndex) {
            return groups['default'].bossLevels[bossIndex];
        },
        getTotalLevels: function getTotalLevels() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            var group = groups[groupName];
            return group.levels.length;
        },
        levelStartAnalytics: function levelStartAnalytics(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            var levelName = group.levels[i];
            var qualifiedLevelNumber = groupName === 'default' ? '' + (i + 1) : groupName + '-' + (i + 1);
            EventSystem.fire('GameAnalytics-addProgressionEvent', {
                progressionStatus: 1, // Start
                progression01: 'level-' + qualifiedLevelNumber
            });

            EventSystem.fire('FirebaseAnalytics-logEvent', {
                name: 'level_start',
                properties: {
                    level_id: qualifiedLevelNumber,
                    amount_of_moves_left: Globals.maxSlices,
                    objective_type: 'chest',
                    amount_of_objective_left: Utils.clamp(0, SaveManager.load('chestProgress'), 4) / 4,
                    does_have_tutorial: levelName === 'test2' || levelName === 'testf11' || levelName === 'testh7',
                    attempt_number: Globals.retryCount
                }
            });
        },
        levelRetryAnalytics: function levelRetryAnalytics(i) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var qualifiedLevelNumber = groupName === 'default' ? '' + (i + 1) : groupName + '-' + (i + 1);
            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'game:retry'
            });
            EventSystem.fire('GameAnalytics-addProgressionEvent', {
                progressionStatus: 3, // Fail
                progression01: 'level-' + qualifiedLevelNumber
            });

            EventSystem.fire('FirebaseAnalytics-logEvent', {
                name: 'level_attempt',
                properties: {
                    level_id: i + 1,
                    count: Globals.retryCount
                }
            });
        },
        levelCompleteAnalytics: function levelCompleteAnalytics(i, newStars) {
            var groupName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'default';

            var qualifiedLevelNumber = groupName === 'default' ? '' + (i + 1) : groupName + '-' + (i + 1);
            EventSystem.fire('GameAnalytics-addProgressionEvent', {
                progressionStatus: 2, // Complete
                progression01: 'level-' + qualifiedLevelNumber,
                score: newStars
            });

            if (groupName !== 'default') {
                // the remaining analytics are only relevant to the default levels.
                return;
            }

            var skipped = newStars === 0;
            var totalRewarded = SaveManager.load('totalRewarded');
            var totalInterstitials = SaveManager.load('totalInterstitials');
            var totalAds = totalRewarded + totalInterstitials;

            EventSystem.fire('GameAnalytics-addDesignEvent', {
                eventId: 'level:' + (skipped ? 'skip' : 'win') + ':' + qualifiedLevelNumber
            });

            EventSystem.fire('FirebaseAnalytics-logEvent', {
                name: 'level_complete',
                properties: {
                    level_id: qualifiedLevelNumber,
                    duration: Globals.levelTimer / 60,
                    boost_used_in_level: Globals.hintsRevealed > 0,
                    amount_of_moves_left: Globals.maxSlices - Globals.sliceCount,
                    amount_of_objective_left: Utils.clamp(0, SaveManager.load('chestProgress'), 4) / 4
                }
            });

            if (SaveManager.load('firstPlayedVersion') >= Utils.versionToHex('1.1.0')) {
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'AdsCount:rewarded:' + qualifiedLevelNumber,
                    value: totalRewarded
                });
                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'AdsCount:interstitial:' + qualifiedLevelNumber,
                    value: totalInterstitials
                });
            }

            if (!LevelManager.hasCleared(i)) {

                EventSystem.fire('GameAnalytics-addDesignEvent', {
                    eventId: 'level:firstComplete:' + qualifiedLevelNumber
                });

                var n = LevelManager.getNumberCleared() + 1;
                if (n === 5) {
                    EventSystem.fire('Tenjin', { action: '5_levels_completed' });
                } else if (n === 10) {
                    EventSystem.fire('Tenjin', { action: '10_levels_completed' });
                } else if (n === 20) {
                    EventSystem.fire('Tenjin', { action: '20_levels_completed' });
                }

                if (n === 1) {
                    EventSystem.fire('FirebaseAnalytics-logEvent', { name: 'level_1_passed' });
                } else if (n === 10) {
                    EventSystem.fire('FirebaseAnalytics-logEvent', { name: 'level_10_passed' });
                } else if (n === 20) {
                    EventSystem.fire('FirebaseAnalytics-logEvent', { name: 'level_20_passed' });
                } else if (n === 50) {
                    EventSystem.fire('FirebaseAnalytics-logEvent', { name: 'level_50_passed' });
                }
            }
        },
        gotoNextLevel: function gotoNextLevel() {
            // TODO: adapt this for other groups??
            var group = groups['default'];
            Globals.hintsRevealed = 0;
            Globals.retryCount = 0;
            Globals.failCount = 0;
            Globals.levelTimer = 0;
            var next = LevelManager.getNext(Utils.currentLevel, 'default');
            if (next !== -1) {
                var w = worldByLevelIndex(next);
                if (worldIsLocked(w, group) && !Utils.isDev()) {
                    // player doesn't have enough stars to progress to next world.
                    bento.require(['gui/needstarsdialog'], function (NeedStarsDialog) {
                        Bento.objects.attach(new NeedStarsDialog({
                            starsNeeded: starsToUnlockWorld(w)
                        }));
                    });
                } else {
                    // go to next level
                    Utils.currentLevel = next;
                    Bento.screens.show('screens/main');
                }
            } else {
                // player beat every level
                bento.require(['gui/dialog'], function (Dialog) {
                    Bento.objects.attach(new Dialog({
                        name: 'clearedAllLevels',
                        type: 'ok',
                        title: 'CONGRATS!',
                        text: 'Wow, it looks like you cleared every level... for now! Thanks for playing, and stay tuned for the next update.',
                        onComplete: function onComplete() {
                            Bento.screens.show('screens/titlescreen');
                        }
                    }));
                });
            }
        },
        getWorldInfo: function getWorldInfo(w) {
            var groupName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

            var group = groups[groupName];
            var fromLevel = w * levelsPerWorld;
            var toLevel = Math.min((w + 1) * levelsPerWorld, group.levels.length);
            var starsEarned = 0;
            for (var _i2 = fromLevel; _i2 < toLevel; _i2++) {
                starsEarned += LevelManager.getStars(_i2, groupName);
            }
            return {
                fromLevel: fromLevel,
                toLevel: toLevel,
                starsToUnlock: starsToUnlockWorld(w),
                starsEarned: starsEarned
            };
        },
        getNumWorlds: function getNumWorlds() {
            var groupName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'default';

            var group = groups[groupName];
            return Math.ceil(group.levels.length / levelsPerWorld);
        },
        getCurrentChallenge: function getCurrentChallenge() {
            for (var _i3 = 0; _i3 < challenges.length; _i3++) {
                var groupName = challenges[_i3];
                var _group = groups[groupName];
                if (_group.totalCleared < _group.levels.length) {
                    return groupName;
                }
            }
            return null;
        },
        getGroupInfo: function getGroupInfo(groupName) {
            return groups[groupName];
        },
        isChallenge: function isChallenge(groupName) {
            return groups[groupName].isChallenge;
        }
    };

    // self-init
    LevelManager.init();

    return LevelManager;
});