'use strict';

/**
 * Helper module to schedule notifications. Game specific.
 * @moduleName NotificationScheduler 
 * @snippet NotificationScheduler.init()|snippet
NotificationScheduler.init({
    position: new Vector2(0, 0)
})
 */
bento.define('modules/notificationscheduler', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/tween', 'modules/notifications', 'modules/savemanager'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Tween, Notifications, SaveManager) {
    'use strict';
    /**
     * @snippet Notification|schedule
    EventSystem.fire('scheduleEvent', {
    id: '$1',
    title: Localization.getText('notifications')['$1Title'],
    body: Localization.getText('notifications')['$1Body'],
    time: ${2:24*60*60*1000}
    });
     */

    var onSchedule = function onSchedule(settings) {
        var id = settings.id;
        var time = settings.time;
        var title = settings.title;
        var body = settings.body;
        var userData = settings.userData || {};

        var savedNotifications = SaveManager.load('notifications');

        if (!body) {
            Utils.log("ERROR: cannot schedule notification without body text");
            return;
        }
        if (!title) {
            Utils.log("ERROR: cannot schedule notification without title text");
            return;
        }

        // reset badge
        Notifications.resetBadgeNumber();

        // cancel old notification if needed
        var oldId = savedNotifications[id];
        if (oldId) {
            Notifications.cancel(oldId);
        }
        console.log('scheduling notification: ' + id);

        // schedule notification
        var notificationObject = {
            title: title, // ios title text
            message: Utils.isNativeIos() ? body : title, // ios body text, but android title text
            soundEnabled: true,
            badgeNumber: 1,
            userData: userData,
            contentBody: Utils.isNativeIos() ? "" : body, // android body text
            contentTitle: "",
            date: new Date().valueOf() + time, // notify again in X hours
            icon: 'notification_icon'
        };
        Notifications.send(notificationObject, function (notificationId) {
            // save id so it can be cancelled if needed
            savedNotifications[id] = notificationId;
            SaveManager.save('notifications', savedNotifications);
            console.log('Scheduled notification: ' + id + ', ' + notificationId);
        });
    };

    return {
        init: function init() {
            EventSystem.on('scheduleNotification', onSchedule);
        }
    };
});