'use strict';

bento.define('modules/skinmanager', ['bento', 'bento/utils', 'modules/savemanager', 'globals', 'bento/eventsystem'], function (Bento, Utils, SaveManager, Globals, EventSystem) {

    var b = 0; // boss number counter

    // characters specified in the order that they appear in the shop (if relevant)
    // 'id'   - string which matches the asset name
    // 'ball' - which kind of gacha ball color scheme goes with this character?
    // 'cost' - this character must be bought with coins
    // 'vip'  - this character char is only available when vip subscription is active
    // 'bossNumber' - this character is the Nth boss that you encounter
    /**
     * This module is responsible for all characters, scissors and themes.
     * Should be imported in preloader but no earlier (i.e. nativestorage must be ready!)
     */

    var characters = [{ id: "1", ball: '01', coins: true }, // cap default

    // ADS

    { id: '11', ball: '01', ad: true }, // tophat default
    { id: "12", ball: '02', ad: true }, // tophat green
    { id: "19", ball: '09', ad: true }, // tophat white
    { id: "15", ball: '06', ad: true }, // tophat bluetint

    { id: "83", ball: '01', ad: true }, // paper boat default
    { id: "84", ball: '01', ad: true }, // military default
    { id: "51", ball: '01', ad: true }, // pirate default
    { id: "87", ball: '01', ad: true }, // floral default

    { id: '73', ball: '01', ad: true }, // apple default
    { id: "75", ball: '08', ad: true }, // apple purple
    { id: "76", ball: '09', ad: true }, // apple white
    { id: "74", ball: '03', ad: true }, // apple gb

    // COINS

    { id: "102", ball: '07', coins: true }, // flowerbun lime
    { id: "103", ball: '04', coins: true }, // flowerbun cyan
    { id: "104", ball: '08', coins: true }, // flowerbun purple
    { id: "105", ball: '09', coins: true }, // flowerbun white

    { id: "70", ball: '10', coins: true }, // skull dark
    { id: "72", ball: '04', coins: true }, // skull cyan
    { id: '71', ball: '02', coins: true }, // skull green
    { id: "69", ball: '01', coins: true }, // skull default

    { id: '27', ball: '07', coins: true }, // cake lime
    { id: "26", ball: '05', coins: true }, // cake pink
    { id: "23", ball: '04', coins: true }, // cake cyan
    { id: "30", ball: '10', coins: true }, // cake dark

    { id: "91", ball: '03', coins: true }, // boat gb
    { id: "92", ball: '05', coins: true }, // boat pink
    { id: "93", ball: '07', coins: true }, // boat lime
    { id: "94", ball: '06', coins: true }, // boat bluetint

    { id: "59", ball: '09', coins: true }, // pirate white
    { id: "52", ball: '02', coins: true }, // pirate green
    { id: '56', ball: '05', coins: true }, // pirate pink
    { id: "60", ball: '10', coins: true }, // pirate dark

    { id: "106", ball: '09', coins: true }, // floral brown
    { id: "108", ball: '02', coins: true }, // floral green
    { id: "109", ball: '08', coins: true }, // floral peach
    { id: "110", ball: '03', coins: true }, // floral gb

    { id: "95", ball: '02', coins: true }, // military green
    { id: "96", ball: '06', coins: true }, // military bluetint
    { id: "97", ball: '10', coins: true }, // military red
    { id: "98", ball: '04', coins: true }, // military blue

    { id: "85", ball: '01', coins: true }, // teapot default
    { id: "99", ball: '03', coins: true }, // teapot gb
    { id: "100", ball: '05', coins: true }, // teapot pink
    { id: "101", ball: '10', coins: true }, // teapot dark

    { id: "116", ball: '08', coins: true }, // bee purple
    { id: "117", ball: '02', coins: true }, // bee green
    { id: "118", ball: '07', coins: true }, // bee lime
    { id: "119", ball: '11', coins: true }, // bee teal

    { id: "111", ball: '01', coins: true }, // santa 
    { id: "112", ball: '04', coins: true }, // santa 
    { id: "113", ball: '07', coins: true }, // santa 
    { id: "114", ball: '05', coins: true }, // santa 

    // BOSS

    { id: "41", ball: '01', bossNumber: b++ }, // shroom default
    { id: "89", ball: '02', bossNumber: b++ }, // soda green
    { id: '66', ball: '06', bossNumber: b++ }, // rabbit bluetint
    { id: "37", ball: '07', bossNumber: b++ }, // crown lime

    { id: "45", ball: '06', bossNumber: b++ }, // shroom bluetint
    { id: "88", ball: '04', bossNumber: b++ }, // soda cyan
    { id: "67", ball: '05', bossNumber: b++ }, // rabbit pink
    { id: "38", ball: '08', bossNumber: b++ }, // crown purple

    { id: '44', ball: '03', bossNumber: b++ }, // shroom gb
    { id: "82", ball: '01', bossNumber: b++ }, // soda default
    { id: "68", ball: '07', bossNumber: b++ }, // rabbit lime
    { id: '39', ball: '09', bossNumber: b++ }, // crown white

    { id: "43", ball: '04', bossNumber: b++ }, // shroom cyan
    { id: "90", ball: '09', bossNumber: b++ }, // soda white
    { id: "65", ball: '03', bossNumber: b++ }, // rabbit zombie
    { id: "40", ball: '10', bossNumber: b++ }, // crown dark

    { id: "3", ball: '04', bossNumber: b++ }, // cap cyan
    { id: "5", ball: '06', bossNumber: b++ }, // cap bluetint
    { id: "7", ball: '07', bossNumber: b++ }, // cap lime
    { id: "9", ball: '09', bossNumber: b++ }, // cap white

    { id: "48", ball: '08', bossNumber: b++ }, // shroom purple
    { id: "4", ball: '03', bossNumber: b++ }, // cap gb
    { id: "115", ball: '01', bossNumber: b++ }, // bee default
    { id: "10", ball: '10', bossNumber: b++ }, // cap dark

    { id: "2", ball: '02', bossNumber: b++ }, // cap green
    { id: "86", ball: '01', bossNumber: b++ }, // flowerbun default
    { id: "8", ball: '08', bossNumber: b++ }, // cap purple
    { id: "31", ball: '01', bossNumber: b++ }, // crown default

    // 7 day challenge
    { id: '120', challenge: 'challenge1' },

    // vip
    { id: '77', vip: true }, // brainjar
    { id: '78', vip: true }, // baseball
    { id: '79', vip: true }, // goji
    { id: '80', vip: true }, // witch
    { id: '81', vip: true }];

    var scissors = [
    // normal
    { id: '1-1', cost: 0 }, { id: '1-2', cost: 150 },
    // zigzag
    { id: '4-1', cost: 250 }, { id: '4-2', cost: 250 },
    // fatties
    { id: '2-1', cost: 200 }, { id: '2-3', cost: 200 },
    // cats
    { id: '8-1', cost: 500 }, { id: '8-2', cost: 500 },
    // piranha plants
    { id: '9-1', cost: 500 }, { id: '9-2', cost: 500 },
    // tweesers
    { id: '6-1', cost: 250 }, { id: '6-3', cost: 250 },
    // quills
    { id: '10-1', cost: 350 }, { id: '10-3', cost: 350 },
    // beetles
    { id: '7-1', cost: 350 }, { id: '7-3', cost: 350 },
    // hooks
    { id: '11-2', cost: 400 }, { id: '11-3', cost: 400 },
    // blades
    { id: '12-1', cost: 400 }, { id: '12-2', cost: 400 },
    // short
    { id: '3-2', cost: 250 }, { id: '3-3', cost: 250 },
    // bent-outwards
    { id: '5-2', cost: 250 }, { id: '5-3', cost: 250 }];

    var themes = [{ id: 'wave-blue', cost: 0 }, { id: 'wave-pink', cost: 500 }, { id: 'wave-green', cost: 300 }, { id: 'wave-purple', cost: 500 }, { id: 'diagonal-pink', cost: 500 }, { id: 'diagonal-green', cost: 300 }, { id: 'diagonal-blue', cost: 300 }, { id: 'diagonal-orange', cost: 700 }, { id: 'spiky-green', cost: 300 }, { id: 'spiky-blue', cost: 300 }, { id: 'spiky-orange', cost: 700 }, { id: 'spiky-dark', cost: 700 }, { id: 'stars-blue', cost: 300 }, { id: 'stars-orange', cost: 700 }, { id: 'stars-dark', cost: 700 }, { id: 'stars-pink', cost: 500 }, { id: 'spot-orange', cost: 700 }, { id: 'spot-dark', cost: 700 }, { id: 'spot-pink', cost: 500 }, { id: 'spot-purple', cost: 500 }, { id: 'ring-dark', cost: 700 }, { id: 'ring-pink', cost: 500 }, { id: 'ring-purple', cost: 500 }, { id: 'ring-brown', cost: 600 }, { id: 'stone-pink', cost: 500 }, { id: 'stone-purple', cost: 500 }, { id: 'stone-brown', cost: 600 }, { id: 'stone-cyan', cost: 800 }];

    var charactersById = {}; // all characters, mapped by ID
    var scissorsById = {};
    var themesById = {};

    var shopCharacters = []; // list of all characters that appear in the shop (i.e. all free/coin/boss/vip characters)
    var shopScissors = [];
    var shopThemes = [];

    var bossCharacters = []; // list of all characters which are bosses
    var bossNumberByCharacterId = {};

    var adCharacters = {}; // set of all character ids which can be unlocked in the victory screen

    var cheapestScissorCost = Infinity;
    var cheapestThemeCost = Infinity;

    var numBoughtCharacters = 0;
    var numBoughtScissors = 0;
    var numBoughtThemes = 0;

    var unlockedCharacters = []; // list of all characters ids which are unlocked by any means
    var charactersByChallenge = {}; // mapping of challenge names to character ids

    // populate arrays
    var init = function init() {

        charactersById = {};
        scissorsById = {};
        themesById = {};

        shopCharacters = [];
        shopScissors = [];
        shopThemes = [];

        bossCharacters = [];
        bossNumberByCharacterId = {};

        adCharacters = {};

        cheapestScissorCost = Infinity;
        cheapestThemeCost = Infinity;

        numBoughtCharacters = 0;
        numBoughtScissors = 0;
        numBoughtThemes = 0;

        unlockedCharacters = [];
        charactersByChallenge = {};

        characters.forEach(function (data) {
            charactersById[data.id] = data;
            var locked = !SaveManager.load('hasChar-' + data.id);
            if (data.coins) {
                shopCharacters.push(data);
                if (!locked && data.id !== "1") {
                    // TODO: try removing the 'coins' property from the default skin so we don't have to special-case it here.
                    numBoughtCharacters++;
                }
            } else if (data.bossNumber != null) {
                shopCharacters.push(data);
                bossCharacters.push(data.id);
                bossNumberByCharacterId[data.id] = data.bossNumber;
            } else if (data.ad) {
                shopCharacters.push(data);
                if (locked) {
                    adCharacters[data.id] = true;
                }
            } else if (data.challenge) {
                shopCharacters.push(data);
                charactersByChallenge[data.challenge] = data.id;
            } else if (data.vip) {
                shopCharacters.push(data);
            }
            if (!locked) {
                unlockedCharacters.push(data.id);
            }
        });
        scissors.forEach(function (data) {
            scissorsById[data.id] = data;
            var hasScissor = SaveManager.load('hasScissor-' + data.id);
            if (Utils.isDefined(data.cost)) {
                shopScissors.push(data);
                if (hasScissor) {
                    numBoughtScissors++;
                } else if (data.cost < cheapestScissorCost) {
                    cheapestScissorCost = data.cost;
                }
            }
        });
        themes.forEach(function (data) {
            themesById[data.id] = data;
            var hasTheme = SaveManager.load('hasTheme-' + data.id);
            if (Utils.isDefined(data.cost)) {
                shopThemes.push(data);
                if (hasTheme) {
                    numBoughtThemes++;
                } else if (data.cost < cheapestThemeCost) {
                    cheapestThemeCost = data.cost;
                }
            }
        });
    };

    // self init
    init();

    var manager = {

        init: init, getShopCharacters: function getShopCharacters() {
            return shopCharacters;
        },
        getShopScissors: function getShopScissors() {
            return shopScissors;
        },
        getShopThemes: function getShopThemes() {
            return shopThemes;
        },
        getAllCharacters: function getAllCharacters() {
            return characters;
        },
        getCharacterById: function getCharacterById(id) {
            return charactersById[id];
        },
        getScissorById: function getScissorById(id) {
            return scissorsById[id];
        },
        getThemeById: function getThemeById(id) {
            return themesById[id];
        },
        unlockCharacter: function unlockCharacter(id) {
            SaveManager.save('hasChar-' + id, true);
            EventSystem.fire('saveToCloud');
            var data = charactersById[id];
            if (data.coins) {
                numBoughtCharacters++;
            } else if (data.ad) {
                delete adCharacters[id];
            }
            unlockedCharacters.push(id);
        },
        unlockScissor: function unlockScissor(id) {
            SaveManager.save('hasScissor-' + id, true);
            EventSystem.fire('saveToCloud');
            numBoughtScissors++;
            var data = scissorsById[id];
            if (data && data.cost === cheapestScissorCost) {
                cheapestScissorCost = Infinity;
                shopScissors.forEach(function (s) {
                    if (s.cost && s.cost < cheapestScissorCost && !SaveManager.load('hasScissor-' + s.id)) {
                        cheapestScissorCost = s.cost;
                    }
                });
            }
        },
        unlockTheme: function unlockTheme(id) {
            SaveManager.save('hasTheme-' + id, true);
            EventSystem.fire('saveToCloud');
            numBoughtThemes++;
            var data = themesById[id];
            if (data && data.cost === cheapestThemeCost) {
                cheapestThemeCost = Infinity;
                shopThemes.forEach(function (t) {
                    if (t.cost && t.cost < cheapestThemeCost && !SaveManager.load('hasTheme-' + t.id)) {
                        cheapestThemeCost = t.cost;
                    }
                });
            }
        },
        getCheapestItemCost: function getCheapestItemCost() {
            return Math.min(manager.getCharacterCost(), cheapestScissorCost, cheapestThemeCost);
        },
        getCheapestScissorCost: function getCheapestScissorCost() {
            return cheapestScissorCost;
        },
        getCheapestThemeCost: function getCheapestThemeCost() {
            return cheapestThemeCost;
        },
        getCharacterCost: function getCharacterCost() {
            switch (numBoughtCharacters) {
                case 0:
                    return 100;
                case 1:
                    return 200;
                case 2:
                    return 300;
                case 3:
                    return 400;
                case 4:
                    return 450;
                case 5:
                    return 500;
                case 6:
                    return 550;
                case 7:
                    return 600;
                default:
                    return 600 + (numBoughtCharacters - 7) * 35;
            }
        },
        getNumBoughtCharacters: function getNumBoughtCharacters() {
            return numBoughtCharacters;
        },
        getNumBoughtScissors: function getNumBoughtScissors() {
            return numBoughtScissors;
        },
        getNumBoughtThemes: function getNumBoughtThemes() {
            return numBoughtThemes;
        },
        getRandomUnlockableCharacter: function getRandomUnlockableCharacter() {
            var keys = Object.keys(adCharacters);
            return Utils.pickRandomFrom(keys);
        },
        getCharacterIdByBossNumber: function getCharacterIdByBossNumber(bossNumber) {
            return bossCharacters[bossNumber];
        },
        getNumBossCharacters: function getNumBossCharacters() {
            return bossCharacters.length;
        },
        getRandomUnlockedCharacter: function getRandomUnlockedCharacter() {
            return Utils.pickRandomFrom(unlockedCharacters);
        },
        getCharacterByChallenge: function getCharacterByChallenge(challengeName) {
            return charactersByChallenge[challengeName];
        }
    };

    return manager;
});