'use strict';

/**
 * Time manager, manages the time for energy regen and other timed events
 *
 * @moduleName TimeManager
 * @snippet TimeManager.getTime
TimeManager.getTime();
 * @snippet TimeManager.getLocalTime
TimeManager.getLocalTime();
 */
bento.define('modules/timemanager', ['bento', 'utils', 'bento/entity', 'modules/savemanager', 'bento/eventsystem'], function (Bento, Utils, Entity, SaveManager, EventSystem) {
    'use strict';

    var started = false;

    var counter = 0;
    var counterTotal = 0;

    var counterBehavior = {
        start: function start() {
            console.log('globalTimer started');
        },
        destroy: function destroy() {
            console.log('Warning: attempted to remove global timer');
            // put self back??
        },
        now: Date.now(),
        update: function update(data) {
            var deltaT;
            var now;

            counter++;

            if (counter === 60) {
                // get delta time
                now = Date.now();
                deltaT = now - counterBehavior.now;

                // limit deltaT to 10seconds to be sure
                if (deltaT > 10000) {
                    deltaT = 10000;
                }

                // update now
                counterBehavior.now = now;
                manager.time += deltaT;
                counter = 0;
                counterTotal += 1;
                // save time every 60 second
                if (counterTotal % 60 === 0) {
                    manager.saveTime();
                }
            }
        }
    };
    var timerEntity = new Entity({
        z: 0,
        name: 'globalTimer',
        global: true,
        updateWhenPaused: Infinity,
        components: [counterBehavior]
    });

    var initialize = function initialize() {
        if (!started) {
            manager.time = SaveManager.load('lastKnownTime');
            Bento.objects.attach(timerEntity);
            getServerTime(serverComplete, serverFail);

            if (Utils.isDev()) {
                window.timeManager = manager;
            }

            started = true;
        }
    };
    var isRequestingTime = false;
    var getServerTime = function getServerTime(onComplete, onFail) {
        var dateObject;
        var date;
        var todayNow = Date.now();
        var today = Math.floor(todayNow / 1000 / 60 / 60 / 24);
        var attempt = 0;
        var getAwsTime = function getAwsTime() {
            var xhr = new window.XMLHttpRequest();
            xhr.open("GET", Utils.urls.getDate);
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status >= 200 && xhr.status < 300) {
                        try {
                            // get date from server
                            dateObject = JSON.parse(xhr.responseText);
                            date = dateObject.date;

                            // calculate new today
                            todayNow = date;
                            today = Math.floor(todayNow / 1000 / 60 / 60 / 24);

                            console.log("Received date from server: " + date);
                            onComplete(date);
                        } catch (e) {
                            if (onFail) {
                                Utils.log('ERROR: could not retrieve or parse date from server.' + xhr.responseText);
                                if (attempt === 0) {
                                    attempt += 1;
                                    getHeaderTime();
                                } else {
                                    onFail();
                                }
                            }
                        }
                    } else {
                        if (onFail) {
                            Utils.log('ERROR: bad server response.');
                            if (attempt === 0) {
                                attempt += 1;
                                getHeaderTime();
                            } else {
                                onFail();
                            }
                        }
                    }
                }
            };
            // xhr.ontimeout = function (e) {
            //     if (onFail) {
            //         Utils.log('ERROR: timed out.');
            //         // onFail();
            //         getHeaderTime();
            //     }
            // };
            xhr.send();
        };
        var getHeaderTime = function getHeaderTime() {
            var dateString;
            var xhr = new window.XMLHttpRequest();
            xhr.open('HEAD', 'https://s3.amazonaws.com/lucky-kat/beatstreet/data/date.json', true);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status >= 200 && xhr.status < 300) {
                        try {
                            // get date from server
                            dateString = this.getResponseHeader("Date");
                            if (dateString.indexOf('GMT') === -1) {
                                dateString += ' GMT';
                            }
                            date = Date.parse(dateString);

                            // calculate new today
                            todayNow = date;
                            today = Math.floor(todayNow / 1000 / 60 / 60 / 24);

                            console.log('Date from server: ' + dateString + ' -> ' + date);

                            // headers received
                            onComplete(date);
                        } catch (e) {
                            if (onFail) {
                                Utils.log('ERROR: could not retrieve or parse date from server.');
                                onFail();
                                if (attempt === 0) {
                                    attempt += 1;
                                    getAwsTime();
                                } else {
                                    onFail();
                                }
                            }
                        }
                    } else {
                        // should be 200 on success
                        if (onFail) {
                            if (attempt === 0) {
                                attempt += 1;
                                getAwsTime();
                            } else {
                                onFail();
                            }
                        }
                    }
                }
            };
            xhr.send();
        };

        manager.hasUpdatedTime = false;
        if (!isRequestingTime) {
            isRequestingTime = true;
            console.log("Getting server time...");
            if (Utils.WEB || Utils.iDreamSky) {
                attempt = 1;
                // getAwsTime();
                getHeaderTime();
            } else {
                if (Utils.isNativeIos() || Utils.isNativeAndroid()) {
                    // getAwsTime();
                    getHeaderTime();
                } else {
                    // getAwsTime();
                    getHeaderTime();
                }
            }
        }
    };

    var serverComplete = function serverComplete(time) {
        isRequestingTime = false;
        manager.online = true;
        manager.time = time;
        manager.hasUpdatedTime = true;
        manager.saveTime();

        // we save the time difference between server and local once
        var timeOffset = SaveManager.load('timeOffset');
        if (timeOffset === null) {
            timeOffset = manager.time - Date.now();
            SaveManager.save('timeOffset', timeOffset);
        }

        // save the initial playing time (note: not accurate for softlaunch players)
        var initialStartingTime = SaveManager.load('initialStartingTime');
        if (initialStartingTime === 0) {
            initialStartingTime = time;
            SaveManager.save('initialStartingTime', initialStartingTime);
        }

        EventSystem.fire('serverTimeUpdated');
    };

    var serverFail = function serverFail() {
        var currentScreen = Bento.screens.getCurrentScreen();
        manager.online = false;
        isRequestingTime = false;

        // try again in 10 seconds
        window.setTimeout(function () {
            if (manager.tryToConnect) {
                manager.updateServerTime();
            }
        }, 10000);
    };

    var getTime = function getTime() {
        return manager.time;
    };

    var resumeEvent = function resumeEvent() {
        manager.hasWarned = false;
        // update now because there can be a big gap between the counterBehavior.now and the real time
        counterBehavior.now = Date.now();
        // update server time
        getServerTime(serverComplete, serverFail);
    };

    var manager = {
        hasWarned: false,
        online: false,
        time: SaveManager.load('lastKnownTime'),
        elapsed: 0,
        initialize: initialize,
        getTime: getTime,
        hasUpdatedTime: false,
        tryToConnect: true,
        getLocalTime: function getLocalTime() {
            // retrieves server time corrected with offset
            var offset = SaveManager.load('timeOffset') || 0;
            return manager.time - offset;
        },
        resumeEvent: resumeEvent,
        saveTime: function saveTime() {
            // save last known time
            SaveManager.save('lastKnownTime', manager.time);
        },
        updateServerTime: function updateServerTime(onComplete) {
            // update server time
            getServerTime(function (date) {
                if (onComplete) {
                    onComplete(true);
                }
                serverComplete(date);
            }, function () {
                if (onComplete) {
                    onComplete(false);
                }
                serverFail();
            });
        }
    };

    return manager;
});