'use strict';

bento.define('screens/main', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'entities/gui/gamecounter', 'entities/gui/gamebutton', 'entities/gui/gametext', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'entities/luckykatlogo', 'globals', 'bento/components/fill', 'bento/components/eventlistener', 'entities/camera', 'modules/particlesystem', 'components/wiggle', 'modules/box2d', 'bento/tiled', 'entities/lemming', 'entities/collectible', 'entities/goal', 'entities/terrain', 'entities/joint', 'entities/hintarea', 'entities/failregion', 'entities/cage', 'entities/gui/starsearned', 'entities/gui/fastforwardbutton', 'entities/gui/tutorialhand', 'entities/gui/hintline', 'entities/gui/hintbutton', 'entities/gridbg', 'gui/gui', 'gui/guimain', 'modules/levelmanager', 'components/debug', 'modules/tapticengine', 'gui/dialog', 'modules/savemanager', 'modules/notifications', 'modules/confetti', 'gui/starsforadoverlay', 'gui/guivictory', 'gui/guichallengevictory', 'modules/cloudsave', 'modules/admanager', 'modules/textstyles', 'modules/localization', 'gui/askrating', 'gui/characterunlock', 'gui/cageprogressbar', 'modules/skinmanager', 'modules/timemanager', 'modules/cloudsavechecker'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, GameCounter, GameButton, GameText, Text, Utils, Screen, Tween, LuckyKat, Globals, Fill, EventListener, Camera, ParticleSystem, Wiggle, Box2D, Tiled, Lemming, Collectible, Goal, Terrain, Joint, HintArea, FailRegion, Cage, StarsEarned, FastForwardButton, TutorialHand, HintLine, HintButton, GridBg, Gui, GuiMain, LevelManager, Debug, TapticEngine, Dialog, SaveManager, Notifications, Confetti, StarsForAdOverlay, GuiVictory, GuiChallengeVictory, CloudSave, AdManager, TextStyles, Localization, AskRating, CharacterUnlock, CageProgressBar, SkinManager, TimeManager, CloudSaveChecker) {

    function onShow() {
        var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};


        var viewport = Bento.getViewport();

        if (!LevelManager.isValid(Utils.currentLevel, Utils.currentLevelGroup)) {
            // last resort in case we messed up and tried to load a level out of bounds.
            Utils.currentLevel = LevelManager.getNextUncleared(Utils.currentLevelGroup);
        }

        Globals.hasWon = false;
        Globals.sliceCount = 0;
        Globals.paths = {};

        // set later when loading level
        Globals.hintsAvailable = 0;
        Globals.maxSlices = 0;
        Globals.theme = null;

        Globals.failOnTurn = false;
        Globals.alreadyCleared = LevelManager.hasCleared(Utils.currentLevel, Utils.currentLevelGroup);

        // set up for character unlock on level complete (if this level happens to be a 'boss fight')
        Globals.cageCharacter = null;
        if (Utils.currentLevelGroup === 'default') {
            var bossNumber = LevelManager.getBossNumber(Utils.currentLevel);
            if (bossNumber != null) {
                Globals.cageCharacter = SkinManager.getCharacterIdByBossNumber(bossNumber);
            }
        }

        var guiIsLoaded = false;
        var levelIsLoaded = false;

        var guiLoadedListener = new EventListener({
            eventName: 'guiLoaded',
            onEvent: function onEvent() {
                if (guiIsLoaded) {
                    // only the initial load counts (i.e. we're not interested in the victory UI)
                    return;
                }
                if (Globals.failCount >= 2 || Globals.retryCount >= 2) {
                    // remind user of skip for ad?
                    Gui.get('buttonSkip', function (buttonSkip) {
                        var skipHand = new TutorialHand({
                            moveAmount: 5,
                            squishAmount: 1.12,
                            position: new Vector2(-20, 2),
                            textOffset: new Vector2(0, 38)
                        });
                        skipHand.rotation = Math.PI * 0.5;
                        skipHand.attach(new EventListener({
                            eventName: 'beginSlicing',
                            onEvent: skipHand.hide
                        }));
                        buttonSkip.attach(skipHand);
                    });
                }

                guiIsLoaded = true;
                if (levelIsLoaded && guiIsLoaded) {
                    EventSystem.fire('mainLoaded');
                }
            }
        });

        var mainLoadedListener = new EventListener({
            eventName: 'mainLoaded',
            onEvent: function onEvent() {
                LevelManager.levelStartAnalytics(Utils.currentLevel, Utils.currentLevelGroup);
                if (Utils.currentLevelGroup === 'default') {
                    Bento.objects.attach(new CageProgressBar({
                        position: new Vector2(viewport.width * 0.5, 44)
                    }));
                    SaveManager.save('nextLevel', Utils.currentLevel);
                }

                Bento.objects.attach({
                    name: 'levelTimer',
                    update: function update() {
                        Globals.levelTimer++;
                    }
                });
            }
        });

        var nextLevelListener = new EventListener({
            eventName: 'nextLevel',
            onEvent: function onEvent() {
                LevelManager.gotoNextLevel(Utils.currentLevelGroup);
            }
        });

        var showVictoryScreen = function showVictoryScreen() {

            // the main game is fully hidden at this point,
            // so we can stop the simulation and remove some things
            Box2D.isPaused = true;
            Bento.objects.getByFamily('lemming').slice().forEach(function (e) {
                return e.removeSelf();
            });
            Bento.objects.getByFamily('terrain').slice().forEach(function (e) {
                return e.removeSelf();
            });
            Bento.objects.getByFamily('goal').slice().forEach(function (e) {
                return e.removeSelf();
            });
            Bento.objects.getByFamily('cage').slice().forEach(function (e) {
                return e.removeSelf();
            });
            Bento.objects.getByFamily('gib').slice().forEach(function (e) {
                return e.removeSelf();
            });

            // Note: it's OK for victory overlay to replace the whole UI, since they have nothing in common
            var doShowVictory = function doShowVictory() {
                AdManager.canShowRewarded(function (canShow) {
                    var group = LevelManager.getGroupInfo(Utils.currentLevelGroup);
                    if (group.isChallenge) {
                        Gui.load({
                            assetName: 'ui/challengevictory',
                            handlers: GuiChallengeVictory,
                            canShowRewardedAds: canShow
                        });
                    } else {
                        Gui.load({
                            assetName: 'ui/victory',
                            handlers: GuiVictory,
                            canShowRewardedAds: canShow
                        });
                    }
                });
            };

            var tryShowInterstitial = function tryShowInterstitial() {
                // let interstitialRate = Bento.saveState.load('AB_InterstitialRate', '');
                // let rate = (interstitialRate === 'InterstitialPer5Levels') ? 5 : 3;

                if (Utils.currentLevelGroup === 'default' && Globals.starsEarned === 3 && AdManager.incrementCounter() >= 2 && SaveManager.load('winCount') >= 2) {
                    console.log('Trying to show interstitial');
                    AdManager.tryShowInterstitial(function (success, err) {
                        if (err) {
                            console.log('Can\'t show interstitial: ' + err);
                        }
                        doShowVictory();
                    }, 'Level_Win_Interstitial');
                } else {
                    doShowVictory();
                }
            };

            var tryCharacterUnlock = function tryCharacterUnlock() {
                // show character unlock if player didn't skip, and this is a boss level, and they don't already have the character
                if (Globals.hasWon && Globals.cageCharacter && !SaveManager.load('hasChar-' + Globals.cageCharacter)) {
                    Bento.objects.attach(new CharacterUnlock({
                        characterId: Globals.cageCharacter,
                        onComplete: function onComplete() {
                            tryShowInterstitial();
                        }
                    }));
                } else {
                    tryShowInterstitial();
                }
            };

            tryCharacterUnlock();
        };

        var winListener = new EventListener({
            eventName: 'win',
            onEvent: function onEvent() {
                var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
                var _data$skipped = data.skipped,
                    skipped = _data$skipped === undefined ? false : _data$skipped;


                var oldStars = LevelManager.getStars(Utils.currentLevel, Utils.currentLevelGroup);
                var slicesRemaining = Globals.maxSlices - Globals.sliceCount;

                if (skipped) {
                    Globals.starsEarned = 0;
                } else {
                    Globals.hasWon = true;
                    Globals.starsEarned = Utils.clamp(1, slicesRemaining + 1, 3);
                }
                Globals.oldCoins = SaveManager.load('coins');

                var starDiff = Globals.starsEarned - oldStars;
                var group = LevelManager.getGroupInfo(Utils.currentLevelGroup);
                var coinsPerStar = group.coinsPerStar;
                if (starDiff > 0) {
                    SaveManager.save('coins', Globals.oldCoins + starDiff * coinsPerStar);
                }
                LevelManager.levelCompleteAnalytics(Utils.currentLevel, Globals.starsEarned, Utils.currentLevelGroup);
                LevelManager.levelWin(Utils.currentLevel, Globals.starsEarned, Utils.currentLevelGroup);
                Gui.hide('buttonRetryWide');

                CloudSave.saveToCloud(function (success, errorMsg) {
                    if (!success) {
                        console.log(errorMsg);
                        return;
                    }
                });

                if (Utils.currentLevelGroup === 'default') {
                    // regular level
                    // save nextLevel if we have enough stars
                    var next = LevelManager.getNext(Utils.currentLevel, Utils.currentLevelGroup);
                    if (next !== -1 && !LevelManager.isLocked(next, Utils.currentLevelGroup)) {
                        SaveManager.save('nextLevel', next);
                    }
                } else if (LevelManager.isChallenge(Utils.currentLevelGroup)) {
                    // 7-day challenge level
                    // schedule the time that the next level will become available


                    // next day midnight local time
                    var globalTime = TimeManager.getTime();
                    var localTime = TimeManager.getLocalTime();
                    var localDay = Math.floor(localTime / (24 * 60 * 60 * 1000));
                    var nextDay = localDay + 1;
                    var nextTime = nextDay * (24 * 60 * 60 * 1000);
                    var nextTimeGlobal = nextTime + (globalTime - localTime);
                    SaveManager.save('scheduledChallenge', nextTimeGlobal);

                    // testing: add 15 seconds
                    // let scheduledTime = TimeManager.getTime() + 15 * 1000;
                    // SaveManager.save('scheduledChallenge', scheduledTime);

                    var levelsComplete = LevelManager.getNumberCleared(Utils.currentLevelGroup);
                    var totalLevels = LevelManager.getTotalLevels(Utils.currentLevelGroup);

                    // if granted permission, and we didn't finish the challenge, set a reminder!
                    if (levelsComplete < totalLevels && SaveManager.load('notificationsAllowed') && SaveManager.load('notifications7DayChallenge')) {
                        EventSystem.fire('scheduleNotification', {
                            id: 'challengeReminder',
                            title: Localization.getText('challengeTitle').replace('{NUMBER}', totalLevels),
                            body: Localization.getText('notificationChallengeBody'),
                            time: 24 * 60 * 60 * 1000
                        });
                    }
                }

                Utils.timeout(20, function () {
                    var confetti = new Confetti({
                        updateWhenPaused: 1,
                        speedY: 0.7,
                        particleScale: 0.08,
                        z: Globals.Layers.CONFETTI,
                        global: true
                    });
                    Bento.objects.attach(confetti);
                });

                var gridOverlay = new GridBg({});
                gridOverlay.name = 'gridOverlay';
                gridOverlay.z = Globals.Layers.GUI_FILL;
                gridOverlay.alpha = 0;

                Gui.hide('anchorTopMain');
                Gui.hide('anchorBottomMain');

                var victoryDelay = 65;
                Gui.get('root', function (root) {
                    if (!root.visible) {
                        victoryDelay += 80; // longer transition when cheat enabled.
                    }
                });

                if (!skipped && Utils.currentLevelGroup === 'default' && LevelManager.getStars(Utils.currentLevel, Utils.currentLevelGroup) < 3) {
                    Utils.timeout(victoryDelay, function () {
                        Bento.objects.attach(new StarsForAdOverlay({
                            onComplete: function onComplete() {
                                // quickly go to victory
                                new Tween({
                                    from: 0, to: 1, in: 20,
                                    ease: 'easeInQuad',
                                    onStart: function onStart() {
                                        Bento.objects.attach(gridOverlay);
                                    },
                                    onUpdate: function onUpdate(v, t) {
                                        gridOverlay.alpha = v;
                                    },
                                    onComplete: function onComplete() {
                                        showVictoryScreen();
                                    }
                                });
                            }
                        }));
                    });
                } else {
                    // longer fade then go to victory
                    new Tween({
                        delay: victoryDelay,
                        from: 0, to: 1, in: 50,
                        ease: 'easeInQuad',
                        onStart: function onStart() {
                            Bento.objects.attach(gridOverlay);
                        },
                        onUpdate: function onUpdate(v, t) {
                            gridOverlay.alpha = v;
                        },
                        onComplete: function onComplete() {
                            showVictoryScreen();
                        }
                    });
                }

                // Rating popup after lv6
                if (Utils.currentLevelGroup === 'default' && Utils.currentLevel + 1 === 6 && AskRating.canAsk()) {
                    Utils.timeout(30, function () {
                        // ask for a review
                        if (Utils.isNativeIos()) {
                            AskRating.askIos10();
                        } else if (Utils.isNativeAndroid() || Bento.isDev()) {
                            // use old review funnel
                            Bento.objects.attach(new AskRating({
                                z: Globals.Layers.DIALOGS,
                                onComplete: function onComplete() {}
                            }));
                        }
                    });
                }
            }
        });

        var failListener = new EventListener({
            eventName: 'fail',
            onEvent: function onEvent() {
                Bento.objects.remove(failListener);
                Bento.objects.remove(outOfSlicesListener);
                Globals.failCount++;
                Utils.timeout(60, function () {
                    if (!Globals.hasWon) {
                        Gui.get('buttonRetry', function (buttonRetry) {
                            var retryHand = new TutorialHand({
                                moveAmount: 5,
                                squishAmount: 1.12,
                                // position: new Vector2(0, 16),
                                position: new Vector2(0, 26)
                            });
                            buttonRetry.attach(retryHand);
                            var retryText = new Text({
                                text: Localization.getText('retry'),
                                position: buttonRetry.position.add(new Vector2(0, 16)),
                                scale: new Vector2(0.5, 0.5),
                                fontSettings: TextStyles.get('retrySmall'),
                                align: 'center',
                                maxWidth: 64,
                                maxHeight: 32,
                                drawDebug: false
                            });
                            var retryWiggle = new Wiggle({});
                            new Tween({
                                from: 0, to: 1, in: 20,
                                ease: 'easeInOutQuad',
                                onStart: function onStart() {
                                    buttonRetry.parent.attach(retryText);
                                    buttonRetry.dimension.height += 10;
                                    // buttonRetry.attach(retryWiggle);
                                },
                                onUpdate: function onUpdate(v, t) {
                                    retryText.alpha = v;
                                    retryWiggle.amplitude = v * 0.03;
                                }
                            });
                        });
                    }
                });

                if (Utils.currentLevelGroup === 'default') {
                    EventSystem.fire('FirebaseAnalytics-logEvent', {
                        name: 'level_fail',
                        properties: {
                            level_id: Utils.currentLevel + 1,
                            duration: Globals.levelTimer / 60,
                            boost_used_in_level: Globals.hintsRevealed > 0,
                            amount_of_moves_left: Globals.maxSlices - Globals.sliceCount,
                            amount_of_objective_left: Utils.clamp(0, SaveManager.load('chestProgress'), 4) / 4
                        }
                    });
                }
            }
        });

        var outOfSlicesListener = new EventListener({
            eventName: 'outOfSlices',
            onEvent: function onEvent() {
                Bento.objects.remove(failListener);
                Bento.objects.remove(outOfSlicesListener);
                Globals.failCount++;
                Utils.timeout(60, function () {
                    if (!Globals.hasWon) {
                        Gui.show('buttonRetryWide');
                    }
                });
            }
        });

        var sliceListener = new EventListener({
            eventName: 'slice',
            onEvent: function onEvent() {
                Gui.hide('buttonHintContainer');
                Gui.show('buttonRetry');
                TapticEngine.impact({ type: 'medium' });
                Bento.audio.playSound('sfx_scissor');
            }
        });

        Bento.objects.attach(guiLoadedListener);
        Bento.objects.attach(nextLevelListener);
        Bento.objects.attach(winListener);
        Bento.objects.attach(failListener);
        Bento.objects.attach(outOfSlicesListener);
        Bento.objects.attach(sliceListener);
        Bento.objects.attach(mainLoadedListener);

        /**
         * Fastforward key shortcut
         */
        if (Utils.isDev()) {
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-z',
                onEvent: function onEvent() {
                    Bento.setGameSpeed(Bento.getGameSpeed() === 1 ? 3 : 1);
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-x',
                onEvent: function onEvent() {
                    Bento.setGameSpeed(Bento.getGameSpeed() === 1 ? 0.25 : 1);
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-c',
                onEvent: function onEvent() {
                    if (!Globals.hasWon) {
                        EventSystem.fire('win');
                    }
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-left',
                onEvent: function onEvent() {
                    var next = LevelManager.getPrevious(Utils.currentLevel, Utils.currentLevelGroup);
                    if (next !== -1) {
                        Utils.currentLevel = next;
                        Bento.screens.show('screens/main');
                    }
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-right',
                onEvent: function onEvent() {
                    var next = LevelManager.getNext(Utils.currentLevel, Utils.currentLevelGroup);
                    if (next !== -1) {
                        Utils.currentLevel = next;
                        Bento.screens.show('screens/main');
                    }
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-up',
                onEvent: function onEvent() {
                    var next = LevelManager.getPrevious(Utils.currentLevel - 9, Utils.currentLevelGroup);
                    if (next !== -1) {
                        Utils.currentLevel = next;
                        Bento.screens.show('screens/main');
                    }
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'buttonDown-down',
                onEvent: function onEvent() {
                    var next = LevelManager.getNext(Utils.currentLevel + 9, Utils.currentLevelGroup);
                    if (next !== -1) {
                        Utils.currentLevel = next;
                        Bento.screens.show('screens/main');
                    }
                }
            }));
            Bento.objects.attach(EventListener({
                eventName: 'win',
                onEvent: function onEvent() {
                    Bento.setGameSpeed(1);
                }
            }));
        }

        Box2D.setup({
            gravity: new Vector2(0, Box2D.toPixels(9.8)),
            canSleep: true,
            // debugDraw: true,
            baseSpeed: Globals.simulationSpeed,
            debugLayer: Globals.Layers.BOX2D_DEBUG
        });
        Bento.objects.attach(Box2D);

        Bento.objects.attach(new GridBg({}));

        var camera = new Camera({
            startLayer: Globals.Layers.CAMERA_START,
            endLayer: Globals.Layers.CAMERA_END,
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            zoom: 1.0,
            rotation: 0
        });

        Globals.camera = camera;
        Bento.objects.attach(camera);

        // let btnFastForward = new FastForwardButton({
        //     position: new Vector2(viewport.width - 16, viewport.height - 16),
        //     // position: new Vector2(40, viewport.height - 16),
        // });
        // Bento.objects.attach(btnFastForward);


        // TODO: slicing function that allows for concave bodies (made of multiple convex shapes)
        var slice2 = function slice2(p1, p2) {
            var bodiesSliced = 0;
            var entryHits = [];
            var exitHits = [];

            var isSliceable = function isSliceable(fixture) {
                return fixture.filter && fixture.filter.categoryBits & Globals.Categories.SLICEABLE;
            };
            Box2D.rayCast(p1, p2, function (fixture, point, normal, fraction) {
                if (isSliceable(fixture)) {
                    entryHits.push({ fixture: fixture, point: point, normal: normal });
                }
            });
            Box2D.rayCast(p2, p1, function (fixture, point, normal, fraction) {
                if (isSliceable(fixture)) {
                    exitHits.push({ fixture: fixture, point: point, normal: normal });
                }
            });

            // a single slice may:
            // - pass through multiple fixtures in the same body  (the sliced parts of those fixtures should be grouped together)
            // - pass through the same body more than once  (e.g. if the body is U shaped, then the resulting fixtures should _not_ be grouped together)
            // 
            // mapping from body entity IDs to array of slice data
            // each slice data contains a list of entry hits and exit hits
            // entity ID -> array[{entryHits, exitHits}]
            var slices = {};
        };

        /**
         * Slice all terrain entities from p1 to p2.
         * This causes additional terrain entities to spawn.
         * Properties such as friction, density etc. are copied.
         * Return how many shapes were sliced.
         */
        var slice = function slice(p1, p2) {
            var shapesSliced = 0;
            var entryHits = [];
            var exitHits = [];

            console.log('slice', p1, p2);

            var isSliceable = function isSliceable(fixture) {
                return fixture.filter && fixture.filter.categoryBits & Globals.Categories.SLICEABLE;
            };
            Box2D.rayCast(p1, p2, function (fixture, point, normal, fraction) {
                if (isSliceable(fixture)) {
                    entryHits.push({ fixture: fixture, point: point, normal: normal });
                    // console.log('entry', Box2D.toVector2(normal).angle());
                }
            });
            Box2D.rayCast(p2, p1, function (fixture, point, normal, fraction) {
                if (isSliceable(fixture)) {
                    exitHits.push({ fixture: fixture, point: point, normal: normal });
                    // console.log('exit', Box2D.toVector2(normal).angle());
                }
            });

            if (entryHits.length > exitHits.length) {
                for (var i = 0; i < entryHits.length; i++) {}
            }

            // Debug draw points
            /*
                entryHits.forEach(hit => {
                    let p = Box2D.toBento(hit.point);
                    Bento.objects.attach({
                        z: Globals.Layers.SLICER,
                        draw ({renderer}) {
                            renderer.fillCircle(Utils.color('#2222ff'), p.x, p.y, 1);
                        }
                    });
                });
                exitHits.forEach(hit => {
                    let p = Box2D.toBento(hit.point);
                    Bento.objects.attach({
                        z: Globals.Layers.SLICER,
                        draw ({renderer}) {
                            renderer.fillCircle(Utils.color('#ff2222'), p.x, p.y, 1);
                        }
                    });
                });
            */

            if (entryHits.length < 1 || exitHits.length < 1) {
                // no complete slices
                return shapesSliced;
            }
            // console.log('Before fixing:', entryHits, exitHits);
            console.log('Before fixing:', entryHits.map(function (hit) {
                return hit.fixture.entity.id;
            }), exitHits.map(function (hit) {
                return hit.fixture.entity.id;
            }));

            // if we started or ended inside a wall, the arrays can be misaligned so we try to fix them
            // if (entryHits[0].fixture !== exitHits[0].fixture) {
            //     if (entryHits[1] && entryHits[1].fixture === exitHits[0].fixture) {
            //         entryHits.shift();
            //     } else if (exitHits[1] && entryHits[0].fixture === exitHits[1].fixture) {
            //         exitHits.shift();
            //     }
            // }
            var minLen = Math.min(entryHits.length, exitHits.length);
            for (var _i = 0; _i < minLen; _i++) {
                if (entryHits[_i].fixture !== exitHits[_i].fixture) {
                    if (entryHits[_i + 1] && entryHits[_i + 1].fixture === exitHits[_i].fixture) {
                        entryHits.splice(_i, 1);
                        minLen = Math.min(entryHits.length, exitHits.length);
                    } else if (exitHits[_i + 1] && exitHits[_i + 1].fixture === entryHits[_i].fixture) {
                        exitHits.splice(_i, 1);
                        minLen = Math.min(entryHits.length, exitHits.length);
                    }
                }
            }

            // console.log('After fixing:', entryHits, exitHits);
            console.log('After fixing:', entryHits.map(function (hit) {
                return hit.fixture.entity.id;
            }), exitHits.map(function (hit) {
                return hit.fixture.entity.id;
            }));

            // Mapping from entity id to array of fixtures.
            // This way we can group all fixtures that came from the same body
            // However, we should check for fixtures that have no edges in common, and put those into new bodies.
            var newFixtures = {};

            for (var _i2 = 0; _i2 < entryHits.length; _i2++) {
                var entry = entryHits[_i2];
                var exit = exitHits[_i2];

                if (!entry || !exit) {
                    console.log('Missing slice entry or exit', entry, exit);
                    break;
                }

                var fixture = entry.fixture;
                var body = fixture.body;
                var entity = fixture.entity;

                if (fixture !== exit.fixture) {
                    Utils.log('Mismatch of entry/exit fixtures?');
                    console.warn(entryHits.map(function (hit) {
                        return hit.fixture.entity.id;
                    }), exitHits.map(function (hit) {
                        return hit.fixture.entity.id;
                    }));
                    continue;
                }

                var shapes = splitShape(fixture, entry.point, exit.point);
                if (!shapes) {
                    console.warn('Shape is not sliceable', fixture);
                    continue;
                }

                // ensure that shapes[0] is always the bigger shape
                var m0 = new Box2D.b2MassData();
                var m1 = new Box2D.b2MassData();
                shapes[0].ComputeMass(m0, 1.0);
                shapes[1].ComputeMass(m1, 1.0);
                if (m0.mass < m1.mass) {
                    var _ref = [shapes[1], shapes[0]];
                    shapes[0] = _ref[0];
                    shapes[1] = _ref[1];
                }

                // like body.createFixture but sets up collision handlers for haptic feedback.
                // entity.addTerrainFixture({
                body.createFixture({
                    filter: Terrain.filter,
                    shape: shapes[0],
                    friction: fixture.friction,
                    restitution: fixture.restitution,
                    density: fixture.density
                });
                var bodyType = body.getType();
                var autoType = bodyType !== 'dynamic';
                var newSettings = entity.getInheritableSettings();
                Utils.extend(newSettings, {
                    name: entity.name,
                    type: bodyType,
                    autoType: autoType,
                    position: body.getPosition(),
                    rotation: body.getAngle(),
                    fixture: {
                        filter: Terrain.filter,
                        shape: shapes[1],
                        friction: fixture.friction,
                        restitution: fixture.restitution,
                        density: fixture.density
                    }
                });
                var newTerrain = new Terrain(newSettings);
                Bento.objects.attach(newTerrain);
                body.destroyFixture(fixture);
                entity.identifyEdges();
                if (autoType) {
                    entity.chooseType();
                }

                // entity.addScar(entry.point, exit.point);
                // newTerrain.addScar(entry.point, exit.point);

                // preserve dynamic properties, code based on b2Body.Split()
                var b2Math = Box2D.b2Math;
                var b1 = body.getB2Body();
                var b2 = newTerrain.getComponent('body').getB2Body();
                var linearVelocity = b1.GetLinearVelocity().Copy();
                var angularVelocity = b1.GetAngularVelocity();
                var center = b1.GetWorldCenter();
                b1.ResetMassData();
                b2.ResetMassData();
                var center1 = b1.GetWorldCenter();
                var center2 = b2.GetWorldCenter();
                var velocity1 = b2Math.AddVV(linearVelocity, b2Math.CrossFV(angularVelocity, b2Math.SubtractVV(center1, center)));
                var velocity2 = b2Math.AddVV(linearVelocity, b2Math.CrossFV(angularVelocity, b2Math.SubtractVV(center2, center)));
                b1.SetLinearVelocity(velocity1);
                b2.SetLinearVelocity(velocity2);
                b1.SetAngularVelocity(angularVelocity);
                b2.SetAngularVelocity(angularVelocity);
                b1.SynchronizeFixtures();
                b2.SynchronizeFixtures();

                EventSystem.fire('bodySlice', [body, newTerrain.getComponent('body')]);

                shapesSliced++;
            }

            return shapesSliced;
        };

        /**
         * Polygon splitting algorithm from Velcro Physics engine, based on daid's BoxCutter
         *   https://github.com/VelcroPhysics/VelcroPhysics/blob/master/VelcroPhysics/Tools/Cutting/Simple/CuttingTools.cs
         * Returns two smaller shapes.
         */
        var splitShape = function splitShape(fixture, entryPoint, exitPoint) {
            var b2Math = Box2D.b2Math;
            var shape = fixture.shape;
            var b2body = fixture.body.getB2Body();
            var localEntryPoint = b2body.GetLocalPoint(entryPoint);
            var localExitPoint = b2body.GetLocalPoint(exitPoint);

            if (!Box2D.isPolygonShape(shape)) {
                return null;
            }

            var vertices = shape.GetVertices().slice();
            var newPolygon = [[], []];

            // offset entry/exit points if they are too close to the vertices
            vertices.forEach(function (v) {
                if (v.x === localEntryPoint.x && v.y === localEntryPoint.y) {
                    localEntryPoint.y -= Utils.EPSILON;
                }
                if (v.x === localExitPoint.x && v.y === localExitPoint.y) {
                    localExitPoint.y += Utils.EPSILON;
                }
            });

            var cutAdded = [-1, -1];
            var last = -1;
            for (var i = 0; i < vertices.length; i++) {

                var entryToExit = localExitPoint.Copy();
                entryToExit.Subtract(localEntryPoint);

                var entryToVertex = vertices[i].Copy();
                entryToVertex.Subtract(localEntryPoint);

                // find out if this vertex is on the old or new shape
                var n = b2Math.Dot(b2Math.CrossVF(entryToExit, 1), entryToVertex) > Utils.EPSILON ? 0 : 1;

                if (last !== n) {
                    if (last === 0) {
                        if (cutAdded[0] !== -1) throw "???";
                        cutAdded[0] = newPolygon[last].length;
                        newPolygon[last].push(localExitPoint);
                        newPolygon[last].push(localEntryPoint);
                    } else if (last === 1) {
                        if (cutAdded[last] !== -1) throw "???";
                        cutAdded[last] = newPolygon[last].length;
                        newPolygon[last].push(localEntryPoint);
                        newPolygon[last].push(localExitPoint);
                    }
                }
                newPolygon[n].push(vertices[i]);
                last = n;
            }

            // add the cut in case it has not been added yet.
            if (cutAdded[0] === -1) {
                cutAdded[0] = newPolygon[0].length;
                newPolygon[0].push(localExitPoint);
                newPolygon[0].push(localEntryPoint);
            }
            if (cutAdded[1] === -1) {
                cutAdded[1] = newPolygon[1].length;
                newPolygon[1].push(localEntryPoint);
                newPolygon[1].push(localExitPoint);
            }

            for (var _n = 0; _n < 2; _n++) {
                var offset = new Box2D.b2Vec2();
                if (cutAdded[_n] > 0) {
                    offset.Set(newPolygon[_n][cutAdded[_n] - 1]);
                    offset.Subtract(newPolygon[_n][cutAdded[_n]]);
                } else {
                    offset.Set(newPolygon[_n][newPolygon[_n].length - 1]);
                    offset.Subtract(newPolygon[_n][0]);
                }
                offset.Normalize();
                if (!offset.IsValid()) {
                    offset.Set(1, 1);
                }
                offset.Multiply(Utils.EPSILON);
                newPolygon[_n][cutAdded[_n]].Add(offset);

                if (cutAdded[_n] < newPolygon[_n].length - 2) {
                    offset.Set(newPolygon[_n][cutAdded[_n] + 2]);
                    offset.Subtract(newPolygon[_n][cutAdded[_n] + 1]);
                } else {
                    offset.Set(newPolygon[_n][0]);
                    offset.Subtract(newPolygon[_n][newPolygon[_n].length - 1]);
                }
                offset.Normalize();
                if (!offset.IsValid()) {
                    offset.Set(1, 1);
                }
                offset.Multiply(Utils.EPSILON);
                newPolygon[_n][cutAdded[_n] + 1].Add(offset);
            }

            return [new Box2D.PolygonShapeRaw(newPolygon[0]), new Box2D.PolygonShapeRaw(newPolygon[1])];
        };

        var slicer = {
            z: Globals.Layers.SLICER,
            name: 'slicer',
            isPressed: false,
            startPos: new Vector2(0, 0),
            endPos: new Vector2(0, 0),
            prevStartPos: new Vector2(0, 0),
            prevEndPos: new Vector2(0, 0),
            graphics: new window.PIXI.Graphics(),
            helpers: [],
            start: function start() {
                EventSystem.on('pointerDown', slicer.onPointerDown);
                EventSystem.on('pointerUp', slicer.onPointerUp);
                EventSystem.on('pointerMove', slicer.onPointerMove);
            },
            destroy: function destroy() {
                EventSystem.off('pointerDown', slicer.onPointerDown);
                EventSystem.off('pointerUp', slicer.onPointerUp);
                EventSystem.off('pointerMove', slicer.onPointerMove);
            },
            onPointerDown: function onPointerDown(data) {
                if (Gui.hasPosition(data.localPosition)) {
                    return;
                }
                var slicesRemaining = Globals.maxSlices - Globals.sliceCount;
                if (slicesRemaining <= 0) {
                    return;
                }

                var basePos = Globals.camera.worldCoords(data.position);
                var pos = basePos.clone();
                slicer.helpers.forEach(function (_ref2) {
                    var p1 = _ref2.p1,
                        p2 = _ref2.p2,
                        power = _ref2.power,
                        limit = _ref2.limit;

                    var closestDist = Utils.closestPointOnEdge(basePos, p1, p2).subtractFrom(basePos);
                    var correctionAmount = Math.min(1 / closestDist.sqrMagnitude() * power, limit);
                    console.log(correctionAmount);
                    pos.addTo(closestDist.scale(correctionAmount));
                });

                slicer.isPressed = true;
                slicer.startPos = pos.clone();
                slicer.endPos = pos.clone();
                EventSystem.fire('beginSlicing');
            },
            onPointerUp: function onPointerUp(data) {
                if (!slicer.isPressed) {
                    return;
                }
                var slicesRemaining = Globals.maxSlices - Globals.sliceCount;
                if (slicesRemaining <= 0) {
                    return;
                }
                slicer.isPressed = false;
                slicer.endPos = Globals.camera.worldCoords(data.position);
                var shapesSliced = slice(slicer.startPos, slicer.endPos);
                if (shapesSliced > 0) {
                    Globals.sliceCount++;
                    if (Globals.sliceCount === 1) {
                        EventSystem.fire('firstSlice'); // start the player moving
                    }
                    EventSystem.fire('slice');
                    if (Globals.sliceCount >= Globals.maxSlices) {
                        EventSystem.fire('outOfSlices');
                    }
                }
                EventSystem.fire('endSlicing');
            },
            onPointerMove: function onPointerMove(data) {
                slicer.endPos = Globals.camera.worldCoords(data.position);
            },
            addHelper: function addHelper(helperLine) {
                slicer.helpers.push(helperLine);
            },
            draw: function draw(_ref3) {
                var renderer = _ref3.renderer;


                if (slicer.isPressed) {
                    // let diff = slicer.endPos.subtract(slicer.startPos);
                    // slicer.graphics.clear();
                    // slicer.graphics.moveTo(slicer.startPos.x, slicer.startPos.y);

                    // // draw black/white dashed line
                    // let segments = diff.magnitude() / 3;
                    // for (let i = 1; i < segments; i++) {
                    //     let x = slicer.startPos.x + diff.x * (i/segments);
                    //     let y = slicer.startPos.y + diff.y * (i/segments);
                    //     slicer.graphics.lineStyle(0.7, (i % 2 ? 0x000000 : 0xffffff));
                    //     slicer.graphics.lineTo(x, y);
                    // }
                    // renderer.drawPixi(slicer.graphics);

                    // only redraw if the line changed
                    if (slicer.prevEndPos.x !== slicer.endPos.x || slicer.prevEndPos.y !== slicer.endPos.y || slicer.prevStartPos.x !== slicer.startPos.x || slicer.prevStartPos.y !== slicer.startPos.y) {
                        slicer.endPos.copyInto(slicer.prevEndPos);
                        slicer.startPos.copyInto(slicer.prevStartPos);
                        slicer.graphics.clear().lineStyle(0.5, 0x000000).moveTo(slicer.startPos.x, slicer.startPos.y).lineTo(slicer.endPos.x, slicer.endPos.y);
                    }

                    // slicer.helpers.forEach(({p1,p2}) => {
                    //     slicer.graphics.moveTo(p1.x, p1.y).lineTo(p2.x, p2.y);
                    // });

                    renderer.drawPixi(slicer.graphics);
                }
            }
        };
        Bento.objects.attach(slicer);

        // let levelName = 'testd4';
        var levelName = LevelManager.getName(Utils.currentLevel, Utils.currentLevelGroup);

        var lemmingCount = 0;

        new Tiled({
            assetName: levelName,
            drawTiles: false,
            merge: false,
            spawnEntities: true, // require the module (asynchronously)
            spawnBackground: true, // spawn background entities (drawn tile layers)
            attachEntities: true, onInit: function onInit(tiledJson, externalTilesets) {
                // Callback after initial parsing

                // default properties for all levels
                var properties = {
                    idealSlices: 1
                };

                if (Utils.isArray(tiledJson.properties)) {
                    tiledJson.properties.forEach(function (prop) {
                        properties[prop.name] = prop.value;
                    });
                } else if (Utils.isObject(tiledJson.properties)) {
                    for (var k in tiledJson.properties) {
                        properties[k] = tiledJson.properties[k];
                    }
                }

                Globals.maxSlices = properties.idealSlices + 2;
                Globals.failOnTurn = properties.failOnTurn || false;
                Globals.theme = properties.theme || null;

                var width = tiledJson.width,
                    height = tiledJson.height,
                    tilewidth = tiledJson.tilewidth,
                    tileheight = tiledJson.tileheight;

                var w = width * tilewidth;
                var h = height * tileheight;
                Globals.camera.zoom = viewport.width / w;
                Globals.camera.position.x = w / 2;
                Globals.camera.position.y = h / 2;
                Globals.camera.zoom *= Math.min(Gui.scaleFactor + 0.075, 1.0);
                Globals.camera.position.y -= 10 * Math.min(Gui.scaleFactor + 0.075, 1.0);
                // Globals.camera.zoom = 0.4;
                // Globals.camera.position.x = w / 2 - 60;
                // Globals.camera.position.y = h / 2 + 60;

                // preprocess all layers to find edges
                // this is needed so that terrain knows whether to draw a border
                Terrain.clearGlobalEdges();
                tiledJson.layers.forEach(function (layer) {
                    if (layer.objects) {
                        layer.objects.forEach(function (obj) {
                            if (obj.type === 'wall' || obj.type === 'terrain') {
                                if (obj.polygon) {
                                    Utils.forEachEdge(obj.polygon, function (a, b) {
                                        Terrain.addGlobalEdge(new Vector2(obj.x + a.x, obj.y + a.y), new Vector2(obj.x + b.x, obj.y + b.y));
                                    });
                                } else {
                                    var a = new Vector2(obj.x, obj.y);
                                    var b = new Vector2(obj.x + obj.width, obj.y);
                                    var c = new Vector2(obj.x + obj.width, obj.y + obj.height);
                                    var d = new Vector2(obj.x, obj.y + obj.height);
                                    Terrain.addGlobalEdge(a, b);
                                    Terrain.addGlobalEdge(b, c);
                                    Terrain.addGlobalEdge(c, d);
                                    Terrain.addGlobalEdge(d, a);
                                }
                            }
                        });
                    }
                });
            },
            onLayer: function onLayer(layer) {
                // Callback when the reader passes a layer
            },
            onTile: function onTile(tileX, tileY, tilesetJson, tileIndex) {
                // Callback after tile is drawn
            },
            onObject: function onObject(objectJson, tilesetJson, tileIndex) {
                // Callback when the reader passes a Tiled object
                // console.log(objectJson);

                var properties = {};

                if (Utils.isArray(objectJson.properties)) {
                    objectJson.properties.forEach(function (prop) {
                        properties[prop.name] = prop.value;
                    });
                } else if (Utils.isObject(objectJson.properties)) {
                    for (var k in objectJson.properties) {
                        properties[k] = objectJson.properties[k];
                    }
                }

                // extend properties with those embedded in the name (a:b,c:d)
                if (objectJson.name.indexOf(':') !== -1) {
                    var pairs = objectJson.name.split(',');
                    for (var i = 0; i < pairs.length; i++) {
                        var pair = pairs[i].split(':');
                        properties[pair[0]] = pair[1];
                    }
                }

                // other common properties

                properties.position = new Vector2(objectJson.x, objectJson.y);

                /**
                 * Combine universal properties with entity-specific ones,
                 * then use these settings to create a new entity.
                 */
                var create = function create(Fn, props) {
                    var spawnSettings = Utils.extend(properties, props);
                    return new Fn(spawnSettings);
                };

                var spawn = function spawn(Fn, props) {
                    Bento.objects.attach(create(Fn, props));
                };

                switch (objectJson.type) {
                    case 'player':
                        lemmingCount++;
                        spawn(Lemming, {
                            randomizeCharacterId: lemmingCount > 1
                        });
                        break;
                    case 'goal':
                        spawn(Goal, {});
                        break;
                    case 'wall':
                        spawn(Terrain, {
                            type: 'static',
                            tiled: objectJson,
                            isWall: true
                        });
                        break;
                    case 'terrain':
                        spawn(Terrain, {
                            tiled: objectJson
                        });
                        break;
                    case 'dynamic':
                        spawn(Terrain, {
                            type: 'dynamic',
                            tiled: objectJson,
                            isWall: true
                        });
                        break;
                    case 'kinematic':
                        spawn(Terrain, {
                            type: 'kinematic',
                            tiled: objectJson,
                            isWall: true
                        });
                        break;
                    case 'path':
                        var name = objectJson.name,
                            polyline = objectJson.polyline;

                        if (!name) {
                            Utils.log('Path needs a name!');
                            return;
                        }
                        if (!polyline) {
                            Utils.log('Path should be a polyline!');
                            return;
                        }
                        Globals.paths[name] = {
                            name: name,
                            points: polyline, // read-only
                            timePerPoint: properties.timePerPoint,
                            ease: properties.ease
                        };
                        break;
                    case 'hint':
                        var _properties$hintLevel = properties.hintLevel,
                            hintLevel = _properties$hintLevel === undefined ? 1 : _properties$hintLevel;

                        if (hintLevel > Globals.hintsAvailable) {
                            Globals.hintsAvailable = hintLevel;
                        }
                        spawn(HintLine, {
                            position: new Vector2(objectJson.x, objectJson.y),
                            points: objectJson.polyline
                        });
                        // slicer.addHelper({
                        //     p1: new Vector2(objectJson.x+objectJson.polyline[0].x, objectJson.y+objectJson.polyline[0].y),
                        //     p2: new Vector2(objectJson.x+objectJson.polyline[1].x, objectJson.y+objectJson.polyline[1].y),
                        // });
                        break;
                    case 'helper':
                        // slicer.addHelper({
                        //     p1: new Vector2(objectJson.x+objectJson.polyline[0].x, objectJson.y+objectJson.polyline[0].y),
                        //     p2: new Vector2(objectJson.x+objectJson.polyline[1].x, objectJson.y+objectJson.polyline[1].y),
                        //     power: objectJson.power || 120,
                        //     limit: objectJson.limit || 0.6,
                        // });
                        break;
                    case 'tutorial-region':
                        // somewhere that could pop up text, or suggest the user to retry?
                        // spawn(HintArea, {
                        //     tiled: objectJson
                        // });
                        break;
                    case 'fail':
                        spawn(FailRegion, {
                            tiled: objectJson
                        });
                        break;
                    case 'collectible':
                        // spawn(Collectible, {});
                        break;
                    case 'weldJoint':
                    case 'revoluteJoint':
                    case 'lineJoint':
                    case 'prismaticJoint':
                    case 'distanceJoint':
                        spawn(Joint, {
                            type: objectJson.type,
                            tiled: objectJson
                        });
                        break;
                    // case 'hinge':
                    //     spawn(Joint.Hinge, {
                    //         type: objectJson.type,
                    //         tiled: objectJson,
                    //     });
                    //     break;
                    case 'cage':
                        spawn(Cage, {
                            tiled: objectJson
                        });
                        break;
                    default:
                        if (objectJson.type !== '') {
                            Utils.log('Unknown object (name: \'' + objectJson.name + '\', type: \'' + objectJson.type + '\')');
                        }
                }
            },
            onComplete: function onComplete() {
                // Synchronous callback when the reader passed all layers
                // `this` references the tiled object (to get width and height)
                Bento.objects.attach({
                    z: Globals.Layers.END,
                    name: 'tiledComplete',
                    start: function start() {
                        // called at the end of the frame
                        EventSystem.fire('setupJoints');
                        EventSystem.fire('levelLoaded');

                        levelIsLoaded = true;
                        if (levelIsLoaded && guiIsLoaded) {
                            EventSystem.fire('mainLoaded');
                        }
                    }
                });
            },
            onLayerMergeCheck: function onLayerMergeCheck(layer) {
                // called for each layer when merge: true
                // return false if layer should not merge
                return true;
            },
            onSpawn: function onSpawn(entity) {
                // called after all a module is spawned (asynchronous)
            },
            onSpawnComplete: function onSpawnComplete() {
                // called after all modules are spawned (asynchronous)
            }
        });

        Gui.load({
            assetName: 'ui/main',
            handlers: GuiMain
        });

        Bento.objects.attach({
            name: 'bannerMaker',
            t: 0,
            update: function update() {
                if (this.t++ > 60) {
                    this.t = 0;
                    AdManager.showBanner(); // show if allowed and not already visible.
                }
            }
        });

        Bento.objects.attach(CloudSaveChecker);
    } /**
       * Main screen
       */


    function onHide() {}

    return new Screen({
        onShow: onShow,
        onHide: onHide
    });
});