'use strict';

bento.define('screens/preloader', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/components/fill', 'bento/gui/text', 'bento/entity', 'bento/eventsystem', 'bento/utils', 'bento/screen', 'bento/tween', 'entities/luckykatlogo', 'globals', 'modules/localization', 'modules/levelmanager', 'modules/notifications', 'modules/savemanager', 'modules/cloudsave', 'gui/uiloader', 'gui/iphonexoverlay', 'modules/admanager', 'modules/vipmanager', 'modules/textstyles', 'components/mask', 'modules/crosspromo/crosspromo', 'gui/vippopup', 'modules/skinmanager', 'modules/gamecenter', 'modules/googleplaygames'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Fill, Text, Entity, EventSystem, Utils, Screen, Tween, LuckyKat, Globals, Localization, LevelManager, Notifications, SaveManager, CloudSave, UILoader, IPhoneXOverlay, AdManager, VipManager, TextStyles, Mask, CrossPromo, VipPopup, SkinManager, GameCenter, GooglePlayGames) {
    /**
     * Screen description
     */
    var initPostPreloader = function initPostPreloader() {

        // hide cordova splashscreen if exists
        if (navigator.splashscreen) {
            navigator.splashscreen.hide();
        }

        Text.disposeCanvas = true;

        // Localization language is set after localized json files are loaded
        Localization.init();

        Localization.setLanguage();
        if (Utils.isDev()) {}
        // Localization.setLanguage('ja');
        // Localization.setLanguage('ko');
        // Localization.setLanguage('pt');
        // Localization.setLanguage('es');
        // Localization.setLanguage('fr');
        // Localization.setLanguage('ru');
        // Text.drawDebug = true;


        // Fix input bug (when suspending the game while touching the screen)
        EventSystem.on('touchcancel', Bento.input.resetPointers);

        console.log('Go Slice Version', window.VERSION);

        initSaveData();
    };

    // This may also be called when restoring from cloudsave

    var initSaveData = function initSaveData() {

        // Ensure players don't lose any shop items that they bought, even if that item is no longer available to buy.
        // Returns true if the key belongs to this base
        var addIfMissing = function addIfMissing(array, base, k) {
            if (!k.startsWith(base)) {
                return false;
            }
            var id = k.substring(base.length);
            if (!array.find(function (obj) {
                return obj.id === id;
            })) {
                array.push({ id: id, cost: 0 }); // NOTE, take care that this default object remains valid if new fields are added to characters/scissors/themes in the future
            }
            return true;
        };
        // This solution isn't ideal now that we have SkinManager...
        // Legacy skins should really still be defined there even if they aren't in the shop by default.
        Bento.saveState.getKeys().forEach(function (k) {
            return addIfMissing(SkinManager.getShopCharacters(), 'hasChar-', k) || addIfMissing(SkinManager.getShopScissors(), 'hasScissor-', k) || addIfMissing(SkinManager.getShopThemes(), 'hasTheme-', k);
        });

        var firstPlayedVersion = SaveManager.load('firstPlayedVersion');
        var lastPlayedVersion = SaveManager.load('lastPlayedVersion');
        var version = Utils.versionToHex(window.VERSION);
        if (firstPlayedVersion === 0) {
            firstPlayedVersion = lastPlayedVersion !== 0 ? lastPlayedVersion : version;
            SaveManager.save('firstPlayedVersion', firstPlayedVersion);
        }
        SaveManager.save('lastPlayedVersion', version);

        if (lastPlayedVersion < Utils.versionToHex('1.2.0')) {
            // now we have different kinds of notifications, carry over the player's old preference
            if (SaveManager.load('notificationsAllowed')) {
                SaveManager.save('notificationsFreeGift', true);
            }
        }

        // Go back to the level we were last playing (or the one after that if we beat it)
        // But if that level is locked, go to a sensible alternative instead.
        Utils.currentLevel = SaveManager.load('nextLevel');
        if (Utils.currentLevel < 0 || Utils.currentLevel >= LevelManager.getTotalLevels() || LevelManager.isLocked(Utils.currentLevel)) {
            Utils.currentLevel = LevelManager.getNextUncleared();
            SaveManager.save('nextLevel', Utils.currentLevel);
        }
    };

    var PIXI = window.PIXI;

    var onShow = function onShow() {
        /* Screen starts here */
        var viewport = Bento.getViewport(true);
        var loaded = false;

        if (Utils.isNativeIos()) {
            GameCenter.login();
        } else if (Utils.isNativeAndroid()) {
            // GooglePlayGames.login()
        }

        var bgScale = 0.5;
        var bgSprite = new PIXI.extras.TilingSprite(Utils.newTexture('texture/slice-white'), viewport.width / bgScale, viewport.height / bgScale);
        var background = new Entity({
            z: 0,
            name: 'bg',
            float: true,
            scale: new Vector2(bgScale, bgScale),
            components: [{
                name: 'behavior',
                draw: function draw(_ref) {
                    var renderer = _ref.renderer;

                    renderer.drawPixi(bgSprite);
                }
            }]
        });

        var barTexture = Utils.newTexture('temp/loadingbar2');
        var bar = new PIXI.Sprite(barTexture);
        var baseWidth = barTexture.width;
        barTexture.frame.width = 0;

        var loadingBar = new Entity({
            name: 'loadingBar',
            position: new Vector2(-barTexture.width / 2, -barTexture.height / 2),
            components: {
                name: 'behavior',
                draw: function draw(_ref2) {
                    var renderer = _ref2.renderer;

                    renderer.drawPixi(bar);
                }
            }
        });

        // Tween({
        //     from: 0,
        //     to: 1,
        //     in: 60,
        //     delay: 0,
        //     applyOnDelay: 0,
        //     ease: 'linear',
        //     onUpdate: function (v, t) {
        //         barTexture.frame.width = v * baseWidth;
        //         barTexture._updateUvs();
        //     }
        // });

        var loadStatus = 0;
        var fakeLoad = 0;
        var loadingPercentage = 0;
        var loadDisplay = new Entity({
            z: 0,
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5 + 10),
            scale: new Vector2(0.16, 0.16),
            components: [{
                name: 'behavior',
                update: function update() {
                    // if (!loaded) {
                    //     if (loadingPercentage === 0 && fakeLoad < 0.4) {
                    //         fakeLoad += 0.05;
                    //     }
                    //     // loadingBar.scale.x = fakeLoad + (0.9 - fakeLoad) * loadingPercentage;
                    // }
                    // barTexture.frame.width = loadingPercentage * baseWidth;
                    // console.log(loadingPercentage * baseWidth);
                    // barTexture._updateUvs();
                }
            }, new Sprite({
                imageName: 'temp/loadingbar-border',
                originRelative: new Vector2(0.5, 0.5)
            }), loadingBar]
        });

        var end = function end() {
            if (loaded) {
                // Utils.setDev(true);

                Utils.assetsLoaded = true;
                EventSystem.fire('assetsLoaded');

                // ensure character textures are always sharp
                // let pixelart = Bento.assets.getAssetGroups()['pixelart'];
                // let imgs = pixelart['images'] || pixelart['packed-images'];
                // if (imgs) {
                //     console.log(JSON.stringify(imgs));
                //     Object.keys(imgs).forEach(img => {
                //         Utils.newTexture(img, window.PIXI.SCALE_MODES.NEAREST);
                //     });
                // } else {
                //     Utils.log('Couldn\'t identify pixelart images folder');
                // }
                SkinManager.getAllCharacters().forEach(function (char) {
                    Utils.newTexture('character' + char.id + '-idle', PIXI.SCALE_MODES.NEAREST);
                    Utils.newTexture('character' + char.id + '-victory', PIXI.SCALE_MODES.NEAREST);
                    Utils.newTexture('character' + char.id + '-walk', PIXI.SCALE_MODES.NEAREST);
                });
                // Utils.newTexture(`main/gem`, PIXI.SCALE_MODES.NEAREST);


                // iPhone X Overlay
                if (Utils.isDev()) {
                    Bento.objects.attach(new IPhoneXOverlay({
                        z: Globals.Layers.IPHONE_X_OVERLAY,
                        orientation: 'portrait',
                        autoHide: true
                    }));
                }

                if (CrossPromo.canShow()) {
                    Globals.crossPromo = new CrossPromo({});
                }
                EventSystem.on('levelLoaded', function () {
                    // load new crosspromo every time we start a new level
                    // disabled for now due to performance issues... :|
                    // CrossPromo.shuffleApp();
                    // Globals.crossPromo = new CrossPromo({});
                });

                VipPopup.loadTerms();

                Bento.screens.show('screens/splash', {
                    loadDisplay: loadDisplay
                });

                // Clear textures on screen change.
                // EventSystem.on('screenHidden', () => {
                //     var images = Bento.assets.getAssets().images;
                //     Object.keys(images).forEach(function (key) {
                //         var image = images[key];
                //         if (image.texture) {
                //             image.texture.destroy();
                //             image.texture = null;
                //         }
                //     });
                // });

                AdManager.showBanner();
            }
        };
        var loadFonts = function loadFonts() {
            var fonts = Bento.assets.getAssetGroups().preloader.fonts;
            var font = void 0;
            if (!fonts) {
                return;
            }
            for (font in fonts) {
                if (!fonts.hasOwnProperty(font)) {
                    continue;
                }
                new Text({
                    position: new Vector2(1000, 1000),
                    text: '.',
                    font: font
                });
            }
        };

        initPostPreloader();
        loadFonts();

        Bento.objects.attach(background);

        var text = new Text({
            text: Localization.getText('loading').toUpperCase(),
            position: new Vector2(0, -80),
            scale: new Vector2(3.5, 3.5),
            fontSettings: TextStyles.get('loading'),
            align: 'center',
            maxWidth: 120,
            maxHeight: 32,
            drawDebug: false
        });
        Tween({
            from: 0, to: 1, in: 30,
            ease: 'easeInOutQuad',
            onStart: function onStart() {
                loadDisplay.attach(text);
            },
            onUpdate: function onUpdate(v, t) {
                text.alpha = v;
            }
        });
        Bento.objects.attach(loadDisplay);

        // we will now load all other assets
        Bento.assets.loadAllAssets({
            exceptions: ['preloader'], onLoaded: function onLoaded(current, total) {
                // show how many assets still to be loaded
                // text.setText('Loading ' + current + '/' + total);
                loadingPercentage = Math.min(current / total, 1);
                barTexture.frame.width = loadingPercentage * baseWidth;
                barTexture._updateUvs();
            },
            onReady: function onReady() {
                UILoader.preload({
                    onComplete: function onComplete() {
                        loaded = true;
                        end();
                    }
                });
            }
        });
    };

    var preloader = new Screen({
        onShow: onShow
    });

    preloader.initSaveData = initSaveData;

    return preloader;
});