'use strict';

bento.define('screens/victoryscreen', ['bento', 'bento/math/vector2', 'bento/math/rectangle', 'bento/components/sprite', 'bento/components/clickable', 'bento/entity', 'bento/eventsystem', 'bento/gui/clickbutton', 'bento/gui/counter', 'bento/gui/text', 'bento/utils', 'bento/screen', 'bento/tween', 'bento/components/fill', 'bento/components/eventlistener', 'entities/gridbg', 'globals', 'modules/levelmanager', 'gui/gui', 'gui/guivictory'], function (Bento, Vector2, Rectangle, Sprite, Clickable, Entity, EventSystem, ClickButton, Counter, Text, Utils, Screen, Tween, Fill, EventListener, GridBg, Globals, LevelManager, Gui, GuiVictory) {

    /**
     * Generally, GuiVictory appears as an overlay on the main screen.
     * Therefore this screen just exists for returning to GuiVictory from other screens such as the shop.
     */

    var onShow = function onShow() {
        var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var restore = settings.restore;


        var gridOverlay = Bento.objects.get('gridOverlay');
        if (gridOverlay) {
            // move grid overlay into background (avoid flicker)
            gridOverlay.global = false;
            gridOverlay.z = Globals.Layers.GRID;
        } else {
            // create a new background
            Bento.objects.attach(new GridBg({}));
        }

        if (restore) {
            if (GuiVictory.savedState) {
                Gui.restoreState(GuiVictory.savedState);
            } else {
                Utils.log('Failed to return to victory screen, no stored root');
                Bento.screens.show('titlescreen');
            }
        } else {
            Gui.load({
                assetName: 'ui/victoryscreen',
                handlers: GuiVictory
            });
        }

        Bento.objects.attach(new EventListener({
            eventName: 'nextLevel',
            onEvent: function onEvent() {
                LevelManager.gotoNextLevel();
            }
        }));
    };

    return new Screen({
        onShow: onShow
    });
});